#include "EthernetOperation.hpp"



EthernetOperation::EthernetOperation(OPERATION_TYPE oprationType, string helpContent) : OperationBaseClass(oprationType, helpContent)
{

}



EthernetOperation::~EthernetOperation()
{
  
}



int
EthernetOperation::executeOperation(StringParser command_line)
{
	  string parameter_1;
	  int    err = 0;


    command_line.getParameter(&parameter_1, 0);

    //If the first parameter is "help" or space, show help message         
    if (parameter_1 == HELP || parameter_1 == "")
    {
        Help_parameter(command_line);
        err = 0;
        return err;
    }

    //Execute operation
    switch (getOperationType())
    {
        case OPERATION_ALL:
        case OPERATION_NODE:
        case OPERATION_LINK:
        case OPERATION_SHOW_SPEED:
        case OPERATION_FW:
            err = show_all(command_line);
            break;
        case OPERATION_MDI:
            err = mdi(command_line);
            break;
        case OPERATION_INTERRUPT:
            err = interrupt(command_line);
            break;
        case OPERATION_INTERNAL_LOOPBACK_TEST:
            err = internalLoopback(command_line);
            break;
        case OPERATION_SINGLE_LOOPBACK_TEST:
            err = singleLoopback(command_line);
            break;
        case OPERATION_CROSS_LOOPBACK_TEST:
            err = crossLoopback(command_line);
            break;
        case OPERATION_READ_MAC:
            err = readMAC(command_line);
            break;
        case OPERATION_WRITE_MAC:
            err = writeMAC(command_line);
            break;
        case OPERATION_READ_EEPROM:
            err = readEEPROM(command_line);
            break;
        case OPERATION_WRITE_EEPROM:
            err = writeEEPROM(command_line);
            break;
        case OPERATION_VALIDATE_EEPROM:
            err = validateEEPROM(command_line);
            break;
        case OPERATION_PROGRAM:
            err = programEEPROM(command_line);
            break;
        case OPERATION_READ_SFP:
            err = readSFP(command_line);
            break;
        case OPERATION_WRITE_SFP:
            err = writeSFP(command_line);
            break;
        case OPERATION_ERROR_ALL:
            err = show_all_error(command_line);
            break;
        case OPERATION_ERROR_STATISTICS:
            err = statistic(command_line);
            break;
        case OPERATION_ERROR_PCI:
            err = pciError(command_line);
            break;
        case OPERATION_CLEAR:
            err = clear(command_line);
            break;
        default:
            cout << "Unsupport operation" << endl;
            err = -1; 
            break;
    }

    return err;
}



int
EthernetOperation::Help_parameter(StringParser command_line)
{
    string module, command, operation;

    command_line.getModule(&module);
    command_line.getCommand(&command);
    getName(&operation);

    cout << "Usage:	lemur " << module << " " << command << " " << operation << " ";
    //Show parameter help message depend on operation type
    switch (getOperationType())
    {
        case OPERATION_ALL:
            cout << "P1" << endl;
            cout << "Show all ethernet information" << endl;
            cout << " Syntax: all P1" << endl;
            cout << " P1: Device name" << endl;
            break;
        case OPERATION_NODE:
            cout << "P1" << endl;
            cout << "Show ethernet node name" << endl;
            cout << " Syntax: node P1" << endl;
            cout << " P1: Device name" << endl;
            break;
        case OPERATION_LINK:
            cout << "P1" << endl;
            cout << "Show ethernet link status" << endl;
            cout << " Syntax: link P1" << endl;
            cout << " P1: Device name" << endl;
            break;
        case OPERATION_SHOW_SPEED:
            cout << "P1" << endl;
            cout << "Show ethernet current speed" << endl;
            cout << " Syntax: link P1" << endl;
            cout << " P1: Device name" << endl;
            break;
        case OPERATION_FW:
            cout << "P1" << endl;
            cout << "Show ethernet F.W version" << endl;
            cout << " Syntax: link P1" << endl;
            cout << " P1: Device name" << endl;
            break;
        case OPERATION_MDI:
            cout << "P1" << endl;
            cout << "Check the link is up or not" << endl;
            cout << " Syntax: mdi p1" << endl;
            cout << " P1: Device name" << endl;
            break;
        case OPERATION_INTERRUPT:
            cout << "P1" << endl;
            cout << "Test device interrupt" << endl;
            cout << " Syntax: interrupt p1" << endl;
            cout << " P1: Device name" << endl;
            break;
        case OPERATION_INTERNAL_LOOPBACK_TEST:
            cout << "P1" << endl;
            cout << "Test device internal loopback" << endl;
            cout << " Syntax: interrupt p1" << endl;
            cout << " P1: Device name" << endl;
            break;
        case OPERATION_SINGLE_LOOPBACK_TEST:
            cout << "P1 P2" << endl;
            cout << "Test device single loopback" << endl;
            cout << " Syntax: interloopback p1 p2" << endl;
            cout << " P1: test speed" << endl;
            cout << " P2: Device name" << endl;
            break;
        case OPERATION_CROSS_LOOPBACK_TEST:
            cout << "P1 P2 P3" << endl;
            cout << "Test device cross loopback" << endl;
            cout << " Syntax: interloopback p1 p2 p3" << endl;
            cout << " P1: test speed" << endl;
            cout << " P2: Source device name" << endl;
            cout << " P3: Destination device name" << endl;
            break;
        case OPERATION_READ_MAC:
            cout << "P1" << endl;
            cout << "Read a MAC address from Etherent" << endl;
            cout << " Syntax: read p1" << endl;
            cout << " P1: Device name" << endl;
            break;
        case OPERATION_WRITE_MAC:
            cout << "P1 P2" << endl;
            cout << "Write a MAC address to Etherent" << endl;
            cout << " Syntax: write p1 p2" << endl;
            cout << " P1: MAC address" << endl;
            cout << " P2: Device name" << endl;
            break;
        case OPERATION_READ_EEPROM:
            cout << "P1 P2 P3 P4" << endl;
            cout << "Read data from EEPROM" << endl;
            cout << " Syntax: read p1 p2 p3 p4" << endl;
            cout << " P1: Start address(Hex)" << endl;
            cout << " P2: Unit, 8/16/32(Dec)" << endl;
            cout << " P3: Length(Hex)" << endl;
            cout << " P4: Device name" << endl;
            break;
        case OPERATION_WRITE_EEPROM:
            cout << "P1 P2 P3 P4" << endl;
            cout << "Write data to EEPROM" << endl;
            cout << " Syntax: write p1 p2 p3 p4" << endl;
            cout << " P1: Start address(Hex)" << endl;
            cout << " P2: Unit, 8/16/32(Dec)" << endl;
            cout << " P3: Data(Hex)" << endl;
            cout << " P4: Device name" << endl;
            break;
        case OPERATION_VALIDATE_EEPROM:
            cout << "P1" << endl;
            cout << "Validate the content by calculate checksum" << endl;
            cout << " Syntax: validate P1" << endl;
            cout << " P1: Device name" << endl;
            break;
        case OPERATION_PROGRAM:
            cout << "P1 P2" << endl;
            cout << "Program EEPROM Image" << endl;
            cout << " Syntax: program P1 P2" << endl;
            cout << " P1: Path/FileName" << endl;
            cout << " P2: Device name" << endl;
            break;
        case OPERATION_READ_SFP:
            cout << "P1 P2 P3 P4" << endl;
            cout << "Read data from SFP" << endl;
            cout << " Syntax: program P1 P2 P3" << endl;
            cout << " P1: Device Address(Hex)" << endl;
            cout << " P2: Start Address(Hex)" << endl;
            cout << " P3: Length/byte(Hex)" << endl;
            cout << " P4: Device Name" << endl;
            break;
        case OPERATION_WRITE_SFP:
            cout << "P1 P2 P3" << endl;
            cout << "Read data from SFP" << endl;
            cout << " Syntax: program P1 P2 P3" << endl;
            cout << " P1: Device Address(Hex)" << endl;
            cout << " P2: Start Address(Hex)" << endl;
            cout << " P3: Data(Hex)" << endl;
            cout << " P4: Device name" << endl;
            break;
        case OPERATION_ERROR_ALL:
            cout << "P1" << endl;
            cout << "Show ethernet all error(statistics and PCI)" << endl;
            cout << " Syntax: all P1" << endl;
            cout << " P1: Device name" << endl;
            break;
        case OPERATION_ERROR_STATISTICS:
            cout << "P1" << endl;
            cout << "Show ethernet statistic" << endl;
            cout << " Syntax: statistics P1" << endl;
            cout << " P1: Device name" << endl;
            break;
        case OPERATION_ERROR_PCI:
            cout << "P1" << endl;
            cout << "Show ethernet PCI error count" << endl;
            cout << " Syntax: pci P1" << endl;
            cout << " P1: Device name" << endl;
            break;
        case OPERATION_CLEAR:
            cout << "P1" << endl;
            cout << "Clear statistic and PCI error count" << endl;
            cout << " Syntax: clear P1" << endl;
            cout << " P1: Device name" << endl;
            break;
       default:
            cout << "not define parameter help message" << endl;
            break;
    }
    
    return 0;
}



int
EthernetOperation::show_all(StringParser command_line)
{
    int           err = 0;
    EthernetDAPI  *pEthernetDAPI = NULL;
    ErrorMsg      *error_code;
    string        WellKnownName, Node, Link, Speed, FWVersion, error_massage;
    string        PCI_BDF, PCI_Speed, PCI_Width;
    string        Operation;


    error_code = NO_ERROR_MSG;

    err = Syntax_check(command_line);

    if (err == 0)
    {
        //Get Ethernet device name
        command_line.getParameter(&WellKnownName, 0);
        //cout << "Name: " <<  WellKnownName << endl;


        //Creat Ethernet DAPI object
        pEthernetDAPI = new EthernetDAPI("EthernetDAPI");

        //Get device node by API
        error_code = pEthernetDAPI->GetNode(WellKnownName, &Node);
        if (error_code != NO_ERROR_MSG)
        {
            printError(error_code);
            return -1;
        }

        //Get device link by API
        error_code = pEthernetDAPI->GetLinkStatus(WellKnownName, &Link);
        if (error_code != NO_ERROR_MSG)
        {
            printError(error_code);
            return -1;
        }

        //Get device speed by API
        error_code = pEthernetDAPI->GetSpeedDuplex(WellKnownName, &Speed);
        if (error_code != NO_ERROR_MSG)
        {
            printError(error_code);
            return -1;
        }

        //Get device F.W version by API
        error_code = pEthernetDAPI->GetFirmwareVersion(WellKnownName, &FWVersion);
        if (error_code != NO_ERROR_MSG)
        {
            printError(error_code);
            return -1;
        }

        //Get device PCI BDF by API
        error_code = pEthernetDAPI->GetPCIBDF(WellKnownName, &PCI_BDF);
        if (error_code != NO_ERROR_MSG)
        {
            printError(error_code);
            return -1;
        }

        //Get device PCI speed by API
        error_code = pEthernetDAPI->GetPCISpeed(WellKnownName, &PCI_Speed);
        if (error_code != NO_ERROR_MSG)
        {
            printError(error_code);
            return -1;
        }

        //Get device PCI width by API
        error_code = pEthernetDAPI->GetPCIWidth(WellKnownName, &PCI_Width);
        if (error_code != NO_ERROR_MSG)
        {
            printError(error_code);
            return -1;
        }

        //Get the opration type then decied which type data to show
        command_line.getOperation(&Operation);

        if (Operation == "all")
        {
            cout << "Node: " << Node << endl;
            cout << "Link: " << Link << endl;
            cout << "Speed: " << Speed << endl;
            cout << "FW Version: " << FWVersion << endl;
            cout << "PCI BDF: " << PCI_BDF << endl;
            cout << "PCI Speed: " << PCI_Speed << " GT/s" << endl;
            cout << "PCI Width: x" << PCI_Width << endl;
        }
        else if (Operation == "node")
        {
            cout << "Node: " << Node << endl;
        }
        else if (Operation == "link")
        {
            cout << "Link: " << Link << endl;
        }
        else if (Operation == "speed")
        {
            cout << "Speed: " << Speed << endl;
        }
        else if (Operation == "fw")
        {
            cout << "FW Version: " << FWVersion << endl;
        }

        delete pEthernetDAPI;
    }

    return err;
}



int
EthernetOperation::mdi(StringParser command_line)
{
    int           err = 0;
    EthernetDAPI  *pEthernetDAPI = NULL;
    ErrorMsg      *error_code;
    string        WellKnownName;
    int           result = 0;

    error_code = NO_ERROR_MSG;

    err = Syntax_check(command_line);

    if (err == 0)
    {
        //Get Ethernet device name
        command_line.getParameter(&WellKnownName, 0);
        //cout << "Name: " <<  WellKnownName << endl;


        //Creat Ethernet DAPI object
        pEthernetDAPI = new EthernetDAPI("EthernetDAPI");

        //Start test by API
        error_code = pEthernetDAPI->MDIRegisterTest(WellKnownName, &result);

        if (error_code != NO_ERROR_MSG)
        {
            printError(error_code);
            err = -1;
        }

        delete pEthernetDAPI;
    }

    return err;
}



int
EthernetOperation::interrupt(StringParser command_line)
{
    int           err = 0;
    EthernetDAPI  *pEthernetDAPI = NULL;
    ErrorMsg      *error_code;
    string        WellKnownName;
    int           result = 0;

    error_code = NO_ERROR_MSG;

    err = Syntax_check(command_line);

    if (err == 0)
    {
        //Get Ethernet device name
        command_line.getParameter(&WellKnownName, 0);
        //cout << "Name: " <<  WellKnownName << endl;


        //Creat Ethernet DAPI object
        pEthernetDAPI = new EthernetDAPI("EthernetDAPI");

        //Start test by API
        error_code = pEthernetDAPI->InterruptTest(WellKnownName, &result);
        if (error_code != NO_ERROR_MSG)
        {
            printError(error_code);
            err = -1;
        }

        delete pEthernetDAPI;
    }

    return err;
}



int
EthernetOperation::internalLoopback(StringParser command_line)
{
    int           err = 0;
    EthernetDAPI  *pEthernetDAPI = NULL;
    ErrorMsg      *error_code;
    string        WellKnownName;
    int           result = 0;

    error_code = NO_ERROR_MSG;

    err = Syntax_check(command_line);

    if (err == 0)
    {
        //Get Ethernet device name
        command_line.getParameter(&WellKnownName, 0);
        //cout << "Name: " <<  WellKnownName << endl;


        //Creat Ethernet DAPI object
        pEthernetDAPI = new EthernetDAPI("EthernetDAPI");

        //Start test by API
        error_code = pEthernetDAPI->InternalLoopbackTest(WellKnownName, &result);

        if (error_code != NO_ERROR_MSG)
        {
            printError(error_code);
            err = -1;
        }

        delete pEthernetDAPI;
    }

    return err;
}



int
EthernetOperation::singleLoopback(StringParser command_line)
{
    int           err = 0, speed;
    EthernetDAPI  *pEthernetDAPI = NULL;
    ErrorMsg      *error_code;
    string        WellKnownName, temp_string, statistics;
    float         throughput;

    error_code = NO_ERROR_MSG;

    err = Syntax_check(command_line);

    if (err == 0)
    {
        //Get Ethernet device name
        command_line.getParameter(&WellKnownName, 1);
        //cout << "Name: " <<  WellKnownName << endl;

        //Get test speed
        command_line.getParameter(&temp_string, 0);
        command_line.convertStringToDec(temp_string, &speed);
        //cout << "speed: " <<  speed << endl;

        //Creat Ethernet DAPI object
        pEthernetDAPI = new EthernetDAPI("EthernetDAPI");

        //Start test by API
        error_code = pEthernetDAPI->Extrenal_loopback(WellKnownName, speed, &throughput, &statistics);
        delete pEthernetDAPI;
//        if (error_code != NO_ERROR_MSG)
//        {
//            printError(error_code);
//            return -1;
//        }

        cout << "Throughput: " << throughput << " Mb/s" << endl;
        cout << statistics ;

        printError(error_code);

        if (error_code != NO_ERROR_MSG)
        {
            err = -1;
        }
    }

    return err;
}



int
EthernetOperation::crossLoopback(StringParser command_line)
{
    int           err = 0, speed;
    EthernetDAPI  *pEthernetDAPI = NULL;
    ErrorMsg      *error_code;
    string        WellKnownNameSource, WellKnownNameDestination, temp_string, statistics;
    float         throughput;

    error_code = NO_ERROR_MSG;

    err = Syntax_check(command_line);

    if (err == 0)
    {
        //Get Ethernet source device name
        command_line.getParameter(&WellKnownNameSource, 1);
        //cout << "Source Name: " <<  WellKnownNameSource << endl;

        //Get Ethernet destination device name
        command_line.getParameter(&WellKnownNameDestination, 2);
        //cout << "Destination Name: " <<  WellKnownNameDestination << endl;

        //Get test speed
        command_line.getParameter(&temp_string, 0);
        command_line.convertStringToDec(temp_string, &speed);
        //cout << "speed: " <<  speed << endl;

        //Creat Ethernet DAPI object
        pEthernetDAPI = new EthernetDAPI("EthernetDAPI");

        //Start test by API
        error_code = pEthernetDAPI->Extrenal_loopback(WellKnownNameSource, WellKnownNameDestination, speed, &throughput, &statistics);
        delete pEthernetDAPI;
//        if (error_code != NO_ERROR_MSG)
//        {
//            printError(error_code);
//            return -1;
//        }

        cout << "Throughput: " << throughput << " Mb/s" << endl;
        cout << statistics ;

        printError(error_code);

        if (error_code != NO_ERROR_MSG)
        {
            err = -1;
        }
    }

    return err;
}



int
EthernetOperation::readMAC(StringParser command_line)
{
    int           err = 0;
    EthernetDAPI  *pEthernetDAPI = NULL;
    ErrorMsg      *error_code;
    string        WellKnownName, MAC;

    error_code = NO_ERROR_MSG;

    err = Syntax_check(command_line);

    if (err == 0)
    {
        //Get Ethernet device name
        command_line.getParameter(&WellKnownName, 0);
        //cout << "Name: " <<  WellKnownName << endl;


        //Creat Ethernet DAPI object
        pEthernetDAPI = new EthernetDAPI("EthernetDAPI");

        //Start read MAC by API
        error_code = pEthernetDAPI->ReadMacAddress(WellKnownName, &MAC);

        if (error_code == NO_ERROR_MSG)
        {
            cout << MAC << endl;
        }
        printError(error_code);

        if (error_code != NO_ERROR_MSG)
        {
            err = -1;
        }

        delete pEthernetDAPI;
    }

    return err;
}



int
EthernetOperation::writeMAC(StringParser command_line)
{
    int           err = 0, index, find = 0;
    EthernetDAPI  *pEthernetDAPI = NULL;
    ErrorMsg      *error_code;
    string        WellKnownName, MACStress, MAC[6];

    error_code = NO_ERROR_MSG;

    err = Syntax_check(command_line);

    if (err == 0)
    {
        //Get Ethernet device name
        command_line.getParameter(&WellKnownName, 1);
        //cout << "Name: " <<  WellKnownName << endl;

        //Get MAC address
        command_line.getParameter(&MACStress, 0);
        //cout << "MAC: " <<  MAC << endl;

        for (index = 0; index < 6; index++)
        {
            MAC[index] = MACStress.substr(find, 2);
            find = MACStress.find(":", find);
            find++;
        }

        //Creat Ethernet DAPI object
        pEthernetDAPI = new EthernetDAPI("EthernetDAPI");

        //Start write MAC by API
        error_code = pEthernetDAPI->WriteMacAddress(WellKnownName, MAC);
        if (error_code != NO_ERROR_MSG)
        {
            printError(error_code);
            return -1;
        }

        //Show write MAC content
        MAC[0] = "";
        error_code = pEthernetDAPI->ReadMacAddress(WellKnownName, &MAC[0]);

        cout << WellKnownName << " MAC:" << MAC[0] << endl;
        printError(error_code);

        if (error_code != NO_ERROR_MSG)
        {
            err = -1;
        }

        delete pEthernetDAPI;
    }

    return err;
}



int
EthernetOperation::readEEPROM(StringParser command_line)
{
    int           err = 0, i, data[16 * 1024];
    EthernetDAPI  *pEthernetDAPI = NULL;
    ErrorMsg      *error_code;
    string        WellKnownName, temp_string;
    int           StartAddress, Unit, Length;

    error_code = NO_ERROR_MSG;

    err = Syntax_check(command_line);

    if (err == 0)
    {
        //Get Ethernet device name
        command_line.getParameter(&WellKnownName, 3);
        //cout << "Name: " <<  WellKnownName << endl;

        //Get start address
        command_line.getParameter(&temp_string, 0);
        command_line.convertStringToHex(temp_string, &StartAddress);
        //cout << "Start address: " <<  StartAddress << endl;

        //Get unit value
        command_line.getParameter(&temp_string, 1);
        command_line.convertStringToDec(temp_string, &Unit);
        //cout << "Unit: " <<  Unit << endl;

        //Get length value
        command_line.getParameter(&temp_string, 2);
        command_line.convertStringToHex(temp_string, &Length);
        //cout << "Length: " <<  Length << endl;


        //Creat Ethernet DAPI object
        pEthernetDAPI = new EthernetDAPI("EthernetDAPI");

        //Start read data from EEPROM by API
        error_code = pEthernetDAPI->ReadEEPROM(WellKnownName, StartAddress, Length, data);
        if (error_code != NO_ERROR_MSG)
        {
            printError(error_code);
            return -1;
        }

        //Convert unit from byte to byte length
        switch (Unit)
        {
            case 8:
                Unit = 1;
                break;
            case 16:
                Unit = 2;
                break;
            case 32:
                Unit = 4;
                break;
            default:
                Unit = 1;
        }


        //Show getted data content
        for (i=0; i < Length ; i++)
        {
            if (i == 0 || (i % 16) == 0 )
            {
                printf("%08x: ", StartAddress+i);
            }

            printf("%02x", data[i]);
            if (((i+1) % Unit) == 0)
            {
                printf(" ");
            }

            if (((i+1) % 16) == 0  && i != (Length - 1))
            {
                printf("\n");
             }
        }
        printf("\n");

        delete pEthernetDAPI;
    }

    return err;
}



int
EthernetOperation::writeEEPROM(StringParser command_line)
{
    int           err = 0, i;
    EthernetDAPI  *pEthernetDAPI = NULL;
    ErrorMsg      *error_code;
    string        WellKnownName, temp_string;
    int           StartAddress, Unit, Data[16 * 1024], Length;

    error_code = NO_ERROR_MSG;

    err = Syntax_check(command_line);

    if (err == 0)
    {
        //Get Ethernet device name
        command_line.getParameter(&WellKnownName, 3);
        //cout << "Name: " <<  WellKnownName << endl;

        //Get start address
        command_line.getParameter(&temp_string, 0);
        command_line.convertStringToHex(temp_string, &StartAddress);
        //cout << "Start address: " <<  StartAddress << endl;

        //Get unit value
        command_line.getParameter(&temp_string, 1);
        command_line.convertStringToDec(temp_string, &Unit);
        //cout << "Unit: " <<  Unit << endl;

        //Get write data value
        command_line.getParameter(&temp_string, 2);
        command_line.convertHexStringToArrary(temp_string, Data, &Length);
        /*for (i = 0; i < Length; i++)
            printf("%02x ", Data[i]);
        printf("\n");*/

        //Creat Ethernet DAPI object
        pEthernetDAPI = new EthernetDAPI("EthernetDAPI");

        //Start Write data to EEPROM by API
        error_code = pEthernetDAPI->WriteEEPROM(WellKnownName, StartAddress, Data, Length);
        if (error_code != NO_ERROR_MSG)
        {
            printError(error_code);
            return -1;
        }

        //Convert unit from byte to byte length
        switch (Unit)
        {
            case 8:
                Unit = 1;
                break;
            case 16:
                Unit = 2;
                break;
            case 32:
                Unit = 4;
                break;
            default:
                Unit = 1;
        }

        //Read data from EEPROM by API
        error_code = pEthernetDAPI->ReadEEPROM(WellKnownName, StartAddress, Length, Data);
        if (error_code != NO_ERROR_MSG)
        {
            printError(error_code);
            return -1;
        }


        //Show getted data content
        for (i=0; i < Length ; i++)
        {
            if (i == 0 || (i % 16) == 0 )
            {
                printf("%08x: ", StartAddress+i);
            }

            printf("%02x", Data[i]);
            if (((i+1) % Unit) == 0)
            {
                printf(" ");
            }

            if (((i+1) % 16) == 0  && i != (Length - 1))
            {
                printf("\n");
             }
        }
        printf("\n");

        delete pEthernetDAPI;
    }

    return err;
}



int
EthernetOperation::validateEEPROM(StringParser command_line)
{
    int           err = 0;
    EthernetDAPI  *pEthernetDAPI = NULL;
    ErrorMsg      *error_code;
    string        WellKnownName;
    int           result = 0;

    error_code = NO_ERROR_MSG;

    err = Syntax_check(command_line);

    if (err == 0)
    {
        //Get Ethernet device name
        command_line.getParameter(&WellKnownName, 0);
        //cout << "Name: " <<  WellKnownName << endl;


        //Creat Ethernet DAPI object
        pEthernetDAPI = new EthernetDAPI("EthernetDAPI");

        //Start test by API
        error_code = pEthernetDAPI->EepromValidationTest(WellKnownName, &result);
        if (error_code != NO_ERROR_MSG)
        {
            printError(error_code);
            return -1;
        }

        delete pEthernetDAPI;
    }

    return err;
}



int
EthernetOperation::programEEPROM(StringParser command_line)
{
    int           err = 0;
    EthernetDAPI  *pEthernetDAPI = NULL;
    ErrorMsg      *error_code;
    string        WellKnownName, FilePath;

    error_code = NO_ERROR_MSG;

    err = Syntax_check(command_line);

    if (err == 0)
    {
        //Get Ethernet device name
        command_line.getParameter(&WellKnownName, 1);
        //cout << "Name: " <<  WellKnownName << endl;

        //Get Ethernet device name
        command_line.getParameter(&FilePath, 0);
        //cout << "Name: " <<  WellKnownName << endl;


        //Creat Ethernet DAPI object
        pEthernetDAPI = new EthernetDAPI("EthernetDAPI");

        //Start program by API
        error_code = pEthernetDAPI->Program(WellKnownName, FilePath);
        if (error_code != NO_ERROR_MSG)
        {
            printError(error_code);
            return -1;
        }

        cout << "Completed!" << endl;

        delete pEthernetDAPI;
    }

    return err;
}



int
EthernetOperation::readSFP(StringParser command_line)
{
    int           err = 0, i, data[1024];
    EthernetDAPI  *pEthernetDAPI = NULL;
    ErrorMsg      *error_code;
    string        WellKnownName, temp_string;
    int           StartAddress, Unit, Length, DeviceAddress;

    error_code = NO_ERROR_MSG;

    err = Syntax_check(command_line);

    if (err == 0)
    {
        //Get Ethernet device name
        command_line.getParameter(&WellKnownName, 3);
        //cout << "Name: " <<  WellKnownName << endl;

        //Device address
        command_line.getParameter(&temp_string, 0);
        command_line.convertStringToHex(temp_string, &DeviceAddress);

        //Get start address
        command_line.getParameter(&temp_string, 1);
        command_line.convertStringToHex(temp_string, &StartAddress);
        //cout << "Start address: " <<  StartAddress << endl;

        //Get length value
        command_line.getParameter(&temp_string, 2);
        command_line.convertStringToHex(temp_string, &Length);
        //cout << "Length: " <<  Length << endl;


        //Creat Ethernet DAPI object
        pEthernetDAPI = new EthernetDAPI("EthernetDAPI");

        //Start read data from EEPROM by API
        error_code = pEthernetDAPI->ReadSFP(WellKnownName, DeviceAddress, StartAddress, Length, data);
        if (error_code != NO_ERROR_MSG)
        {
            printError(error_code);
            return -1;
        }

        Unit = 1;

        //Show getted data content
        for (i=0; i < Length ; i++)
        {
            if (i == 0 || (i % 16) == 0 )
            {
                printf("%08x: ", StartAddress+i);
            }

            printf("%02x", data[i]);
            if (((i+1) % Unit) == 0)
            {
                printf(" ");
            }

            if (((i+1) % 16) == 0  && i != (Length - 1))
            {
                printf("\n");
             }
        }
        printf("\n");

        delete pEthernetDAPI;
    }

    return err;
}



int
EthernetOperation::writeSFP(StringParser command_line)
{
    int           err = 0, data;
    EthernetDAPI  *pEthernetDAPI = NULL;
    ErrorMsg      *error_code;
    string        WellKnownName, temp_string;
    int           StartAddress, DeviceAddress;

    error_code = NO_ERROR_MSG;

    err = Syntax_check(command_line);

    if (err == 0)
    {
        //Get Ethernet device name
        command_line.getParameter(&WellKnownName, 3);
        //cout << "Name: " <<  WellKnownName << endl;

        //Device address
        command_line.getParameter(&temp_string, 0);
        command_line.convertStringToHex(temp_string, &DeviceAddress);

        //Get start address
        command_line.getParameter(&temp_string, 1);
        command_line.convertStringToHex(temp_string, &StartAddress);
        //cout << "Start address: " <<  StartAddress << endl;

        //Get length value
        command_line.getParameter(&temp_string, 2);
        command_line.convertStringToHex(temp_string, &data);
        //cout << "Length: " <<  Length << endl;


        //Creat Ethernet DAPI object
        pEthernetDAPI = new EthernetDAPI("EthernetDAPI");

        //Start write data to EEPROM by API
        error_code = pEthernetDAPI->writeSFP(WellKnownName, DeviceAddress, StartAddress, data);
        if (error_code != NO_ERROR_MSG)
        {
            printError(error_code);
            return -1;
        }

        //Start read data from EEPROM by API
        error_code = pEthernetDAPI->ReadSFP(WellKnownName, DeviceAddress, StartAddress, 1, &data);
        if (error_code != NO_ERROR_MSG)
        {
            printError(error_code);
            return -1;
        }

        printf("%03x: %02x\n", StartAddress, data);

        delete pEthernetDAPI;
    }

    return err;
}



int
EthernetOperation::show_all_error(StringParser command_line)
{
    int           err = 0;
    EthernetDAPI  *pEthernetDAPI = NULL;
    ErrorMsg      *error_code;
    string        WellKnownName, str_result;

    error_code = NO_ERROR_MSG;

    err = Syntax_check(command_line);

    if (err == 0)
    {
        //Get Ethernet device name
        command_line.getParameter(&WellKnownName, 0);
        //cout << "Name: " <<  WellKnownName << endl;


        //Creat Ethernet DAPI object
        pEthernetDAPI = new EthernetDAPI("EthernetDAPI");

        //Start read statistic by API
        error_code = pEthernetDAPI->GetStatistics(WellKnownName, &str_result);

        cout << "==== statistics ====" << endl;
        cout << str_result;
        if (error_code != NO_ERROR_MSG)
        {
            printError(error_code);
            return -1;
        }

        str_result.clear();

        //Start read PCI error by API
        error_code = pEthernetDAPI->GetPCIError(WellKnownName, &str_result);

        //cout << "==== PCI error ====" << endl;
        cout << str_result << endl;
        if (error_code != NO_ERROR_MSG)
        {
            printError(error_code);
            return -1;
        }

        delete pEthernetDAPI;
    }

    return err;
}



int
EthernetOperation::statistic(StringParser command_line)
{
    int           err = 0;
    EthernetDAPI  *pEthernetDAPI = NULL;
    ErrorMsg      *error_code;
    string        WellKnownName, str_statistic;

    error_code = NO_ERROR_MSG;

    err = Syntax_check(command_line);

    if (err == 0)
    {
        //Get Ethernet device name
        command_line.getParameter(&WellKnownName, 0);
        //cout << "Name: " <<  WellKnownName << endl;


        //Creat Ethernet DAPI object
        pEthernetDAPI = new EthernetDAPI("EthernetDAPI");

        //Start read statistic by API
        error_code = pEthernetDAPI->GetStatistics(WellKnownName, &str_statistic);

        cout << str_statistic << endl;
        if (error_code != NO_ERROR_MSG)
        {
            printError(error_code);
            return -1;
        }

        delete pEthernetDAPI;
    }

    return err;
}



int
EthernetOperation::pciError(StringParser command_line)
{
    int           err = 0;
    EthernetDAPI  *pEthernetDAPI = NULL;
    ErrorMsg      *error_code;
    string        WellKnownName, pciError;

    error_code = NO_ERROR_MSG;

    err = Syntax_check(command_line);

    if (err == 0)
    {
        //Get Ethernet device name
        command_line.getParameter(&WellKnownName, 0);
        //cout << "Name: " <<  WellKnownName << endl;


        //Creat Ethernet DAPI object
        pEthernetDAPI = new EthernetDAPI("EthernetDAPI");

        //Start read PCI error by API
        error_code = pEthernetDAPI->GetPCIError(WellKnownName, &pciError);

        cout << pciError << endl;
        if (error_code != NO_ERROR_MSG)
        {
            printError(error_code);
            return -1;
        }

        delete pEthernetDAPI;
    }

    return err;
}



int
EthernetOperation::clear(StringParser command_line)
{
    int           err = 0;
    EthernetDAPI  *pEthernetDAPI = NULL;
    ErrorMsg      *error_code;
    string        WellKnownName;

    error_code = NO_ERROR_MSG;

    err = Syntax_check(command_line);

    if (err == 0)
    {
        //Get Ethernet device name
        command_line.getParameter(&WellKnownName, 0);
        //cout << "Name: " <<  WellKnownName << endl;


        //Creat Ethernet DAPI object
        pEthernetDAPI = new EthernetDAPI("EthernetDAPI");

        //Start clear by API
        error_code = pEthernetDAPI->Clear(WellKnownName);

        if (error_code != NO_ERROR_MSG)
        {
            printError(error_code);
            return -1;
        }

        delete pEthernetDAPI;
    }

    return err;
}



int
EthernetOperation::Syntax_check(StringParser command_line)
{
    int    i, parameter_number = 0;
    string parameter[7];
    int    err = 0;
    string operation_name;

    getName(&operation_name);

    command_line.getParameterNumber(&parameter_number);

    //Get parameter
    for (i=0; i<parameter_number; i++)
    {
        command_line.getParameter(&parameter[i], i);
    }

    switch (getOperationType())
    {
        case OPERATION_ALL:
        case OPERATION_NODE:
        case OPERATION_LINK:
        case OPERATION_SHOW_SPEED:
        case OPERATION_FW:
        case OPERATION_MDI:
        case OPERATION_INTERRUPT:
        case OPERATION_READ_MAC:
        case OPERATION_VALIDATE_EEPROM:
        case OPERATION_INTERNAL_LOOPBACK_TEST:
        case OPERATION_ERROR_ALL:
        case OPERATION_ERROR_STATISTICS:
        case OPERATION_ERROR_PCI:
        case OPERATION_CLEAR:
            //Check parameter number
            if (parameter_number != 1)
            {
                cout << "Parameter numbers isn't 1" << endl;
                return -1;
            }
            break;
        case OPERATION_WRITE_MAC:
        case OPERATION_PROGRAM:
            //Check parameter number
            if (parameter_number != 2)
            {
                cout << "Parameter numbers isn't 2" << endl;
                return -1;
            }
            break;
        case OPERATION_SINGLE_LOOPBACK_TEST:
            //Check parameter number
            if (parameter_number != 2)
            {
                cout << "Parameter numbers isn't 2" << endl;
                return -1;
            }

            //Check parameter 1 is numeric
            err = command_line.isHEXString(parameter[0]);
            if (err != 0)
            {
                cout << "parameter 2 isn't numeric." << endl;
                return -1;
            }
            break;
        case OPERATION_CROSS_LOOPBACK_TEST:
            //Check parameter number
            if (parameter_number != 3)
            {
                cout << "Parameter numbers isn't 3" << endl;
                return -1;
            }

            //Check parameter 1 is numeric
            err = command_line.isHEXString(parameter[0]);
            if (err != 0)
            {
                cout << "parameter 3 isn't numeric." << endl;
                return -1;
            }
            break;
        case OPERATION_READ_EEPROM:
        case OPERATION_WRITE_EEPROM:
            //Check parameter number
            if (parameter_number != 4)
            {
                cout << "Parameter numbers isn't 4" << endl;
                return -1;
            }

            //Check parameter 1 is numeric
            err = command_line.isHEXString(parameter[0]);
            if (err != 0)
            {
                cout << "parameter 1 isn't numeric." << endl;
                return -1;
            }

            //Check parameter 2 is numeric
            err = command_line.isHEXString(parameter[1]);
            if (err != 0)
            {
                cout << "parameter 2 isn't numeric." << endl;
                return -1;
            }
            //Check parameter 2 is '8', '16', or '32'
            if (parameter[1] != "8" && parameter[1] != "16" && parameter[1] != "32")
            {
                cout << "parameter 2 isn't '8', '16', or '32'" << endl;
                return -1;
            }

            //Check parameter 3 is numeric
            err = command_line.isHEXString(parameter[2]);
            if (err != 0)
            {
                cout << "parameter 3 isn't numeric." << endl;
                return -1;
            }
            break;
        case OPERATION_READ_SFP:
        case OPERATION_WRITE_SFP:
            //Check parameter number
            if (parameter_number != 4)
            {
                cout << "Parameter numbers isn't 3" << endl;
                return -1;
            }

            //Check parameter 1 is numeric
            err = command_line.isHEXString(parameter[0]);
            if (err != 0)
            {
                cout << "parameter 1 isn't numeric." << endl;
                return -1;
            }

            //Check parameter 2 is numeric
            err = command_line.isHEXString(parameter[1]);
            if (err != 0)
            {
                cout << "parameter 2 isn't numeric." << endl;
                return -1;
            }

            //Check parameter 3 is numeric
            err = command_line.isHEXString(parameter[2]);
            if (err != 0)
            {
                cout << "parameter 2 isn't numeric." << endl;
                return -1;
            }
            break;

       default:
            cout << "Not define syntax_check for " << operation_name << endl;
            err = -1;
            break;
    }

    return err;
}
