
#include "FPGACommand.hpp"
#include "CommonCLIFunctions.hpp"
#include "FPGADAPI.hpp"

FPGACommand::FPGACommand(COMMAND_TYPE   in_eCommandType,
                             string         in_strHelpContent) : CommandBaseClass(in_eCommandType, in_strHelpContent)
{
    switch (in_eCommandType)
    {
        case CMD_SHOW:
            addOperation(new FPGAOperation(OPERATION_ALL, "Show all of FPGA's information"));
            addOperation(new FPGAOperation(OPERATION_FW, "Show FPGA's revision"));
            addOperation(new FPGAOperation(OPERATION_DESIGN_ID, "Show FPGA's design ID"));
            break;
        case CMD_SPAD:
            addOperation(new FPGAOperation(OPERATION_RANGE, "Get register range of scratch pad"));
            addOperation(new FPGAOperation(OPERATION_READ, "Read value from the scratch pad register"));
            addOperation(new FPGAOperation(OPERATION_WRITE, "Write value to the scratch pad register"));
            break;
        case CMD_UTILITY:
            addOperation(new FPGAOperation(OPERATION_GET, "Get value from the register"));
            addOperation(new FPGAOperation(OPERATION_SET, "Set value to the register"));
            break;
        default:
            break;
    }
}

FPGACommand::~FPGACommand()
{

}

int
FPGACommand::executeCommand(StringParser in_parser)
{
	COMMAND_TYPE    eCommand = NO_COMMAND;

	in_parser.getCommandType(&eCommand);

    switch (eCommand)
    {
        case CMD_LIST:
            return executeCommandList(in_parser);
        case CMD_SHOW:
        case CMD_SPAD:
        case CMD_UTILITY:
            return findAndExecuteOperation(in_parser);
        default:
            cout << UNKNOWN_COMMAND_MSG << endl;
            break;
    }

    return -1;
}

int
FPGACommand::executeCommandList(StringParser in_parser)
{
    FPGADAPI        fpgaDAPI("FPGADAPI");
    ErrorMsg*       pErrorMsg = NO_ERROR_MSG;
    list<string>    deviceList;
    
    if ((pErrorMsg = fpgaDAPI.getErrorMsg()))
    {
        printError(pErrorMsg);
        return -1;
    }

    pErrorMsg = fpgaDAPI.getDeviceList(&deviceList);
    if (pErrorMsg != NO_ERROR_MSG)
    {
        printError(pErrorMsg);
        return -1;
    }

    while (!deviceList.empty())
    {
        cout << deviceList.front() << endl;
        deviceList.pop_front();
    }

    return 0;
}
