
readonly PROGRAM="ibcfg"
readonly VERSION="1.0.0"
readonly RELEASEDATE="12/23/2015 10:14:46"
readonly AUTHOR="Ian Fang"
readonly COPYRIGHT="Copyright(c)2005-2011 Foxconn IND., Group."

function version_show()
{
    echo
    echo "Program Name:   $PROGRAM"
    echo "version:        $VERSION"
    echo "Release date:   $RELEASEDATE"
    echo "Author:         $AUTHOR"
    echo "$COPYRIGHT"
    echo
}
function usage()
{
    echo "Usage: [-h|help] [-v] [-g|get] [-w|write]"
        echo "  -h"
        echo "   Display this tool information."
        echo "       Example:"
        echo "           ./ibcfg -h"
	echo ""
        echo "  -v"
        echo "   Display IB cfg version."
        echo "       Example:"
        echo "           ./ibcfg -v"
	echo ""
	echo "	-r | read [mac|guid|fw] [name|file name]"
	echo "   Read MAC or GUID or Backup FW image."
	echo "    parameter1:mac"
	echo "    parameter2:name(all|port1|port2)"
	echo "	    all:read all MAC address."
	echo "		Example:"
	echo "		  ./ibcfg -r mac all"
	echo "	    port1|port2:read designation port MAC address."
	echo "		  ./ibcfg -r mac port1"
	echo "		  ./ibcfg -r mac port2"
	echo "    parameter1:guid"
	echo "    parameter2:name(all|port1|port2|si)"
	echo "	    all:read all GUID address."
	echo "	    	Example:"
	echo "		  ./ibcfg -r guid all"
	echo "	    node|port1|port2|si:read designation port GUID address."
	echo "		Example:"
	echo "		  ./ibcfg -r guid node"
	echo "		  ./ibcfg -r guid port1"
	echo "		  ./ibcfg -r guid port2"
	echo "		  ./ibcfg -r guid node"
	echo "    parameter1:fw"
	echo "    parameter2:file name"
	echo "	    Backup firmware image."
	echo "		Example:"
	echo "		  ./ibcfg -r fw MT275XXbackup.bin"
	echo ""
        echo "  -w | write [mac|guid|fw] [mac address | guid address ] [name|file name]"
	echo "   Program MAC or GUID or FW image."
        echo "    parameter1:mac"
        echo "    parameter2:mac address"
        echo "    parameter3:name (all|port1|port2) "
	echo "	    all:programm all MAC address by base value. 2 MACs are automatically assigned to the"
        echo "    	following values:"
        echo "      	mac   -> port1"
        echo "     	mac+1 -> port2"
        echo "       	Example:"
        echo "           ./ibcfg -w mac 00A09830A4CC all"
	echo "	    port1|port2: program mac address to designation port"
	echo "		Example:"
        echo "           ./ibcfg -w mac 00A09830A4CC port1"
        echo "           ./ibcfg -w mac 00A09830A4CD port2"
        echo "    parameter1:guid"
	echo "    parameter2:guid address"
	echo "    parameter3:name(all|port1|port2|si)"
	echo " 	    all:programm all GUID address by base value. 4 GUIDs are automatically assigned to the"
	echo "		following values:"
        echo "	        guid   -> node GUID"
        echo "    	guid+1 -> port 1"
        echo "      	guid+2 -> port 2"
        echo "      	guid+3 -> system image GUID"
	echo "  	Example:"
	echo "		  ./ibcfg -w guid 00A09800000A27F4 all"
        echo "	    node|port1|port2|si:program guid address to designation port"
	echo "		Example:"
	echo "		  ./ibcfg -w guid 00A09800000A27F4 node"
	echo "		  ./ibcfg -w guid 00A09800000A27F5 port1"
	echo "		  ./ibcfg -w guid 00A09800000A27F6 port2"
	echo "		  ./ibcfg -w guid 00A09800000A27F7 si"
        echo "    parameter1:fw"
	echo "    parameter2:file name"
	echo " 　   Burn firmware image - *bin"
	echo "		Example:"
	echo "		  ./ibool -w fw MT275xx-image.bin"
        echo ""
}

function listDataElement()
{
	echo -e "  ALL"
	echo -e "  LOG_PATH"
	echo -e "  MTUSB_MODE [0|1|2|3]"
	echo -e "  SERVER_HOST_NAME (Read only)"
	echo -e "  DUT_HOST_NAME (Read only)"
	echo -e "  BIST_MODE [on|off]"
	echo -e "  FLASH_MODE [on|off]"
	echo -e "  FLASH_TEST_TYPE [fw|rw]"
	echo -e "  FLASH_NUM [1|2]"
	echo -e "  FLASH_SIZE [0x200000|0x400000]"
	echo -e "  GPIO_MODE [on|off]"
	echo -e "  GPIO_MASK"
	echo -e "  GPIO_EXP_VAL"
	echo -e "  PCIE_MODE [on|off]"
	echo -e "  PCIE_SPEED [0x1|0x2|0x3]"
	echo -e "  PCIE_WIDTH [0x4|0x8]"
	echo -e "  IB_LINK_MODE [on|off]"
	echo -e "  IB_SPEED [QDR|FDR10|FDR14]"
	echo -e "  IB_WIDTH [1x|4x|8x]"
	echo -e "  PORTS [1|2]"
	echo -e "  IB_STRESS_MODE [on|off]"
	echo -e "  STRESS_RUNS [1-10000]"
	echo -e "  STRESS_MIN_BW [1-10000]"
}


function getConfiguration()
{
#	echo -e "getConfiguration"
	#echo -e "1st:${1}"
	case $1 in
		all|ALL)
		getAllDataElementFromFile
		;;
		MTUSB_MODE | mtusb_mode)
		getDataElementFromFile Setup
		;;
		SERVER_HOST_NAME | server_host)
		getDataElementFromFile SERVER_STATION_HOST_NAME
		;;
		DUT_HOST_NAME | dut_host_name)
		getDataElementFromFile DUT_HOST_NAME
		;;
		BIST_MODE | bist_mode)
		getDataElementFromFile BIST_test
		;;
		FLASH_MODE | flash_mode)
		getDataElementFromFile Flash_test
		;;
		GPIO_MODE | gpio_mode)
		getDataElementFromFile GPIO_test
		;;
		GPIO_EXP_VAL | gpio_exp_val)
		getDataElementFromFile GPIO_expected_value
		;;
		PCIE_MODE | pcie_mode)
		getDataElementFromFile PCIe_test
		;;
		IB_LINK_MODE | ib_link_mode)
		getDataElementFromFile IB_link_test
		;;
		IB_STRESS_MODE | ib_stress_mode)
		getDataElementFromFile Stress_test
		;;
		STRESS_RUNS | stress_runs)
		getDataElementFromFile INTERATIONS
		;;
		STRESS_MIN_BW | stress_min_bw)
		getDataElementFromFile MIN_BW
		;;
		LOG_PATH | log_path | \
		FLASH_TEST_TYPE | flash_test_type | FLASH_NUM | flash_num | FLASH_SIZE | flash_size | \
		GPIO_MASK | gpio_mask | \
		PCIE_SPEED | pcie_speed | PCIE_WIDTH | pcie_swidth | \
		IB_SPEED | ib_speed | IB_WIDTH | ib_width | \
		PORTS | ports)
		getDataElementFromFile $1
		;;
		*)
		;;
	esac
}

function getDataElementFromFile()
{
#	echo -e "getDataElementFromFile"
#	echo -e "cat $CONFIGURATION_FILE  | grep -i $1 | grep -v '#' | grep -v section | awk '{print $2}'"
	Result=`cat $CONFIGURATION_FILE  | grep -iw $1 | grep -v \# | grep -v section | sed 's/^[     ]*//;s/[     ]*$//' | sed -e 's/\ /\:  /g'`
	echo -e ${Result}
	

}
function getAllDataElementFromFile()
{
		getDataElementFromFile LOG_PATH
		getDataElementFromFile Setup
		getDataElementFromFile SERVER_STATION_HOST_NAME
		getDataElementFromFile DUT_HOST_NAME
		
		getDataElementFromFile BIST_test
		
		getDataElementFromFile Flash_test
		getDataElementFromFile FLASH_TEST_TYPE
		getDataElementFromFile FLASH_NUM
		getDataElementFromFile FLASH_SIZE
		
		getDataElementFromFile GPIO_test
		getDataElementFromFile GPIO_MASK
		getDataElementFromFile GPIO_expected_value
		
		getDataElementFromFile PCIe_test
		getDataElementFromFile PCIE_SPEED
		getDataElementFromFile PCIE_WIDTH
		
		getDataElementFromFile IB_link_test
		getDataElementFromFile IB_SPEED
		getDataElementFromFile IB_WIDTH
		getDataElementFromFile PORTS
		
		getDataElementFromFile Stress_test
		getDataElementFromFile INTERATIONS
		getDataElementFromFile MIN_BW
}



function setConfiguration()
{
	#echo -e "setConfiguration"
	#echo -e "1st:${1}"
	#echo -e "2nd:${2}"
	# check IS vaild mac address length
	
	#
	
	case $1 in
		LOG_PATH | log_path)
		setDataElementToFile LOG_PATH $2
		;;
		MTUSB_MODE | mtusb_mode)
		if [ "${2}" == "0" ] || [ "${2}" == "1" ] || [ "${2}" == "2" ] || [ "${2}" == "3" ]; then
			setDataElementToFile Setup $2
		else
			echo -e "The parameter is 0 ~ 3"
		fi
		;;
		SERVER_HOST_NAME | server_host)
		setDataElementToFile SERVER_STATION_HOST_NAME $2
		;;
		DUT_HOST_NAME | dut_host_name)
		setDataElementToFile DUT_HOST_NAME $2
		;;
		BIST_MODE | bist_mode)
		if [ "${2}" == "on" ] || [ "${2}" == "ON" ] || [ "${2}" == "off" ] || [ "${2}" == "OFF" ]; then
			setDataElementToFile BIST_test $2
		else
			echo -e "The parameter is not expected(on or off)."
		fi
		;;
		FLASH_MODE | flash_mode)
		if [ "${2}" == "on" ] || [ "${2}" == "ON" ] || [ "${2}" == "off" ] || [ "${2}" == "OFF" ]; then
			setDataElementToFile Flash_test $2
		else
			echo -e "The parameter is not expected(on or off)."
		fi
		;;
		FLASH_TEST_TYPE | flash_test_type)
		if [ "${2}" == "fw" ] || [ "${2}" == "FW" ] || [ "${2}" == "rw" ] || [ "${2}" == "RW" ]; then
			setDataElementToFile FLASH_TEST_TYPE $2
		else
			echo -e "The parameter is not expected(fw or rw)"
		fi
		;;
		FLASH_NUM | flash_num)
		if [ "${2}" == "1" ] || [ "${2}" == "2" ]; then
  			setDataElementToFile FLASH_NUM $2
		else
			echo -e "The parameter is not expected(1 or 2)"
		fi
		;;
		FLASH_SIZE | flash_size)
		if [ "${2}" == "0x200000" ] || [ "${2}" == "0x400000" ]; then
 			setDataElementToFile FLASH_SIZE $2
		else
			echo -e "The parameter is not expected(0x200000 or 0x400000)"
		fi
		
		;;
		GPIO_MODE | gpio_mode)
		if [ "${2}" == "on" ] || [ "${2}" == "ON" ] || [ "${2}" == "off" ] || [ "${2}" == "OFF" ]; then
			setDataElementToFile GPIO_test $2
		else
			echo -e "The parameter is not expected(on or off)."
		fi
		;;
		GPIO_MASK | gpio_mask)
		setDataElementToFile GPIO_MASK $2
		;;
		GPIO_EXP_VAL | gpio_exp_val)
		setDataElementToFile GPIO_expected_value $2
		;;
		PCIE_MODE | pcie_mode)
		if [ "${2}" == "on" ] || [ "${2}" == "ON" ] || [ "${2}" == "off" ] || [ "${2}" == "OFF" ]; then
			setDataElementToFile PCIe_test $2
		else
			echo -e "The parameter is not expected(on or off)."
		fi
		;;
		PCIE_SPEED | pcie_speed)
		if [ "${2}" == "0x1" ] || [ "${2}" == "0x2" ] || [ "${2}" == "0x3" ]; then
			setDataElementToFile PCIE_SPEED $2
		else
			echo -e "The parameter is not expected(0x1 or 0x2 or 0x3)"
		fi
		;;
		PCIE_WIDTH | pcie_swidth)
		if [ "${2}" == "0x4" ] || [ "${2}" == "0x8" ]; then
			setDataElementToFile PCIE_WIDTH $2
		else
			echo -e "The parameter is not expected(0x4 or 0x8)"
		fi
		;;
		IB_LINK_MODE | ib_link_mode)
		if [ "${2}" == "on" ] || [ "${2}" == "ON" ] || [ "${2}" == "off" ] || [ "${2}" == "OFF" ]; then
			setDataElementToFile IB_link_test $2
		else
			echo -e "The parameter is not expected(on or off)."
		fi
		;;
		IB_SPEED | ib_speed)
		if [ "${2}" == "QDR" ] || [ "${2}" == "FDR10" ] || [ "${2}" == "FDR14" ]; then
			setDataElementToFile IB_SPEED $2
		else
			echo -e "The parameter is not expected(QDR or FDR10 or FDR14)."
		fi
		;;
		IB_WIDTH | ib_width)
		if [ "${2}" == "1x" ] || [ "${2}" == "4x" ] || [ "${2}" == "8x" ]; then
			setDataElementToFile IB_WIDTH $2
		else
			echo -e "The parameter is not expected(1x or 4x or 8x)."
		fi
		;;
		PORTS | ports)
		if [ "${2}" == "1" ] || [ "${2}" == "2" ]; then
			setDataElementToFile PORTS $2
		else
			echo -e "The parameter is not expected(1 or 2)."
		fi
		;;
		IB_STRESS_MODE | ib_stress_mode)
		if [ "${2}" == "on" ] || [ "${2}" == "ON" ] || [ "${2}" == "off" ] || [ "${2}" == "OFF" ]; then
			setDataElementToFile Stress_test $2
		else
			echo -e "The parameter is not expected(on or off)."
		fi
		;;
		STRESS_RUNS | stress_runs)
		if [ $2 -le 10000 ] && [ $2 -ge 1 ]; then
			setDataElementToFile INTERATIONS $2
		else
			echo -e "The parameter is between 1 and 10000"
		fi
		;;
		STRESS_MIN_BW | stress_min_bw)
		if [ $2 -le 10000 ] && [ $2 -ge 1 ]; then
			setDataElementToFile MIN_BW $2
		else
			echo -e "The parameter is between 1 and 10000"
		fi
		;;
		*)
		;;
	esac
	getConfiguration $1
}

function setDataElementToFile()
{
#	echo -e "getDataElementFromFile"
#	echo -e "cat $CONFIGURATION_FILE  | grep -i $1 | grep -v '#' | grep -v section | awk '{print $2}'"
	Result=`cat $CONFIGURATION_FILE  | grep -iw $1 | grep -v \# | grep -v section | sed 's/^[     ]*//;s/[     ]*$//'`

	sed -i 's/'"$Result"'/'"${1}"' '"$2"'/g' $CONFIGURATION_FILE
	#echo -e "sed -i 's/$Result/$1 $2/g' $CONFIGURATION_FILE"
	#echo -e $Result
	

}


DIR=.
FILE=prod_ib.cfg

if [ ! -e $DIR/$FILE ]; then
	echo -e "NOT FOUND CONFIGURATION FILE"
	exit 0
fi
CONFIGURATION_FILE=$DIR/$FILE
#dev=$(ls $DIR | grep 'cr0' | cut -f 1) 
#device=$DIR/$dev

if [ $# == 0 ]; then
    usage
fi

case $1 in
-h | help)
	usage
	;;
-l | list | LIST)
	listDataElement
	;;
-g | get)
	if [ $# != 2 ]; then
		echo -e "get option necessary 1 parameters"
		usage
		exit 0
	fi 
	getConfiguration $2
	;;
-s | set)
	if [ $# != 3 ]; then
		echo -e "get option necessary 2 parameters"
		usage
		exit 0
	fi 
	setConfiguration $2 $3
;;
-v)
	version_show
	;;  
*)
	usage
	;;
esac

