
#include "LEDCommand.hpp"

LEDCommand::LEDCommand(COMMAND_TYPE   in_eCommandType,
                       string         in_strHelpContent) : CommandBaseClass(in_eCommandType, in_strHelpContent)
{
    switch (in_eCommandType)
    {
        case CMD_UTILITY:
            addOperation(new LEDOperation(OPERATION_GET, "Get LED status"));
            addOperation(new LEDOperation(OPERATION_SET, "Set on/off to LED"));
            break;
        default:
            break;
    }
}

LEDCommand::~LEDCommand()
{

}

int
LEDCommand::executeCommand(StringParser in_parser)
{
	COMMAND_TYPE    eCommand = NO_COMMAND;

	in_parser.getCommandType(&eCommand);

    switch (eCommand)
    {
        case CMD_LIST:
            return executeCommandList(in_parser);
        case CMD_UTILITY:
            return findAndExecuteOperation(in_parser);
        default:
            cout << UNKNOWN_COMMAND_MSG << endl;
            break;
    }

    return -1;
}

int
LEDCommand::executeCommandList(StringParser in_parser)
{
    LEDDAPI         ledDAPI("LEDDAPI");
    ErrorMsg*       pErrorMsg = NO_ERROR_MSG;
    list<string>    deviceList;

    if ((pErrorMsg = ledDAPI.getErrorMsg()))
    {
        printError(pErrorMsg);
        return -1;
    }

    pErrorMsg = ledDAPI.getDeviceList(&deviceList);
    if (pErrorMsg)
    {
        printError(pErrorMsg);
        return -1;
    }

    while (!deviceList.empty())
    {
        cout << deviceList.front() << endl;
        deviceList.pop_front();
    }

    return 0;
}
