#include <unistd.h>
#include <algorithm>
#include "LEDOperation.hpp"

LEDOperation::LEDOperation(OPERATION_TYPE   in_eOpType,
                           string           in_strHelp) : OperationBaseClass(in_eOpType, in_strHelp)
{

}

LEDOperation::~LEDOperation()
{
  
}

int
LEDOperation::executeOperation(StringParser in_parser)
{
    string      strParameter;
    int         count;
    string      strP1, strP2, strStatus;
    ErrorMsg    *pErrorMsg = NO_ERROR_MSG;
    LEDDAPI     ledDAPI("LEDDAPI");

    in_parser.getParameter(&strParameter, 0);

    if (strParameter == HELP)
    {
        showOperationHelp();
        return 0;
    }

    in_parser.getParameterNumber(&count);

    switch (getOperationType())
    {
        case OPERATION_GET:
            // Check syntax
            if (count != 1)
            {
                cout << SYNTAX_ERROR_MSG << endl;
                return -1;
            }

            if ((pErrorMsg = ledDAPI.getErrorMsg()))
            {
                printError(pErrorMsg);
                return -1;
            }

            in_parser.getParameter(&strP1, 0);

            if ((pErrorMsg = ledDAPI.getLEDStatus(strP1, &strStatus)))
            {
                printError(pErrorMsg);
                return -1;
            }

            cout << strStatus << endl;
            break;
        case OPERATION_SET:
            // Check syntax
            if (count != 2)
            {
                cout << SYNTAX_ERROR_MSG << endl;
                return -1;
            }

            if ((pErrorMsg = ledDAPI.getErrorMsg()))
            {
                printError(pErrorMsg);
                return -1;
            }

            in_parser.getParameter(&strP1, 0);
            in_parser.getParameter(&strP2, 1);

            if ((pErrorMsg = ledDAPI.setLEDStatus(strP2, strP1)))
            {
                printError(pErrorMsg);
                return -1;
            }
            
           usleep(50000); 

            if ((pErrorMsg = ledDAPI.getLEDStatus(strP2, &strStatus)))
            {
                printError(pErrorMsg);
                return -1;
            }

            cout << strStatus << endl;
            break;
        default:
            cout << UNKNOWN_OPERATION_MSG << endl;
            return -1;
    }

    return 0;
}

void
LEDOperation::showOperationHelp()
{
    string strOpName, strHelp;

    getName(&strOpName);
    getHelp(&strHelp);

    cout << strHelp << endl;
    cout << "Syntax " << strOpName << " ";

    switch (getOperationType())
    {
        case OPERATION_GET:
            cout << "P1" << endl;
            cout << "P1: LED name" << endl;
            break;
        case OPERATION_SET:
            cout << "P1 P2" << endl;
            cout << "P1: On/Off" << endl;
            cout << "P2: LED name" << endl;
            break;
        default:
            break;
    }
}
