#include "NVMeCommand.hpp"

NVMeCommand::NVMeCommand(COMMAND_TYPE commandType, string strHelpContent) :
        CommandBaseClass(commandType, strHelpContent)
{
    //Base on command type to add different operations
    switch(commandType)
    {
        case CMD_LIST:
            addOperation(new NVMeOperation(NO_OPRATION, "Display NVMe Controller Information."));

            break;
        case CMD_SHOW:
            addOperation(new NVMeOperation(OPERATION_ALL, "Display All Information."));
            addOperation(new NVMeOperation(OPERATION_HW, "Display NVME Hardware Info."));
            addOperation(new NVMeOperation(OPERATION_FW, "Display Firmware Version."));
            addOperation(new NVMeOperation(OPERATION_MODEL, "Display NVME Model"));
            addOperation(new NVMeOperation(OPERATION_SN, "Display NVME Serial Number."));
            addOperation(new NVMeOperation(OPERATION_CAPACITY, "Display NVME Capacity (GB)."));
            addOperation(new NVMeOperation(OPERATION_TEMP, "Display NVME Temperature."));
            addOperation(new NVMeOperation(OPERATION_BDF, "Display Controller Bus:Device.Function."));
            addOperation(new NVMeOperation(OPERATION_SHOW_SPEED, "Display PCI Speed."));
            addOperation(new NVMeOperation(OPERATION_SHOW_WIDTH, "Display PCI width."));
            addOperation(new NVMeOperation(OPERATION_NODE_NAME, "Display NVME Node Name on OS."));
            addOperation(new NVMeOperation(OPERATION_BLOCK_NAME, "Display NVME Block Name on OS."));
            addOperation(new NVMeOperation(OPERATION_BLOCK_SIZE, "Display NVME Block size"));
            addOperation(new NVMeOperation(OPERATION_NS_ID, "Display NVME NameSpace ID"));

            break;
        case CMD_ERROR:
            addOperation(new NVMeOperation(OPERATION_ERROR_ALL, "Get PCI/NVME Error statistics."));
            addOperation(new NVMeOperation(OPERATION_ERROR_STATISTICS, "Get NVME Error statistics."));
            addOperation(new NVMeOperation(OPERATION_ERROR_PCI, "Get PCI Error statistics."));
            addOperation(new NVMeOperation(OPERATION_CLEAR, "Clear PCI Error statistics."));

            break;
        case CMD_UTILITY:
            addOperation(new NVMeOperation(OPERATION_RESET_CONTROLLER, "Reset NVME Controller."));
            addOperation(new NVMeOperation(OPERATION_RESET_SUBSYS, "Reset NVME Subsystem."));
            addOperation(new NVMeOperation(OPERATION_IDENTITY_CONTROLLER, "Send NVME Identify Controller, display structure."));
            addOperation(new NVMeOperation(OPERATION_IDENTITY_NAMESPACE, "Send NVME Identify Namespace, display structure."));
            addOperation(new NVMeOperation(OPERATION_WRITE_ZERO, "Write Zero to specified block."));
            addOperation(new NVMeOperation(OPERATION_NVME_SMART, "Display NVMe Smart Log"));
            //addOperation(new NVMeOperation(OPERATION_GET_FEATURES, "Get an NVMe feature"));
            //addOperation(new NVMeOperation(OPERATION_SET_FEATURES, "Set an NVMe feature"));

            break;
        case CMD_TEST:
            addOperation(new NVMeOperation(OPERATION_READPFMC, "NVME Read Performance Test"));
            addOperation(new NVMeOperation(OPERATION_WRITEPFMC, "NVME Write Performance Test"));
            addOperation(new NVMeOperation(OPERATION_WRC, "NVME Read/Write Test"));

            break;
        case CMD_FW:
            addOperation(new NVMeOperation(OPERATION_FIRMWARE_ACTIVATE, "Activate new firmware slot."));
            addOperation(new NVMeOperation(OPERATION_FIRMWARE_DOWNLOAD, "Download firmware to NVME Controller."));
            addOperation(new NVMeOperation(OPERATION_FW_LOG, "Display Firmware Log."));

            break;
        default:

            break;
    }
}

NVMeCommand::~NVMeCommand()
{

}

int NVMeCommand::executeCommand(StringParser commandLine)
{
    COMMAND_TYPE    commandType = NO_COMMAND;
    string 			strParserOperationName;
	
    commandLine.getCommandType(&commandType);
    switch (commandType)
    {
        case CMD_LIST:
        case CMD_SHOW:
        case CMD_TEST:
        case CMD_ERROR:
        case CMD_UTILITY:
        case CMD_FW:
            return findAndExecuteOperation(commandLine);
        default:
            cout << UNKNOWN_COMMAND_MSG << endl;
            break;
    }

    return -1;
}

