#include "NVMeOperation.hpp"
extern void showSmartLog(struct nvme_smart_log *smart);
extern void showFWLog(struct nvme_firmware_log_page *fw_log);
extern void showNvmeIdentifyCtrl(struct nvme_id_ctrl *ctrl);
extern void showNvmeIdentifyNS(struct nvme_id_ns *ns);
extern void showErrorLog(struct nvme_error_log_page *err_log);
extern void nvmeFeatureShowFields(unsigned int fid, unsigned int result, unsigned char *buf);

#define LOG_ENTRIES 64

NVMeOperation::NVMeOperation(OPERATION_TYPE oprationType, string helpContent) :OperationBaseClass(oprationType, helpContent)
{
    switch(oprationType)
    {
        case OPERATION_HW:
            setName("hw");
            break;
        case OPERATION_BDF:
            setName("bdf");
            break;
        case OPERATION_BLOCK_SIZE:
            setName("blocksize");
            break;
        case OPERATION_NODE_NAME:
            setName("nodename");
            break;
        case OPERATION_BLOCK_NAME:
            setName("blockname");
            break;
        case OPERATION_NS_ID:
            setName("nsid");
            break;
        case OPERATION_MODEL:
            setName("model");
            break;
        case OPERATION_FW_LOG:
            setName("fwlog");
            break;
        case OPERATION_NVME_SMART:
            setName("smart");
            break;
        case OPERATION_CAPACITY:
            setName("capacity");
            break;
        case OPERATION_IDENTITY_NAMESPACE:
            setName("idns");
            break;
        case OPERATION_IDENTITY_CONTROLLER:
            setName("idctrl");
            break;
        case OPERATION_READPFMC:
            setName("readpfmc");
            break;
        case OPERATION_WRITEPFMC:
            setName("writepfmc");
            break;
        case OPERATION_FIRMWARE_ACTIVATE:
            setName("activate");
            break;
        case OPERATION_FIRMWARE_DOWNLOAD:
            setName("download");
            break;
        case OPERATION_WRITE_ZERO:
            setName("writezero");
            break;
		case OPERATION_GET_FEATURES:
			setName("getfeat");
			break;
		case OPERATION_SET_FEATURES:
			setName("setfeat");
			break;
		case OPERATION_RESET_CONTROLLER:
			setName("resetctrl");
			break;
		case OPERATION_RESET_SUBSYS:
			setName("resetsubsys");
			break;
        default:
            break;
    }
}

NVMeOperation::~NVMeOperation()
{
}

int NVMeOperation::executeOperation(StringParser commandLine)
{
    string  parameter;
    int     err = 0;
    
    commandLine.getParameter(&parameter, 0);

    if(parameter == HELP)
    {
        showOperationHelp();
        return 0;
    }

    err = checkSyntax(commandLine);
    if (err == 0)
    {
        //Execute operation
        switch (getOperationType())
        {
            case NO_OPRATION:
                return executeListNVMe(commandLine);
            case OPERATION_ALL:
            case OPERATION_HW:
            case OPERATION_BDF:
			case OPERATION_SHOW_SPEED:
			case OPERATION_SHOW_WIDTH:
            case OPERATION_FW:
            case OPERATION_FW_LOG:
            case OPERATION_ERROR_LOG:
            case OPERATION_TEMP:
            case OPERATION_NODE_NAME:
            case OPERATION_BLOCK_NAME:
            case OPERATION_SN:
            case OPERATION_BLOCK_SIZE:
            case OPERATION_NS_ID:
            case OPERATION_MODEL:
            case OPERATION_CAPACITY:
            case OPERATION_NVME_SMART:
                return executeShowController(commandLine);
            case OPERATION_ERROR_ALL:
            case OPERATION_ERROR_STATISTICS:
            case OPERATION_ERROR_PCI:
                return executeError(commandLine);
            case OPERATION_CLEAR:
                return executeClearError(commandLine);
            case OPERATION_RESET_CONTROLLER:
			case OPERATION_RESET_SUBSYS:
            case OPERATION_IDENTITY_CONTROLLER:
            case OPERATION_IDENTITY_NAMESPACE:
            case OPERATION_WRITE_ZERO:
                return executeUtility(commandLine);
            case OPERATION_FIRMWARE_ACTIVATE:
                return executeActivateFW(commandLine);
            case OPERATION_FIRMWARE_DOWNLOAD:
                return executeDownloadFW(commandLine);
            case OPERATION_READPFMC:
            case OPERATION_WRITEPFMC:
            case OPERATION_WRC:
                return executeTest(commandLine);
            case OPERATION_GET_FEATURES:
                return executeGetFeatures(commandLine);
			case OPERATION_SET_FEATURES:
				return executeSetFeatures(commandLine);
            default:
                cout << UNKNOWN_OPERATION_MSG << endl;
                err = -1; 
                break;
        }
    }
    
    return err;
}

void NVMeOperation::showOperationHelp()
{
    string strOpName, strHelp;

    getName(&strOpName);
    getHelp(&strHelp);

    cout << strHelp << endl;
    cout << "Syntax " << strOpName << " ";
    
    switch (getOperationType())
    {
        case OPERATION_ALL:
        case OPERATION_HW:
		case OPERATION_FW:
		case OPERATION_MODEL:
		case OPERATION_SN:
		case OPERATION_CAPACITY:
		case OPERATION_TEMP:
        case OPERATION_BDF:
		case OPERATION_SHOW_SPEED:
		case OPERATION_SHOW_WIDTH:
		case OPERATION_NODE_NAME:
		case OPERATION_BLOCK_NAME:
		case OPERATION_BLOCK_SIZE:
		case OPERATION_NS_ID:
        case OPERATION_FW_LOG:
        case OPERATION_ERROR_LOG:
        case OPERATION_ERROR_ALL:
        case OPERATION_ERROR_STATISTICS:
        case OPERATION_ERROR_PCI:
        case OPERATION_CLEAR:
		case OPERATION_RESET_CONTROLLER:
		case OPERATION_RESET_SUBSYS:
		case OPERATION_IDENTITY_CONTROLLER:
		case OPERATION_IDENTITY_NAMESPACE:
		case OPERATION_NVME_SMART:
            cout << "P1" << endl;
            cout << "P1: Device Name" << endl;
            break;
        case OPERATION_READPFMC:
		case OPERATION_WRITEPFMC:
		case OPERATION_WRC:
            cout << "P1 P2 P3" << endl;
            cout << "P1: Start Sector (Hex)" << endl;
            cout << "P2: Test Size/MB (Dec)" << endl;
            cout << "P3: Device Name" << endl;
            break;           
        case OPERATION_FIRMWARE_ACTIVATE:
            cout << "P1 P2 P3" << endl;
            cout << "P1: [0-7]: Firmware Slot to Activate" << endl;
			cout << "    - 0: The controller shall choose the firmware slot (slot 1 �V7)" << endl;
			cout << "    - 1: Slot 1" << endl;
			cout << "    - 2: Slot 2" << endl;
			cout << "    - 3: Slot 3" << endl;
			cout << "    - 4: Slot 4" << endl;
			cout << "    - 5: Slot 5" << endl;
			cout << "    - 6: Slot 6" << endl;
			cout << "    - 7: Slot 7" << endl;
            cout << "P2: [0-3]: Replacement Action" << endl;
			cout << "    - 0: Downloaded image replaces the image specified by the Firmware Slot field." << endl;
			cout << "         This image is not activated." << endl; 
			cout << "    - 1: Downloaded image replaces the image specified by the Firmware Slot field." << endl;
			cout << "         This image is activated at the next reset." << endl; 
			cout << "    - 2: The image specified by the Firmware Slot field is activated at the next reset. " << endl;
			cout << "    - 3: The image specified by the Firmware Slot field is requested" << endl;
			cout << "         to be activated immediately without reset." << endl;
            cout << "P3: Device Name" << endl;
            break;
        case OPERATION_FIRMWARE_DOWNLOAD:
            cout << "P1 P2" << endl;
            cout << "P1: Firmware file path" << endl;
            cout << "P2: Device Name" << endl;
            break;
        case OPERATION_WRITE_ZERO:
            cout << "P1 P2 P3" << endl;
            cout << "P1: Start Block (Hex)" << endl;
            cout << "P2: Block (Dec)" << endl;
            cout << "P3: Device Name" << endl;
            break;
		case OPERATION_GET_FEATURES:
			cout << "P1 P2 P3" << endl;
			cout << "P1: Select" << endl;
			cout << "    - 0: Current" << endl; 
			cout << "    - 1: Default" << endl;
			cout << "    - 2: Saved" << endl;
			cout << "    - 3: Supported capabilities" << endl;
			cout << "P2: Feature ID" << endl;
			cout << "    - 0x01: Arbitration" << endl; 
			cout << "    - 0x02: Power Management" << endl; 
			cout << "    - 0x03: LBA Range Type (Optional)" << endl; 
			cout << "    - 0x04: Temperature Threshold" << endl; 
			cout << "    - 0x05: Error Recovery" << endl; 
			cout << "    - 0x06: Volatile Write Cache (Optional)" << endl; 
			cout << "    - 0x07: Number of Queues" << endl; 
			cout << "    - 0x08: Interrupt Coalescing" << endl; 
			cout << "    - 0x09: Interrupt Vector Configuration" << endl; 
			cout << "    - 0x0A: Write Atomicity Normal" << endl; 
			cout << "    - 0x0B: Asynchronous Event Configuration" << endl; 
			cout << "    - 0x0C: Autonomous Power State Transition (Optional)" << endl; 
            //!!
			cout << "    - 0x0D: Host Memory Buffer (Optional)" << endl; 
            //!!
			cout << "    - 0x0F: Keep Alive Timer (Optional)" << endl; 
            cout << "    - 0x80: Software Progress Marker (Optional)" << endl;
            //!!
            cout << "    - 0x81: Host Identifier (Optional)" << endl;
            cout << "    - 0x82: Reservation Notification Mask (Optional)" << endl;
            cout << "    - 0x83: Reservation Persistance (Optional)" << endl;
            cout << "    - 0xC0: Get Endurance Management Disable Setting (Intel Only)" << endl;
            cout << "    - 0xC1: Get Max LBA (Intel Only)" << endl;
            cout << "    - 0xC2: Get Native Max LBA (Intel Only)" << endl;
            cout << "    - 0xC6: Get Power Governor Setting (Intel Only)" << endl;
            cout << "    - 0xC8: Get SMB ASIC Address (Intel Only)" << endl;
            cout << "    - 0xC9: Get Blink Activity for LED (Intel Only)" << endl;
            cout << "    - 0xE2: Get Enable Latency Tracking (Intel Only)" << endl;
			cout << "P3: Device Name" << endl;
			break;
		case OPERATION_SET_FEATURES:
			cout << "P1 P2 P3" << endl;
			cout << "P1: Feature ID" << endl;
			cout << "    - 0x01: Arbitration" << endl; 
			cout << "    - 0x02: Power Management" << endl; 
			cout << "    - 0x03: LBA Range Type" << endl; 
			cout << "    - 0x04: Temperature Threshold" << endl; 
			cout << "    - 0x05: Error Recovery" << endl; 
			cout << "    - 0x06: Volatile Write Cache" << endl; 
			cout << "    - 0x07: Number of Queues" << endl; 
			cout << "    - 0x08: Interrupt Coalescing" << endl; 
			cout << "    - 0x09: Interrupt Vector Configuration" << endl; 
			cout << "    - 0x0A: Write Atomicity Normal" << endl; 
			cout << "    - 0x0B: Asynchronous Event Configuration" << endl; 
			cout << "    - 0x0C: Autonomous Power State Transition" << endl; 
			cout << "    - 0x0D: Host Memory Buffer" << endl; 
			cout << "    - 0x0F: Keep Alive Timer" << endl; 
			cout << "P2: Value" << endl;
			cout << "P3: Device Name" << endl;
			break;

        default:
            break;
    }
}

int NVMeOperation::executeListNVMe(StringParser commandLine)
{
    vector<NVMe_DEVICE_INFO>            nvmeList;
    vector<NVMe_DEVICE_INFO>::iterator  nvmeIter;
    ErrorMsg                            *pErrorMsg = NO_ERROR_MSG;
    NVMeDAPI                            nvmeDAPI("NVMeDAPI");
    if ((pErrorMsg = nvmeDAPI.getErrorMsg()) != NO_ERROR_MSG)
    {
        printError(pErrorMsg);
        return -1;
    }
   
    if((pErrorMsg = nvmeDAPI.getInformationList(&nvmeList)) != NO_ERROR_MSG)
    {
        printError(pErrorMsg);
        return -1;
    }

    for(nvmeIter = nvmeList.begin(); nvmeIter != nvmeList.end(); ++nvmeIter)
    {
        cout << nvmeIter->wellKnowName << "(" << nvmeIter->domindBusDeviceFunction << ")" << endl;
    }

	return 0;
}

int NVMeOperation::executeShowController(StringParser commandLine)
{
    UINT8                       logicalID;
    int                         nsid;
    vector<NVMe_DEVICE_INFO>    nvmeList;
    UINT32                      temperature;
    long long int               lba;
    uint64_t                    nsze;
    uint64_t                    tmpValue;
    UINT32                      tmpIndex;
    static const char           sizeUint[] = " KMGTP";
    CHAR                        capStr[64];
    CHAR                        siStr[64];
    string                      targetName;
    string                      capacityString;
    string                      tempString;
    ErrorMsg                    *pErrorMsg = NO_ERROR_MSG;
    NVMeDAPI                    nvmeDAPI("NVMeDAPI");

    if ((pErrorMsg = nvmeDAPI.getErrorMsg()) != NO_ERROR_MSG)
    {
        printError(pErrorMsg);
        return -1;
    }

    commandLine.getParameter(&targetName, 0);

    if((pErrorMsg = nvmeDAPI.getInformationList(&nvmeList)) == NO_ERROR_MSG)
    {
        if((pErrorMsg = nvmeDAPI.getLogicalIDByName(targetName, &logicalID)) == NO_ERROR_MSG)
        {
            if((pErrorMsg = nvmeDAPI.getBarResourceByController(logicalID, &nvmeList[logicalID].barInfo)) == NO_ERROR_MSG)
                if((pErrorMsg = nvmeDAPI.getIdentityCTRLByController(logicalID, &nvmeList[logicalID].idCtrl)) == NO_ERROR_MSG)
                    if((pErrorMsg = nvmeDAPI.getIdentityNSByController(logicalID, &nvmeList[logicalID].idNs)) == NO_ERROR_MSG)
                        if((pErrorMsg = nvmeDAPI.getFirmwareLogByController(logicalID, &nvmeList[logicalID].fwLog)) == NO_ERROR_MSG)
                            if((pErrorMsg = nvmeDAPI.getSmartLogByController(logicalID, &nvmeList[logicalID].smartLog)) == NO_ERROR_MSG)
                                pErrorMsg = nvmeDAPI.getNameSpaceIDByController(logicalID, &nsid);
        }
    }

    if(pErrorMsg != NO_ERROR_MSG)
    {
        printError(pErrorMsg);
        return -1;
    }
    
    // get temperature from smart log
    temperature = ((nvmeList[logicalID].smartLog.temperature[1] << 8) | nvmeList[logicalID].smartLog.temperature[0]) - 273;
    lba = nvmeList[logicalID].idNs.lbaf[(nvmeList[logicalID].idNs.flbas & 0x0f)].ds;
    lba  = (1 << lba);
    nsze = nvmeList[logicalID].idNs.nsze;
    nsze *= lba;
    snprintf(capStr, sizeof(capStr), "%lu", nsze);
    
    tmpIndex = 0;
    tmpValue = nsze;
    do
    {
        if (tmpValue / 1000 > 0)
        {
            tmpValue /= 1000;
            tmpIndex++;
        }
    }while((tmpValue / 1000 > 0) && tmpIndex < sizeof(sizeUint));
    snprintf(siStr, sizeof(siStr), "%lu %cB", tmpValue, sizeUint[tmpIndex]);
    capacityString = capacityString.assign((const char*)capStr) + " bytes ";
    capacityString += "[" + tempString.assign((const char*)siStr) + "]";

    switch(getOperationType())
    {
        case OPERATION_ALL:
            cout << "Node Name      : " << nvmeList[logicalID].chardeviceNodeName << endl;
            cout << "Block Node Name: " << nvmeList[logicalID].blockdeviceNodeName << endl;
            printf( "Model Name     : %-40.40s\n", nvmeList[logicalID].idCtrl.mn);
            printf( "Serial Number  : %-20.20s\n", nvmeList[logicalID].idCtrl.sn);
            cout << "HW Version     : " << (nvmeList[logicalID].barInfo->vs >> 16) << "."
                                        << ((nvmeList[logicalID].barInfo->vs >> 8) & 0xff) << endl;
            printf( "FW Version     : %-8.8s\n", nvmeList[logicalID].idCtrl.fr);
            cout << "PCI BDF        : " << nvmeList[logicalID].domindBusDeviceFunction << endl;
            printf( "PCI Speed      : %.1f GT/s\n", nvmeList[logicalID].pciSpeed);
            cout << "PCI Width      : x" <<  nvmeList[logicalID].pciWidth << endl;
            cout << "Namespace ID   : " << nsid << endl;
            cout << "User Capacity  : " << capacityString << endl;
            cout << "Block Size     : " << lba << " bytes" << endl;
            cout << "Temperature    : " << temperature << " C" << endl;

            break;
        case OPERATION_HW:
            cout << "HW Version : " << (nvmeList[logicalID].barInfo->vs >> 16) << "."
                                    << ((nvmeList[logicalID].barInfo->vs >> 8) & 0xff) << endl;
            break;
        case OPERATION_FW:
            printf( "FW Version : %-8.8s\n", nvmeList[logicalID].idCtrl.fr);

            break;
        case OPERATION_SN:
            printf( "Serial Number : %-20.20s\n", nvmeList[logicalID].idCtrl.sn);

            break;
        case OPERATION_MODEL:
            printf( "Model Name : %-40.40s\n", nvmeList[logicalID].idCtrl.mn);

            break;
        case OPERATION_BDF:
            cout << "PCI BDF : " <<  nvmeList[logicalID].domindBusDeviceFunction << endl;
			break;
		case OPERATION_SHOW_SPEED:
			printf("PCI Speed : %.1f GT/s\n", nvmeList[logicalID].pciSpeed);
			
			break;
		case OPERATION_SHOW_WIDTH:
			cout << "PCI Width : x" <<  nvmeList[logicalID].pciWidth << endl;
			
            break;
        case OPERATION_TEMP:
            cout << "Temperature : " <<  temperature << " C" << endl;

            break;
        case OPERATION_CAPACITY:
            cout << "User Capacity : " << capacityString << endl;

            break;
        case OPERATION_BLOCK_SIZE:
            cout << "Block Size : " << lba << " bytes" << endl;

            break;
        case OPERATION_NODE_NAME:
            cout << "Node Name      : " << nvmeList[logicalID].chardeviceNodeName << endl;
            break;
        case OPERATION_BLOCK_NAME:
            cout << "Block Node Name: " << nvmeList[logicalID].blockdeviceNodeName << endl;
            break;
        case OPERATION_NS_ID:
            cout << "Namespace ID   : " << nsid << endl;
            break;
        case OPERATION_FW_LOG:
            showFWLog(&nvmeList[logicalID].fwLog);
            break;
        case OPERATION_NVME_SMART:
            showSmartLog(&nvmeList[logicalID].smartLog);
            break;
        default:
            cout << "Unsupported operation parameter" << endl;
            return -1;
    }

    return 0;
}

int NVMeOperation::executeError(StringParser commandLine)
{
    UINT8                       logicalID = -1;
    string                      targetName;
    PFA                         bdf;
    vector<NVMe_DEVICE_INFO>    nvmeList;
    PCIDAPI                     pciDAPI("PCIDAPI");
    NVMeDAPI                    nvmeDAPI("NVMeDAPI");
    ErrorMsg                    *pErrorMsg = NO_ERROR_MSG;

    if ((pErrorMsg = nvmeDAPI.getErrorMsg()) != NO_ERROR_MSG)
    {
        printError(pErrorMsg);
        return -1;
    }
    
    commandLine.getParameter(&targetName, 0);
    pErrorMsg = nvmeDAPI.getInformationList(&nvmeList);
    if(pErrorMsg == NO_ERROR_MSG)
    {
        if((pErrorMsg = nvmeDAPI.getLogicalIDByName(targetName, &logicalID)) == NO_ERROR_MSG)
        {
            if((pErrorMsg = nvmeDAPI.getErrorLogByController(logicalID, &nvmeList[logicalID].errLog)) == NO_ERROR_MSG)
            {
                sscanf(nvmeList[logicalID].domindBusDeviceFunction.c_str(), "%x:%x:%x.%x", &bdf.u16Domain, &bdf.u16Bus, &bdf.u16Device, &bdf.u8Function);
                switch(getOperationType())
                {
                    case OPERATION_ERROR_STATISTICS:
                        showErrorLog(&nvmeList[logicalID].errLog);
                        break;
                    case OPERATION_ERROR_ALL:
                        showErrorLog(&nvmeList[logicalID].errLog);
                    case OPERATION_ERROR_PCI:
                        pciDAPI.showErrorStatistics(bdf, PCI_COMPATIBLE_ERROR, FALSE);
                        pciDAPI.showErrorStatistics(bdf, PCIE_ERROR, FALSE);
                        pciDAPI.showErrorStatistics(bdf, AER_ERROR, FALSE);
                        break;
                    default:
                        cout << "Unsupported operation parameter" << endl;
                        return -1;
                }
            }
        }
    }

    if(pErrorMsg != NO_ERROR_MSG)
    {
        printError(pErrorMsg);
        return -1;
    }	

    return 0;
}

int NVMeOperation::executeClearError(StringParser commandLine)
{
    UINT8                       logicalID = -1;
    string                      targetName;
    PFA                         bdf;
    vector<NVMe_DEVICE_INFO>    nvmeList;
    PCIDAPI                     pciDAPI("PCIDAPI");
    NVMeDAPI                    nvmeDAPI("NVMeDAPI");
    ErrorMsg                    *pErrorMsg = NO_ERROR_MSG;

    if ((pErrorMsg = nvmeDAPI.getErrorMsg()) != NO_ERROR_MSG)
    {
        printError(pErrorMsg);
        return -1;
    }

    commandLine.getParameter(&targetName, 0);
    pErrorMsg = nvmeDAPI.getInformationList(&nvmeList);
    if(pErrorMsg == NO_ERROR_MSG)
    {
        if((pErrorMsg = nvmeDAPI.getLogicalIDByName(targetName, &logicalID)) == NO_ERROR_MSG)
        {
            if((pErrorMsg = nvmeDAPI.getErrorLogByController(logicalID, &nvmeList[logicalID].errLog)) == NO_ERROR_MSG)
            {
                sscanf(nvmeList[logicalID].domindBusDeviceFunction.c_str(), "%x:%x:%x.%x", &bdf.u16Domain, &bdf.u16Bus, &bdf.u16Device, &bdf.u8Function);
                pciDAPI.clearErrors(bdf);
                pciDAPI.showErrorStatistics(bdf, PCI_COMPATIBLE_ERROR, FALSE);
                pciDAPI.showErrorStatistics(bdf, PCIE_ERROR, FALSE);
                pciDAPI.showErrorStatistics(bdf, AER_ERROR, FALSE);
            }
        }
    }

    if(pErrorMsg != NO_ERROR_MSG)
    {
        printError(pErrorMsg);
        return -1;
    }

    return 0;
}

int NVMeOperation::executeUtility(StringParser commandLine)
{
    UINT8                       logicalID;
    UINT64                      startBlock = 0;
    int                         blockCount = 0;
    string                      targetName;
    string                      tempString;
    vector<NVMe_DEVICE_INFO>    nvmeList;
    ErrorMsg                    *pErrorMsg = NO_ERROR_MSG;
    NVMeDAPI                    nvmeDAPI("NVMeDAPI");

    if ((pErrorMsg = nvmeDAPI.getErrorMsg()) != NO_ERROR_MSG)
    {
        printError(pErrorMsg);
        return -1;
    }

    if(getOperationType() == OPERATION_WRITE_ZERO)
    {
        commandLine.getParameter(&tempString, 0);
        commandLine.convertStringToHex(tempString, &startBlock);

        commandLine.getParameter(&tempString, 1);
        commandLine.convertStringToDec(tempString, &blockCount);

        commandLine.getParameter(&targetName, 2);
    }
    else
        commandLine.getParameter(&targetName, 0);

    if((pErrorMsg = nvmeDAPI.getLogicalIDByName(targetName, &logicalID)) == NO_ERROR_MSG)
    {
        switch(getOperationType())
        {
            case OPERATION_RESET_CONTROLLER:
                pErrorMsg = nvmeDAPI.resetCtrl(logicalID);
                break;
			case OPERATION_RESET_SUBSYS:
                pErrorMsg = nvmeDAPI.resetSubSystem(logicalID);
				break;
            case OPERATION_IDENTITY_CONTROLLER:
                if((pErrorMsg = nvmeDAPI.getInformationList(&nvmeList)) == NO_ERROR_MSG)
                    if((pErrorMsg = nvmeDAPI.getIdentityCTRLByController(logicalID, &nvmeList[logicalID].idCtrl)) == NO_ERROR_MSG)
                        showNvmeIdentifyCtrl(&nvmeList[logicalID].idCtrl);
                break;
            case OPERATION_IDENTITY_NAMESPACE:
                if((pErrorMsg = nvmeDAPI.getInformationList(&nvmeList)) == NO_ERROR_MSG)
                    if((pErrorMsg = nvmeDAPI.getIdentityNSByController(logicalID, &nvmeList[logicalID].idNs)) == NO_ERROR_MSG)
                        showNvmeIdentifyNS(&nvmeList[logicalID].idNs);
                break;
            case OPERATION_WRITE_ZERO:
                pErrorMsg = nvmeDAPI.writeZeroByController(logicalID, startBlock, blockCount);
                break;
            default:
                cout << "Unsupported operation parameter" << endl;
                return -1;
        }
    }

    if(pErrorMsg != NO_ERROR_MSG)
    {
        printError(pErrorMsg);
        return -1;
    }

    return 0;
}

int NVMeOperation::executeActivateFW(StringParser commandLine)
{    
    UINT8                   logicalID;
    string                  tempString;
    string                  targetName;
    struct nvme_fw_active   activate;
    ErrorMsg                *pErrorMsg = NO_ERROR_MSG;
    NVMeDAPI                nvmeDAPI("NVMeDAPI");

    if ((pErrorMsg = nvmeDAPI.getErrorMsg()) != NO_ERROR_MSG)
    {
        printError(pErrorMsg);
        return -1;
    }

    //Get parameter
    commandLine.getParameter(&tempString, 0);
    commandLine.convertStringToDec(tempString, (int*)&activate.slot);
    if (activate.slot > 7)
    {
        cout << "invalid slot: " << tempString << endl;
        return -1;
    }

    commandLine.getParameter(&tempString, 1);
    commandLine.convertStringToDec(tempString, (int*)&activate.action);
    if (activate.action > 3)
    {
        cout << "invalid action: " << tempString << endl;

        return -1;
    }
    
    commandLine.getParameter(&targetName, 2);
    
    if((pErrorMsg = nvmeDAPI.getLogicalIDByName(targetName, &logicalID)) == NO_ERROR_MSG)
    {
        pErrorMsg = nvmeDAPI.activateFirmwareByController(logicalID, activate);
    }

    if(pErrorMsg != NO_ERROR_MSG)
    {
        printError(pErrorMsg);
        return -1;
    }

    return 0;
}

int NVMeOperation::executeDownloadFW(StringParser commandLine)
{
    UINT8       logicalID;
    string      parameter[2];
    ErrorMsg    *pErrorMsg = NO_ERROR_MSG;
    NVMeDAPI    nvmeDAPI("NVMeDAPI");

    if ((pErrorMsg = nvmeDAPI.getErrorMsg()) != NO_ERROR_MSG)
    {
        printError(pErrorMsg);
        return -1;
    }

    //Get parameter
    commandLine.getOrigParameter(&parameter[0], 0);
    commandLine.getParameter(&parameter[1], 1);

    if((pErrorMsg = nvmeDAPI.getLogicalIDByName(parameter[1], &logicalID)) == NO_ERROR_MSG)
    {
        pErrorMsg = nvmeDAPI.downloadFirmwareByController(logicalID, parameter[0].c_str());
    }

    if(pErrorMsg != NO_ERROR_MSG)
    {
        printError(pErrorMsg);
        return -1;
    }

    return 0;
}

int NVMeOperation::executeTest(StringParser commandLine)
{
    UINT32      u32LogicalID;
    UINT64      u64StartSector = 0;
    INT32       s32TestSize = 0;
    string      targetName;
    string      tempString;
    ErrorMsg    *pErrorMsg = NO_ERROR_MSG;
    NVMeDAPI    nvmeDAPI("NVMeDAPI");
    StorageDAPI storageDAPI("StorageDAPI");
    
    if ((pErrorMsg = nvmeDAPI.getErrorMsg()) != NO_ERROR_MSG)
    {
        printError(pErrorMsg);
        return -1;
    }
    
    if ((pErrorMsg = storageDAPI.getErrorMsg()) != NO_ERROR_MSG)
    {
        printError(pErrorMsg);
        return -1;
    }

    commandLine.getParameter(&tempString, 0);
    commandLine.convertStringToHex(tempString, &u64StartSector);

    commandLine.getParameter(&tempString, 1);
    commandLine.convertStringToDec(tempString, &s32TestSize);

    commandLine.getParameter(&targetName, 2);

    if((pErrorMsg = storageDAPI.getIndexByName(targetName, &u32LogicalID)) != NO_ERROR_MSG)
    {
        printError(pErrorMsg);
        return -1;
    }
	
	thread progressHandler(&NVMeOperation::handleTestProgress, this, (UINT32)s32TestSize);
    switch(getOperationType())
    {
        case OPERATION_READPFMC:
            pErrorMsg = storageDAPI.readPerformanceTest(u32LogicalID, u64StartSector, (UINT32)s32TestSize);
            break;
        case OPERATION_WRITEPFMC:
            pErrorMsg = storageDAPI.writePerformanceTest(u32LogicalID, u64StartSector, (UINT32)s32TestSize);
            break;
        case OPERATION_WRC:
            pErrorMsg = storageDAPI.readwriteTest(u32LogicalID, u64StartSector, (UINT32)s32TestSize);
            break;
        default:
            cout << "Unsupported operation parameter" << endl;
            return -1;
    }
        
	if(pErrorMsg != NO_ERROR_MSG)
	{
		printError(pErrorMsg);
		return -1;
	}
	progressHandler.join();
	cout << storageDAPI.m_strOutputResultString;


    return 0;
}

int NVMeOperation::checkSyntax(StringParser commandLine)
{
    int                 index;
    int                 err = 0;
    int                 parameterNumber = 0;
    string              *parameter;
    string              operation_name;
    unsigned long long  number;
    
    getName(&operation_name);

    commandLine.getParameterNumber(&parameterNumber);
    parameter = new string [parameterNumber];

    //Get parameter
    for (index = 0; index < parameterNumber; index++)
    {
        commandLine.getParameter(&parameter[index], index);
    }
    
    switch(getOperationType())
    {
        case NO_OPRATION:
            if(parameterNumber > 0)
            {
                cout << "Too many parameter." << endl;
                err = -1;
            }
            break;

        case OPERATION_ALL:
        case OPERATION_NODE_NAME:
        case OPERATION_BLOCK_NAME:
        case OPERATION_SN:
        case OPERATION_MODEL:
        case OPERATION_CAPACITY:
        case OPERATION_TEMP:
        case OPERATION_BDF:
		case OPERATION_SHOW_SPEED:
		case OPERATION_SHOW_WIDTH:
        case OPERATION_HW:
        case OPERATION_BLOCK_SIZE:
        case OPERATION_NS_ID:
        case OPERATION_FW:
        case OPERATION_FW_LOG:
        case OPERATION_ERROR_ALL:
        case OPERATION_ERROR_STATISTICS:
        case OPERATION_ERROR_PCI:
        case OPERATION_CLEAR:
        case OPERATION_RESET_CONTROLLER:
		case OPERATION_RESET_SUBSYS:
        case OPERATION_IDENTITY_CONTROLLER:
        case OPERATION_IDENTITY_NAMESPACE:
        case OPERATION_NVME_SMART:
            if(parameterNumber != 1)
            {
                cout << "The number of parameter isn't 1" << endl;
                err = -1;
            }
            break;

        case OPERATION_FIRMWARE_ACTIVATE:
        case OPERATION_READPFMC:
        case OPERATION_WRITEPFMC:
        case OPERATION_WRC:
        case OPERATION_WRITE_ZERO:
		case OPERATION_GET_FEATURES:
		case OPERATION_SET_FEATURES:

            if(parameterNumber != 3)
            {
                cout << "The number of parameter isn't 3" << endl;
                err = -1;
            }
            
            if (err == 0)
            {
                //Check parameter 1 is numeric
                err = commandLine.convertStringToHex(parameter[0], &number);
                if(err != 0)
                {
                    cout << "Parameter 1 isn't numeric." << endl;
                    err = -1;
                }
            }
            
            if (err == 0)
            {
                //Check parameter 2 is numeric
                err = commandLine.convertStringToHex(parameter[1], &number);
                if(err != 0)
                {
                    cout << "Parameter 2 isn't numeric." << endl;
                    err = -1;
                }
            }
            break;

        case OPERATION_FIRMWARE_DOWNLOAD:
            if(parameterNumber != 2)
            {
                cout << "The number of parameter isn't 2" << endl;
                err = -1;
            }
            break;
       default:
            cout << "Not define syntax_check for " << operation_name << endl;
            err = -1;
            break;
    }

    delete [] parameter;

    return err;
}

void NVMeOperation::handleTestProgress(UINT32 in_u32TestSizeMB)
{
    INT32 		progress = 0;
	StorageDAPI	storageDAPI("StorageDAPI");
	
    sleep(1);
    while(progress < 100)
    {
        sleep(1);
        progress = storageDAPI.getProgress();
        printf("Progressing: %3u%% \r", progress);
        fflush(stdout);
    }
    printf("\n");
}

int NVMeOperation::executeGetFeatures(StringParser commandLine)
{
    UINT8                       logicalID;
    string                      tempString;
    string                      targetName;
    struct nvme_get_features    getFeatures;
    ErrorMsg                    *pErrorMsg = NO_ERROR_MSG;
    NVMeDAPI                    nvmeDAPI("NVMeDAPI");
    
    if ((pErrorMsg = nvmeDAPI.getErrorMsg()) != NO_ERROR_MSG)
    {
        printError(pErrorMsg);
        return -1;
    }

	//Default value
	getFeatures.nsid     = 1;
	getFeatures.cdw11    = 0;
	getFeatures.data_len = 0;
    getFeatures.data_buf = NULL;

    commandLine.getParameter(&tempString, 0);
    commandLine.convertStringToDec(tempString, (int*)&getFeatures.sel);
    
    commandLine.getParameter(&tempString, 1);
    commandLine.convertStringToHex(tempString, (int*)&getFeatures.fid);
    
    commandLine.getParameter(&targetName, 2);
    if((pErrorMsg = nvmeDAPI.getLogicalIDByName(targetName, &logicalID)) == NO_ERROR_MSG)
    {
        pErrorMsg = nvmeDAPI.getFeatures(logicalID, &getFeatures);
    }
    
    if(pErrorMsg != NO_ERROR_MSG)
    {
        printError(pErrorMsg);
        return -1;
    }
	/*
	else
	{
	   nvmeFeatureShowFields(getFeatures.fid, (unsigned int)getFeatures.result, (unsigned char *)getFeatures.data_buf);
	}
	*/

    return 0;

}

int NVMeOperation::executeSetFeatures(StringParser commandLine)
{
    UINT8                       logicalID;
    string                      tempString;
    string                      targetName;
    struct nvme_set_features    setFeatures;
    ErrorMsg                    *pErrorMsg = NO_ERROR_MSG;
    NVMeDAPI                    nvmeDAPI("NVMeDAPI");
    
    if ((pErrorMsg = nvmeDAPI.getErrorMsg()) != NO_ERROR_MSG)
    {
        printError(pErrorMsg);
        return -1;
    }

	//Default value
	setFeatures.nsid     = 1;
	setFeatures.cdw11    = 0;
	setFeatures.data_len = 0;

    commandLine.getParameter(&tempString, 0);
    commandLine.convertStringToHex(tempString, (int*)&setFeatures.fid);
    
    commandLine.getParameter(&tempString, 1);
    commandLine.convertStringToHex(tempString, (int*)&setFeatures.cdw11);
    
    commandLine.getParameter(&targetName, 2);
    
    if((pErrorMsg = nvmeDAPI.getLogicalIDByName(targetName, &logicalID)) == NO_ERROR_MSG)
    {
        pErrorMsg = nvmeDAPI.setFeatures(logicalID, &setFeatures);
    }
    
    if(pErrorMsg != NO_ERROR_MSG)
    {
        printError(pErrorMsg);
        return -1;
    }

    return 0;

}
