#include "NVSRAMCommand.hpp"

NVSRAMCommand::NVSRAMCommand(COMMAND_TYPE CommandType, string strHelpContent) :
        CommandBaseClass(CommandType, strHelpContent)
{
    //Base on command type to add different operations
    switch(CommandType)
    {
        case CMD_SHOW:
            addOperation(new NVSRAMOperation(OPERATION_TEST_RANGE, "Show testable NVSRAM range"));

            break;
        case CMD_TEST:
            addOperation(new NVSRAMOperation(OPERATION_DATALINE, "NVSRAM Data Line Test"));
            addOperation(new NVSRAMOperation(OPERATION_ADDRLINE, "NVSRAM Address Line Test"));
            addOperation(new NVSRAMOperation(OPERATION_NVSTEST, "NVSRAM Non-volatile Test"));

            break;
        default:
            break;
    }
}

NVSRAMCommand::~NVSRAMCommand()
{

}

int NVSRAMCommand::executeCommand(StringParser CommandLine)
{
    INT32 err = 0;
    INT32 OperationIndex;
    COMMAND_TYPE CommandType;
    string strParserOperationName;
    string OperationName;

    CommandLine.getOperation(&strParserOperationName);
    CommandLine.getCommandType(&CommandType);

    switch(CommandType)
    {
        case CMD_SHOW:
        case CMD_TEST:
            if(strParserOperationName == "help" || strParserOperationName == "")
            {
                listOperation(CommandLine);
                return -1;
            }

            break;
        default:
            cout << "Not supported command" << endl;
            return -1;
    }

    for(OperationIndex = 0; OperationIndex < m_operation_count; OperationIndex++)
    {
        m_operation[OperationIndex]->getName(&OperationName);

        if (OperationName == strParserOperationName)
        {
            err = m_operation[OperationIndex]->executeOperation(CommandLine);
            break;
        }

    }

    if(OperationIndex == m_operation_count)
    {
        cout << strParserOperationName << " operation not found" << endl;
        return -1;
    }

    return err;
}

