#include "PCIOperation.hpp"


PCIOperation::PCIOperation(OPERATION_TYPE   in_eOprationType, 
                           string           in_strHelpContent) : OperationBaseClass(in_eOprationType, in_strHelpContent)
{
    m_pciDAPI = new PCIDAPI("PCIDAPI");
}


PCIOperation::~PCIOperation()
{
    if(m_pciDAPI != NULL)
    {
        delete m_pciDAPI;
        m_pciDAPI = NULL;
    }
}


int 
PCIOperation::convertStringToBytes(string   in_string,
                                   UINT8*   out_u8pBuffer)
{
    UINT32  u32ByteCount;
    UINT32  u32Index;
    UINT32  u32StringLength;
    string  strTemp;
    string  strToConvert;


    u32StringLength = in_string.size();

    u32ByteCount = 0;
    for(u32Index = 0; u32Index < u32StringLength; u32Index = u32Index + 2)
    {
        strTemp = in_string.substr(u32Index, u32StringLength + 1);
        strToConvert = strTemp.substr(0,2);
        sscanf(strToConvert.c_str(), "%x", (UINT32*)&out_u8pBuffer[u32ByteCount]);
        u32ByteCount++;
    }

    return 0;
}

int 
PCIOperation::convertStringToDWords(string      in_string,
                                    UINT32*     out_u32pBuffer)
{
    UINT32  u32DWordCount;
    UINT32  u32Index;
    UINT32  u32StringLength;
    string  strTemp;
    string  strToConvert;

    u32StringLength = in_string.size();

    u32DWordCount = 0;
    for(u32Index = 0; u32Index < u32StringLength; u32Index = u32Index + 8)
    {
        strTemp = in_string.substr(u32Index, u32StringLength + 1);
        strToConvert = strTemp.substr(0,8);
        sscanf(strToConvert.c_str(), "%x", &out_u32pBuffer[u32DWordCount]);
        u32DWordCount++;
    }

    return 0;
}

int  
PCIOperation::convertStringToBDF(string in_strParameter,
                                 PFA*   out_pBDF)
{
    string  strHexHeader = "0x";
    string  strTemp;
    string  strBDF;
    UINT8   u8Function;
    UINT32  s32ReturnPosition;
    bool    check;

    check = isBDF(in_strParameter);
    if(!check)
    {
        cout << "Invalid PCI address format!" << endl;
        return -1; 
    }

    s32ReturnPosition = in_strParameter.find(':', 0);
    if(s32ReturnPosition == 2)
    {
        strBDF = "0000:";
        strBDF.append(in_strParameter);
    }
    else
    {
        strBDF = in_strParameter;
    }
    //printf("PCIOperation::convertStringToBDF, strBDF: %s\n", strBDF.c_str());//check 
    
    //get domain 
    strTemp = strBDF.substr(0,4);
    strHexHeader.append(strTemp);
    sscanf(strHexHeader.c_str(), "%x", (UINT16*)&out_pBDF->u16Domain);
    strHexHeader = "0x"; //reset hex header 


    //get bus 
    strTemp = strBDF.substr(5,2);
    strHexHeader.append(strTemp);
    sscanf(strHexHeader.c_str(), "%x", (UINT16*)&out_pBDF->u16Bus);
    strHexHeader = "0x"; //reset hex header

    //get device
    strTemp = strBDF.substr(8,2);
    strHexHeader.append(strTemp);
    sscanf(strHexHeader.c_str(), "%x", (UINT16*)&out_pBDF->u16Device);
    strHexHeader = "0x"; //reset hex header

    //get function 
    strTemp = strBDF.substr(11,1);
    strHexHeader.append(strTemp);
    sscanf(strHexHeader.c_str(), "%x", &u8Function);
    out_pBDF->u8Function = u8Function;

    return 0; 
}


int 
PCIOperation::showPciInfo(StringParser in_classCommandLine)
{
    string      strTemp;
    ERRORCODE   eErrorCode = NO_ERROR;
    int         s32Result;
    PFA         bdf;

    if(m_s32ParmCount != 1)
    {
        cout << SYNTAX_ERROR_MSG << endl;
        return -1;
    }

    s32Result = convertStringToBDF(m_strParameter[0], &bdf);
    if(s32Result != NO_ERROR)
    {
        cout << SYNTAX_ERROR_MSG << endl;
        return s32Result;
    }

    eErrorCode = m_pciDAPI->checkPresenceOfDevice(bdf);
    if(eErrorCode != NO_ERROR)
    {
        cout << "Device is not found" << endl;
        return -1;
    }

    cout << "PCI information: " << endl;
    cout << "-------------------------------------------" << endl;
    if(getOperationType() == OPERATION_ALL)
    {
        s32Result = showSpecificPciInfo(bdf, OPERATION_ID); 
        if(s32Result != 0)//only need to check if the device is found at the first time
        {
            return s32Result;
        }
        showSpecificPciInfo(bdf, OPERATION_CLASS_TYPE);
        showSpecificPciInfo(bdf, OPERATION_PORT_TYPE);
        showSpecificPciInfo(bdf, OPERATION_SHOW_LINK);
        showSpecificPciInfo(bdf, OPERATION_SHOW_SPEED);
        showSpecificPciInfo(bdf, OPERATION_SHOW_WIDTH);
    }
    else 
    {
        s32Result = showSpecificPciInfo(bdf, getOperationType());
    }

    return s32Result;
}


int 
PCIOperation::showSpecificPciInfo(PFA               in_bdf,
                                  OPERATION_TYPE    in_eOprationType)
{
    string      strTemp;
    bool        bIsPCIe;

    bIsPCIe = m_pciDAPI->isPCIe(in_bdf);
    switch(in_eOprationType)
    {
        case OPERATION_ID:
            printf("Vendor/Device ID:\t %04x/%04x\n",
                    m_pciDAPI->GetDataByBDF(in_bdf, PCI_DATA_TYPE_VENDOR_ID),
                    m_pciDAPI->GetDataByBDF(in_bdf, PCI_DATA_TYPE_DEVICE_ID));
            break;

        case OPERATION_CLASS_TYPE:
            strTemp = m_pciDAPI->getReadableStringByBDF(in_bdf, PCI_DATA_TYPE_CLASS_TYPE);
            printf("ClassType:\t\t %s\n", strTemp.c_str());
            break;
        
        case OPERATION_PORT_TYPE:
            strTemp = m_pciDAPI->getReadableStringByBDF(in_bdf, PCI_DATA_TYPE_PORT_TYPE);
            printf("PortType:\t\t %s\n", strTemp.c_str());
            break;

        case OPERATION_SHOW_LINK:
            if(bIsPCIe)
            {
                strTemp = m_pciDAPI->getReadableStringByBDF(in_bdf, PCI_DATA_TYPE_LINK_STATUS);
                printf("Link:\t\t\t %s\n", strTemp.c_str());
            }
            else 
            {
                printf("Link:\t\t\t -\n");
            }
            break;

        case OPERATION_SHOW_SPEED:
            if(bIsPCIe)
            {
                printf("Speed(MAX/CUR):\t\t %.1f/%.1f GT/s\n",
                        m_pciDAPI->GetReadableSpeed(in_bdf, PCI_DATA_TYPE_MAX_SPEED),
                        m_pciDAPI->GetReadableSpeed(in_bdf, PCI_DATA_TYPE_CURRENT_SPEED));
            }
            else
            {
                printf("Speed(MAX/CUR):\t\t -/-\n");
            }
            break;

        case OPERATION_SHOW_WIDTH:
            if(bIsPCIe)
            {
                printf("Width(MAX/CUR):\t\t x%d/x%d\n",
                        m_pciDAPI->GetDataByBDF(in_bdf, PCI_DATA_TYPE_MAX_WIDTH),
                        m_pciDAPI->GetDataByBDF(in_bdf, PCI_DATA_TYPE_CURRENT_WIDTH));
            }
            else 
            {
                printf("Width(MAX/CUR):\t\t -/-\n");
            }
        
        default:
            break;
    }
    return 0;
}

#if 0
int //error code  
PCIOperation::showSpecificErrorStatistics(PFA               in_bdf,
                                          OPERATION_TYPE    in_eOprationType,
                                          bool              in_showSummary)
{
    UINT32  u32ErrorStatistics = 0x0;

    ErrorMsg*                   pErrorMsg;
    list<PCI_ERROR>             pciErrorList;
    list<PCI_ERROR>::iterator   it;

    pErrorMsg =  m_pciDAPI->getErrorStatistics(in_bdf, in_eOprationType, &pciErrorList);
    switch(in_eOprationType)
    {
        case OPERATION_PCI_COMPATIBLE:
            pErrorMsg = m_pciDAPI->getErrorStatistics(in_bdf, PCI_COMPATIBLE_ERROR, &pciErrorList);
            if(pErrorMsg == NO_ERROR_MSG)
            {
                cout << "==== PCI-compatible ====" << endl;
            }
            break;

        case OPERATION_PCIE:
            pErrorMsg = m_pciDAPI->getErrorStatistics(in_bdf, PCIE_ERROR, &pciErrorList);
            if(pErrorMsg == NO_ERROR_MSG)
            {
                cout << "==== PCIE ====" << endl;
            }
            break;

        case OPERATION_AER:
            pErrorMsg = m_pciDAPI->getErrorStatistics(in_bdf, AER_ERROR, &pciErrorList);
            if(pErrorMsg == NO_ERROR_MSG)
            {
                cout << "==== AER ====" << endl;
                for(it = pciErrorList.begin(); it != pciErrorList.end(); ++it)
                {
                    printf("%s: 0x%08x\n",it->strErrorName.c_str(), it->u32Data);
                }
            }
            return 0;

        default:
            break;
    }

    if(pErrorMsg != NO_ERROR_MSG)
    {
        deleteErrorMsg(pErrorMsg);
        return 1;
    }

    if(in_showSummary)
    {
        for(it = pciErrorList.begin(); it != pciErrorList.end(); ++it)
        {
            u32ErrorStatistics = u32ErrorStatistics + it->u32Data;
        }
        cout << it->strErrorName << " Error Statistics: " << "0x" << u32ErrorStatistics << endl;
    }
    else
    {
        for(it = pciErrorList.begin(); it != pciErrorList.end(); ++it)
        {
            cout << it->strErrorName << ": " << it->u32Data << endl;
        }
    }

    return 0;
}
#endif 

int 
PCIOperation::showPlxNtbErrorStatistics(PFA             in_bdf,
                                        OPERATION_TYPE  in_op,
                                        UINT8           in_u8Interface)
{
    if(in_op == OPERATION_ERROR_ALL)
    {
        m_pciDAPI->showPlxNtbErrorStatistics(in_bdf, PCI_COMPATIBLE_ERROR, FALSE, in_u8Interface);
        m_pciDAPI->showPlxNtbErrorStatistics(in_bdf, PCIE_ERROR, FALSE, in_u8Interface);
        m_pciDAPI->showPlxNtbErrorStatistics(in_bdf, AER_ERROR, FALSE, in_u8Interface);
    }
    else 
    {
        switch(in_op)
        {
            case OPERATION_PCI_COMPATIBLE:
                m_pciDAPI->showPlxNtbErrorStatistics(in_bdf, PCI_COMPATIBLE_ERROR, FALSE, in_u8Interface);
                break;
            case OPERATION_PCIE:
                m_pciDAPI->showPlxNtbErrorStatistics(in_bdf, PCIE_ERROR, FALSE, in_u8Interface);
                break;
            case OPERATION_AER:
                m_pciDAPI->showPlxNtbErrorStatistics(in_bdf, AER_ERROR, FALSE, in_u8Interface);
                break;
            default:
                break;
        }
    }
   
    return 0;
}

int //error code  
PCIOperation::showPciErrorStatistics()
{
    PFA             bdf;
    int             s32Result;
    ERRORCODE       eErrorCode = NO_ERROR;
    UINT32          u32VendorID;
    UINT32          u32DeviceID;
    PCI_ERROR_TYPE  eErrorType;

    if(m_s32ParmCount != 1)
    {
        cout << SYNTAX_ERROR_MSG << endl;
        return -1;
    }

    if (strstr(m_strParameter[0].c_str(), "sw") != NULL)
    {
        if(getOperationType() == OPERATION_ERROR_ALL)
        {
            eErrorType = ALL_ERROR; 
        }
        else 
        {
            switch(getOperationType())
            {
                case OPERATION_PCI_COMPATIBLE:
                    eErrorType = PCI_COMPATIBLE_ERROR;
                    break;
                case OPERATION_PCIE:
                    eErrorType = PCIE_ERROR;
                    break;
                case OPERATION_AER:
                    eErrorType = AER_ERROR;
                    break;
                default:
                    break;
            }   
        }
        m_pErrorMsg = m_pciDAPI->showSwitchErrorStatistics(m_strParameter[0], eErrorType);
        if(m_pErrorMsg != NO_ERROR_MSG)
        {
            printError(PCI_DEBUG_LEVEL, m_pErrorMsg);
            return -1;
        }
        return 0;
    }

    s32Result = convertStringToBDF(m_strParameter[0], &bdf);
    if(s32Result != NO_ERROR)
    {
        cout << SYNTAX_ERROR_MSG << endl;
        return s32Result;
    }

    eErrorCode = m_pciDAPI->checkPresenceOfDevice(bdf);
    if(eErrorCode != NO_ERROR)
    {
        cout << "Device is not found" << endl;
        return -1;
    }

    //PLX NTB 
    u32VendorID = m_pciDAPI->GetDataByBDF(bdf, PCI_DATA_TYPE_VENDOR_ID);
    u32DeviceID = m_pciDAPI->GetDataByBDF(bdf, PCI_DATA_TYPE_DEVICE_ID);
    if(u32VendorID == PCI_VENDOR_ID_PLX &&
      (u32DeviceID == PLX_DEVICE_ID_NTB0 || u32DeviceID == PLX_DEVICE_ID_NTB1))
    {
        cout << "NTB Virtual Side Error Statistics:" << endl;
        s32Result = showPlxNtbErrorStatistics(bdf, getOperationType(), PLX_PCIE_SWITCH_NTB_VIRTUAL);
        cout << endl;
        cout << "NTB Link Side Error Statistics:" << endl;
        s32Result = showPlxNtbErrorStatistics(bdf, getOperationType(), PLX_PCIE_SWITCH_NTB_LINK);
        
        return s32Result;
    }

    if(getOperationType() == OPERATION_ERROR_ALL)
    {
        m_pciDAPI->showErrorStatistics(bdf, PCI_COMPATIBLE_ERROR, FALSE);
        m_pciDAPI->showErrorStatistics(bdf, PCIE_ERROR, FALSE);
        m_pciDAPI->showErrorStatistics(bdf, AER_ERROR, FALSE);
    }
    else 
    {
        switch(getOperationType())
        {
            case OPERATION_PCI_COMPATIBLE:
                m_pciDAPI->showErrorStatistics(bdf, PCI_COMPATIBLE_ERROR, FALSE);
                break;
            case OPERATION_PCIE:
                m_pciDAPI->showErrorStatistics(bdf, PCIE_ERROR, FALSE);
                break;
            case OPERATION_AER:
                m_pciDAPI->showErrorStatistics(bdf, AER_ERROR, FALSE);
                break;
            default:
                break;
        }
    }

    return 0;
}


int
PCIOperation::clearErrors()
{
    PFA         bdf;
    UINT32      u32VendorID;
    UINT32      u32DeviceID;
    ERRORCODE   eErrorCode = NO_ERROR;
    ErrorMsg*   pErrorMsg;

    if (strstr(m_strParameter[0].c_str(), "sw") != NULL)
    {
        pErrorMsg = m_pciDAPI->clearPCIeSwitchPortErrors(m_strParameter[0]);
        if(pErrorMsg != NO_ERROR_MSG)
        {
            printError(PCI_DEBUG_LEVEL, pErrorMsg);
            return -1;
        }
        m_pciDAPI->showSwitchErrorStatistics(m_strParameter[0], ALL_ERROR);
        return 0;
    }

    convertStringToBDF(m_strParameter[0], &bdf);

    eErrorCode = m_pciDAPI->checkPresenceOfDevice(bdf);
    if(eErrorCode != NO_ERROR)
    {
        cout << "Device is not found" << endl;
        return -1;
    }

    m_pciDAPI->clearErrors(bdf);

    //check errors
    u32VendorID = m_pciDAPI->GetDataByBDF(bdf, PCI_DATA_TYPE_VENDOR_ID);
    u32DeviceID = m_pciDAPI->GetDataByBDF(bdf, PCI_DATA_TYPE_DEVICE_ID);
    if(u32VendorID == PCI_VENDOR_ID_PLX &&
      (u32DeviceID == PLX_DEVICE_ID_NTB0 || u32DeviceID == PLX_DEVICE_ID_NTB1))
    {
        cout << "check virtual side errors..." << endl;
        showPlxNtbErrorStatistics(bdf, OPERATION_ERROR_ALL, PLX_PCIE_SWITCH_NTB_VIRTUAL);
        cout << endl;
        cout << "check link side errors..." << endl;
        showPlxNtbErrorStatistics(bdf, OPERATION_ERROR_ALL, PLX_PCIE_SWITCH_NTB_LINK);
    }
    else 
    {
        m_pciDAPI->showErrorStatistics(bdf, PCI_COMPATIBLE_ERROR, TRUE);
        m_pciDAPI->showErrorStatistics(bdf, PCIE_ERROR, TRUE);
        m_pciDAPI->showErrorStatistics(bdf, AER_ERROR, TRUE);
    }

    return 0; 
}


void  
PCIOperation::showConfig(UINT8* in_u8pBuffer, 
                         UINT32 in_u32StartOffset,
                         UINT32 in_u32Length)
{
    UINT32  u32Index;

    for(u32Index = 0; u32Index < in_u32Length; u32Index++)
    {
        if(u32Index == 0 || u32Index % 16 == 0)
        {
            printf("%03x: ", in_u32StartOffset + u32Index);
        }
        printf("%02x ", in_u8pBuffer[u32Index]);
        if((u32Index + 1) % 16 == 0 && u32Index != in_u32Length - 1)
        {
            printf("\n");
        }
    }
    printf("\n");
}


int 
PCIOperation::readConfig(StringParser in_classCommandLine)
{
    int         s32StartOffset;
    int         s32EndOffset;
    int         s32Length;
    int         s32Result;
    UINT8*      u8pBuffer;
    ERRORCODE   eErrorCode = NO_ERROR;
    PFA         bdf;

    //syntax check 
    if(m_s32ParmCount != 3)
    {
        cout << SYNTAX_ERROR_MSG << endl;
        return -1;
    }
    s32Result = convertStringToBDF(m_strParameter[2], &bdf);
    if(s32Result != NO_ERROR)
    {
        cout << SYNTAX_ERROR_MSG << endl;
        return s32Result;
    }

    eErrorCode = m_pciDAPI->checkPresenceOfDevice(bdf);
    if(eErrorCode != NO_ERROR)
    {
        cout << "Device is not found" << endl;
        return -1;
    }

    in_classCommandLine.convertStringToHex(m_strParameter[0], &s32StartOffset);
    in_classCommandLine.convertStringToHex(m_strParameter[1], &s32Length);
   
    s32EndOffset = s32StartOffset + s32Length - 1;

    u8pBuffer = (UINT8*)malloc(sizeof(UINT8)*s32Length);


    m_pciDAPI->readConfig(bdf, (UINT32)s32StartOffset, (UINT32)s32EndOffset, u8pBuffer);

    //check 
    /*
    int i;
    for(i=0; i<s32Length; i++)
    {
        printf("u8pBuffer[%d] --- 0x%x\n", i, u8pBuffer[i]);
    }
    */

    showConfig(u8pBuffer, s32StartOffset, s32Length);

    free((UINT8*)u8pBuffer);

    return 0;
}


int
PCIOperation::writeConfig(StringParser in_classCommandLine)
{
    int         s32StartOffset;
    int         s32Length;
    int         s32Result;
    ERRORCODE   eErrorCode = NO_ERROR;
    PFA         bdf;
    UINT8*      u8pBuffer;
    UINT32      u32ByteCount;
    UINT32      u32Index;
    string      strTemp;

    //syntax check 
    if(m_s32ParmCount != 3)
    {
        cout << SYNTAX_ERROR_MSG << endl;
        return -1;
    }
    s32Result = convertStringToBDF(m_strParameter[2], &bdf);
    if(s32Result != NO_ERROR)
    {
        cout << SYNTAX_ERROR_MSG << endl;
        return s32Result;
    }

    eErrorCode = m_pciDAPI->checkPresenceOfDevice(bdf);
    if(eErrorCode != NO_ERROR)
    {
        cout << "Device is not found" << endl;
        return -1;
    }

    in_classCommandLine.convertStringToHex(m_strParameter[0], &s32StartOffset);
    if(s32StartOffset < 0 || s32StartOffset > 0xfff)
    {
        cout << "Invalid address range" << endl;
        cout << SYNTAX_ERROR_MSG << endl;
        return -1;
    }

    s32Length = m_strParameter[1].size();
    //printf("size of write data: %d\n", s32Length);//check
    
    if(m_strParameter[1].compare(0,2,"0x") == 0)
    {
        s32Length = s32Length - 2;
        strTemp = m_strParameter[1].substr(2, s32Length);
    }
    else
    {
        strTemp = m_strParameter[1];
    }
    
    if(s32Length % 2 != 0)
    {
        u32ByteCount = s32Length/2 + 1;
    }
    else
    {
        u32ByteCount = s32Length/2;
    }
   
    u8pBuffer = (UINT8*)malloc(sizeof(UINT8)*u32ByteCount);

    convertStringToBytes(strTemp, u8pBuffer);
    m_pciDAPI->writeConfig(bdf, s32StartOffset, u8pBuffer, u32ByteCount);

    //check data
    //--reset buffer
    for(u32Index = 0; u32Index < u32ByteCount; u32Index++)
    {
        u8pBuffer[u32Index] = 0x0;
    }
    m_pciDAPI->readConfig(bdf, (UINT32)s32StartOffset, (UINT32)(s32StartOffset + u32ByteCount - 1), u8pBuffer);
    showConfig(u8pBuffer, s32StartOffset, u32ByteCount);

    free((UINT8*)u8pBuffer);

    return 0;
}


int
PCIOperation::writeEeprom(StringParser in_classCommandLine)
{
    int     s32Result;
    int     s32Length;
    UINT32  u32Address;
    UINT32  *u32pBuffer;
    UINT32  u32DWordCount; //buffer size 
    string  strDataString;
    string  strTemp;

    if(m_s32ParmCount != 3)
    {
        cout << SYNTAX_ERROR_MSG << endl;
        return -1;
    }

    s32Result = in_classCommandLine.convertStringToHex(m_strParameter[0], (int*)&u32Address);
    if(s32Result != 0)
    {
        return s32Result;
    }
    if(u32Address % 4 != 0)
    {
        cout << "Invalid address" << endl;
        return -1;
    }

    if(m_strParameter[1].compare(0,2,"0x") == 0)
    {
        strDataString = m_strParameter[1].substr(2, -1);
    }
    else 
    {
        strDataString = m_strParameter[1];
    }

    s32Length = strDataString.size();
    if(s32Length % 8 != 0)
    {
        cout << "data length must be multiple of 4 (DWord)" << endl;
        return -1;
    }
    
    u32DWordCount = (s32Length/8);

    u32pBuffer = (UINT32*)malloc( sizeof(UINT32)*u32DWordCount);

    convertStringToDWords(strDataString, u32pBuffer);

    ////check 
    //for(int i=0; i < u32DWordCount; i++)
    //{
    //    printf("0x%x\n", u32pBuffer[i]);
    //}

    m_pErrorMsg = m_pciDAPI->writeEeprom(m_strParameter[2], u32Address, u32DWordCount, u32pBuffer);
    if(m_pErrorMsg != NO_ERROR_MSG)
    {
        printError(PCI_DEBUG_LEVEL, m_pErrorMsg);
        s32Result = -1;
    }
    
    //check data 
    if(s32Result == 0)
    {
        memset(u32pBuffer, 0x0, u32DWordCount);
        s32Result = readEepromRun(m_strParameter[2], 32, u32Address, u32DWordCount, u32pBuffer);
    }

    free(u32pBuffer);
    
    return s32Result;
}


int 
PCIOperation::dumpEeprom(StringParser in_classCommandLine)
{
    int     s32Result;
    char    eepromName[8];
    UINT32  u32EepromSize;
    UINT32  u32DWordCount;
    UINT32* u32pBuffer;

    if(m_s32ParmCount != 1)
    {
        cout << SYNTAX_ERROR_MSG << endl;
        return -1;
    }

    sprintf(eepromName, "%s", m_strParameter[0].c_str());
    in_classCommandLine.convertToLowcase(eepromName);

    m_pErrorMsg = m_pciDAPI->getEepromSize(eepromName, &u32EepromSize);
    //printf("PCIOperation::dumpEeprom, u32EepromSize: %d\n", u32EepromSize);//check 

    u32DWordCount = u32EepromSize/4;
    //printf("PCIOperation::dumpEeprom, u32DWordCount: %d\n", u32DWordCount);//check 

    u32pBuffer = (UINT32*)malloc(sizeof(UINT32) * u32DWordCount);

    s32Result = readEepromRun(eepromName, 32, 0x0, u32DWordCount, u32pBuffer);

    free(u32pBuffer);

    return s32Result;
}


void
PCIOperation::showEepromContentInByte(UINT32  in_u32StartAddress,
                                      UINT32  in_u32BufferSize, //Dword aligned 
                                      UINT32* in_u32pBuffer)
{
    UINT8*  u8pBuffer;
    UINT32  u32ByteCount;
    UINT32  u32DwordCount;
    UINT32  u32Index = 0;

    //convert buffer data from UINT32 to UINT8
    u32ByteCount = in_u32BufferSize * 4;
    u8pBuffer = (UINT8*)malloc(sizeof(UINT8) * u32ByteCount);

    for(u32DwordCount = 0; u32DwordCount < in_u32BufferSize; u32DwordCount++)
    {
        u8pBuffer[u32Index] = in_u32pBuffer[u32DwordCount] & 0xff;
        u32Index++;
        u8pBuffer[u32Index] = (in_u32pBuffer[u32DwordCount] & 0xff00) >> 8;
        u32Index++;
        u8pBuffer[u32Index] = (in_u32pBuffer[u32DwordCount] & 0xff0000) >> 16;
        u32Index++;
        u8pBuffer[u32Index] = (in_u32pBuffer[u32DwordCount] & 0xff000000) >> 24;
        u32Index++;
    }
    
    for(u32Index = 0; u32Index < u32ByteCount; u32Index++)
    {
        if(u32Index == 0 || u32Index % 16 == 0)
        {
            printf("%08X: ", (in_u32StartAddress + u32Index)/4 * 4);
        }

        printf("%02X ", u8pBuffer[u32Index]);

        if(u32Index != 0 && (u32Index + 1) % 16 == 0)
        {
            printf("\n");
        }
    }

    free(u8pBuffer);
}


void
PCIOperation::showEepromContentInWord(UINT32  in_u32StartAddress,
                                      UINT32  in_u32BufferSize, //Dword aligned 
                                      UINT32* in_u32pBuffer)
{
    UINT16*     u16pBuffer;
    UINT32      u32WordCount;
    UINT32      u32DwordCount;
    UINT32      u32Index = 0;

    //convert buffer data from UINT32 to UINT16
    u32WordCount = in_u32BufferSize * 2;
    u16pBuffer = (UINT16*)malloc(sizeof(UINT16) * u32WordCount);

    for(u32DwordCount = 0; u32DwordCount < in_u32BufferSize; u32DwordCount++)
    {
        u16pBuffer[u32Index] = in_u32pBuffer[u32DwordCount] & 0xffff;
        u32Index++;
        u16pBuffer[u32Index] = (in_u32pBuffer[u32DwordCount] & 0xffff0000) >> 16;
        u32Index++;
    }
    
    for(u32Index = 0; u32Index < u32WordCount; u32Index++)
    {
        if(u32Index == 0 || u32Index % 8 == 0)
        {
            printf("%08X: ", (in_u32StartAddress + u32Index*2)/4 * 4);
        }

        printf("%04X ", u16pBuffer[u32Index]);

        if(u32Index != 0 && (u32Index + 1) % 8 == 0)
        {
            printf("\n");
        }
    }

    free(u16pBuffer);
}


int 
PCIOperation::showEepromContent(UINT32  in_u32Unit,
                                UINT32  in_u32StartAddress,
                                UINT32  in_u32BufferSize, //Dword aligned
                                UINT32* in_u32pBuffer)
{
    UINT32  u32DwordCount;
    bool    bUnit32 = false;

    switch(in_u32Unit)
    {
        case 8:
            showEepromContentInByte(in_u32StartAddress, in_u32BufferSize, in_u32pBuffer);
            break;
        case 16:
            showEepromContentInWord(in_u32StartAddress, in_u32BufferSize, in_u32pBuffer);
            break;
        case 32:
            bUnit32 = true;
            break;
        default:
            cout << "Invalid unit" << endl;
            return -1;            
    }

    if(bUnit32)
    {
        for(u32DwordCount = 0; u32DwordCount < in_u32BufferSize; u32DwordCount++)
        {
            if(u32DwordCount == 0 || u32DwordCount % 4 == 0)
            {
                printf("%08X: ", (in_u32StartAddress + u32DwordCount*4)/4 * 4);
            }
            printf("%08X ", in_u32pBuffer[u32DwordCount]);
            if(u32DwordCount != 0 && (u32DwordCount + 1) % 4 == 0)
            {
                printf("\n");
            }
        }
    }

    if(in_u32BufferSize % 4 != 0)
    {
        printf("\n");
    }

    return 0;
}

int
PCIOperation::showEepromFW(StringParser in_classCommandLine)
{
    char    eepromName[8];

    if(m_s32ParmCount != 1)
    {
        cout << SYNTAX_ERROR_MSG << endl;
        return -1;
    }

    sprintf(eepromName, "%s", m_strParameter[0].c_str());
    in_classCommandLine.convertToLowcase(eepromName);

    m_pErrorMsg = m_pciDAPI->showEepromImageVersion(eepromName);
    if(m_pErrorMsg != NO_ERROR_MSG)
    {
        printError(PCI_DEBUG_LEVEL, m_pErrorMsg);
        return -1;
    }

    return 0;
}

int
PCIOperation::readEeprom(StringParser in_classCommandLine)
{
    int         s32Result;
    UINT32      u32StartAddress;
    UINT32*     u32pBuffer;
    UINT32      u32DWordCount;
    UINT32      u32Unit;

    if(m_s32ParmCount != 4)
    {
        cout << SYNTAX_ERROR_MSG << endl;
        return -1;
    }

    s32Result = in_classCommandLine.convertStringToHex(m_strParameter[0], (int*)&u32StartAddress);
    if(s32Result != 0)
    {
        showHelp();
        return s32Result;
    }

    //get unit 
    stringstream(m_strParameter[1]) >> u32Unit;
    //printf("u32Unit: %d\n", u32Unit);//check
    
    //get u32DWordCount, the Buffer Size must be DWord aligned 
    stringstream(m_strParameter[2]) >> u32DWordCount;
    if(u32DWordCount == 0)
    {
        cout << "Bad parameter." << endl;
        showHelp();
        return -1;
    }
    printf("Read from 0x%x, %d DWords.\n", u32StartAddress, u32DWordCount);//check 
    
    u32pBuffer = (UINT32*)malloc(sizeof(UINT32)*u32DWordCount);
    s32Result = readEepromRun(m_strParameter[3], u32Unit, u32StartAddress, u32DWordCount, u32pBuffer);
    free(u32pBuffer);

    return s32Result;
}

int 
PCIOperation::readEepromRun(string   in_strEepromName,
                            UINT32   in_u32Unit,
                            UINT32   in_u32StartAddress,
                            UINT32   in_u32BufferSize, //DWord aligned
                            UINT32  *out_u32pBuffer)
{
    int         s32Result;
    ErrorMsg*   pErrorMsg;
    UINT32      u32EepromSize; //image size, byte aligned
    UINT32      u32InputEndAddress;
    UINT32      u32ImageEndAddress;

    //get eeprom/image size 
    pErrorMsg = m_pciDAPI->getEepromSize(in_strEepromName, &u32EepromSize);
    if(pErrorMsg != NO_ERROR_MSG)
    {
        printError(PCI_DEBUG_LEVEL, pErrorMsg);
        return -1;
    }
    //printf("PCIOperation::readEepromRun, in_u32BufferSize: %d\n", in_u32BufferSize);//check 
    //printf("PCIOperation::readEepromRun, u32EepromSize: %d\n", u32EepromSize);//check 

    u32ImageEndAddress = u32EepromSize; 
    u32InputEndAddress = in_u32StartAddress + (in_u32BufferSize * 4);
    //printf("PCIOperation::readEepromRun, u32ImageEndAddress: 0x%x\n", u32ImageEndAddress);//check 
    //printf("PCIOperation::readEepromRun, u32InputEndAddress: 0x%x\n", u32InputEndAddress);//check

    if(u32InputEndAddress > u32ImageEndAddress)
    {
        printf("Read size is out of range\n");
        printf("The image is from 0x0 - 0x%x\n", u32ImageEndAddress);
        printf("Your read range is from 0x%x - 0x%x\n", in_u32StartAddress, u32InputEndAddress);
        return -1;
    }

    pErrorMsg = m_pciDAPI->readEeprom(in_strEepromName, in_u32StartAddress, in_u32BufferSize, out_u32pBuffer);
    if(pErrorMsg != NO_ERROR_MSG)
    {
        printError(PCI_DEBUG_LEVEL, pErrorMsg);
        return  -1;
    }

    ////check 
    //for(u32Index = 0; u32Index < in_u32BufferSize; u32Index++)
    //{
    //    printf("0x%08x\n", out_u32pBuffer[u32Index]);
    //}

    s32Result = showEepromContent(in_u32Unit, in_u32StartAddress, in_u32BufferSize, out_u32pBuffer);
    if(s32Result != 0)
    {
        showHelp();
    }
    return s32Result;
}


int 
PCIOperation::testEeprom(StringParser in_classCommandLine)
{
    int s32Result = 0;

    if(m_s32ParmCount != 1)
    {
        cout << SYNTAX_ERROR_MSG << endl;
        return -1;
    }
    
    m_pErrorMsg = m_pciDAPI->testEeprom(m_strParameter[0]);
    if(m_pErrorMsg != NO_ERROR_MSG)
    {
        printError(PCI_DEBUG_LEVEL, m_pErrorMsg);
        s32Result = -1;
    }
    
    return s32Result;
}

int 
PCIOperation::programEeprom(StringParser in_classCommandLine)
{
    int  s32Result = 0;
    
    if(m_s32ParmCount != 2)
    {
        cout << SYNTAX_ERROR_MSG << endl;
        return -1;
    }

    m_pErrorMsg = m_pciDAPI->programEeprom(m_strParameter[1], m_strParameter[0]);
    if(m_pErrorMsg != NO_ERROR_MSG)
    {
        printError(PCI_DEBUG_LEVEL, m_pErrorMsg);
        s32Result = -1;
    }
    return s32Result;
}



int 
PCIOperation::saveEeprom(StringParser in_classCommandLine)
{
    int s32Result = 0;
    
    if(m_s32ParmCount != 2)
    {
        cout << SYNTAX_ERROR_MSG << endl;
        return -1;
    }
  
    m_pErrorMsg = m_pciDAPI->saveEeprom(m_strParameter[1], m_strParameter[0].c_str());
    if(m_pErrorMsg != NO_ERROR_MSG)
    {
        printError(PCI_DEBUG_LEVEL, m_pErrorMsg);
        s32Result = -1;
    }

    return s32Result;
}

void  
PCIOperation::showTestRetrainResult(vector<TEST_RESULT>   in_testResult)
{
    bool                            bIsNTB;
    bool                            bDisplayDomain = false;
    RetrainInfo                     retrainInfo;
    PCIInformantion                 pciInfo;
    vector<TEST_RESULT>::iterator   it;

    //check if domain is used
    for(it = in_testResult.begin(); it != in_testResult.end(); ++it)
    {
        if(it->retrainLinkInfo.PciInfo.u16Domain != 0x0)
        {
            bDisplayDomain = true;
        }
    }

    if(bDisplayDomain)
    {
        for(it = in_testResult.begin(); it != in_testResult.end(); ++it)
        {
            retrainInfo = it->retrainLinkInfo;
            pciInfo = retrainInfo.PciInfo;
            bIsNTB = m_pciDAPI->isNtbDevice(retrainInfo);
            if(retrainInfo.LinkStatus)
            {
                printf("%04x:%02x:%02x.%x: %s\n",
                       pciInfo.u16Domain, pciInfo.u16Bus, pciInfo.u16Device, pciInfo.u8Function,
                       (it->bTestPass) ? "Pass" : "Fail");
            }
            else //link is down  
            {
                printf("%04x:%02x:%02x.%x: no link, not test\n",
                        pciInfo.u16Domain, pciInfo.u16Bus, pciInfo.u16Device, pciInfo.u8Function);
            }
        }
    }
    else //bDisplayDomain = false: do not show domain 
    {
        for(it = in_testResult.begin(); it != in_testResult.end(); ++it)
        {
            retrainInfo = it->retrainLinkInfo;
            pciInfo = retrainInfo.PciInfo;
            bIsNTB = m_pciDAPI->isNtbDevice(retrainInfo);
            if(retrainInfo.LinkStatus)
            {
                printf("%02x:%02x.%x: %s\n",
                        pciInfo.u16Bus, pciInfo.u16Device, pciInfo.u8Function,
                        (it->bTestPass) ? "Pass" : "Fail");
            }
            else //link is down  
            {
                printf("%02x:%02x.%x: no link, not test\n",
                        pciInfo.u16Bus, pciInfo.u16Device, pciInfo.u8Function);
            }
        }
     }
}


void
PCIOperation::progressHandler()
{
    int     s32Progress = 0;
    UINT32  u32Index;

    for(u32Index = 0; u32Index < (UINT32)m_s32TotalSteps; u32Index++)
    {
        usleep(25000);
        s32Progress = (u32Index/100.0) * 100;
        //s32Progress += getProgress()/100;
        //printf("PCIDAPI::progressHandler, s32Progress: %d\n", s32Progress);//check
        if(s32Progress > 100)
        {
            //printf("Progressing: 100\%\n");
            break;
        }
        printf("progress:%3d%%\r", s32Progress);
        fflush(stdout);
    }
}


int 
PCIOperation::testRetrain(StringParser in_classCommandLine)
{
    PFA         bdf;
    bool        bDeviceFound = false;
    bool        bTestPass = true;
    thread*     pProgressHandler = NULL;
    ErrorMsg*   pErrorMsg;
    int         s32ParameterAmount;

    TEST_RESULT                     testResult;
    vector<TEST_RESULT>             testResultList;
    RetrainInfo                     retrainInfo;                    
    vector<RetrainInfo>             retrainDeviceList;
    vector<RetrainInfo>::iterator   it;

    m_s32TotalSteps = 0;

    m_pciDAPI->getRetrainSupportedDeviceList(&retrainDeviceList);

    in_classCommandLine.getParameterNumber(&s32ParameterAmount);
    if(s32ParameterAmount == 0)
    {
        pProgressHandler = new thread(&PCIOperation::progressHandler, this);
        
        for(it = retrainDeviceList.begin(); it != retrainDeviceList.end(); ++it)
        {
            m_s32TotalSteps = m_s32TotalSteps + 10;

            testResult.retrainLinkInfo = *it;

            testResult.bTestPass = true;

            bdf.u16Domain   = it->PciInfo.u16Domain;
            bdf.u16Bus      = it->PciInfo.u16Bus;
            bdf.u16Device   = it->PciInfo.u16Device;
            bdf.u8Function  = it->PciInfo.u8Function;
          
            if(it->LinkStatus)
            {
                pErrorMsg = m_pciDAPI->testRetrain(bdf);
                if(pErrorMsg != NO_ERROR_MSG)
                {
                    bTestPass = false;
                    testResult.bTestPass = false;
                }
            }
            testResultList.push_back(testResult);
        }
        
        pProgressHandler->join();
        delete pProgressHandler;
    }
    else //test single device 
    {
        pProgressHandler = new thread(&PCIOperation::progressHandler, this);
        
        m_s32TotalSteps = m_s32TotalSteps + 10;
        
        convertStringToBDF(m_strParameter[0], &bdf);

        for(it = retrainDeviceList.begin(); it != retrainDeviceList.end(); ++it)
        {
            if(bdf.u16Domain == it->PciInfo.u16Domain && bdf.u16Bus == it->PciInfo.u16Bus &&
               bdf.u16Device == it->PciInfo.u16Device && bdf.u8Function == it->PciInfo.u8Function)
            {
                bDeviceFound = true;
                testResult.retrainLinkInfo = *it;
            }
        }
        if(!bDeviceFound)
        {
            cout << "Device is not found or retrain is unsupported" << endl;
            return -1;
        }
        
        bTestPass = true;
        pErrorMsg = m_pciDAPI->testRetrain(bdf);
        if(pErrorMsg != NO_ERROR_MSG)
        {
            bTestPass = false;
            printError(PCI_DEBUG_LEVEL, pErrorMsg);
        }
        testResult.bTestPass = bTestPass;
        testResultList.push_back(testResult);
        
        pProgressHandler->join();
        delete pProgressHandler;
    }
   
    //show test result
    showTestRetrainResult(testResultList);
    
    if(!bTestPass)
    {
        return -1;
    }
    return 0;
}


int 
PCIOperation::searchHiddenDevices(StringParser in_classCommandLine)
{
    bool    bDisplayDomain = false;

    vector<PCIInformantion>             hiddenDeviceList;
    vector<PCIInformantion>::iterator   it;

    m_pErrorMsg = m_pciDAPI->scanHiddenDevices(0, &hiddenDeviceList);
    if(m_pErrorMsg != NO_ERROR_MSG)
    {
        printError(PCI_DEBUG_LEVEL, m_pErrorMsg);
        return -1;
    }

    //show devices
    for(it = hiddenDeviceList.begin(); it != hiddenDeviceList.end(); ++it)
    {
        if(it->u16Domain != 0x0)
        {
            bDisplayDomain = true;
        }
    }

    if(bDisplayDomain)
    {
        for(it = hiddenDeviceList.begin(); it != hiddenDeviceList.end(); ++it)
        {
            printf("%04x:%02x:%02x.%x - %s\n", 
                    it->u16Domain, it->u16Bus, it->u16Device, it->u8Function,
                    it->szDeviceName.c_str());
        }
    }
    else
    {
        for(it = hiddenDeviceList.begin(); it != hiddenDeviceList.end(); ++it)
        {
            printf("%02x:%02x.%x - %s\n", 
                    it->u16Bus, it->u16Device, it->u8Function,
                    it->szDeviceName.c_str());
        }
    }

    return 0;
}


bool 
PCIOperation::isHex(STRING in_constStrHex)
{
    int s32StringLength;
    int s32Index = 0;

    s32StringLength = strlen(in_constStrHex);

    while(s32Index < s32StringLength)
    {
        if(isxdigit(in_constStrHex[s32Index]) == 0)
        {
            return false;
        }
        s32Index++;
    }

    return true;
}


bool 
PCIOperation::isBDF(string in_strParameter)
{
    int     s32ReturnPosition;
    string  strTemp;
    string  strBDF;
    int     s32Length;

    s32ReturnPosition = in_strParameter.find(':', 0);
    //printf("s32ReturnPosition - %d\n", s32ReturnPosition);//check
    if(s32ReturnPosition == 2)
    {
        strBDF = "0000:";
        strBDF.append(in_strParameter);
    }
    else if(s32ReturnPosition == 4)
    {
        strBDF = in_strParameter;
    }
    else
    {
        return false;
    }
    //printf("strBDF: %s\n", strBDF.c_str());//check 

    s32Length = strlen(strBDF.c_str());
    //printf("s32Length -- %d\n", s32Length);//check
    if(s32Length != 12)
    {
        return false; 
    }

    strTemp = strBDF.substr(0,4);
    //printf("PCIOperation::isBDF, Domain: %s\n", strTemp.c_str()); //check
    if(isHex(strTemp.c_str())) //check domain 
    {
        strTemp = strBDF.substr(5,2);
        //printf("PCIOperation::isBDF, Bus: %s\n", strTemp.c_str()); //check
        if(isHex(strTemp.c_str())) //check bus 
        {
            strTemp = strBDF.substr(8,2);
            //printf("PCIOperation::isBDF, Device: %s\n", strTemp.c_str()); //check 
            if(isHex(strTemp.c_str())) //check device
            {
                strTemp = strBDF.substr(11,1);
                //printf("PCIOperation::isBDF, Function: %s\n", strTemp.c_str()); //check
                return isHex(strTemp.c_str()); //check function 
            }
        }
    }
    
    return false;
}


int 
PCIOperation::checkBDF(string in_strParameter)
{
    //cout << "PCIOperation::checkBDF" << endl; //check 

    int     s32ReturnPosition;
    string  strTemp;
    string  strBDF;


    //check the format of BDF
    s32ReturnPosition = in_strParameter.find(':', 0);
    if(s32ReturnPosition != 4 && s32ReturnPosition != 2)
    {
        return -1;
    }
    if(s32ReturnPosition == 2)
    {
        strBDF = "0000:";
        strBDF.append(in_strParameter);
    }
    else 
    {
        strBDF = in_strParameter;
    }
    
    strTemp = strTemp.assign(strBDF, 5, 12);
    if(strTemp.size() != 7)
    {
        return -1;
    }
    s32ReturnPosition = strTemp.find(':', 0);
    if(s32ReturnPosition != 2)
    {
        return -1;
    }

    s32ReturnPosition = strTemp.find('.', 0);
    if(s32ReturnPosition != 5)
    {
        return -1;
    }

    //check BDF value
    if(isBDF(strBDF))
    {
        return 0;
    }
    else 
    {
        return -1;
    }
}

int 
PCIOperation::showPciDevicesByClass(string in_strClass)
{
    bool                                displayDomain = false;
    int                                 s32Index;
    UINT32                              u32ClassCode;
    vector<PCIInformantion>             pciInfoList;
    vector<PCIInformantion>::iterator   it;
    
    ErrorMsg*   pErrorMsg;

    pErrorMsg = m_pciDAPI->convertStringToClassCode(in_strClass, &u32ClassCode);
    if(pErrorMsg != NO_ERROR_MSG)
    {
        printError(PCI_DEBUG_LEVEL, pErrorMsg);
        return -1;
    }
    //printf("PCICommand::listPciDevicesByClass, u32ClassCode: 0x%x\n", u32ClassCode);//check 

    m_pciDAPI->getDevices(&pciInfoList);
       
    //check if domain is used 
    for(it = pciInfoList.begin(); it != pciInfoList.end(); ++it)
    {
        if(it->u16Domain != 0x0)
        {
            displayDomain = true;
        }
    }

    s32Index = 0;
    printf("Device List:\n");
    printf("-------------------------------------------------------\n");

    for(it = pciInfoList.begin(); it != pciInfoList.end(); ++it)
    {
        //printf("PCICommand::listPciDevicesByClass, it->u32ClassCode: 0x%x\n", it->u32ClassCode);//check 
        if(it->u32ClassCode == u32ClassCode)
        {
            if(displayDomain)
            {
                printf("%04x:%02x:%02x.%x %s\n",
                        it->u16Domain, it->u16Bus, it->u16Device, it->u8Function, it->szDeviceName.c_str());
            }
            else 
            {
                printf("%02x:%02x.%x %s\n",
                        it->u16Bus, it->u16Device, it->u8Function, it->szDeviceName.c_str());
            }
            s32Index++;
        }
    }
        
    printf("-------------------------------------------------------\n");
    printf("%d devices are found\n", s32Index);

    return 0;
}


void 
PCIOperation::showPciDevices()
{
    bool                                displayDomain = false;
    int                                 s32Index;
    vector<PCIInformantion>             pciInfoList;
    vector<PCIInformantion>::iterator   it;
  
    m_pciDAPI->getDevices(&pciInfoList);
    
    //check if domain is used 
    for(it = pciInfoList.begin(); it != pciInfoList.end(); ++it)
    {
        if(it->u16Domain != 0x0)
        {
            displayDomain = true;
        }
    }
        
    s32Index = 0;
    printf("Device List:\n");
    printf("-------------------------------------------------------\n");

    for(it = pciInfoList.begin(); it != pciInfoList.end(); ++it)
    {
        if(displayDomain)
        {
            printf("%04x:%02x:%02x.%x %s\n",
                    it->u16Domain, it->u16Bus, it->u16Device, it->u8Function, it->szDeviceName.c_str());
        }
        else 
        {
            printf("%02x:%02x.%x %s\n",
                    it->u16Bus, it->u16Device, it->u8Function, it->szDeviceName.c_str());
        }
        s32Index++;
    }
        
    printf("-------------------------------------------------------\n");
    printf("%d devices are found\n", s32Index);
}


int 
PCIOperation::listPCIeSwitch()
{
    vector<SWITCH_UNIT>             switchList;
    vector<SWITCH_UNIT>::iterator   it;

    ErrorMsg*   pErrorMsg;

    pErrorMsg = m_pciDAPI->getPCIeSwitchList(&switchList);
    if(pErrorMsg != NO_ERROR_MSG)
    {
        printError(PCI_DEBUG_LEVEL, pErrorMsg);
        return -1;
    }

    if(switchList.empty())
    {
        cout << "No PCIe switch is found" << endl;
        return 0;
    }

    cout << endl;
    cout << "###  " << "device_name" << endl;
    cout << "------------------" << endl;
   
    for(it = switchList.begin(); it != switchList.end(); ++it)
    {
        if((it->bIsVirtualSwitch))
        {
            continue;
        }

        if (it->name.length() > 7)
        {
            printf("%s %s\n", it->name.c_str(), it->deviceName.c_str());
        }
        else
        {
            printf("%s\t %s\n", it->name.c_str(), it->deviceName.c_str());
        }
    }

    cout << endl;

    return 0;
}


int 
PCIOperation::listPciDevices(StringParser in_classCommandLine)
{
    int         sResult = 0;
    string      strClass;
    int         s32ParameterAmount;

    in_classCommandLine.getParameterNumber(&s32ParameterAmount);
    if(s32ParameterAmount == 1)
    {
        in_classCommandLine.getParameter(&strClass, 0);
        if(strcmp(strClass.c_str(), "type") == 0)
        {
            m_pciDAPI->showPciDefinedTypes();
        }
        else 
        {
            sResult = showPciDevicesByClass(strClass);
        }
    }
    else 
    {
        showPciDevices();
    }

    return sResult;
}

int
PCIOperation::listDevices(StringParser in_classCommandLine)
{
    switch(getOperationType())
    {
        case OPERATION_EEPROM:
            m_pciDAPI->DisplayPCIeSwitchEepromList();
            break;
        case OPERATION_HBA:
             m_pciDAPI->displayHBAs();
             break;
        case OPERATION_PCI:
            return listPciDevices(in_classCommandLine);
        case OPERATION_SWITCH:
            return listPCIeSwitch();
        default:
            break;
    }

    return 0;
}


void 
PCIOperation::showRetrainInfo(StringParser in_classCommandLine)
{
    bool    bIsNTB = false;
    bool    bDisplayDomain = false; 

    vector<RetrainInfo>             retrainDeviceList;
    vector<RetrainInfo>::iterator   it;
        
    m_pciDAPI->getRetrainSupportedDeviceList(&retrainDeviceList);

    for(it = retrainDeviceList.begin(); it != retrainDeviceList.end(); ++it)
    {
        if(it->PciInfo.u16Domain != 0x0)
        {
            bDisplayDomain = true;
            break;
        }
    } 

    cout << "avaiable retrain paths:" << endl;
    if(bDisplayDomain)
    {
        for(it = retrainDeviceList.begin(); it != retrainDeviceList.end(); ++it)
        {
            bIsNTB = m_pciDAPI->isNtbDevice(*it);
            if(it->LinkStatus)
            {
                if(bIsNTB)
                {
                    printf("%04x:%02x:%02x.%x <-----> NTB link \n",
                            it->PciInfo.u16Domain, it->PciInfo.u16Bus, it->PciInfo.u16Device, it->PciInfo.u8Function);
                }
                else 
                {
                    printf("%04x:%02x:%02x.%x <-----> %04x:%02x:%02x.%x \n",
                            it->PciInfo.u16Domain, it->PciInfo.u16Bus, it->PciInfo.u16Device, it->PciInfo.u8Function,
                            it->u16DomainDown, it->u16BusDown, it->u16DeviceDown, it->u8FunctionDown);
                }
            }
            else 
            {
                if(bIsNTB)
                {
                    printf("%04x:%02x:%02x.%x <-----> NTB (link is down)\n",
                            it->PciInfo.u16Domain, it->PciInfo.u16Bus, it->PciInfo.u16Device, it->PciInfo.u8Function);
                }
                else 
                {
                printf("%04x:%02x:%02x.%x <-----> link is down\n",
                        it->PciInfo.u16Domain, it->PciInfo.u16Bus, it->PciInfo.u16Device, it->PciInfo.u8Function);
                }
            }
        }
    }
    else //do not show domain  
    {
        for(it = retrainDeviceList.begin(); it != retrainDeviceList.end(); ++it)
        {
            bIsNTB = m_pciDAPI->isNtbDevice(*it);
            if(it->LinkStatus)
            {
                if(bIsNTB)
                {
                    printf("%02x:%02x.%x <-----> NTB link \n",
                            it->PciInfo.u16Bus, it->PciInfo.u16Device, it->PciInfo.u8Function);
                }
                else 
                {
                    printf("%02x:%02x.%x <-----> %02x:%02x.%x \n",
                            it->PciInfo.u16Bus, it->PciInfo.u16Device, it->PciInfo.u8Function,
                            it->u16BusDown, it->u16DeviceDown, it->u8FunctionDown);
                }
            }
            else 
            {
                if(bIsNTB)
                {
                    printf("%02x:%02x.%x <-----> NTB link (link is down)\n",
                            it->PciInfo.u16Bus, it->PciInfo.u16Device, it->PciInfo.u8Function);
                }
                else 
                {
                    printf("%02x:%02x.%x <-----> (link is down)\n",
                            it->PciInfo.u16Bus, it->PciInfo.u16Device, it->PciInfo.u8Function);
                }
            }
        }
    }
}


int 
PCIOperation::checkSyntax(StringParser in_classCommandLine)
{
    //bool    bIsHex;
    int     s32Index;
    int     s32ParameterAmount;
    int     u32Result;

    in_classCommandLine.getParameterNumber(&s32ParameterAmount);
    
    for(s32Index = 0; s32Index < s32ParameterAmount; s32Index++)
    {
        in_classCommandLine.getParameter(&m_strParameter[s32Index], s32Index);
        //printf("m_strParameter[%d]: %s\n",s32Index, m_strParameter[s32Index].c_str()); //check 
    }

    switch(getOperationType())
    {

        case OPERATION_ALL:
        case OPERATION_ID:
        case OPERATION_CLASS_TYPE:
        case OPERATION_PORT_TYPE:
        case OPERATION_SHOW_LINK:
        case OPERATION_SHOW_SPEED:
        case OPERATION_SHOW_WIDTH:
        case OPERATION_CLEAR:
        case OPERATION_ERROR_ALL:
        case OPERATION_PCI_COMPATIBLE:
        case OPERATION_PCIE:
        case OPERATION_AER:
            return checkBDF(m_strParameter[0]);

        //retrain test 
        case OPERATION_RETRAIN:
            if(s32ParameterAmount == 0)
            {
                return 0;
            }
            else if(s32ParameterAmount == 1)
            {
                return checkBDF(m_strParameter[0]);
            }
            else 
            {
                return -1;
            }

        case OPERATION_CONFIG_READ:
        case OPERATION_CONFIG_WRITE:
            u32Result = checkBDF(m_strParameter[2]);
            if(u32Result != 0)
            {
                cout << "Bad BDF format." << endl;
                return -1;
            }
            u32Result = in_classCommandLine.isHEXString(m_strParameter[0]);
            if(u32Result != 0)
            {
                cout << "Parameter 2 isn't a hexadecimal number." << endl;
                return -1;
            }
            u32Result = in_classCommandLine.isHEXString(m_strParameter[1]);
            if(u32Result != 0)
            {
                cout << "Parameter 3 isn't a hexadecimal number." << endl;
                return -1;
            }
            return 0;

        case OPERATION_SEARCH:
            //bIsHex = isHex(m_strParameter[0].c_str());
            //if(!bIsHex)
            //{
            //    return -1;
            //}
            return 0;

        case OPERATION_DUMP:
        case OPERATION_TEST:
            if(s32ParameterAmount != 1)
            {
                return -1;
            }
            return 0;

        case OPERATION_READ:
            if(s32ParameterAmount != 4)
            {
                return -1;
            }
            return 0;

        case OPERATION_WRITE:
            if(s32ParameterAmount != 3)
            {
                return -1;
            }
            return 0;

        case OPERATION_PROGRAM:
        case OPERATION_SAVE: 
            if(s32ParameterAmount != 2)
            {
                return -1;
            }
            if(m_strParameter[0][0] != '/')
            {
                return -1;
            }
            return 0;

        default:
            return 0;                
    }
}


int 
PCIOperation::executeOperation(StringParser in_classCommandLine)
{
    int s32Index;

    m_s32Result = -1;
    
    //get parameter amount
    in_classCommandLine.getParameterNumber(&m_s32ParmCount);

    //get paramters
    for(s32Index = 0; s32Index < m_s32ParmCount; s32Index++)
    {
        in_classCommandLine.getParameter(&m_strParameter[s32Index], s32Index);
        //printf("m_strParameter[%d]: %s\n",s32Index, m_strParameter[s32Index].c_str()); //check 
    }
    
    in_classCommandLine.getParameter(&m_strParameter[0], 0);
    if(m_strParameter[0] == HELP)
    {
        showHelp();
        return 0;
    }

    switch(getOperationType())
    {
        case OPERATION_EEPROM:
        case OPERATION_HBA:
        case OPERATION_PCI:
        case OPERATION_SWITCH:
            return listDevices(in_classCommandLine);

        case OPERATION_RETRAIN_LINK:
            showRetrainInfo(in_classCommandLine);
            return 0;
        case OPERATION_ALL:
        case OPERATION_ID:
        case OPERATION_CLASS_TYPE:
        case OPERATION_PORT_TYPE:
        case OPERATION_SHOW_LINK:
        case OPERATION_SHOW_SPEED:
        case OPERATION_SHOW_WIDTH:
            return showPciInfo(in_classCommandLine);
        
        //for ERROR operation 
        case OPERATION_ERROR_ALL:
        case OPERATION_PCI_COMPATIBLE:
        case OPERATION_PCIE:
        case OPERATION_AER:
            return showPciErrorStatistics();
        case OPERATION_CLEAR:
            return clearErrors();
       
        //for UTILITY operation
        case OPERATION_CONFIG_READ:
            return readConfig(in_classCommandLine);
        case OPERATION_CONFIG_WRITE:
            return writeConfig(in_classCommandLine);
        case OPERATION_SEARCH:
            return searchHiddenDevices(in_classCommandLine);

        //for EEPROM operation 
        case OPERATION_DUMP:
            return dumpEeprom(in_classCommandLine);
        case OPERATION_READ:
            return readEeprom(in_classCommandLine);
        case OPERATION_WRITE:
            return writeEeprom(in_classCommandLine);
        case OPERATION_PROGRAM:
            return programEeprom(in_classCommandLine);
        case OPERATION_SAVE:
            return saveEeprom(in_classCommandLine);
        case OPERATION_TEST:
            m_s32Result = testEeprom(in_classCommandLine);
            break;
        case OPERATION_FW:
            return showEepromFW(in_classCommandLine);

        //retrain test 
        case OPERATION_RETRAIN:
            m_s32Result = testRetrain(in_classCommandLine);
            break;

        default:
            break;
    }
   
    //m_s32Result == 0 ? cout << "Pass" : cout << "Fail";
    //cout << endl;

    return m_s32Result;    
}




void 
PCIOperation::showHelp()
{
    string strHelpContent;

    getHelp(&strHelpContent);
    cout << strHelpContent << endl;
    cout << "Parameters required:" << endl;

    switch(getOperationType())
    {
        case OPERATION_ALL:
        case OPERATION_ID:
        case OPERATION_CLASS_TYPE:
        case OPERATION_PORT_TYPE:
        case OPERATION_SHOW_LINK:
        case OPERATION_SHOW_SPEED:
        case OPERATION_SHOW_WIDTH:
            cout << "  P1: PCI device's address (xxxx:xx:xx.x or xx:xx.x, Hex)" << endl;
            break;

        case OPERATION_CONFIG_READ:
            cout << "  P1: Start address (Hex)" << endl;
            cout << "  P2: Read length (Hex)" << endl;
            cout << "  P3: PCI device's address (xxxx:xx:xx.x or xx:xx.x, Hex)" << endl;
            break;

        case OPERATION_CONFIG_WRITE:
            cout << "  P1: Start address (Hex)" << endl;
            cout << "  P2: Write data (Hex)" << endl;
            cout << "  P3: PCI device's address (xxxx:xx:xx.x or xx:xx.x, Hex)" << endl;
            break;

        case OPERATION_READ:
            cout << "  P1: Start Address (Hex)" << endl;
            cout << "  P2: Unit, 8/16/32 (Dec)" << endl;
            cout << "  P3: Length in DWord (Dec)" << endl;
            cout << "  P4: EEPROM Name" << endl;
            break;

        case OPERATION_WRITE:
            cout << "  P1: Start address (Hex)" << endl;
            cout << "      Address must be DWord aligned" << endl;
            cout << "  P2: Write data (Hex)" << endl;
            cout << "      The data must be DWord base." << endl;
            cout << "  P3: EEPROM name" << endl;
            cout << endl;
            cout << "Examples:" << endl;
            cout << "     lemur pci eeprom write 0x0 0x12345678 eeprom0" << endl;
            cout << "     lemur pci eeprom write 0x0 0x1234567800000000 eeprom0" << endl;
            cout << "     lemur pci eeprom write 0x4 0x12345678 eeprom0" << endl;
            break;

        case OPERATION_SEARCH:
            cout << "  No parameter is need" << endl;
            //cout << " P1: domain to be searched (Hex)" << endl;
            break;

        case OPERATION_DUMP:
        case OPERATION_TEST:
        case OPERATION_FW:
            cout << "  P1: EEPROM name, (EEPROM0, EEPROM1...etc)" << endl; 
            break;

        case OPERATION_SAVE:
        case OPERATION_PROGRAM:
            cout << "  P1: /path/file_name" << endl;
            cout << "  P2: EEPROM name, (EEPROM0, EEPROM1...etc)" << endl;
            break;

        case OPERATION_RETRAIN:
            cout << "  P1: PCI device's address (xxxx:xx:xx.x or xx:xx.x, Hex, optional)" << endl;
            cout << "      If P1 is not given, test all retrain supported devices" << endl;
            cout << endl;
            cout << " Examples:" << endl;
            cout << "      lemur pci test retrain" << endl;
            cout << "      lemur pci test retrain 00:02.1" << endl;
        
        //for ERROR operation 
        case OPERATION_ERROR_ALL:
        case OPERATION_PCI_COMPATIBLE:
        case OPERATION_PCIE:
        case OPERATION_AER:
        case OPERATION_CLEAR:
            cout << "  P1: PCI device's address (xxxx:xx:xx.x or xx:xx.x, Hex)" << endl;
            break;

        default:
            break;
    }
}


