#include "RTCOperation.hpp"
#include <string.h>
#include <stdlib.h>

RTCOperation:: RTCOperation(OPERATION_TYPE oprationType, string helpContent) : OperationBaseClass(oprationType, helpContent)
{

}

RTCOperation::~ RTCOperation()
{

}

int
RTCOperation::executeOperation(StringParser commandLine)
{
    string  parameter;
    int     err = 0;

    switch (getOperationType())
    {
        case OPERATION_DATE:
        case OPERATION_TIME:
        case OPERATION_DATETIME:
            commandLine.getParameter(&parameter, 0);
            if (parameter == HELP)
            {
                helpParameter(commandLine);
                return 0;
            }
            break;
        default:
            break;
    }
    
    err = checkSyntax(commandLine);
    if (err == 0)
    {
        //Execute operation
        switch (getOperationType())
        {
            case OPERATION_DATE:
                err = date(commandLine);
                break;
            case OPERATION_TIME:
                err = time(commandLine);
                break;
            case OPERATION_DATETIME:
                err = dateTime(commandLine);
                break;
            case OPERATION_ACCURACY:
                err = accuracy(commandLine);
                break;
            case OPERATION_WDT:
                err = wdt(commandLine);
                break;
            default:
                cout << UNKNOWN_OPERATION_MSG << endl;
                err = -1; 
                break;
        }
    }
    
    return err;
}

void
RTCOperation::helpParameter(StringParser commandLine)
{
    string strOpName, strHelp;

    getName(&strOpName);
    getHelp(&strHelp);

    cout << strHelp << endl;
    cout << "Syntax " << strOpName << " ";
    
    //Show parameter help message depend on operation type
    switch (getOperationType())
    {
        case OPERATION_DATE:
            cout << "P1 P2 P3 P4" << endl;
            cout << "P1: Year" << endl;
            cout << "P2: Month" << endl;
            cout << "P3: Date" << endl;
            cout << "P4: Day" << endl;
            break;
        case OPERATION_TIME:
            cout << "P1 P2 P3" << endl;
            cout << "P1: Hour" << endl;
            cout << "P2: Minute" << endl;
            cout << "P3: Second" << endl;
            break;
        case OPERATION_DATETIME:
            cout << "P1 P2 P3 P4 P5 P6 P7" << endl;
            cout << "P1: Year" << endl;
            cout << "P2: Month" << endl;
            cout << "P3: Date" << endl;
            cout << "P4: Day" << endl;
            cout << "P5: Hour" << endl;
            cout << "P6: Minute" << endl;
            cout << "P7: Second" << endl;
            break;
        case OPERATION_ACCURACY:
            cout << endl;
            break;
        case OPERATION_WDT:
            cout << endl;
            break;
        default:
            break;
    }
}

int
RTCOperation::date(StringParser commandLine)
{
    RTCDAPI     RTCDAPI("RTCDAPI");
    ErrorMsg*   pErrorMsg = NO_ERROR_MSG;
    string      tempString;
    int         year, month, date;
    char        day[4];

    commandLine.getParameter(&tempString, 0);
    year = atoi(tempString.c_str());
    commandLine.getParameter(&tempString, 1);
    month = atoi(tempString.c_str());
    commandLine.getParameter(&tempString, 2);
    date = atoi(tempString.c_str());
    commandLine.getParameter(&tempString, 3);
    strcpy(day, tempString.c_str());
    
    for (INT32 index = 0; day[index]; index++)
    {
        day[index] = toupper(day[index]);
    }

    if ((pErrorMsg = RTCDAPI.getErrorMsg()))
    {
        printError(pErrorMsg);
        return -1;
    }
    
    //Get  RTC device data by API
    pErrorMsg = RTCDAPI.setDate(year, month, date, day);
    if (pErrorMsg != NO_ERROR_MSG)
    {
        printError(pErrorMsg);
        return -1;
    }

    return 0;
}

int
RTCOperation::time(StringParser commandLine)
{
    RTCDAPI     RTCDAPI("RTCDAPI");
    ErrorMsg*   pErrorMsg = NO_ERROR_MSG;
    string      tempString;
    int         hour, minute, second;

    commandLine.getParameter(&tempString, 0);
    hour = atoi(tempString.c_str());
    commandLine.getParameter(&tempString, 1);
    minute = atoi(tempString.c_str());
    commandLine.getParameter(&tempString, 2);
    second = atoi(tempString.c_str());

    if ((pErrorMsg = RTCDAPI.getErrorMsg()))
    {
        printError(pErrorMsg);
        return -1;
    }
    //Get  RTC device data by API
    pErrorMsg = RTCDAPI.setTime(hour, minute, second);
    if (pErrorMsg != NO_ERROR_MSG)
    {
        printError(pErrorMsg);
        return -1;
    }

    return 0;
}

int
RTCOperation::dateTime(StringParser commandLine)
{
    RTCDAPI     RTCDAPI("RTCDAPI");
    ErrorMsg*   pErrorMsg = NO_ERROR_MSG;
    string      tempString;
    int         year, month, date;
    char        day[4];
    int         hour, minute, second;
	
    commandLine.getParameter(&tempString, 0);
    year = atoi(tempString.c_str());
    commandLine.getParameter(&tempString, 1);
    month = atoi(tempString.c_str());
    commandLine.getParameter(&tempString, 2);
    date = atoi(tempString.c_str());
    commandLine.getParameter(&tempString, 3);
    strcpy(day, tempString.c_str());
    commandLine.getParameter(&tempString, 4);
    hour = atoi(tempString.c_str());
    commandLine.getParameter(&tempString, 5);
    minute = atoi(tempString.c_str());
    commandLine.getParameter(&tempString, 6);
    second = atoi(tempString.c_str());
    
    for (INT32 index = 0; day[index]; index++)
    {
        day[index] = toupper(day[index]);
    }
    
    if ((pErrorMsg = RTCDAPI.getErrorMsg()))
    {
        printError(pErrorMsg);
        return -1;
    }
    
    //Get  RTC device data by API
    pErrorMsg = RTCDAPI.setDateTime(year, month, date, day,
                                    hour, minute, second);
    if (pErrorMsg != NO_ERROR_MSG)
    {
        printError(pErrorMsg);
        return -1;
    }

    return 0;
}

//!!
int
RTCOperation::accuracy(StringParser commandLine)
{
    RTCDAPI     RTCDAPI("RTCDAPI");
    ErrorMsg*   pErrorMsg = NO_ERROR_MSG;

    if ((pErrorMsg = RTCDAPI.getErrorMsg()))
    {
        printError(pErrorMsg);
        return -1;
    }
    
    pErrorMsg = RTCDAPI.rtcAccuracyTest();

    if (pErrorMsg)
    {
        printError(pErrorMsg);
        return -1;
    }

    return 0;
}

//!!
int
RTCOperation::wdt(StringParser commandLine)
{
    RTCDAPI     RTCDAPI("RTCDAPI");
    ErrorMsg*   pErrorMsg = NO_ERROR_MSG;

    if ((pErrorMsg = RTCDAPI.getErrorMsg()))
    {
        printError(pErrorMsg);
        return -1;
    }
    
    pErrorMsg = RTCDAPI.rtcWDTTest();

    if (pErrorMsg)
    {
        printError(pErrorMsg);
        return -1;
    }

    return 0;
}

int
RTCOperation::checkSyntax(StringParser commandLine)
{
    int     index;
    int     err = 0;
    int     number;
    int     parameterNumber = 0;
    string  *parameter;
    string  operation_name;

    getName(&operation_name);

    commandLine.getParameterNumber(&parameterNumber);
    parameter = new string [parameterNumber];

    //Get parameter
    for (index = 0; index < parameterNumber; index++)
    {
        commandLine.getParameter(&parameter[index], index);
    }

    switch (getOperationType())
    {
        case OPERATION_DATE:
            //Check parameter number
            if (parameterNumber == 4)
            {
               //Check parameter 1 is numeric
                err = commandLine.convertStringToDec(parameter[0], &number);
                if (err != 0)
                {
                    cout << "Parameter 1 isn't numeric." << endl;
                    err = -1;
                }

                if (err == 0)
                {
                    //Check parameter 2 is numeric
                    err = commandLine.convertStringToDec(parameter[1], &number);
                    if (err != 0)
                    {
                        cout << "Parameter 2 isn't numeric." << endl;
                        err = -1;
                    }
                }
                
                if (err == 0)
                {
                    //Check parameter 3 is numeric
                    err = commandLine.convertStringToDec(parameter[2], &number);
                    if (err != 0)
                    {
                        cout << "Parameter 3 isn't numeric." << endl;
                        err = -1;
                    }
                }
            }
            else
            {
                cout << "The number of parameter isn't 4" << endl;
                err = -1;
            }
            break;
        case OPERATION_TIME:           
            //Check parameter number
            if (parameterNumber == 3)
            {
               //Check parameter 1 is numeric
                err = commandLine.convertStringToDec(parameter[0], &number);
                if (err != 0)
                {
                    cout << "Parameter 1 isn't numeric." << endl;
                    err = -1;
                }

                if (err == 0)
                {
                    //Check parameter 2 is numeric
                    err = commandLine.convertStringToDec(parameter[1], &number);
                    if (err != 0)
                    {
                        cout << "Parameter 2 isn't numeric." << endl;
                        err = -1;
                    }
                }
                
                if (err == 0)
                {
                    //Check parameter 3 is numeric
                    err = commandLine.convertStringToDec(parameter[2], &number);
                    if (err != 0)
                    {
                        cout << "Parameter 3 isn't numeric." << endl;
                        err = -1;
                    }
                }
            }
            else
            {
                cout << "The number of parameter isn't 3" << endl;
                err = -1;
            }
            break;
        case OPERATION_DATETIME:            
            //Check parameter number
            if (parameterNumber == 7)
            {
                //Check parameter 1 is numeric
                err = commandLine.convertStringToDec(parameter[0], &number);
                if (err != 0)
                {
                    cout << "Parameter 1 isn't numeric." << endl;
                    err = -1;
                }

                if (err == 0)
                {
                    //Check parameter 2 is numeric
                    err = commandLine.convertStringToDec(parameter[1], &number);
                    if (err != 0)
                    {
                        cout << "Parameter 2 isn't numeric." << endl;
                        err = -1;
                    }
                }
                
                if (err == 0)
                {
                    //Check parameter 3 is numeric
                    err = commandLine.convertStringToDec(parameter[2], &number);
                    if (err != 0)
                    {
                        cout << "Parameter 3 isn't numeric." << endl;
                        err = -1;
                    }
                }

                if (err == 0)
                {
                    //Check parameter 5 is numeric
                    err = commandLine.convertStringToDec(parameter[4], &number);
                    if (err != 0)
                    {
                        cout << "Parameter 5 isn't numeric." << endl;
                        err = -1;
                    }
                }
                
                if (err == 0)
                {
                    //Check parameter 6 is numeric
                    err = commandLine.convertStringToDec(parameter[5], &number);
                    if (err != 0)
                    {
                        cout << "Parameter 6 isn't numeric." << endl;
                        err = -1;
                    }
                }
                
                if (err == 0)
                {
                    //Check parameter 7 is numeric
                    err = commandLine.convertStringToDec(parameter[6], &number);
                    if (err != 0)
                    {
                        cout << "Parameter 7 isn't numeric." << endl;
                        err = -1;
                    }
                }
            }
            else
            {
                cout << "The number of parameter isn't 7" << endl;
                err = -1;
            }
            
            break;       
        case OPERATION_ACCURACY:
            //Check parameter number
            if (parameterNumber != 0)
            {
                cout << "Too many parameter." << endl;
                err = -1;
            }
            break;
        case OPERATION_WDT:
            //Check parameter number
            if (parameterNumber != 0)
            {
                cout << "Too many parameter." << endl;
                err = -1;
            }
            break;
       default:
            cout << "Not define syntax_check for " << operation_name << endl;
            err = -1;
            break;
    }

    delete [] parameter;

    return err;
}
