#include "SASCommand.hpp"

SASCommand::SASCommand(COMMAND_TYPE CommandType, string strHelpContent) :
        CommandBaseClass(CommandType, strHelpContent)
{
    //Base on command type to add different operations
    switch(CommandType)
    {
        case CMD_LIST:
            addOperation(new SASOperation(OPERATION_CONTROLLER, "Display SAS Controller Information."));
            addOperation(new SASOperation(OPERATION_PHY, "Display SAS Controller PHY Information."));
            addOperation(new SASOperation(OPERATION_DRIVE, "Display SAS Controller Attached Drive Information."));

            break;
        case CMD_SHOW:
            addOperation(new SASOperation(OPERATION_ALL, "Display All Information.(Controller | PHY | Drive)"));
            addOperation(new SASOperation(OPERATION_HW, "Display Hardware Version.(Controller | Drive)"));
            addOperation(new SASOperation(OPERATION_FW, "Display Firmware Version.(Controller | Drive)"));
            addOperation(new SASOperation(OPERATION_DRIVER, "Display Controller Driver Version."));
            addOperation(new SASOperation(OPERATION_BIOS, "Display Controller BIOS Version."));
            //addOperation(new SASOperation(OPERATION_PORT, "Display Port Number.(Controller | PHY)"));
            addOperation(new SASOperation(OPERATION_ADDRESS, "Display SAS Address.(Controller | PHY | Drive)"));
            addOperation(new SASOperation(OPERATION_TEMP, "Display Temperature.(Controller | Drive)"));
            addOperation(new SASOperation(OPERATION_BDF, "Display Controller Bus:Device.Function."));
            //addOperation(new SASOperation(OPERATION_RAMDISK, "Display Controller Attached RAM Disk."));
            addOperation(new SASOperation(OPERATION_ATTACHED_TYPE, "Display PHY Attached Devices Type."));
            addOperation(new SASOperation(OPERATION_LINK, "Display PHY Link Status."));
            addOperation(new SASOperation(OPERATION_SPEED, "Display Link Speed.(PHY | Drive)"));
            addOperation(new SASOperation(OPERATION_NODE, "Display Drive Node Name on OS."));
            addOperation(new SASOperation(OPERATION_SN, "Display Drive Serial Number."));
            addOperation(new SASOperation(OPERATION_LOGICAL_BLOCK_SIZE, "Display Drive Logical Block Size."));
            addOperation(new SASOperation(OPERATION_PHYSICAL_BLOCK_SIZE, "Display Drive Physical Block Size."));
            addOperation(new SASOperation(OPERATION_MODEL, "Display Drive Model"));
            addOperation(new SASOperation(OPERATION_CAPACITY, "Display Drive Capacity (size)."));
            addOperation(new SASOperation(OPERATION_FIRMWARE_PID, "Display Firmware Product ID."));
            addOperation(new SASOperation(OPERATION_NVDATA_PID, "Display NVDATA Product ID."));
            addOperation(new SASOperation(OPERATION_FCODE_VERSION, "Display FCODE version."));
            addOperation(new SASOperation(OPERATION_UEFI_BSD_VERSION, "Display UEFI BSD version."));
            addOperation(new SASOperation(OPERATION_NVDATA_VENDOR, "Display NVDATA vendor name."));
            addOperation(new SASOperation(OPERATION_DEFAULT_NVDATA_VERSION, "Display default NVDATA version."));
            addOperation(new SASOperation(OPERATION_PERSISTENT_NVDATA_VERSION, "Display persistent NVDATA version."));

            break;
        case CMD_ERROR:
            addOperation(new SASOperation(OPERATION_ERROR_ALL, "Get PCI/SAS Error statistics."));
            addOperation(new SASOperation(OPERATION_ERROR_STATISTICS, "Get SAS Error statistics.(Controller | PHY)"));
            addOperation(new SASOperation(OPERATION_ERROR_PCI, "Get SAS Chip PCI Error statistics."));
            addOperation(new SASOperation(OPERATION_DISPARITY_ERR, "Get Disparity Error Count.(Controller | PHY)"));
            addOperation(new SASOperation(OPERATION_INVALID_DWORD_ERR, "Get Invalid Dword Error Count.(Controller | PHY)"));
            addOperation(new SASOperation(OPERATION_LOSS_DWORD_SYNC, "Get Loss Dword Synch Error Count.(Controller | PHY)"));
            addOperation(new SASOperation(OPERATION_PHY_RESET_PRBM, "Get PHY Reset Problem Error Count.(Controller | PHY)"));
            addOperation(new SASOperation(OPERATION_CLEAR, "Clear PCI/SAS Error statistics.(Controller | PHY)"));

            break;
        case CMD_UTILITY:
            addOperation(new SASOperation(OPERATION_RESET_CONTROLLER, "Reset SAS Controller."));
            addOperation(new SASOperation(OPERATION_SMART, "Get SAS Drive Smart Data."));

            break;
        case CMD_TEST:
            addOperation(new SASOperation(OPERATION_READPFMC, "SAS Read Performance Test"));
            addOperation(new SASOperation(OPERATION_WRITEPFMC, "SAS Write Performance Test"));
            addOperation(new SASOperation(OPERATION_WRC, "SAS Read/Write Test"));
            addOperation(new SASOperation(OPERATION_EXTERNAL_LOOPBACK_TEST, "SAS External Loopback Test"));

            break;
        case CMD_PROGRAM:
            addOperation(new SASOperation(OPERATION_PROGRAM_ADDRESS, "Program SAS Address"));
            addOperation(new SASOperation(OPERATION_PROGRAM_FW, "Program SAS Firmware"));
            addOperation(new SASOperation(OPERATION_PROGRAM_SBR, "Program SAS Serial boot loader"));

            break;
        default:

            break;
    }
}

SASCommand::~SASCommand()
{

}

int SASCommand::executeCommand(StringParser CommandLine)
{
    INT32 err = 0;
    INT32 OperationIndex;
    COMMAND_TYPE CommandType;
    string strParserOperationName;
    string OperationName;

    CommandLine.getOperation(&strParserOperationName);
    CommandLine.getCommandType(&CommandType);

    switch(CommandType)
    {
        case CMD_LIST:
        case CMD_SHOW:
        case CMD_TEST:
        case CMD_ERROR:
        case CMD_UTILITY:
        case CMD_PROGRAM:
            if(strParserOperationName == "help" || strParserOperationName == "")
            {
                listOperation(CommandLine);
                return -1;
            }

            break;
        default:
            cout << "Not supported command" << endl;
            return -1;
    }

    for(OperationIndex = 0; OperationIndex < m_operation_count; OperationIndex++)
    {
        m_operation[OperationIndex]->getName(&OperationName);

        if(OperationName == strParserOperationName)
        {
            err = m_operation[OperationIndex]->executeOperation(CommandLine);
            break;
        }

    }

    if(OperationIndex == m_operation_count)
    {
        cout << strParserOperationName << " operation not found" << endl;
        return -1;
    }

    return err;
}

