#include "SASOperation.hpp"

SASOperation::SASOperation(OPERATION_TYPE OprationType, string helpContent) :
        OperationBaseClass(OprationType, helpContent)
{
    pciDAPI = NULL;
    sasDAPI = NULL;
    storageDAPI = NULL;
    progressHandler = NULL;
    setHelp(helpContent);
    operationType = OprationType;

    switch(OprationType)
    {
        case OPERATION_CONTROLLER:
            setName("controller");

            break;
        case OPERATION_PHY:
            setName("phy");

            break;
        case OPERATION_DRIVE:
            setName("drive");

            break;
        case OPERATION_ALL:
            setName("all");

            break;
        case OPERATION_HW:
            setName("hw");

            break;
        case OPERATION_BDF:
            setName("bdf");

            break;
        case OPERATION_DRIVER:
            setName("driver");

            break;
        case OPERATION_BIOS:
            setName("bios");

            break;
        case OPERATION_TEMP:
            setName("temp");

            break;
        case OPERATION_PORT:
            setName("port");

            break;
        case OPERATION_RAMDISK:
            setName("ramdisk");

            break;
        case OPERATION_NODE:
            setName("node");

            break;
        case OPERATION_SN:
            setName("sn");

            break;
        case OPERATION_LOGICAL_BLOCK_SIZE:
            setName("logicalblocksize");

            break;
        case OPERATION_PHYSICAL_BLOCK_SIZE:
            setName("physicalblocksize");

            break;
        case OPERATION_MODEL:
            setName("model");

            break;
        case OPERATION_FW:
            setName("fw");

            break;
        case OPERATION_CAPACITY:
            setName("capacity");

            break;
        case OPERATION_LINK:
            setName("link");

            break;
        case OPERATION_SPEED:
            setName("speed");

            break;
        case OPERATION_READPFMC:
            setName("readpfmc");

            break;
        case OPERATION_WRITEPFMC:
            setName("writepfmc");

            break;
        case OPERATION_WRC:
            setName("wrc");

            break;
        case OPERATION_ADDRESS:
            setName("address");

            break;
        case OPERATION_ATTACHED_TYPE:
            setName("type");

            break;
        case OPERATION_ERROR_ALL:
            setName("all");

            break;
        case OPERATION_ERROR_STATISTICS:
            setName("statistics");

            break;
        case OPERATION_ERROR_PCI:
            setName("pci");

            break;
        case OPERATION_DISPARITY_ERR:
            setName("disparity");

            break;
        case OPERATION_INVALID_DWORD_ERR:
            setName("invalid_dword");

            break;
        case OPERATION_LOSS_DWORD_SYNC:
            setName("loss_dword_sync");

            break;
        case OPERATION_PHY_RESET_PRBM:
            setName("phy_reset_prbm");

            break;
        case OPERATION_CLEAR:
            setName("clear");

            break;
        case OPERATION_RESET_CONTROLLER:
            setName("reset");

            break;
        case OPERATION_EXTERNAL_LOOPBACK_TEST:
            setName("loopback");

            break;
        case OPERATION_FIRMWARE_PID:
            setName("fwpid");

            break;
        case OPERATION_NVDATA_PID:
            setName("nvdatapid");

            break;
        case OPERATION_FCODE_VERSION:
            setName("fcode");

            break;
        case OPERATION_UEFI_BSD_VERSION:
            setName("uefibsd");

            break;
        case OPERATION_NVDATA_VENDOR:
            setName("nvdatavendor");

            break;
        case OPERATION_DEFAULT_NVDATA_VERSION:
            setName("defaultnvdata");

            break;
        case OPERATION_PERSISTENT_NVDATA_VERSION:
            setName("persistentnvdata");

            break;
        case OPERATION_PROGRAM_ADDRESS:
            setName("address");

            break;
        case OPERATION_PROGRAM_FW:
            setName("fw");

            break;
        case OPERATION_PROGRAM_SBR:
            setName("sbr");

            break;
        case OPERATION_SMART:
            setName("smart");

            break;
        default:
            break;
    }
}

SASOperation::~SASOperation()
{
    if(pciDAPI != NULL)
        delete pciDAPI;
    if(sasDAPI != NULL)
        delete sasDAPI;
    if(storageDAPI != NULL)
        delete storageDAPI;
    if(progressHandler != NULL)
        delete progressHandler;
}

int SASOperation::executeOperation(StringParser CommandLine)
{
    string parameter1;
    COMMAND_TYPE CommandType;
    int err = 0;

    CommandLine.getParameter(&parameter1, 0);
    CommandLine.getCommandType(&CommandType);

    //If the first parameter is "help" or space, show help message
    if(parameter1 == HELP)
    {
        helpParameter();
        return 0;
    }

    //Execute operation
    switch(operationType)
    {
        case OPERATION_CONTROLLER:
        case OPERATION_PHY:
            err = executeListIOC(CommandLine);

            break;
        case OPERATION_DRIVE:
            err = executeListDrive(CommandLine);

            break;
        case OPERATION_ALL:
        case OPERATION_HW:
        case OPERATION_BDF:
        case OPERATION_FW:
        case OPERATION_DRIVER:
        case OPERATION_BIOS:
        case OPERATION_TEMP:
        case OPERATION_RAMDISK:
        case OPERATION_NODE:
        case OPERATION_SN:
        case OPERATION_MODEL:
        case OPERATION_CAPACITY:
        case OPERATION_ATTACHED_TYPE:
        case OPERATION_LINK:
        case OPERATION_SPEED:
        case OPERATION_ADDRESS:
        case OPERATION_FIRMWARE_PID:
        case OPERATION_NVDATA_PID:
        case OPERATION_FCODE_VERSION:
        case OPERATION_UEFI_BSD_VERSION:
        case OPERATION_NVDATA_VENDOR:
        case OPERATION_DEFAULT_NVDATA_VERSION:
        case OPERATION_PERSISTENT_NVDATA_VERSION:
        case OPERATION_LOGICAL_BLOCK_SIZE:
        case OPERATION_PHYSICAL_BLOCK_SIZE:
        case OPERATION_SHOW_SPEED:
        case OPERATION_SHOW_WIDTH:
            ConvertStringToUpperCase((char*)parameter1.c_str());
            if((parameter1.find("SAS") != string::npos) && (parameter1.find("DRIVE") != string::npos)) // show drive info
            {
                err = executeShowDrive(CommandLine);
            }
            else if((parameter1.find("SAS") != string::npos) && (parameter1.find("-PHY") != string::npos)) // show PHY info
            {
                err = executeShowPHY(CommandLine);
            }
            else if((parameter1.find("SAS") != string::npos)) // show controller info
            {
                err = executeShowController(CommandLine);
            }

            break;
        case OPERATION_ERROR_ALL:
        case OPERATION_ERROR_STATISTICS:
        case OPERATION_ERROR_PCI:
        case OPERATION_DISPARITY_ERR:
        case OPERATION_INVALID_DWORD_ERR:
        case OPERATION_LOSS_DWORD_SYNC:
        case OPERATION_PHY_RESET_PRBM:
            ConvertStringToUpperCase((char*)parameter1.c_str());
            err = executeError(CommandLine);

            break;
        case OPERATION_CLEAR:
            ConvertStringToUpperCase((char*)parameter1.c_str());
            err = executeClearError(CommandLine);

            break;
        case OPERATION_RESET_CONTROLLER:
        case OPERATION_SMART:
            ConvertStringToUpperCase((char*)parameter1.c_str());
            err = executeUtility(CommandLine);

            break;
        case OPERATION_READPFMC:
        case OPERATION_WRITEPFMC:
        case OPERATION_WRC:
        case OPERATION_EXTERNAL_LOOPBACK_TEST:
            err = executeTest(CommandLine);

            break;
        case OPERATION_PROGRAM_ADDRESS:
        case OPERATION_PROGRAM_FW:
        case OPERATION_PROGRAM_SBR:
            err = executeProgram(CommandLine);

            break;
        default:
            cout << "Unsupport operation" << endl;
            err = -1;
            break;
    }

    return err;
}

int SASOperation::helpParameter()
{
    //Show parameter help message depend on operation type
    switch(operationType)
    {
        case OPERATION_ALL:
        case OPERATION_HW:
        case OPERATION_BDF:
        case OPERATION_FW:
        case OPERATION_BIOS:
        case OPERATION_DRIVER:
        case OPERATION_TEMP:
        case OPERATION_PORT:
        case OPERATION_RAMDISK:
        case OPERATION_NODE:
        case OPERATION_SN:
        case OPERATION_CAPACITY:
        case OPERATION_MODEL:
        case OPERATION_ATTACHED_TYPE:
        case OPERATION_LINK:
        case OPERATION_SPEED:
        case OPERATION_ADDRESS:
        case OPERATION_RESET_CONTROLLER:
        case OPERATION_EXTERNAL_LOOPBACK_TEST:
        case OPERATION_FIRMWARE_PID:
        case OPERATION_NVDATA_PID:
        case OPERATION_FCODE_VERSION:
        case OPERATION_UEFI_BSD_VERSION:
        case OPERATION_NVDATA_VENDOR:
        case OPERATION_DEFAULT_NVDATA_VERSION:
        case OPERATION_PERSISTENT_NVDATA_VERSION:
        case OPERATION_LOGICAL_BLOCK_SIZE:
        case OPERATION_PHYSICAL_BLOCK_SIZE:
        case OPERATION_SMART:
        case OPERATION_SHOW_SPEED:
        case OPERATION_SHOW_WIDTH:
            cout << "Show SAS device characteristic" << endl;
            cout << "Syntax: P1" << endl;
            //cout << "P1: Data Elements" << endl;
            cout << "P1: Controller/PHY Name" << endl;

            break;
        case OPERATION_ERROR_ALL:
        case OPERATION_ERROR_STATISTICS:
        case OPERATION_ERROR_PCI:
        case OPERATION_DISPARITY_ERR:
        case OPERATION_INVALID_DWORD_ERR:
        case OPERATION_LOSS_DWORD_SYNC:
        case OPERATION_PHY_RESET_PRBM:
        case OPERATION_CLEAR:
            cout << "Show PCI/SAS error statistics" << endl;
            cout << "Syntax: P1" << endl;
            cout << "P1: Controller/PHY Name" << endl;

            break;
        case OPERATION_READPFMC:
            cout << "SAS Device Read Performance" << endl;
            cout << "Syntax: readpfc P1 P2 P3" << endl;
            cout << "P1: Start Sector(Hex)" << endl;
            cout << "P2: Test Size/MB(Dec)" << endl;
            cout << "P3: Device Name" << endl;

            break;
        case OPERATION_WRITEPFMC:
            cout << "SAS Device Write Performance" << endl;
            cout << "Syntax: writepfmc P1 P2 P3" << endl;
            cout << "P1: Start Sector(Hex)" << endl;
            cout << "P2: Test Size/MB(Dec)" << endl;
            cout << "P3: Device Name" << endl;

            break;
        case OPERATION_WRC:
            cout << "SAS Device Read/Write Test" << endl;
            cout << "Syntax: wrc P1 P2 P3" << endl;
            cout << "P1: Start Sector(Hex)" << endl;
            cout << "P2: Test Size/MB(Dec)" << endl;
            cout << "P3: Device Name" << endl;

            break;
        case OPERATION_PROGRAM_ADDRESS:
            cout << "Program SAS address" << endl;
            cout << "Syntax: address P1 P2" << endl;
            cout << "P1: SAS address(Hex)" << endl;
            cout << "P2: Device Name" << endl;

            break;
        case OPERATION_PROGRAM_FW:
            cout << "Program SAS Firmware" << endl;
            cout << "Syntax: fw P1 P2" << endl;
            cout << "P1: Firmware file path" << endl;
            cout << "P2: Device Name" << endl;

            break;
        default:
            break;
    }

    return 0;
}

int SASOperation::executeListIOC(StringParser CommandLine)
{
    int err = 0;
    vector<SAS_CONTROLLER_INFO> sasList;
    vector<SAS_CONTROLLER_INFO>::iterator sasIter;
    ErrorMsg* errorMsg = NO_ERROR_MSG;

    sasDAPI = new SASDAPI("SASDAPI");

    err = checkSyntax(CommandLine);
    if(err == 0)
    {
        errorMsg = sasDAPI->getInformationList(&sasList);
        if(errorMsg)
        {
            printError(errorMsg);
            return -1;
        }

        switch(operationType)
        {
            case OPERATION_CONTROLLER:
                for(sasIter = sasList.begin(); sasIter != sasList.end(); ++sasIter)
                {
                    cout << sasIter->wellKnowName << "(" << sasIter->domindBusDeviceFunction << ")" << endl;
                }
                break;

            case OPERATION_PHY:
                for(sasIter = sasList.begin(); sasIter != sasList.end(); ++sasIter)
                {
                    for(int index = 0; index < 8; index++)
                    {
                        cout << sasIter->wellKnowName << "-PHY" << index << endl;
                    }
                }
                break;

            default:
                return -1;
        }
	}

	return err;
}

int SASOperation::executeListDrive(StringParser CommandLine)
{
    int err = 0;
    vector<SAS_CONTROLLER_INFO> sasList;
    vector<SAS_CONTROLLER_INFO>::iterator sasIter;
    vector<SAS_DRIVE_INFO> driveList;
    vector<SAS_DRIVE_INFO>::iterator driveIter;
    ErrorMsg* errorMsg = NO_ERROR_MSG;

    sasDAPI = new SASDAPI("SASDAPI");

    err = checkSyntax(CommandLine);
    if(err == 0)
    {
        errorMsg = sasDAPI->getInformationList(&sasList);

        if(errorMsg)
        {
            printError(errorMsg);
            return -1;
        }

        for(sasIter = sasList.begin(); sasIter != sasList.end(); ++sasIter)
        {
            errorMsg = sasDAPI->getDriveDeviceNodeListByController((sasIter - sasList.begin()), &driveList);
            if (driveList.empty())
                cout << sasIter->wellKnowName << " has NO Attached SAS Drive" << endl;
            else
            {
                for(driveIter = driveList.begin(); driveIter != driveList.end(); ++driveIter)
                {
                    cout << driveIter->wellKnownName << endl;
                }
            }
            driveList.clear();
            cout << endl;
        }
    }

    return err;
}

int SASOperation::executeShowController(StringParser CommandLine)
{
    int err = 0;
    string targetName;
    UINT8 logicalID = -1;
    SasInfo sasInfo;
    UINT32 returnTemp = 0;
    UINT64 hostAddress;
    ErrorMsg* errorMsg = NO_ERROR_MSG;

    targetName.clear();

    sasDAPI = new SASDAPI("SASDAPI");

    err = checkSyntax(CommandLine);
    if(err == 0)
    {
        CommandLine.getParameter(&targetName, 0);

        if((errorMsg = sasDAPI->getLogicalIDByName(targetName, &logicalID)) == NO_ERROR_MSG)
        {
            if((errorMsg = sasDAPI->getInformationByController(logicalID, &sasInfo)) == NO_ERROR_MSG)
                if((errorMsg = sasDAPI->getAddressByController(logicalID, &hostAddress)) == NO_ERROR_MSG)
                    errorMsg = sasDAPI->getTemperatureByController(logicalID, &returnTemp);
        }

        if(errorMsg)
        {
            printError(errorMsg);
            return -1;
        }

        switch(operationType)
        {
            case OPERATION_ALL:
                cout << "Hardware Version           : " << sasInfo.hardwareRevision << endl;
                cout << "Firmware Version           : " << sasInfo.firmwareVersion << endl;
                //cout << "BIOS Version               : " << sasInfo.biosVersion << endl;
                cout << "Driver Version             : " << sasInfo.driverVersion << endl;
                //cout << "NVDATA Version (Default)   : " << sasInfo.defaultNVDATAVersion << endl;
                //cout << "NVDATA Version (Persistent): " << sasInfo.persistentNVDATAVersion << endl;
                //cout << "Firmware Product ID        : " << sasInfo.firmwarePID << endl;
                //cout << "NVDATA Vendor              : " << sasInfo.nvdataVendor << endl;
                //cout << "NVDATA Product ID          : " << sasInfo.nvdataPID << endl;
                //cout << "UEFI BSD Version           : " << sasInfo.uefiBSDVersion << endl;
                //cout << "FCODE Version              : " << sasInfo.fcodeVersion << endl;
                cout << "PCI BDF                    : " << sasInfo.domindBusDeviceFunction << endl;
                printf( "PCI Speed                  : %.1f GT/s\n", sasInfo.pciSpeed);
                cout << "PCI Width                  : x" <<  sasInfo.pciWidth << endl;
                cout << "Temperature                : " << returnTemp << " C" << endl;
                cout << "SAS Address                : " << std::uppercase << std::hex << std::setw(16) <<
                                                           std::setfill('0') << hostAddress << endl;

                break;
            case OPERATION_HW:
                cout << "HW Version : " << sasInfo.hardwareRevision << endl;

                break;
            case OPERATION_FW:
                cout << "FW Version : " << sasInfo.firmwareVersion << endl;

                break;
            case OPERATION_BIOS:
                cout << "BIOS Version : " << sasInfo.biosVersion << endl;
                cout << "No Support" << endl;
                break;
            case OPERATION_DRIVER:
                cout << "Driver Version : " << sasInfo.driverVersion << endl;

                break;
            case OPERATION_BDF:
                cout << "PCI BDF : " << sasInfo.domindBusDeviceFunction << endl;

                break;
            case OPERATION_SHOW_SPEED:
                printf("PCI Speed : %.1f GT/s\n", sasInfo.pciSpeed);

                break;
            case OPERATION_SHOW_WIDTH:
                cout << "PCI Width : x" <<  sasInfo.pciWidth << endl;

                break;
            case OPERATION_FIRMWARE_PID:
                //cout << "Firmware Product ID : " << sasInfo.firmwarePID << endl;
                cout << "No Support" << endl;
                break;
            case OPERATION_NVDATA_PID:
                //cout << "NVDATA Product ID : " << sasInfo.nvdataPID << endl;
                cout << "No Support" << endl;
                break;
            case OPERATION_FCODE_VERSION:
                //cout << "FCODE Version : " << sasInfo.fcodeVersion << endl;
                cout << "No Support" << endl;
                break;
            case OPERATION_UEFI_BSD_VERSION:
                //cout << "UEFI BSD Version : " << sasInfo.uefiBSDVersion << endl;
                cout << "No Support" << endl;
                break;
            case OPERATION_NVDATA_VENDOR:
                //cout << "NVDATA Vendor : " << sasInfo.nvdataVendor << endl;
                cout << "No Support" << endl;
                break;
            case OPERATION_DEFAULT_NVDATA_VERSION:
                //cout << "NVDATA Version (Default) : " << sasInfo.defaultNVDATAVersion << endl;
                cout << "No Support" << endl;
                break;
            case OPERATION_PERSISTENT_NVDATA_VERSION:
                //cout << "NVDATA Version (Persistent) : " << sasInfo.persistentNVDATAVersion << endl;
                cout << "No Support" << endl;
                break;
            case OPERATION_PORT:
                cout << "No Support" << endl;
                break;

            case OPERATION_TEMP:
                cout << "Temperature : " << returnTemp << " C" << endl;

                break;
            case OPERATION_ADDRESS:
                cout << "SAS Address : " << std::uppercase << std::hex << std::setw(16) <<
                                            std::setfill('0') << hostAddress << endl;

                break;
            case OPERATION_RAMDISK:
                cout << "No Support" << endl;
                break;

            default:
                cout << "Unsupport operation parameter" << endl;
                return -1;
        }

        targetName.clear();
    }

    return err;
}

int SASOperation::executeShowPHY(StringParser CommandLine)
{
    int err = 0;
    string targetName,typeName,linkStatus,linkSpeed;
    int phyNumber;
    UINT8 logicalID = -1;
    vector<SAS_CONTROLLER_INFO> sasList;
    ErrorMsg* errorMsg = NO_ERROR_MSG;

    targetName.clear();
    typeName.clear();
    linkStatus.clear();
    linkSpeed.clear();

    sasDAPI = new SASDAPI("SASDAPI");

    err = checkSyntax(CommandLine);
    if(err == 0)
    {
        CommandLine.getParameter(&targetName, 0);

        if((errorMsg = sasDAPI->getInformationList(&sasList)) == NO_ERROR_MSG)
        {
            if((errorMsg = sasDAPI->getLogicalIDByName(targetName.substr(0,targetName.find("-PHY")), &logicalID)) == NO_ERROR_MSG)
            {
                CommandLine.convertStringToHex(targetName.substr(targetName.find_last_of("-PHY")+1), &phyNumber);

                if((errorMsg = sasDAPI->getAttachDeviceByPHY(logicalID, phyNumber, &typeName)) == NO_ERROR_MSG)
                    if((errorMsg = sasDAPI->getLinkSpeedByPHY(logicalID, phyNumber, &linkSpeed)) == NO_ERROR_MSG)
                        errorMsg = sasDAPI->getLinkStatusByPHY(logicalID, phyNumber, &linkStatus);
            }
        }

        if(errorMsg)
        {
            printError(errorMsg);
            return -1;
        }

        switch(operationType)
        {
            case OPERATION_ALL:
                //cout << "Port        : " << endl;
                //cout << "SAS Address : " << endl;
                cout << "Type        : " << typeName << endl;
                cout << "Speed       : " << linkSpeed << endl;
                cout << "Link        : " << linkStatus << endl;

                break;
            case OPERATION_PORT:
                //cout << "Port : " <<  sasInfo.hardwareRevision << endl;

                break;
            case OPERATION_ADDRESS:
                //cout << "SAS Address : " <<  sasInfo.firmwareVersion << endl;

                break;
            case OPERATION_ATTACHED_TYPE:
                cout << "Type : " <<  typeName << endl;

                break;
            case OPERATION_SPEED:
                cout << "Speed : " <<  linkSpeed << endl;

                break;
            case OPERATION_LINK:
                cout << "Link : " <<  linkStatus << endl;

                break;
            default:
                cout << "Unsupport operation parameter" << endl;

                return -1;
        }

        targetName.clear();
        typeName.clear();
        linkStatus.clear();
        linkSpeed.clear();
    }

    return err;
}

int SASOperation::executeShowDrive(StringParser CommandLine)
{
    int err = 0;
    ErrorMsg* errorMsg = NO_ERROR_MSG;
    string targetName, capacityString, tempString;
    vector<SAS_CONTROLLER_INFO> sasList;
    SasDriveInfo driveInfo;
    CHAR cap_str[64];
    CHAR si_str[64];

    sasDAPI = new SASDAPI("SASDAPI");

    err = checkSyntax(CommandLine);
    if(err == 0)
    {
        CommandLine.getParameter(&targetName, 0);

        errorMsg = sasDAPI->getInformationList(&sasList);

        if(errorMsg)
        {
            printError(errorMsg);
            return -1;
        }

        errorMsg = sasDAPI->getInformationByDriveName(targetName, &driveInfo);

        if(errorMsg)
        {
            printError(errorMsg);
            return -1;
        }

        format_with_thousands_sep(cap_str, sizeof(cap_str), driveInfo.capacity);
        format_capacity(si_str, sizeof(si_str), driveInfo.capacity);
        capacityString = capacityString.assign((const char*)cap_str) + " bytes ";
        capacityString += "[" + tempString.assign((const char*)si_str) + "]";

        switch(operationType)
        {
            case OPERATION_ALL:
                cout << "Node Name           : " << driveInfo.deviceNodeName << endl;
                cout << "Controller Name     : " << driveInfo.controllerName << endl;
                cout << "Vendor              : " << driveInfo.vendorName << endl;
                cout << "Serial Number       : " << driveInfo.serialNumber << endl;
                cout << "Model               : " << driveInfo.productName << endl;
                cout << "Firmware Version    : " << driveInfo.revision << endl;
                cout << "Logical Block Size  : " << driveInfo.logicalBlockSize << " bytes" << endl;
                cout << "Physical Block Size : " << driveInfo.physicalBlockSize << " bytes" << endl;
                cout << "SAS Address         : " << std::uppercase << std::hex << std::setw(16) <<
                                                    std::setfill('0') << driveInfo.sasAddress << endl;
                //cout << "Logical Unit ID     : " << driveInfo.logicalUnitID << endl;
                cout << "User Capacity       : " << capacityString << endl;
                cout << "Rotation Rate       : " << driveInfo.rotationRate << endl;
                cout << "Form Factor         : " << driveInfo.formFactor << endl;
                cout << "Transport Protocol  : " << driveInfo.protocalType << endl;
                cout << "Temperature         : " << driveInfo.temperature << endl;
                cout << "Link Speed          : " << driveInfo.sasPhyInfo[0].linkRate << endl;
                break;

            case OPERATION_NODE:
                cout << "Node Name : " << driveInfo.deviceNodeName << endl;
                break;

            case OPERATION_SN:
                cout << "Serial Number : " << driveInfo.serialNumber << endl;
                break;

            case OPERATION_VENDOR:
                cout << "Vendor : " << driveInfo.vendorName << endl;
                break;

            case OPERATION_FW:
                cout << "Firmware : " << driveInfo.revision << endl;
                break;

            case OPERATION_ADDRESS:
                cout << "SAS Address : " << std::uppercase << std::hex << std::setw(16) <<
                                                    std::setfill('0') << driveInfo.sasAddress << endl;
                break;

            case OPERATION_LOGICAL_BLOCK_SIZE:
                cout << "Logical Block Size : " << driveInfo.logicalBlockSize << " bytes" << endl;
                break;

            case OPERATION_PHYSICAL_BLOCK_SIZE:
                cout << "Physical Block Size : " << driveInfo.physicalBlockSize << " bytes" << endl;
                break;

            case OPERATION_CAPACITY:
                cout << "User Capacity : " << capacityString << endl;
                break;

            case OPERATION_MODEL:
                cout << "Model : " << driveInfo.productName << endl;
                break;

            case OPERATION_TEMP:
                cout << "Temperature : " << driveInfo.temperature << endl;
                break;

            default:
                cout << "Unsupport operation parameter" << endl << endl;

                return -1;
        }
    }

    return err;
}

int SASOperation::executeError(StringParser CommandLine)
{
    int err = 0;
    string targetName;
    PFA bdf;
    int phyNumber;
    UINT8 logicalID = -1;
    SasInfo sasInfo;
    ErrorCountInfo errInfo;
    ErrorMsg* errorMsg = NO_ERROR_MSG;

    targetName.clear();

    pciDAPI = new PCIDAPI("PCIDAPI");
    sasDAPI = new SASDAPI("SASDAPI");

    err = checkSyntax(CommandLine);
    if(err == 0)
    {
        CommandLine.getParameter(&targetName, 0);

        if((targetName.find("SAS") != string::npos) && (targetName.find("-PHY") == string::npos)) // show controller error count
        {
            if((errorMsg = sasDAPI->getLogicalIDByName(targetName, &logicalID)) == NO_ERROR_MSG)
            {
                if((errorMsg = sasDAPI->getInformationByController(logicalID, &sasInfo)) == NO_ERROR_MSG)
                {
                    sscanf(sasInfo.domindBusDeviceFunction.c_str(), "%x:%x:%x.%x", &bdf.u16Domain, &bdf.u16Bus, &bdf.u16Device, &bdf.u8Function);

                    switch(operationType)
                    {
                        case OPERATION_ERROR_ALL:
                            pciDAPI->showErrorStatistics(bdf, PCI_COMPATIBLE_ERROR, FALSE);
                            pciDAPI->showErrorStatistics(bdf, PCIE_ERROR, FALSE);
                            pciDAPI->showErrorStatistics(bdf, AER_ERROR, FALSE);
                            cout << endl;
                        case OPERATION_ERROR_STATISTICS:
                        case OPERATION_DISPARITY_ERR:
                        case OPERATION_INVALID_DWORD_ERR:
                        case OPERATION_LOSS_DWORD_SYNC:
                        case OPERATION_PHY_RESET_PRBM:
                            for(int index = 0; index < 8; index++)
                            {
                                if((errorMsg = sasDAPI->getErrorCountByPHY(logicalID, index, &errInfo)) != NO_ERROR_MSG)
                                    break;
                                cout << "PHY" << index << endl;
                                switch(operationType)
                                {
                                    case OPERATION_ERROR_ALL:
                                    case OPERATION_ERROR_STATISTICS:
                                        cout << "INVALID_DWORD_ERR : " << errInfo.InvalidDwordError << endl;
                                        cout << "DISPARITY_ERR     : " << errInfo.DisparityError << endl;
                                        cout << "LOSS_DWORD_SYNC   : " << errInfo.LossDwordSync << endl;
                                        cout << "PHY_RESET_PRBM    : " << errInfo.PhyResetProblem << endl << endl;

                                        break;
                                    case OPERATION_DISPARITY_ERR:
                                        cout << "DISPARITY_ERR : " <<  errInfo.DisparityError << endl << endl;

                                        break;
                                    case OPERATION_INVALID_DWORD_ERR:
                                        cout << "INVALID_DWORD_ERR : " <<  errInfo.InvalidDwordError << endl << endl;

                                        break;
                                    case OPERATION_LOSS_DWORD_SYNC:
                                        cout << "LOSS_DWORD_SYNC : " <<  errInfo.LossDwordSync << endl << endl;

                                        break;
                                    case OPERATION_PHY_RESET_PRBM:
                                        cout << "PHY_RESET_PRBM : " <<  errInfo.PhyResetProblem << endl << endl;

                                        break;
                                }
                            }
                            break;
                        case OPERATION_ERROR_PCI:
                            pciDAPI->showErrorStatistics(bdf, PCI_COMPATIBLE_ERROR, FALSE);
                            pciDAPI->showErrorStatistics(bdf, PCIE_ERROR, FALSE);
                            pciDAPI->showErrorStatistics(bdf, AER_ERROR, FALSE);

                            break;
                        default:
                            cout << "Unsupport operation parameter" << endl;

                            return -1;
                    }
                }
            }
        }
        else // show PHY error count
        {
            if((errorMsg = sasDAPI->getLogicalIDByName(targetName.substr(0,targetName.find("-PHY")), &logicalID)) == NO_ERROR_MSG)
            {
                if(targetName.substr(targetName.find_last_of("-PHY")+1) == "")
                {
                    cout << "Invalid target name" << endl;
                    return -1;
                }
                CommandLine.convertStringToDec(targetName.substr(targetName.find_last_of("-PHY")+1), &phyNumber);

                if((errorMsg = sasDAPI->getErrorCountByPHY(logicalID, phyNumber, &errInfo)) == NO_ERROR_MSG)
                {
                    switch(operationType)
                    {
                        case OPERATION_ERROR_STATISTICS:
                            cout << "INVALID_DWORD_ERR : " << errInfo.InvalidDwordError << endl;
                            cout << "DISPARITY_ERR     : " << errInfo.DisparityError << endl;
                            cout << "LOSS_DWORD_SYNC   : " << errInfo.LossDwordSync << endl;
                            cout << "PHY_RESET_PRBM    : " << errInfo.PhyResetProblem << endl;

                            break;
                        case OPERATION_DISPARITY_ERR:
                            cout << "DISPARITY_ERR : " <<  errInfo.DisparityError << endl;

                            break;
                        case OPERATION_INVALID_DWORD_ERR:
                            cout << "INVALID_DWORD_ERR : " <<  errInfo.InvalidDwordError << endl;

                            break;
                        case OPERATION_LOSS_DWORD_SYNC:
                            cout << "LOSS_DWORD_SYNC : " <<  errInfo.LossDwordSync << endl;

                            break;
                        case OPERATION_PHY_RESET_PRBM:
                            cout << "PHY_RESET_PRBM : " <<  errInfo.PhyResetProblem << endl;

                            break;
                        default:
                            cout << "Unsupport operation parameter" << endl;

                            return -1;
                    }
                }
            }
        }
    }

    if(errorMsg)
    {
        printError(errorMsg);
        return -1;
    }

    return err;
}

int SASOperation::executeClearError(StringParser CommandLine)
{
    int err = 0;
    string targetName;
    PFA bdf;
    int phyNumber;
    UINT8 logicalID = -1;
    SasInfo sasInfo;
    ErrorMsg* errorMsg = NO_ERROR_MSG;
    ErrorCountInfo errInfo;

    pciDAPI = new PCIDAPI("PCIDAPI");
    sasDAPI = new SASDAPI("SASDAPI");

    err = checkSyntax(CommandLine);
    if(err == 0)
    {
        CommandLine.getParameter(&targetName, 0);

        if((targetName.find("SAS") != string::npos) && (targetName.find("-PHY") == string::npos)) // show controller error count
        {
            if((errorMsg = sasDAPI->getLogicalIDByName(targetName, &logicalID))== NO_ERROR_MSG)
            {
                if((errorMsg = sasDAPI->getInformationByController(logicalID, &sasInfo)) == NO_ERROR_MSG)
                {
                    sscanf(sasInfo.domindBusDeviceFunction.c_str(), "%x:%x:%x.%x", &bdf.u16Domain, &bdf.u16Bus, &bdf.u16Device, &bdf.u8Function);
                    pciDAPI->clearErrors(bdf);
                    pciDAPI->showErrorStatistics(bdf, PCI_COMPATIBLE_ERROR, TRUE);
                    pciDAPI->showErrorStatistics(bdf, PCIE_ERROR, TRUE);
                    pciDAPI->showErrorStatistics(bdf, AER_ERROR, TRUE);
                }
                if((errorMsg = sasDAPI->clearPHYErrorCountByController(logicalID)) == NO_ERROR_MSG)
                {
                    for(int index = 0; index < 8; index++)
                    {
                        if((errorMsg = sasDAPI->getErrorCountByPHY(logicalID, index, &errInfo)) != NO_ERROR_MSG)
                            break;

                        cout << "PHY" << index << endl;
                        cout << "INVALID_DWORD_ERR : " << errInfo.InvalidDwordError << endl;
                        cout << "DISPARITY_ERR     : " << errInfo.DisparityError << endl;
                        cout << "LOSS_DWORD_SYNC   : " << errInfo.LossDwordSync << endl;
                        cout << "PHY_RESET_PRBM    : " << errInfo.PhyResetProblem << endl << endl;
                    }
                }
            }
        }
        else // show PHY error count
        {
            if((errorMsg = sasDAPI->getLogicalIDByName(targetName.substr(0,targetName.find("-PHY")), &logicalID)) == NO_ERROR_MSG)
            {
                CommandLine.convertStringToDec(targetName.substr(targetName.find_last_of("-PHY")+1), &phyNumber);

                if((errorMsg = sasDAPI->clearPHYErrorCountByPHY(logicalID, phyNumber)) == NO_ERROR_MSG)
                {
                    if((errorMsg = sasDAPI->getErrorCountByPHY(logicalID, phyNumber, &errInfo)) == NO_ERROR_MSG)
                    {
                        cout << "INVALID_DWORD_ERR : " << errInfo.InvalidDwordError << endl;
                        cout << "DISPARITY_ERR     : " << errInfo.DisparityError << endl;
                        cout << "LOSS_DWORD_SYNC   : " << errInfo.LossDwordSync << endl;
                        cout << "PHY_RESET_PRBM    : " << errInfo.PhyResetProblem << endl;
                    }
                }
            }
        }
    }

    if(errorMsg)
    {
        printError(errorMsg);
        return -1;
    }

    return err;
}

int SASOperation::executeUtility(StringParser CommandLine)
{
    int err = 0;
    string targetName;
    UINT8 logicalID = -1;
    vector<SAS_CONTROLLER_INFO> sasList;
    StorageSASSMARTDataStruct smartData;
    ErrorMsg* errorMsg = NO_ERROR_MSG;

    sasDAPI = new SASDAPI("SASDAPI");

    err = checkSyntax(CommandLine);
    if(err == 0)
    {
        CommandLine.getParameter(&targetName, 0);

        errorMsg = sasDAPI->getInformationList(&sasList);

        if(errorMsg == NO_ERROR_MSG)
        {
            switch(operationType)
            {
                case OPERATION_RESET_CONTROLLER:
                    if((targetName.find("SAS") != string::npos) && (targetName.find("-PHY") == string::npos))
                    {
                        if((errorMsg = sasDAPI->getLogicalIDByName(targetName, &logicalID)) == NO_ERROR_MSG)
                            errorMsg = sasDAPI->resetIOC(logicalID);
                    }
                    else
                        errorMsg = sasDAPI->returnErrorMsg((ERRORCODE)ERROR_BAD_PARAMETER_ONE, NULL, NULL);

                    break;
                case OPERATION_SMART:
                    errorMsg = sasDAPI->getSmartDataByDriveName(targetName, &smartData);
                    if(errorMsg == NO_ERROR_MSG)
                    {
                        cout << "SMART Health Status : " << smartData.strHealthStatus << endl;
                        cout << "Non-medium error count : " << smartData.errorLog.nmErrorCount << endl;
                        cout << "=========================================" << endl;
                        cout << "Read error count :" << endl;
                        cout << "fast ECC             : " << smartData.errorLog.read.fastECC << endl;
                        cout << "delayed ECC          : " << smartData.errorLog.read.delayedECC << endl;
                        cout << "reReadWrites         : " << smartData.errorLog.read.reReadWrite << endl;
                        cout << "corrected Error      : " << smartData.errorLog.read.totalCorrected << endl;
                        cout << "uncorrected Error    : " << smartData.errorLog.read.totalUnCorrected << endl;
                        cout << "algorithm Invocation : " << smartData.errorLog.read.algoInvocation << endl;
                        cout << "=========================================" << endl;
                        cout << "Write error count :" << endl;
                        cout << "fast ECC             : " << smartData.errorLog.write.fastECC << endl;
                        cout << "delayed ECC          : " << smartData.errorLog.write.delayedECC << endl;
                        cout << "reReadWrites         : " << smartData.errorLog.write.reReadWrite << endl;
                        cout << "corrected Error      : " << smartData.errorLog.write.totalCorrected << endl;
                        cout << "uncorrected Error    : " << smartData.errorLog.write.totalUnCorrected << endl;
                        cout << "algorithm Invocation : " << smartData.errorLog.write.algoInvocation << endl;
                        cout << "=========================================" << endl;
                        cout << "Verify error count :" << endl;
                        cout << "fast ECC             : " << smartData.errorLog.verify.fastECC << endl;
                        cout << "delayed ECC          : " << smartData.errorLog.verify.delayedECC << endl;
                        cout << "reReadWrites         : " << smartData.errorLog.verify.reReadWrite << endl;
                        cout << "corrected Error      : " << smartData.errorLog.verify.totalCorrected << endl;
                        cout << "uncorrected Error    : " << smartData.errorLog.verify.totalUnCorrected << endl;
                        cout << "algorithm Invocation : " << smartData.errorLog.verify.algoInvocation << endl;
                    }
                    break;
            }
        }
    }

    if(errorMsg)
    {
        printError(errorMsg);
        return -1;
    }

    return err;
}

int SASOperation::executeTest(StringParser CommandLine)
{
    int err = 0;
    string targetName,tempString;
    UINT8 logicalID = -1;
    INT32 startSector = 0;
    INT32 testSize = 0;
    vector<SAS_CONTROLLER_INFO> sasList;
    ErrorMsg* errorMsg = NO_ERROR_MSG;

    sasDAPI = new SASDAPI("SASDAPI");
    storageDAPI = new StorageDAPI("StorageDAPI");

    err = checkSyntax(CommandLine);
    if(err == 0)
    {
        if(operationType != OPERATION_EXTERNAL_LOOPBACK_TEST)
        {
            CommandLine.getParameter(&tempString, 0);
            CommandLine.convertStringToHex(tempString, &startSector);
            tempString.clear();

            CommandLine.getParameter(&tempString, 1);
            CommandLine.convertStringToDec(tempString, &testSize);
            tempString.clear();

            CommandLine.getParameter(&targetName, 2);

            if((errorMsg = storageDAPI->getIndexByName(targetName.c_str(), (UINT32*)&logicalID)) != NO_ERROR_MSG)
            {
                printError(errorMsg);
                return -1;
            }
        }
        else
            CommandLine.getParameter(&targetName, 0);

        errorMsg = sasDAPI->getInformationList(&sasList);

        if(errorMsg == NO_ERROR_MSG)
        {
            switch(operationType)
            {
                case OPERATION_READPFMC:
			        progressHandler = new thread(&SASOperation::handleTestProgress, this, (UINT32)testSize);
                    errorMsg = storageDAPI->readPerformanceTest(logicalID, (UINT64)startSector, (UINT32)testSize);
                    if(errorMsg)
                    {
                        printError(errorMsg);
                        return -1;
                    }
                    progressHandler->join();
                    cout << storageDAPI->m_strOutputResultString;
                    break;

                case OPERATION_WRITEPFMC:
                    progressHandler = new thread(&SASOperation::handleTestProgress, this, (UINT32)testSize);
                    errorMsg = storageDAPI->writePerformanceTest(logicalID, (UINT64)startSector, (UINT32)testSize);
                    if(errorMsg)
                    {
                        printError(errorMsg);
                        return -1;
                    }
                    progressHandler->join();
                    cout << storageDAPI->m_strOutputResultString;
                    break;

                case OPERATION_WRC:
                    progressHandler = new thread(&SASOperation::handleTestProgress, this, (UINT32)testSize);
                    errorMsg = storageDAPI->readwriteTest(logicalID, (UINT64)startSector, (UINT32)testSize);
                    if(errorMsg)
                    {
                        printError(errorMsg);
                        return -1;
                    }
                    progressHandler->join();
                    cout << storageDAPI->m_strOutputResultString;
                    break;

                case OPERATION_EXTERNAL_LOOPBACK_TEST:
                    if((targetName.find("SAS") != string::npos) && (targetName.find("-PHY") == string::npos))
                    {
                        if((errorMsg = sasDAPI->getLogicalIDByName(targetName, &logicalID)) == NO_ERROR_MSG)
                            errorMsg = sasDAPI->doLoopbackTest(logicalID);
                    }
                    else
                        errorMsg = sasDAPI->returnErrorMsg((ERRORCODE)ERROR_FUNCTION_NOT_FINISHED, NULL, NULL);
                    break;

                default:
                    cout << "Unsupport operation" << endl;
                    err = -1;
                    break;
            }
        }
    }

    if(errorMsg)
    {
        printError(errorMsg);
        return -1;
    }

    return err;
}

int SASOperation::executeProgram(StringParser CommandLine)
{
    int err = 0;
    string targetName, tempString;
    UINT8 logicalID = -1;
    UINT64 hostAddress;
    SasInfo sasInfo;
    ErrorMsg* errorMsg = NO_ERROR_MSG;

    sasDAPI = new SASDAPI("SASDAPI");

    err = checkSyntax(CommandLine);
    if(err == 0)
    {
        CommandLine.getParameter(&tempString, 0);
        CommandLine.getParameter(&targetName, 1);
        ConvertStringToUpperCase((char*)targetName.c_str());

        if((errorMsg = sasDAPI->getLogicalIDByName(targetName, &logicalID)) != NO_ERROR_MSG)
        {
            printError(errorMsg);
            return -1;
        }

        switch(operationType)
        {
            case OPERATION_PROGRAM_ADDRESS:
                if((errorMsg = sasDAPI->programSASAddress(logicalID, tempString)) == NO_ERROR_MSG)
                    if((errorMsg = sasDAPI->getAddressByController(logicalID, &hostAddress)) == NO_ERROR_MSG)
                        cout << "Updated SAS Address : " << std::uppercase << std::hex << std::setw(16)
                                                         << std::setfill('0') << hostAddress << endl;

                break;
            case OPERATION_PROGRAM_FW:
                if((errorMsg = sasDAPI->programSASFirmware(logicalID, tempString)) == NO_ERROR_MSG)
                    if((errorMsg = sasDAPI->getInformationByController(logicalID, &sasInfo)) == NO_ERROR_MSG)
                    {
                        cout << "Firmware Version           : " << sasInfo.firmwareVersion << endl;
                        cout << "NVDATA Version (Default)   : " << sasInfo.defaultNVDATAVersion << endl;
                        cout << "NVDATA Version (Persistent): " << sasInfo.persistentNVDATAVersion << endl;
                        cout << "Firmware Product ID        : " << sasInfo.firmwarePID << endl;
                        cout << "NVDATA Vendor              : " << sasInfo.nvdataVendor << endl;
                        cout << "NVDATA Product ID          : " << sasInfo.nvdataPID << endl;
                    }

                break;
            //case OPERATION_PROGRAM_SBR:
            //    errorMsg = sasDAPI->programEEPROM(logicalID, tempString);

            //    break;
            default:
                cout << "Unsupport operation" << endl;
                err = -1;
                break;
        }
    }

    if(errorMsg)
    {
        printError(errorMsg);
        return -1;
    }

    return err;
}

int SASOperation::checkSyntax(StringParser CommandLine)
{
    int i, hex[2], intParameterNumber = 0;
    string parameter[7];
    int err = 0;

    CommandLine.getParameterNumber(&intParameterNumber);

    switch(operationType)
    {
        case NO_OPRATION:
        case OPERATION_CONTROLLER:
        case OPERATION_PHY:
        case OPERATION_DRIVE:
            if(intParameterNumber > 0)
            {
                cout << "Not Necessary Operation Parameter" << endl;
                helpParameter();
                return -1;
            }
            break;
        case OPERATION_ALL:
        case OPERATION_NODE:
        case OPERATION_SN:
        case OPERATION_MODEL:
        case OPERATION_TEMP:
        case OPERATION_CAPACITY:
        case OPERATION_ATTACHED_TYPE:
        case OPERATION_LINK:
        case OPERATION_SPEED:
        case OPERATION_ERROR_ALL:
        case OPERATION_ERROR_STATISTICS:
        case OPERATION_ERROR_PCI:
        case OPERATION_DISPARITY_ERR:
        case OPERATION_INVALID_DWORD_ERR:
        case OPERATION_LOSS_DWORD_SYNC:
        case OPERATION_PHY_RESET_PRBM:
        case OPERATION_RESET_CONTROLLER:
        case OPERATION_EXTERNAL_LOOPBACK_TEST:
        case OPERATION_FIRMWARE_PID:
        case OPERATION_NVDATA_PID:
        case OPERATION_FCODE_VERSION:
        case OPERATION_UEFI_BSD_VERSION:
        case OPERATION_NVDATA_VENDOR:
        case OPERATION_DEFAULT_NVDATA_VERSION:
        case OPERATION_PERSISTENT_NVDATA_VERSION:
        case OPERATION_SMART:
        case OPERATION_SHOW_SPEED:
        case OPERATION_SHOW_WIDTH:
            if(intParameterNumber != 1)
            {
                cout << "Parameter numbers isn't 1" << endl;
                helpParameter();
                return -1;
            }
            break;
        case OPERATION_PROGRAM_ADDRESS:
        {
            UINT64 tempAddress;

            if(intParameterNumber != 2)
            {
                cout << "Parameter numbers isn't 2" << endl;
                helpParameter();
                return -1;
            }
            //Check parameter 1 is numeric
            err = CommandLine.convertStringToHex(parameter[0], &tempAddress);
            if(err != 0)
            {
                cout << "parameter 1 isn't numeric." << endl;
                helpParameter();
                return -1;
            }
            break;
        }
        case OPERATION_PROGRAM_FW:
            if(intParameterNumber != 2)
            {
                cout << "Parameter numbers isn't 2" << endl;
                helpParameter();
                return -1;
            }
            break;
        case OPERATION_READPFMC:
        case OPERATION_WRITEPFMC:
        case OPERATION_WRC:
            if(intParameterNumber != 3)
            {
                cout << "Parameter numbers isn't 3" << endl;
                helpParameter();
                return -1;
            }
            //Get parameter
            for(i = 0; i < intParameterNumber; i++)
            {
                CommandLine.getParameter(&parameter[i], i);
            }
            //Check parameter 1 is numeric
            err = CommandLine.convertStringToHex(parameter[0], &hex[0]);
            if(err != 0)
            {
                cout << "parameter 1 isn't numeric." << endl;
                helpParameter();
                return -1;
            }
            //Check parameter 1 is numeric
            err = CommandLine.convertStringToHex(parameter[1], &hex[1]);
            if(err != 0)
            {
                cout << "parameter 2 isn't numeric." << endl;
                helpParameter();
                return -1;
            }
            break;
        default:
            break;
    }

    return 0;
}

void SASOperation::handleTestProgress(UINT32 in_u32TestSizeMB)
{
    INT32 progress = 0;

    sleep(1);
    while(progress < 100)// <= 100%
    {
        sleep(1);
        progress = storageDAPI->getProgress();
        printf("Progressing: %3u%% \r", progress);
        fflush(stdout);
    }
    printf("\n");
}

// Format integer with thousands separator
const char *
SASOperation::format_with_thousands_sep(char * str, int strsize, uint64_t val,
                                        const char * thousands_sep /* = 0 */)
{
  if (!thousands_sep)
    thousands_sep = ",";

  char num[64];
  snprintf(num, sizeof(num), "%llu", val);
  int numlen = strlen(num);

  int i = 0, j = 0;
  do
    str[j++] = num[i++];
  while (i < numlen && (numlen - i) % 3 != 0 && j < strsize-1);
  str[j] = 0;

  while (i < numlen && j < strsize-1) {
    j += snprintf(str+j, strsize-j, "%s%.3s", thousands_sep, num+i);
    i += 3;
  }

  return str;
}

// Format capacity with SI prefixes
const char *
SASOperation::format_capacity(char * str, int strsize, uint64_t val,
                              const char * decimal_point /* = 0 */)
{
  if (!decimal_point)
    decimal_point = ".";

  const unsigned factor = 1000; // 1024 for KiB,MiB,...
  static const char prefixes[] = " KMGTP";

  // Find d with val in [d, d*factor)
  unsigned i = 0;
  uint64_t d = 1;
  for (uint64_t d2 = d * factor; val >= d2; d2 *= factor) {
    d = d2;
    if (++i >= sizeof(prefixes)-2)
      break;
  }

  // Print 3 digits
  uint64_t n = val / d;
  if (i == 0)
    snprintf(str, strsize, "%u B", (unsigned)n);
  else if (n >= 100) // "123 xB"
    snprintf(str, strsize, "%llu %cB", n, prefixes[i]);
  else if (n >= 10)  // "12.3 xB"
    snprintf(str, strsize, "%llu%s%u %cB", n, decimal_point,
        (unsigned)(((val % d) * 10) / d), prefixes[i]);
  else               // "1.23 xB"
    snprintf(str, strsize, "%llu%s%02u %cB", n, decimal_point,
        (unsigned)(((val % d) * 100) / d), prefixes[i]);

  return str;
}
