#include "StorageOperation.hpp"

StorageOperation::StorageOperation(OPERATION_TYPE OprationType, string helpContent) : OperationBaseClass(OprationType, helpContent)
{
     m_pStorageDAPI = NULL;
     for(UINT32 u32CNT=0; u32CNT<MAX_CURRENT_RUNNING_DRIVE_AMOUNT; u32CNT++)
     {
	  m_pProgressHandler[u32CNT] = NULL;
     }
     switch (OprationType)
     {
     case OPERATION_DRIVE:
	  setName("drive");
	  setHelp(helpContent);
	  m_OperationType = OprationType;
	  break;
     case OPERATION_ALL:
	  setName("all");
	  setHelp(helpContent);
	  m_OperationType = OprationType;
	  break;
     case OPERATION_PORT:
	  setName("port");
	  setHelp(helpContent);
	  m_OperationType = OprationType;
	  break;
     case OPERATION_NODE:
	  setName("node");
	  setHelp(helpContent);
	  m_OperationType = OprationType;
	  break;
     case OPERATION_SN:
	  setName("sn");
	  setHelp(helpContent);
	  m_OperationType = OprationType;
	  break;
     case OPERATION_MODEL:
	  setName("model");
	  setHelp(helpContent);
	  m_OperationType = OprationType;
	  break;
     case OPERATION_FW:
	  setName("fw");
	  setHelp(helpContent);
	  m_OperationType = OprationType;
	  break;
     case OPERATION_LOGICAL_BLOCK_SIZE:
	  setName("block");
	  setHelp(helpContent);
	  m_OperationType = OprationType;
	  break;
     case OPERATION_PHYSICAL_BLOCK_SIZE:
	  setName("physical_block");
	  setHelp(helpContent);
	  m_OperationType = OprationType;
	  break;
     case OPERATION_ADDRESS:
	  setName("address");
	  setHelp(helpContent);
	  m_OperationType = OprationType;
	  break;
     case OPERATION_CAPACITY:
	  setName("capacity");
	  setHelp(helpContent);
	  m_OperationType = OprationType;
	  break;
     case OPERATION_SPEED:
	  setName("speed");
	  setHelp(helpContent);
	  m_OperationType = OprationType;
	  break;
     case OPERATION_TEMP:
	  setName("temp");
	  setHelp(helpContent);
	  m_OperationType = OprationType;
	  break;
     case OPERATION_READPFMC:
	  setName("readpfmc");
	  setHelp(helpContent);
	  m_OperationType = OprationType;
	  break;
     case OPERATION_WRITEPFMC:
	  setName("writepfmc");
	  setHelp(helpContent);
	  m_OperationType = OprationType;
	  break;
     case OPERATION_WRC:
	  setName("wrc");
	  setHelp(helpContent);
	  m_OperationType = OprationType;
	  break;
     case OPERATION_SMART:
	  setName("smart");
	  setHelp(helpContent);
	  m_OperationType = OprationType;
	  break;
     case OPERATION_WZERO:
	  setName("wzero");
	  setHelp(helpContent);
	  m_OperationType = OprationType;
	  break;
     case OPERATION_GLIST:
	  setName("glist");
	  setHelp(helpContent);
	  m_OperationType = OprationType;
	  break;
     case OPERATION_PLIST:
	  setName("plist");
	  setHelp(helpContent);
	  m_OperationType = OprationType;
	  break;
     default:
	  break;
     }
}

StorageOperation::~StorageOperation()
{
     if(m_pStorageDAPI != NULL)
     {
	  delete m_pStorageDAPI;
     }
     for(UINT32 u32CNT=0; u32CNT<MAX_CURRENT_RUNNING_DRIVE_AMOUNT; u32CNT++)
     {
	  if(m_pProgressHandler[u32CNT] != NULL)
	  {
	       delete m_pProgressHandler[u32CNT];
	       m_pProgressHandler[u32CNT] = NULL;
	  }
     }
}

INT32 StorageOperation::executeOperation(StringParser CommandLine)
{
     INT32 s32Err = 0;

     string strStrParameter1 = "";
     CommandLine.getParameter(&strStrParameter1, 0);

     COMMAND_TYPE eCommandType;
     CommandLine.getCommandType(&eCommandType);
     //If the first parameter is "help" or space, show help message
     if(eCommandType == CMD_LIST)
     {
	  strStrParameter1 = NO_OPRATION;
     }
     else
     {
	  if(strStrParameter1==HELP || strStrParameter1=="")
	  {
	       helpParameter();
	       s32Err = -1;
	       return s32Err;
	  }
     }

     //Execute operation
     switch(m_OperationType)
     {
     case OPERATION_DRIVE:
	  s32Err = executeList(CommandLine);
	  break;
     case OPERATION_ALL:
	  if(eCommandType == CMD_LIST)
	  {
	       s32Err = executeList(CommandLine);
	  }
	  else if(eCommandType == CMD_SHOW)
	  {
	       s32Err = executeShow(CommandLine);
	  }
	  else if(eCommandType == CMD_ERROR)
	  {
//	       s32Err = executeError(CommandLine);
	  }
	  break;
     case OPERATION_PORT:
     case OPERATION_NODE:
     case OPERATION_SN:
     case OPERATION_MODEL:
     case OPERATION_FW:
     case OPERATION_LOGICAL_BLOCK_SIZE:
     case OPERATION_ADDRESS:
     case OPERATION_CAPACITY:
     case OPERATION_SPEED:
     case OPERATION_TEMP:
	  s32Err = executeShow(CommandLine);
	  break;
     case OPERATION_READPFMC:
     case OPERATION_WRITEPFMC:
     case OPERATION_WRC:
	  s32Err = executeTest(CommandLine);
	  break;
     case OPERATION_SMART:
     case OPERATION_WZERO:
     case OPERATION_PLIST:
     case OPERATION_GLIST:
	  s32Err = executeUtility(CommandLine);
	  break;
     default:
	  cout << "Unsupport operation" << endl;
	  s32Err = -1; 
	  return s32Err;
	  break;
     }
     return s32Err;
}

INT32 StorageOperation::helpParameter()
{
     //Show parameter help message depend on operation type
     switch (m_OperationType)
     {
     case OPERATION_ALL:
     case OPERATION_PORT:
     case OPERATION_NODE:
     case OPERATION_SN:
     case OPERATION_MODEL:
     case OPERATION_FW:
     case OPERATION_LOGICAL_BLOCK_SIZE:
     case OPERATION_PHYSICAL_BLOCK_SIZE:
     case OPERATION_ADDRESS:
     case OPERATION_CAPACITY:
     case OPERATION_SPEED:
     case OPERATION_TEMP:
	  cout << "Show Storage Device Characteristic" << endl;
	  cout << "Syntax: P1 P2" << endl;
	  cout << "P1: Data Elements" << endl;
	  cout << "P2: Drive definedName" << endl;
	  break;
     case OPERATION_READPFMC:
	  cout << "Storage Device Read Performance Test" << endl;
	  cout << "Syntax: readpfmc P1 P2 P3" << endl;
	  cout << "P1: Start Sector(Hex)" << endl;
	  cout << "P2: Test Size/MB(Dec)" << endl;
	  cout << "P3: Drive definedName" << endl;
	  break;
     case OPERATION_WRITEPFMC:
	  cout << "Storage Device Write Performance Test" << endl;
	  cout << "Syntax: writepfmc P1 P2 P3" << endl;
	  cout << "P1: Start Sector(Hex)" << endl;
	  cout << "P2: Test Size/MB(Dec)" << endl;
	  cout << "P3: Drive definedName" << endl;
	  break;
     case OPERATION_WRC:
	  cout << "Storage Device ReadWrite Test" << endl;
	  cout << "Syntax: wrc P1 P2 P3" << endl;
	  cout << "P1: Start Sector(Hex)" << endl;
	  cout << "P2: Test Size/MB(Dec)" << endl;
	  cout << "P3: Drive definedName" << endl;
	  break;
     case OPERATION_SMART:
	  cout << "Storage Device SMART information" << endl;
	  cout << "Syntax: smart P1 P2 P1" << endl;
	  cout << "P1: Attribute Start ID" << endl;
	  cout << "P2: Attribute End ID" << endl;
	  cout << "P3: Drive definedName" << endl;
	  break;
     case OPERATION_WZERO:
	  cout << "Write zero to Storage Device" << endl;
	  cout << "Syntax: wzero P1 P2" << endl;
	  cout << "P1: Start Sector(Hex)" << endl;
	  cout << "P2: Drive definedName" << endl;
	  break;
     case OPERATION_PLIST:
	  cout << "Get Storage Drive P-List Data" << endl;
	  cout << "Syntax: plist P1 P2" << endl;
	  cout << "P1: P-List Format" << endl;
	  cout << "P2: Drive definedName" << endl;
	  break;
     case OPERATION_GLIST:
	  cout << "Get Storage Drive G-List Data" << endl;
	  cout << "Syntax: glist P1 P2" << endl;
	  cout << "P1: G-List Format" << endl;
	  cout << "P2: Drive definedName" << endl;
	  break;

     default:
	  break;
     }
     return 0;
}

INT32 StorageOperation::executeList(StringParser CommandLine)
{
     INT32 s32Err = 0;
     ErrorMsg *pErrorMsg = NO_ERROR_MSG;
     /*
     m_pStorageDAPI = new StorageDAPI("StorageDAPI");
     if(m_pStorageDAPI->getErrorMsg())
     {
	  printError(pErrorMsg);
	  return -1;
	  }*/

     StorageDAPI m_StorageDAPI("StorageDAPI");
     if((pErrorMsg = m_StorageDAPI.getErrorMsg()) != NO_ERROR_MSG)
     {
	  printError(pErrorMsg);
	  return -1;
     }

     s32Err = checkSyntax(CommandLine);
     if(s32Err == 0)
     {
	  switch(m_OperationType)
	  {
	  case OPERATION_DRIVE:
//	       pErrorMsg = m_pStorageDAPI->IOCTL(0, eStorageListDrive);
	       pErrorMsg = m_StorageDAPI.IOCTL(0, eStorageListDrive);
	       break;
	  default:
	       cout << "no such operation" << endl;
	       s32Err = -1;
	       return s32Err;
	       break;
	  }
	  if(pErrorMsg != NO_ERROR_MSG)
	  {
	       printError(pErrorMsg);
	       s32Err = -1;
	       return s32Err;
	  }
     }
     return s32Err;
}

INT32 StorageOperation::executeShow(StringParser CommandLine)
{
     INT32 s32Err = 0;
     ErrorMsg *pErrorMsg = NO_ERROR_MSG;
     UINT32 u32DefinedIndex = 0;
     StorageDataElement eStorageDataElement = (StorageDataElement)0;

     string strTargetName = "";
     string strReturnData = "";

     m_pStorageDAPI = new StorageDAPI("StorageDAPI");

     s32Err = checkSyntax(CommandLine);
     if(s32Err == 0)
     {
	  CommandLine.getParameter(&strTargetName, 0);
	  strTargetName = convertStringToUpperCase(strTargetName);
	  //printf("2nd Parameter:%s\n", strTargetName.c_str());
	  switch(m_OperationType)
	  {
	  case OPERATION_ALL:
	       eStorageDataElement = eStorageDriveDataAll;
	       break;
	  case OPERATION_PORT:
	       eStorageDataElement = eStorageDrivePortNumber;
	       break;
	  case OPERATION_NODE:
	       eStorageDataElement = eStorageDriveDeviceNodeName;
	       break;
	  case OPERATION_SN:
	       eStorageDataElement = eStorageDriveSerialNumber;
	       break;
	  case OPERATION_MODEL:
	       eStorageDataElement = eStorageDriveModelName;
	       break;
	  case OPERATION_FW:
	       eStorageDataElement = eStorageDriveFirmwareVersion;
	       break;
	  case OPERATION_LOGICAL_BLOCK_SIZE:
	       eStorageDataElement = eStorageDriveLogicalBlockSize;
	       break;
	  case OPERATION_PHYSICAL_BLOCK_SIZE:
	       eStorageDataElement = eStorageDrivePhysicalBlockSize;
	       break;
	  case OPERATION_ADDRESS:
	       eStorageDataElement = eStorageDriveSASAddress;
	       break;
	  case OPERATION_CAPACITY:
	       eStorageDataElement = eStorageDriveCapacity;
	       break;
	  case OPERATION_SPEED:
	       eStorageDataElement = eStorageDriveLinkSpeed;
	       break;
	  case OPERATION_TEMP:
	       eStorageDataElement = eStorageDriveTemperature;
	       break;
	  default:
	       cout << "no such data element" << endl;
	       return -1;
	       break;
	  }
	  if(strTargetName=="SAS" || strTargetName=="NVME" || strTargetName=="USB" || strTargetName=="ALL")
	  {
	       stringstream ssTempI;
	       stringstream ssTempJ;

	       if(strTargetName == "SAS")
	       {
		    cout << setw(15) << left << " ";
		    cout << setw(17) << left << "DeviceNode, ";
		    cout << setw(17) << left << "Vendor, ";
		    cout << setw(17) << left << "SerialNumber, ";
		    cout << setw(17) << left << "ModelName, ";
		    cout << setw(17) << left << "CapacityMB, ";
//		    cout << setw(17) << left << "SASAddr, ";
		    cout << setw(17) << left << "LgBlkSize, ";
		    cout << setw(17) << left << "FWVersion, ";
		    cout << setw(17) << left << "LinkSpeed, ";
//		    cout << setw(17) << left << "RotationRate, ";
//		    cout << setw(17) << left << "FormFactor, ";
		    cout << setw(17) << left << "TransProtocol, ";
		    cout << setw(17) << left << "Temperature, " << endl;

	       }
	       else if(strTargetName == "NVME")
	       {
		    cout << setw(10) << left << " ";
		    cout << setw(20) << left << "DeviceNode, ";
		    cout << setw(20) << left << "LgBlkSize, ";
		    cout << setw(20) << left << "CapacityMB, " << endl;
	       }
	       else if(strTargetName == "USB")
	       {
		    cout << setw(10) << left << " ";
		    cout << setw(20) << left << "DeviceNode, ";
		    cout << setw(20) << left << "Vendor, ";
		    cout << setw(20) << left << "SerialNumber, ";
		    cout << setw(20) << left << "CapacityMB, ";
		    cout << setw(20) << left << "LgBlkSize, ";
		    cout << setw(20) << left << "LinkSpeed, ";
		    cout << setw(20) << left << "Version, " << endl;
	       }

	       for(UINT32 u32I=0; u32I<0xf; u32I++)
	       {
		    ssTempI.str("");
		    ssTempI.clear();
		    ssTempI << u32I;

		    if(strTargetName=="NVME" || strTargetName=="USB")
		    {
			 string strTargetNameTemp = strTargetName + ssTempI.str();
			 if((pErrorMsg = m_pStorageDAPI->getIndexByName(strTargetNameTemp, &u32DefinedIndex)) != NO_ERROR_MSG)
			 {
			      continue;
			 }

			 strReturnData = "";
			 UINT32 u32TempStorageDataElement = 0;
			 UINT32 u32aDataELementArray[(UINT32)eStorageDriveUnknownDataElement];
			 for(UINT32 u32CNT=0; u32CNT<(UINT32)eStorageDriveUnknownDataElement; u32aDataELementArray[u32CNT]=(UINT32)eStorageDriveUnknownDataElement, u32CNT++);
			 if(strTargetName == "NVME")
			 {
			      u32aDataELementArray[0] = (UINT32)eStorageDriveDeviceNodeName;
			      u32aDataELementArray[1] = (UINT32)eStorageDriveLogicalBlockSize;
			      u32aDataELementArray[2] = (UINT32)eStorageDriveCapacity;
			 }
			 else if(strTargetName == "USB")
			 {
			      u32aDataELementArray[0] = (UINT32)eStorageDriveDeviceNodeName;
			      u32aDataELementArray[1] = (UINT32)eStorageDriveVendorName;
			      u32aDataELementArray[2] = (UINT32)eStorageDriveSerialNumber;
			      u32aDataELementArray[3] = (UINT32)eStorageDriveCapacity;
			      u32aDataELementArray[4] = (UINT32)eStorageDriveLogicalBlockSize;
			      u32aDataELementArray[5] = (UINT32)eStorageDriveLinkSpeed;
			      u32aDataELementArray[6] = (UINT32)eStorageDriveUSBVersion;
			 }

			 string strTempRetunData = strTargetNameTemp + ": ";
			 cout << setw(10) << left << strTempRetunData;
			 while(u32aDataELementArray[u32TempStorageDataElement] < (UINT32)eStorageDriveUnknownDataElement)
			 {
			      string strTempRetunData = "";
			      if((pErrorMsg = m_pStorageDAPI->getElementDataByIndex(u32DefinedIndex, (StorageDataElement)u32aDataELementArray[u32TempStorageDataElement], &strTempRetunData)) != NO_ERROR_MSG);

			      strReturnData = strTempRetunData + ", ";
			      cout << setw(20) << left << strReturnData;
			      u32TempStorageDataElement++;
			 }
			 cout << endl;
		    }
		    else
		    {
			 for(UINT32 u32J=0; u32J<0xff; u32J++)
			 {
			      ssTempJ.str("");
			      ssTempJ.clear();
			      ssTempJ << u32J;
			      string strTargetNameTemp = strTargetName + ssTempI.str() + "-DRIVE" + ssTempJ.str();

			      if((pErrorMsg = m_pStorageDAPI->getIndexByName(strTargetNameTemp, &u32DefinedIndex)) != NO_ERROR_MSG)
			      {
				   continue;
			      }
			      strReturnData = "";
			      UINT32 u32TempStorageDataElement = 0;
			      UINT32 u32aDataELementArray[(UINT32)eStorageDriveUnknownDataElement];
			      for(UINT32 u32CNT=0; u32CNT<(UINT32)eStorageDriveUnknownDataElement; u32aDataELementArray[u32CNT]=(UINT32)eStorageDriveUnknownDataElement, u32CNT++);
			      if(strTargetName == "SAS")
			      {
				   u32aDataELementArray[0] = (UINT32)eStorageDriveDeviceNodeName;
				   u32aDataELementArray[1] = (UINT32)eStorageDriveVendorName;
				   u32aDataELementArray[2] = (UINT32)eStorageDriveSerialNumber;
				   u32aDataELementArray[3] = (UINT32)eStorageDriveModelName;
				   u32aDataELementArray[4] = (UINT32)eStorageDriveCapacity;
//				   u32aDataELementArray[5] = (UINT32)eStorageDriveSASAddress;
				   u32aDataELementArray[5] = (UINT32)eStorageDriveLogicalBlockSize;
				   u32aDataELementArray[6] = (UINT32)eStorageDriveFirmwareVersion;
				   u32aDataELementArray[7] = (UINT32)eStorageDriveLinkSpeed;
//				   u32aDataELementArray[9] = (UINT32)eStorageDriveRotationRate;
//				   u32aDataELementArray[10] = (UINT32)eStorageDriveFormFactor;
				   u32aDataELementArray[8] = (UINT32)eStorageDriveProtocolType;
				   u32aDataELementArray[9] = (UINT32)eStorageDriveTemperature;
			      }

			      string strTempRetunData = strTargetNameTemp + ": ";
			      cout << setw(15) << left << strTempRetunData;
			      while(u32aDataELementArray[u32TempStorageDataElement] < (UINT32)eStorageDriveUnknownDataElement)
			      {
				   string strTempRetunData = "";
				   if((pErrorMsg = m_pStorageDAPI->getElementDataByIndex(u32DefinedIndex, (StorageDataElement)u32aDataELementArray[u32TempStorageDataElement], &strTempRetunData)) != NO_ERROR_MSG);

				   strReturnData = strTempRetunData + ", ";
				   cout << setw(17) << left << strReturnData;
				   u32TempStorageDataElement++;
			      }
			      cout << endl;
			 }
		    }
	       }
	  }
	  else
	  {
	       if((pErrorMsg = m_pStorageDAPI->getIndexByName(strTargetName, &u32DefinedIndex)) != NO_ERROR_MSG)
	       {
		    printError(pErrorMsg);
		    return -1;
	       }

	       if((pErrorMsg = m_pStorageDAPI->getElementDataByIndex(u32DefinedIndex, eStorageDataElement, &strReturnData)) != NO_ERROR_MSG)
	       {
		    printError(pErrorMsg);
		    return -1;
	       }
	       else
	       {
		    cout << strReturnData <<endl;
	       }
	  }
     }
     return s32Err;
}

INT32 StorageOperation::executeTest(StringParser CommandLine)
{
     INT32 s32Err = 0;

     ErrorMsg *pErrorMsg = NO_ERROR_MSG;

     UINT32 u32DefinedIndex = 0;
     UINT64 u64StartSector = 0;
     INT32 s32TestSize = 0;

     string strTempString = "";
     strTempString.clear();

     string strTargetName = "";
     strTargetName.clear();

     m_pStorageDAPI = new StorageDAPI("StorageDAPI");

     s32Err = checkSyntax(CommandLine);
     if(s32Err == 0)
     {
	  s32Err = -1;

	  if(m_OperationType != OPERATION_IDENTIFY)
	  {
	       CommandLine.getParameter(&strTempString, 0);
	       CommandLine.convertStringToHex(strTempString, &u64StartSector);
	       strTempString.clear();

	       CommandLine.getParameter(&strTempString, 1);
	       CommandLine.convertStringToDec(strTempString, &s32TestSize);
	       strTempString.clear();

	       CommandLine.getParameter(&strTargetName, 2);
	       strTargetName = convertStringToUpperCase(strTargetName);
	  }

	  if(strTargetName=="SAS" || strTargetName=="NVME" || strTargetName=="USB" || strTargetName=="ALL")
	  {
	       fstream fout;
	       fout.open("/dev/shm/storage.tmp", ios::out);
	       fout.close();

	       stringstream ssTempI;
	       stringstream ssTempJ;

	       if(strTargetName=="NVME" || strTargetName=="USB")
	       {
		    int status[0xf];
		    pid_t pid[0xf];

		    for(UINT32 u32I=0; u32I<0xf; u32I++)
		    {
			 ssTempI.str("");
			 ssTempI.clear();
			 ssTempI << u32I;

			 string strTargetNameTemp = strTargetName + ssTempI.str();
			 if((pErrorMsg = m_pStorageDAPI->getIndexByName(strTargetNameTemp, &u32DefinedIndex)) != NO_ERROR_MSG)
			 {
			      continue;
			 }

			 s32Err = 0; // if loop found any HDDs/controller

			 pid[u32I] = fork();
			 if(pid[u32I] < 0) // if fork faild
			 {
			      s32Err = -1;
			      return -1;
			 }

			 if(pid[u32I] == 0)// child process
			 {
			      // run thread performance test
			      m_pProgressHandler[u32I] = new thread(&StorageOperation::handleTestProgress, this, (UINT32)s32TestSize);
			      switch(m_OperationType)
			      {
			      case OPERATION_READPFMC:
				   if((pErrorMsg = m_pStorageDAPI->getIndexByName(strTargetNameTemp.c_str(), &u32DefinedIndex)) != NO_ERROR_MSG)
				   {
					s32Err = -1;
					break;
				   }
				   if((pErrorMsg = m_pStorageDAPI->readPerformanceTest(u32DefinedIndex, u64StartSector, (UINT32)s32TestSize, true)) != NO_ERROR_MSG)
				   {
					s32Err = -1;
				   }
				   break;
			      case OPERATION_WRITEPFMC:
				   if((pErrorMsg = m_pStorageDAPI->getIndexByName(strTargetNameTemp.c_str(), &u32DefinedIndex)) != NO_ERROR_MSG)
				   {
					s32Err = -1;
					break;
				   }
				   if((pErrorMsg =m_pStorageDAPI->writePerformanceTest(u32DefinedIndex, u64StartSector, (UINT32)s32TestSize, true)) != NO_ERROR_MSG)
				   {
					s32Err = -1;
				   }
				   break;
			      case OPERATION_WRC:
				   if((pErrorMsg = m_pStorageDAPI->getIndexByName(strTargetNameTemp.c_str(), &u32DefinedIndex)) != NO_ERROR_MSG)
				   {
					s32Err = -1;
					break;
				   }
				   if((pErrorMsg = m_pStorageDAPI->readwriteTest(u32DefinedIndex, u64StartSector, (UINT32)s32TestSize, true)) != NO_ERROR_MSG)
				   {
					s32Err = -1;
				   }
				   break;
			      default:
				   cout << "Unsupport operation" << endl;
				   s32Err = -1;
				   break;
			      }

			      if(m_pProgressHandler[u32I] != NULL)
			      {
				   m_pProgressHandler[u32I]->join();
			      }

			      cout << m_pStorageDAPI->m_strOutputResultString;

			      if(s32Err == 0)
			      {
				   cout << "RESULT: PASSED" << endl;
			      }
			      else
			      {
				   cout << strTargetNameTemp << ": ";
				   printError(pErrorMsg);
				   fstream fout;
				   fout.open("/dev/shm/storage.tmp", ios::out);
				   fout << "RESULT: FAILED" << endl;
				   fout.close();
				   cout << "RESULT: FAILED" << endl;
			      }
			      exit(0);
			 }
		    }
		    for(UINT32 u32I=0; u32I<0xf; u32I++)
		    {
			 waitpid((pid_t)pid[u32I], &status[u32I], 0);
		    }
	       }
	       else
	       {
		    for(UINT32 u32I=0; u32I<0xf; u32I++)
		    {
			 ssTempI.str("");
			 ssTempI.clear();
			 ssTempI << u32I;
			 int status[0xff];
			 pid_t pid[0xff];

			 for(UINT32 u32J=0; u32J<0xff; u32J++)
			 {
			      ssTempJ.str("");
			      ssTempJ.clear();
			      ssTempJ << u32J;

			      string strTargetNameTemp = strTargetName + ssTempI.str() + "-DRIVE" + ssTempJ.str();
			      if(strTargetName=="NVME" || strTargetName=="USB")
			      {
				   strTargetNameTemp = strTargetName + ssTempI.str();
			      }

			      if((pErrorMsg = m_pStorageDAPI->getIndexByName(strTargetNameTemp, &u32DefinedIndex)) != NO_ERROR_MSG)
			      {
				   continue;
			      }

			      s32Err = 0; // if loop found any HDDs/controller

			      pid[u32J] = fork();
			      if(pid[u32J] < 0) // if fork faild
			      {
				   s32Err = -1;
				   return -1;
			      }

			      if(pid[u32J] == 0)// child process
			      {
				   // run thread performance test
				   m_pProgressHandler[u32J] = new thread(&StorageOperation::handleTestProgress, this, (UINT32)s32TestSize);
				   switch(m_OperationType)
				   {
				   case OPERATION_READPFMC:
					if((pErrorMsg = m_pStorageDAPI->getIndexByName(strTargetNameTemp.c_str(), &u32DefinedIndex)) != NO_ERROR_MSG)
					{
					     cout << strTargetNameTemp << endl;
					     printError(pErrorMsg);
					     s32Err = -1;
					     break;
					}
					if((pErrorMsg = m_pStorageDAPI->readPerformanceTest(u32DefinedIndex, u64StartSector, (UINT32)s32TestSize, true)) != NO_ERROR_MSG)
					{
					     printError(pErrorMsg);
					     s32Err = -1;
					}
					break;
				   case OPERATION_WRITEPFMC:
					if((pErrorMsg = m_pStorageDAPI->getIndexByName(strTargetNameTemp.c_str(), &u32DefinedIndex)) != NO_ERROR_MSG)
					{
					     cout << strTargetNameTemp << endl;
					     printError(pErrorMsg);
					     s32Err = -1;
					     break;
					}
					if((pErrorMsg =m_pStorageDAPI->writePerformanceTest(u32DefinedIndex, u64StartSector, (UINT32)s32TestSize, true)) != NO_ERROR_MSG)
					{
					     printError(pErrorMsg);
					     s32Err = -1;
					}
					break;
				   case OPERATION_WRC:
					if((pErrorMsg = m_pStorageDAPI->getIndexByName(strTargetNameTemp.c_str(), &u32DefinedIndex)) != NO_ERROR_MSG)
					{
					     cout << strTargetNameTemp << endl;
					     printError(pErrorMsg);
					     s32Err = -1;
					     break;
					}
					if((pErrorMsg = m_pStorageDAPI->readwriteTest(u32DefinedIndex, u64StartSector, (UINT32)s32TestSize, true)) != NO_ERROR_MSG)
					{
					     printError(pErrorMsg);
					     s32Err = -1;
					}
					break;
				   default:
					cout << "Unsupport operation" << endl;
					s32Err = -1;
					break;
				   }

				   if(m_pProgressHandler[u32J] != NULL)
				   {
					m_pProgressHandler[u32J]->join();
				   }

				   cout << m_pStorageDAPI->m_strOutputResultString;

				   if(s32Err == 0)
				   {
					cout << "RESULT: PASSED" << endl;
				   }
				   else
				   {
					fstream fout;
					fout.open("/dev/shm/storage.tmp", ios::out);
					fout << "RESULT: FAILED" << endl;
					fout.close();
					cout << "RESULT: FAILED" << endl;
				   }
				   exit(0);
			      }
			 }
			 for(UINT32 u32J=0; u32J<0xff; u32J++)
			 {
			      waitpid((pid_t)pid[u32J], &status[u32J], 0);
			 }
		    }
	       }

	       fstream fin;
	       fin.open("/dev/shm/storage.tmp", ios::in);
	       string strTempLine = "";
	       while(getline(fin, strTempLine))
	       {
		    if(strTempLine != "")
		    {
			 s32Err = -1;
		    }
	       }
	       fin.close();

	       if(s32Err == -1)// if has error print out
	       {
		    printError(pErrorMsg);
	       }
	  }
	  else
	  {
	       // normal run performance
	       s32Err = 0;

	       if((pErrorMsg = m_pStorageDAPI->getIndexByName(strTargetName.c_str(), &u32DefinedIndex)) != NO_ERROR_MSG)
	       {
		    printError(pErrorMsg);
		    s32Err = -1;
		    return s32Err;
	       }

	       m_pProgressHandler[0] = new thread(&StorageOperation::handleTestProgress, this, (UINT32)s32TestSize);

	       switch(m_OperationType)
	       {
	       case OPERATION_READPFMC:
		    if((pErrorMsg = m_pStorageDAPI->readPerformanceTest(u32DefinedIndex, u64StartSector, (UINT32)s32TestSize)) != NO_ERROR_MSG)
		    {
			 printError(pErrorMsg);
			 s32Err = -1;
		    }
		    break;
	       case OPERATION_WRITEPFMC:
		    if((pErrorMsg =m_pStorageDAPI->writePerformanceTest(u32DefinedIndex, u64StartSector, (UINT32)s32TestSize)) != NO_ERROR_MSG)
		    {
			 printError(pErrorMsg);
			 s32Err = -1;
		    }
		    break;
	       case OPERATION_WRC:
		    if((pErrorMsg = m_pStorageDAPI->readwriteTest(u32DefinedIndex, u64StartSector, (UINT32)s32TestSize)) != NO_ERROR_MSG)
		    {
			 printError(pErrorMsg);
			 s32Err = -1;
		    }
		    break;
	       default:
		    cout << "Unsupport operation" << endl;
		    s32Err = -1;
		    break;
	       }
	       m_pProgressHandler[0]->join();
	       cout << m_pStorageDAPI->m_strOutputResultString;
	  }
     }

     return s32Err;
}

INT32 StorageOperation::executeUtility(StringParser CommandLine)
{
     INT32 s32Err = 0;
     ErrorMsg *pErrorMsg = NO_ERROR_MSG;

     UINT32 u32DefinedIndex = 0;
     INT32 s32StartIDNumber = 0;
     INT32 s32EndIDNumber = 0;
     UINT64 u64StartSector = 0;
     INT32 s32PGListFormat = 0;

     string strTempString = "";
     strTempString.clear();
     string strTargetName = "";
     strTargetName.clear();
     string strReturnData = "";
     strReturnData.clear();

     stringstream ssTemp;
     ssTemp.clear();
     ssTemp << strTempString;
     UINT64 u64CapacityMB = 0;

     m_pStorageDAPI = new StorageDAPI("StorageDAPI");

     s32Err = checkSyntax(CommandLine);
     if(s32Err == 0)
     {
	  switch(m_OperationType)
	  {
	  case OPERATION_SMART:
/*	       CommandLine.getParameter(&strTempString, 0);
	       CommandLine.convertStringToHex(strTempString, &s32StartIDNumber);
	       strTempString.clear();

	       CommandLine.getParameter(&strTempString, 1);
	       CommandLine.convertStringToHex(strTempString, &s32EndIDNumber);
	       strTempString.clear();
*/
//	       CommandLine.getParameter(&strTargetName, 2);
	       CommandLine.getParameter(&strTargetName, 0);
	       strTargetName = convertStringToUpperCase(strTargetName);
	       strTempString.clear();

	       if((pErrorMsg = m_pStorageDAPI->getIndexByName(strTargetName, &u32DefinedIndex)) != NO_ERROR_MSG)
	       {
		    printError(pErrorMsg);
		    return -1;
	       }

	       if((pErrorMsg =m_pStorageDAPI->getSMARTByIndex(u32DefinedIndex)) != NO_ERROR_MSG)
	       {
		    printError(pErrorMsg);
		    return -1;
	       }
	       cout << m_pStorageDAPI->m_strOutputResultString << endl;
/*
  if((pErrorMsg =m_pStorageDAPI->getSMARTByIndex(u32DefinedIndex, (UINT32)s32StartIDNumber, (UINT32)s32EndIDNumber, &strReturnData)) != NO_ERROR_MSG)
  {
  printError(pErrorMsg);
  return -1;
  }
  cout << strReturnData << endl;
*/
	       break;
	  case OPERATION_WZERO:
	       s32Err = 0;

	       CommandLine.getParameter(&strTempString, 0);
	       CommandLine.convertStringToHex(strTempString, &u64StartSector);
	       strTempString.clear();

	       CommandLine.getParameter(&strTargetName, 1);
	       strTargetName = convertStringToUpperCase(strTargetName);

	       if(strTargetName=="SAS" || strTargetName=="NVME" || strTargetName=="USB" || strTargetName=="ALL")
	       {
		    fstream fout;
		    fout.open("/dev/shm/storage.tmp", ios::out);
		    fout.close();

		    stringstream ssTempI;
		    stringstream ssTempJ;

		    if(strTargetName=="NVME" || strTargetName=="USB")
		    {
			 int status[0xf];
			 pid_t pid[0xf];

			 for(UINT32 u32I=0; u32I<0xf; u32I++)
			 {
			      ssTempI.str("");
			      ssTempI.clear();
			      ssTempI << u32I;

			      string strTargetNameTemp = strTargetName + ssTempI.str();
			      if((pErrorMsg = m_pStorageDAPI->getIndexByName(strTargetNameTemp, &u32DefinedIndex)) != NO_ERROR_MSG)
			      {
				   continue;
			      }

			      s32Err = 0; // if loop found any HDDs/controller

			      pid[u32I] = fork();
			      if(pid[u32I] < 0) // if fork faild
			      {
				   s32Err = -1;
				   return -1;
			      }

			      if(pid[u32I] == 0)// child process
			      {
				   // run thread performance test
				   if((pErrorMsg = m_pStorageDAPI->getIndexByName(strTargetNameTemp.c_str(), &u32DefinedIndex)) != NO_ERROR_MSG)
				   {
					s32Err = -1;
					break;
				   }
				   if((pErrorMsg = m_pStorageDAPI->getElementDataByIndex(u32DefinedIndex, eStorageDriveCapacity, &strTempString)) != NO_ERROR_MSG)
				   {
					printError(pErrorMsg);
					s32Err = -1;
					return -1;
				   }
				   ssTemp.str("");
				   ssTemp.clear();
				   ssTemp << strTempString;
				   ssTemp >> u64CapacityMB;

				   m_pProgressHandler[u32I] = new thread(&StorageOperation::handleTestProgress, this, (UINT32)u64CapacityMB);

				   if((pErrorMsg = m_pStorageDAPI->writeZeroByIndex(u32DefinedIndex, (UINT32)u64CapacityMB, true)) != NO_ERROR_MSG)
				   {
					s32Err = -1;
				   }

				   if(m_pProgressHandler[u32I] != NULL)
				   {
					m_pProgressHandler[u32I]->join();
				   }

				   cout << m_pStorageDAPI->m_strOutputResultString;

				   if(s32Err == 0)
				   {
					cout << "RESULT: PASSED" << endl;
				   }
				   else
				   {
					cout << strTargetNameTemp << ": ";
					printError(pErrorMsg);
					fstream fout;
					fout.open("/dev/shm/storage.tmp", ios::out);
					fout << "RESULT: FAILED" << endl;
					fout.close();
					cout << "RESULT: FAILED" << endl;
				   }
				   exit(0);
			      }
			 }
			 for(UINT32 u32I=0; u32I<0xf; u32I++)
			 {
			      waitpid((pid_t)pid[u32I], &status[u32I], 0);
			 }
		    }
		    else
		    {
			 for(UINT32 u32I=0; u32I<0xf; u32I++)
			 {
			      ssTempI.str("");
			      ssTempI.clear();
			      ssTempI << u32I;
			      int status[0xff];
			      pid_t pid[0xff];

			      for(UINT32 u32J=0; u32J<0xff; u32J++)
			      {
				   ssTempJ.str("");
				   ssTempJ.clear();
				   ssTempJ << u32J;

				   string strTargetNameTemp = strTargetName + ssTempI.str() + "-DRIVE" + ssTempJ.str();
				   if(strTargetName=="NVME" || strTargetName=="USB")
				   {
					strTargetNameTemp = strTargetName + ssTempI.str();
				   }

				   if((pErrorMsg = m_pStorageDAPI->getIndexByName(strTargetNameTemp, &u32DefinedIndex)) != NO_ERROR_MSG)
				   {
					continue;
				   }

				   s32Err = 0; // if loop found any HDDs/controller

				   pid[u32J] = fork();
				   if(pid[u32J] < 0) // if fork faild
				   {
					s32Err = -1;
					return -1;
				   }

				   if(pid[u32J] == 0)// child process
				   {
					// run thread performance test
					if((pErrorMsg = m_pStorageDAPI->getElementDataByIndex(u32DefinedIndex, eStorageDriveCapacity, &strTempString)) != NO_ERROR_MSG)
					{
					     printError(pErrorMsg);
					     s32Err = -1;
					     return -1;
					}
					ssTemp.str("");
					ssTemp.clear();
					ssTemp << strTempString;
					ssTemp >> u64CapacityMB;

					m_pProgressHandler[u32J] = new thread(&StorageOperation::handleTestProgress, this, (UINT32)u64CapacityMB);
					if((pErrorMsg = m_pStorageDAPI->getIndexByName(strTargetNameTemp.c_str(), &u32DefinedIndex)) != NO_ERROR_MSG)
					{
					     cout << strTargetNameTemp << endl;
					     printError(pErrorMsg);
					     s32Err = -1;
					     break;
					}
					if((pErrorMsg = m_pStorageDAPI->writeZeroByIndex(u32DefinedIndex, u64StartSector, true)) != NO_ERROR_MSG)
					{
					     printError(pErrorMsg);
					     s32Err = -1;
					}

					if(m_pProgressHandler[u32J] != NULL)
					{
					     m_pProgressHandler[u32J]->join();
					}

					cout << m_pStorageDAPI->m_strOutputResultString;

					if(s32Err == 0)
					{
					     cout << "RESULT: PASSED" << endl;
					}
					else
					{
					     fstream fout;
					     fout.open("/dev/shm/storage.tmp", ios::out);
					     fout << "RESULT: FAILED" << endl;
					     fout.close();
					     cout << "RESULT: FAILED" << endl;
					}
					exit(0);
				   }
			      }
			      for(UINT32 u32J=0; u32J<0xff; u32J++)
			      {
				   waitpid((pid_t)pid[u32J], &status[u32J], 0);
			      }
			 }
		    }

		    fstream fin;
		    fin.open("/dev/shm/storage.tmp", ios::in);
		    string strTempLine = "";
		    while(getline(fin, strTempLine))
		    {
			 if(strTempLine != "")
			 {
			      s32Err = -1;
			 }
		    }
		    fin.close();

		    if(s32Err == -1)// if has error print out
		    {
			 printError(pErrorMsg);
		    }
	       }
	       else
	       {
		    s32Err = 0;
		    // normal run performance
		    if((pErrorMsg = m_pStorageDAPI->getIndexByName(strTargetName, &u32DefinedIndex)) != NO_ERROR_MSG)
		    {
			 printError(pErrorMsg);
			 s32Err = -1;
			 return -1;
		    }

		    if((pErrorMsg = m_pStorageDAPI->getElementDataByIndex(u32DefinedIndex, eStorageDriveCapacity, &strTempString)) != NO_ERROR_MSG)
		    {
			 printError(pErrorMsg);
			 s32Err = -1;
			 return -1;
		    }

		    ssTemp.str("");
		    ssTemp.clear();
		    ssTemp << strTempString;
		    ssTemp >> u64CapacityMB;

		    m_pProgressHandler[0] = new thread(&StorageOperation::handleTestProgress, this, (UINT32)u64CapacityMB);

		    if((pErrorMsg = m_pStorageDAPI->writeZeroByIndex(u32DefinedIndex, u64StartSector)) != NO_ERROR_MSG)
		    {
			 printError(pErrorMsg);
			 return -1;
		    }
		    m_pProgressHandler[0]->join();
		    cout << m_pStorageDAPI->m_strOutputResultString << endl;
	       }
	       break;
	  case OPERATION_GLIST:
	  case OPERATION_PLIST:
	       s32Err = 0;

	       CommandLine.getParameter(&strTempString, 0);
	       CommandLine.convertStringToHex(strTempString, &s32PGListFormat);
	       strTempString.clear();

	       CommandLine.getParameter(&strTargetName, 1);
	       strTargetName = convertStringToUpperCase(strTargetName);
	       strTempString.clear();

	       if(strTargetName=="SAS" || strTargetName=="NVME")
	       {
		    fstream fout;
		    fout.open("/dev/shm/storage.tmp", ios::out);
		    fout.close();

		    stringstream ssTempI;
		    stringstream ssTempJ;

		    if(strTargetName=="NVME")
		    {
			 int status[0xf];
			 pid_t pid[0xf];

			 for(UINT32 u32I=0; u32I<0xf; u32I++)
			 {
			      ssTempI.str("");
			      ssTempI.clear();
			      ssTempI << u32I;

			      string strTargetNameTemp = strTargetName + ssTempI.str();
			      if((pErrorMsg = m_pStorageDAPI->getIndexByName(strTargetNameTemp, &u32DefinedIndex)) != NO_ERROR_MSG)
			      {
				   continue;
			      }

			      s32Err = 0; // if loop found any HDDs/controller

			      pid[u32I] = fork();
			      if(pid[u32I] < 0) // if fork faild
			      {
				   s32Err = -1;
				   return -1;
			      }

			      if(pid[u32I] == 0)// child process
			      {
				   // get nvme p or g-list data
				   if((pErrorMsg = m_pStorageDAPI->getIndexByName(strTargetNameTemp.c_str(), &u32DefinedIndex)) != NO_ERROR_MSG)
				   {
					s32Err = -1;
					break;
				   }

				   if((pErrorMsg = m_pStorageDAPI->getIndexByName(strTargetNameTemp.c_str(), &u32DefinedIndex)) != NO_ERROR_MSG)
				   {
					cout << strTargetNameTemp << endl;
					printError(pErrorMsg);
					s32Err = -1;
					break;
				   }

				   switch(m_OperationType)
				   {
				   case OPERATION_PLIST:
					if((pErrorMsg =m_pStorageDAPI->getPGListByIndex(u32DefinedIndex, 1, 0, (UINT32)s32PGListFormat, true)) != NO_ERROR_MSG)
					{
					     printError(pErrorMsg);
					     s32Err = -1;
					}
					break;
				   case OPERATION_GLIST:
					if((pErrorMsg =m_pStorageDAPI->getPGListByIndex(u32DefinedIndex, 0, 1, (UINT32)s32PGListFormat, true)) != NO_ERROR_MSG)
					{
					     printError(pErrorMsg);
					     s32Err = -1;
					}
					break;
				   default:
					break;
				   }

				   cout << m_pStorageDAPI->m_strOutputResultString;

				   if(s32Err == 0)
				   {
					cout << "\tRESULT: PASSED" << endl;
				   }
				   else
				   {
					cout << strTargetNameTemp << ": ";
					printError(pErrorMsg);
					fstream fout;
					fout.open("/dev/shm/storage.tmp", ios::out);
					fout << "\tRESULT: FAILED" << endl;
					fout.close();
					cout << "\tRESULT: FAILED" << endl;
				   }
				   exit(0);
			      }
			 }
			 for(UINT32 u32I=0; u32I<0xf; u32I++)
			 {
			      waitpid((pid_t)pid[u32I], &status[u32I], 0);
			 }
		    }
		    else
		    {
			 for(UINT32 u32I=0; u32I<0xf; u32I++)
			 {
			      ssTempI.str("");
			      ssTempI.clear();
			      ssTempI << u32I;
			      int status[0xff];
			      pid_t pid[0xff];

			      for(UINT32 u32J=0; u32J<0xff; u32J++)
			      {
				   ssTempJ.str("");
				   ssTempJ.clear();
				   ssTempJ << u32J;

				   string strTargetNameTemp = strTargetName + ssTempI.str() + "-DRIVE" + ssTempJ.str();
				   if((pErrorMsg = m_pStorageDAPI->getIndexByName(strTargetNameTemp, &u32DefinedIndex)) != NO_ERROR_MSG)
				   {
					continue;
				   }

				   s32Err = 0; // if loop found any HDDs/controller

				   pid[u32J] = fork();
				   if(pid[u32J] < 0) // if fork faild
				   {
					s32Err = -1;
					return -1;
				   }

				   if(pid[u32J] == 0)// child process
				   {
					// run thread performance test
					if((pErrorMsg = m_pStorageDAPI->getIndexByName(strTargetNameTemp.c_str(), &u32DefinedIndex)) != NO_ERROR_MSG)
					{
					     cout << strTargetNameTemp << endl;
					     printError(pErrorMsg);
					     s32Err = -1;
					     break;
					}

					switch(m_OperationType)
					{
					case OPERATION_PLIST:
					     if((pErrorMsg = m_pStorageDAPI->getPGListByIndex(u32DefinedIndex, 1, 0, (UINT32)s32PGListFormat, true)) != NO_ERROR_MSG)
					     {
						  printError(pErrorMsg);
						  s32Err = -1;
					     }
					     break;
					case OPERATION_GLIST:
					     if((pErrorMsg = m_pStorageDAPI->getPGListByIndex(u32DefinedIndex, 0, 1, (UINT32)s32PGListFormat, true)) != NO_ERROR_MSG)
					     {
						  printError(pErrorMsg);
						  s32Err = -1;
					     }
					     break;
					default:
					     break;
					}

					cout << m_pStorageDAPI->m_strOutputResultString;

					if(s32Err == 0)
					{
					     cout << "\tRESULT: PASSED" << endl;
					}
					else
					{
					     fstream fout;
					     fout.open("/dev/shm/storage.tmp", ios::out);
					     fout << "\tRESULT: FAILED" << endl;
					     fout.close();
					     cout << "\tRESULT: FAILED" << endl;
					}
					exit(0);
				   }
			      }
			      for(UINT32 u32J=0; u32J<0xff; u32J++)
			      {
				   waitpid((pid_t)pid[u32J], &status[u32J], 0);
			      }
			 }
		    }
		    fstream fin;
		    fin.open("/dev/shm/storage.tmp", ios::in);
		    string strTempLine = "";
		    while(getline(fin, strTempLine))
		    {
			 if(strTempLine != "")
			 {
			      s32Err = -1;
			 }
		    }
		    fin.close();

		    if(s32Err == -1)// if has error print out
		    {
			 printError(pErrorMsg);
		    }
	       }
	       else
	       {
		    // normal single device
		    CommandLine.getParameter(&strTempString, 0);
		    CommandLine.convertStringToHex(strTempString, &s32PGListFormat);
		    strTempString.clear();

		    CommandLine.getParameter(&strTargetName, 1);
		    strTargetName = convertStringToUpperCase(strTargetName);
		    strTempString.clear();

		    if((pErrorMsg = m_pStorageDAPI->getIndexByName(strTargetName, &u32DefinedIndex)) != NO_ERROR_MSG)
		    {
			 printError(pErrorMsg);
			 s32Err = -1;
			 return s32Err;
		    }

		    switch(m_OperationType)
		    {
		    case OPERATION_PLIST:
			 if((pErrorMsg = m_pStorageDAPI->getPGListByIndex(u32DefinedIndex, 1, 0, (UINT32)s32PGListFormat)) != NO_ERROR_MSG)
			 {
			      printError(pErrorMsg);
			      return -1;
			 }
			 break;
		    case OPERATION_GLIST:
			 if((pErrorMsg = m_pStorageDAPI->getPGListByIndex(u32DefinedIndex, 0, 1, (UINT32)s32PGListFormat)) != NO_ERROR_MSG)
			 {
			      printError(pErrorMsg);
			      return -1;
			 }
			 break;
		    default:
			 break;
		    }
	       }
	       cout << m_pStorageDAPI->m_strOutputResultString << endl;
	       break;
	  default:
	       cout << "Unsupport operation" << endl;
	       s32Err = -1;
	       break;
	  }
     }
     return s32Err;
}

INT32 StorageOperation::checkSyntax(StringParser CommandLine)
{
     INT32  s32CNT, s32aHex[2], s32ParameterNumber = 0;
     string straParameter[7];
     int    s32Err = 0;

     CommandLine.getParameterNumber(&s32ParameterNumber);

     COMMAND_TYPE eCommandType = (COMMAND_TYPE)0;
     CommandLine.getCommandType(&eCommandType);

     switch(m_OperationType)
     {
     case NO_OPRATION:
	  if(s32ParameterNumber > 0)
	  {
	       cout << "Not Necessary Operation Parameter" << endl;
	       helpParameter();
	       return -1;
	  }
	  break;
     case OPERATION_DRIVE:
	  if(s32ParameterNumber != 0)
	  {
	       cout << "Number of parameter != 0" << endl;
	       helpParameter();
	       return -1;
	  }
	  break;
     case OPERATION_ALL:
	  if(eCommandType == CMD_LIST)
	  {
	       if (s32ParameterNumber != 0)
	       {
		    cout << "Number of parameter != 0" << endl;
		    helpParameter();
		    return -1;
	       }
	  }
	  else if(eCommandType == CMD_SHOW)
	  {
	       //Check parameter number
	       if (s32ParameterNumber != 1)
	       {
		    cout << "Parameter numbers isn't 1" << endl;
		    helpParameter();
		    return -1;
	       }
	  }
	  else
	  {
	       cout << "Unknown Command" << endl;
	       helpParameter();
	       return -1;
	  }
	  break;
	  // for show
     case OPERATION_PORT:
     case OPERATION_NODE:
     case OPERATION_SN:
     case OPERATION_MODEL:
     case OPERATION_FW:
     case OPERATION_LOGICAL_BLOCK_SIZE:
     case OPERATION_PHYSICAL_BLOCK_SIZE:
     case OPERATION_ADDRESS:
     case OPERATION_CAPACITY:
     case OPERATION_SPEED:
     case OPERATION_TEMP:
     case OPERATION_SMART:
	  //Check parameter number
	  if (s32ParameterNumber != 1)
	  {
	       cout << "Parameter numbers isn't 1" << endl;
	       helpParameter();
	       return -1;
	  }
	  break;
	  // for test
     case OPERATION_READPFMC:
     case OPERATION_WRITEPFMC:
     case OPERATION_WRC:
	  if (s32ParameterNumber != 3)
	  {
	       cout << "Parameter numbers isn't 3" << endl;
	       helpParameter();
	       return -1;
	  }
	  //Get parameter
	  for(s32CNT=0; s32CNT<s32ParameterNumber; s32CNT++)
	  {
	       CommandLine.getParameter(&straParameter[s32CNT], s32CNT);
	  }
	  //Check parameter 1 is numeric
	  s32Err = CommandLine.convertStringToHex(straParameter[0], &s32aHex[0]);
	  if(s32Err != 0)
	  {
	       cout << "parameter 1 isn't numeric." << endl;
	       helpParameter();
	       return -1;
	  }
	  //Check parameter 2 is numeric
	  s32Err = CommandLine.convertStringToHex(straParameter[1], &s32aHex[1]);
	  if(s32Err != 0)
	  {
	       cout << "parameter 2 isn't numeric." << endl;
	       helpParameter();
	       return -1;
	  }
	  break;
	  // for utility
/*
     case OPERATION_SMART:
	  if(s32ParameterNumber != 1)
	  {
	       cout << "Parameter numbers isn't 1" << endl;
	       helpParameter();
	       return -1;
	  }
	  //Get parameter
	  for(s32CNT=0; s32CNT<s32ParameterNumber; s32CNT++)
	  {
	       CommandLine.getParameter(&straParameter[s32CNT], s32CNT);
	  }

	  //Check parameter 1 is numeric
	  s32Err = CommandLine.convertStringToHex(straParameter[0], &s32aHex[0]);
	  if(s32Err != 0)
	  {
	       cout << "parameter 1 isn't numeric." << endl;
	       helpParameter();
	       return -1;
	  }
	  //Check parameter 2 is numeric
	  s32Err = CommandLine.convertStringToHex(straParameter[1], &s32aHex[1]);
	  if(s32Err != 0)
	  {
	       cout << "parameter 2 isn't numeric." << endl;
	       helpParameter();
	       return -1;
	  }
*/
	  break;
     case OPERATION_WZERO:
     case OPERATION_GLIST:
     case OPERATION_PLIST:	  
	  if(s32ParameterNumber != 2)
	  {
	       cout << "Parameter numbers isn't 2" << endl;
	       helpParameter();
	       return -1;
	  }
	  //Get parameter
	  for(s32CNT=0; s32CNT<s32ParameterNumber; s32CNT++)
	  {
	       CommandLine.getParameter(&straParameter[s32CNT], s32CNT);
	  }
	  //Check parameter 1 is numeric
	  s32Err = CommandLine.convertStringToHex(straParameter[0], &s32aHex[0]);
	  if(s32Err != 0)
	  {
	       cout << "parameter 1 isn't numeric." << endl;
	       helpParameter();
	       return -1;
	  }
	  break;
     default:
	  cout << "Unsupport operation" << endl;
	  return -1;
	  break;
     }
     return 0;
}

void StorageOperation::handleTestProgress(UINT32 in_u32TestSizeMB)
{
     INT32 s32Progress = 0;

     sleep(1);
     while(s32Progress < 100)// <= 100%
     {
	  sleep(1);
	  s32Progress = m_pStorageDAPI->getProgress();
	  printf("Progressing: %3u%% \r", s32Progress);
	  fflush(stdout);
     }
     printf("\n");
}

string StorageOperation::convertStringToUpperCase(string in_strInputString)
{
     string strOutputString = "";
     for(string::iterator stringIterator = in_strInputString.begin(); stringIterator != in_strInputString.end(); stringIterator++)
     {
          strOutputString += toupper(*stringIterator);
     }
     return strOutputString;
}
