#include "TPMOperation.hpp"

TPMOperation::TPMOperation(OPERATION_TYPE   in_eOprationType,
                           string           in_strHelpContent) : OperationBaseClass(in_eOprationType, in_strHelpContent)
{
    m_s32Result = 0;

    m_pTPMDAPI = new TPMDAPI("TPMDAPI");
}

TPMOperation::~TPMOperation()
{
    if(m_pTPMDAPI != NULL)
    {
        delete m_pTPMDAPI;
        m_pTPMDAPI = NULL;
    }
}


int 
TPMOperation::TxRxCmd(TPM_CMD_INPUT     in_cmdList,
                      UINT8*            out_u8pRspBuffer,
                      UINT32            in_u32RspBufferSize,
                      UINT32*           out_u32pRspSize) 
{
    ErrorMsg*   pErrorMsg;
    UINT32      u32TpmRC; //TPM_RC 
    
    pErrorMsg = m_pTPMDAPI->commandInterface(in_cmdList, out_u8pRspBuffer, in_u32RspBufferSize, out_u32pRspSize);
    if(pErrorMsg != NO_ERROR_MSG)
    {
        printError(TPM_DEBUG_LEVEL, pErrorMsg);
        return -1;
    }

    u32TpmRC = (out_u8pRspBuffer[6] << 24) | (out_u8pRspBuffer[7] << 16) | (out_u8pRspBuffer[8] << 8) | out_u8pRspBuffer[9];
    //printf("u32TpmRC: 0x%08x\n", u32TpmRC); //check 
    if(u32TpmRC != TPM_RC_SUCCESS)
    {
        return -1;
    }

    return 0;
}

#if 0
int 
TPMOperation::showFirmwareVersion(TPM_CMD_INPUT   in_cmdList)
{
    UINT8   u8CertifiedByte;
    UINT16  u16Major;
    UINT16  u16Minor;
    UINT32  u32BuildNum;
    UINT32  u32Temp;
    int     s32Result = 0;

    //TPM2_GetCapability Response
    //
    //struct TPM_CAP_DATA 
    //{
    //  UINT32  capability;
    //  UINT32  data
    //}
    //
    //UINT16    TMP_ST
    //UINT32    responseSize
    //UINT32    TPM_RC
    //UINT8     TPMI_YES_NO (1 byte)
    //
    //struct TPMS_CAPABILITY_DATA
    //{
    //  UINT32                  tpmCapValue 
    //  vector<TPM_CAP_DATA>    capDataList
    //}
   
    //response header:
    //  in_u8pRspBuffer[0] - in_u8pRspBuffer[9]
    //
    //response parameter1:
    //  in_u8pRspBuffer[10] (TPMI_YES_NO)
    //
    //response parameter2:
    //  in_u8pRspBuffer[11] - in_u8pRspBuffer[14] (TPM_CAP)
    //  in_u8pRspBuffer[15] - in_u8pRspBuffer[18] (propertyCount)
    //
    //  [capability]data:
    //      in_u8pRspBuffer[19] - in_u8pRspBuffer[22] (Property1 - TPM_PT_FIRMWARE_VERSION_1)
    //      in_u8pRspBuffer[23] - in_u8pRspBuffer[26] (Property1 value)
    //      in_u8pRspBuffer[27] - in_u8pRspBuffer[30] (Property2 - TPM_PT_FIRMWARE_VERSION_2)
    //      in_u8pRspBuffer[31] - in_u8pRspBuffer[34] (Property2 value)

    s32Result = TxRxCmd(in_cmdList, u8RspBuffer, TPM_MAX_RSP_BUFFER_SIZE, &u32RspSize);
    if(s32Result == 0)
    {
        u16Major = (u8RspBuffer[23] << 8) | u8RspBuffer[24];
        u16Minor = (u8RspBuffer[25] << 8) | u8RspBuffer[26];

        u32Temp = (u8RspBuffer[31] << 24) | (u8RspBuffer[32] << 16) | (u8RspBuffer[33] << 8) | u8RspBuffer[34];
        u32BuildNum = (u32Temp & 0xffff00 ) >> 8;
        u8CertifiedByte = u32Temp & 0xff;
        printf("Firmware Version: %d.%d.%d.%d\n", u16Major, u16Minor, u32BuildNum, u8CertifiedByte);
    }
    return s32Result;
}


int  
TPMOperation::showVendor(TPM_CMD_INPUT   in_cmdList)
{
    int     s32Result = 0;
   

    s32Result = TxRxCmd(in_cmdList, m_u8pRspBuffer, TPM_MAX_RSP_BUFFER_SIZE, &m_u32RspSize);
    if(s32Result == 0)
    {

        printf("Vendor:           %c%c%c%c %c%c%c%c%c%c%c%c\n", 
                m_u8pRspBuffer[23], u8pRspBuffer[24], m_u8pRspBuffer[25], m_u8pRspBuffer[26], //TPM_PT_MANUFACTURER 
                m_u8pRspBuffer[31], u8pRspBuffer[32], m_u8pRspBuffer[33], m_u8pRspBuffer[34], //TPM_PT_VENDOR_STRING_1
                m_u8pRspBuffer[39], u8pRspBuffer[40], m_u8pRspBuffer[41], m_u8pRspBuffer[42]);//TPM_PT_VENDOR_STRING_1 
        //TPM_PT_VENDOR_STRING_3 and TPM_PT_VENDOR_STRING_4 are NULL for IFX SLB9665
    }
    return s32Result;
}

int 
TPMOperation::showFamily(TPM_CMD_INPUT   in_cmdList)
{
    int     s32Result = 0;
   

    s32Result = TxRxCmd(in_cmdList, m_u8pRspBuffer, TPM_MAX_RSP_BUFFER_SIZE, &m_u32RspSize);
    if(s32Result == 0)
    {

        printf("Family:           %c%c%c%c\n", 
                m_u8pRspBuffer[23], m_u8pRspBuffer[24], m_u8pRspBuffer[25], m_u8pRspBuffer[26]); 
        //TPM_PT_VENDOR_STRING_3 and TPM_PT_VENDOR_STRING_4 are NULL for IFX SLB9665
    }
    return s32Result;
    
}
#endif 

void 
TPMOperation::showBitState(STRING    in_string,    
                           UINT8     in_u8Data)
{
    char    outputString[TPM_MAX_STRING_LENGTH];
    char    space[TPM_MAX_STRING_LENGTH];
    int     s32SpaceNum;
    int     s32StrLength;
    int     s32Index;

    s32StrLength = strlen(in_string);

    s32SpaceNum = TPM_MAX_STRING_LENGTH - s32StrLength;

    for(s32Index = 0; s32Index < TPM_MAX_STRING_LENGTH; s32Index++)
    {
        space[s32Index] = 0x0;
    }
    for(s32Index = 0; s32Index < s32SpaceNum; s32Index++)
    {
        space[s32Index] = 0x20;
    }
    sprintf(outputString, "%s:%s", in_string, space);
    
    printf("%s %s\n", 
            outputString, 
            in_u8Data ? "SET(1)" : "CLEAR(0)");
}

#if 0
int 
TPMOperation::showStatus(TPM_CMD_INPUT   in_cmdList)
{
    UINT8   u8pRspBuffer[TPM_MAX_RSP_BUFFER_SIZE];
    UINT32  u32RspBufferSize;
    UINT32  u32PermanentBits;
    UINT32  u32StartupClearBits;
    int     s32Result = 0;

    
    s32Result = TxRxCmd(in_cmdList, u8pRspBuffer, &u32RspBufferSize);
    if(s32Result == 0)
    {
        u32PermanentBits = (u8pRspBuffer[23] << 24) | (u8pRspBuffer[24] << 16) | (u8pRspBuffer[25] << 8) | u8pRspBuffer[26];
        u32StartupClearBits = (u8pRspBuffer[31] << 24) | (u8pRspBuffer[32] << 16) | (u8pRspBuffer[33] << 8) | u8pRspBuffer[34];

        //TPM_PT_PERMANENT
        showBitState("Owner Auth Set", u32PermanentBits & 0x1);
        showBitState("Endorsement Auth Set", (u32PermanentBits & 0x2) >> 2);
        showBitState("Lockout Auth Set", (u32PermanentBits & 0x4) >> 2);
        showBitState("Disable Clear", (u32PermanentBits & 0x100) >> 8);
        showBitState("In Lockout", (u32PermanentBits & 0x200) >> 9);
        //TPM_PT_STARTUP_CLEAR
        showBitState("Ph Enable", u32StartupClearBits & 0x1);
        showBitState("Sh Enable", (u32StartupClearBits & 0x2) >> 1);
        showBitState("Eh Enable", (u32StartupClearBits & 0x4) >> 2);
    }
    return s32Result;
}
#endif 


int 
TPMOperation::showAllInfo(TPM_CMD_INPUT  in_cmdList,
                         UINT8*          out_u8pRspBuffer,
                         UINT32          in_u32RspBufferSize,
                         UINT32*         out_u32pRspSize)
{
    int s32Result = 0;
    
    cout << "[Device Information]" << endl;
    s32Result = getSpecificInfo(in_cmdList, OPERATION_FAMILY, out_u8pRspBuffer, in_u32RspBufferSize, out_u32pRspSize);
    if(s32Result != 0)
    {
        return s32Result;
    }
    showData(OPERATION_FAMILY, out_u8pRspBuffer);

    s32Result = getSpecificInfo(in_cmdList, OPERATION_VENDOR, out_u8pRspBuffer, in_u32RspBufferSize, out_u32pRspSize);
    if(s32Result != 0)
    {
        return s32Result;
    }
    showData(OPERATION_VENDOR, out_u8pRspBuffer);
    
    s32Result = getSpecificInfo(in_cmdList, OPERATION_CHIP, out_u8pRspBuffer, in_u32RspBufferSize, out_u32pRspSize);
    if(s32Result != 0)
    {
        return s32Result;
    }
    showData(OPERATION_CHIP, out_u8pRspBuffer);

    s32Result = getSpecificInfo(in_cmdList, OPERATION_FW, out_u8pRspBuffer, in_u32RspBufferSize, out_u32pRspSize);
    if(s32Result != 0)
    {
        return s32Result;
    }
    showData(OPERATION_FW, out_u8pRspBuffer);

    cout << endl;
    cout << "[Status]" << endl;
    s32Result = getSpecificInfo(in_cmdList, OPERATION_STATUS, out_u8pRspBuffer, in_u32RspBufferSize, out_u32pRspSize);
    if(s32Result == 0)
    {
        showData(OPERATION_STATUS, out_u8pRspBuffer);
    }

    return s32Result;
}


void 
TPMOperation::showData(OPERATION_TYPE   in_opType,
                       UINT8*           in_u8pRspBuffer)
{
    UINT32  u32Temp;
    UINT32  u32PermanentBits;
    UINT32  u32StartupClearBits;

    switch(in_opType)
    {
        case OPERATION_FW:
            u32Temp = (in_u8pRspBuffer[31] << 24) | (in_u8pRspBuffer[32] << 16) | (in_u8pRspBuffer[33] << 8) | in_u8pRspBuffer[34];
            printf("Firmware Version: %d.%d.%d.%d\n",
                    (in_u8pRspBuffer[23] << 8) | in_u8pRspBuffer[24], //major number
                    (in_u8pRspBuffer[25] << 8) | in_u8pRspBuffer[26], //minor number 
                    (u32Temp & 0xffff00 ) >> 8, //build number
                    u32Temp & 0xff //certified byte 
                    );
            break;

        case OPERATION_VENDOR:
            printf("Vendor:           %c%c%c%c\n", 
                    in_u8pRspBuffer[23], in_u8pRspBuffer[24], in_u8pRspBuffer[25], in_u8pRspBuffer[26]); 
            break;

        case OPERATION_CHIP:
            printf("Chip:             %c%c%c%c%c%c%c%c\n", 
                    in_u8pRspBuffer[23], in_u8pRspBuffer[24], in_u8pRspBuffer[25], in_u8pRspBuffer[26], //TPM_PT_VENDOR_STRING_1
                    in_u8pRspBuffer[31], in_u8pRspBuffer[32], in_u8pRspBuffer[33], in_u8pRspBuffer[34]);//TPM_PT_VENDOR_STRING_2 
            //TPM_PT_VENDOR_STRING_3 and TPM_PT_VENDOR_STRING_4 are NULL for IFX SLB9665
            break;

        case OPERATION_STATUS:
            u32PermanentBits = (in_u8pRspBuffer[23] << 24) | (in_u8pRspBuffer[24] << 16) | (in_u8pRspBuffer[25] << 8) | in_u8pRspBuffer[26];
            u32StartupClearBits = (in_u8pRspBuffer[31] << 24) | (in_u8pRspBuffer[32] << 16) | (in_u8pRspBuffer[33] << 8) | in_u8pRspBuffer[34];
            //TPM_PT_PERMANENT
            showBitState("Owner Auth Set", u32PermanentBits & 0x1);
            showBitState("Endorsement Auth Set", (u32PermanentBits & 0x2) >> 2);
            showBitState("Lockout Auth Set", (u32PermanentBits & 0x4) >> 2);
            showBitState("Disable Clear", (u32PermanentBits & 0x100) >> 8);
            showBitState("In Lockout", (u32PermanentBits & 0x200) >> 9);
            //TPM_PT_STARTUP_CLEAR
            showBitState("Ph Enable", u32StartupClearBits & 0x1);
            showBitState("Sh Enable", (u32StartupClearBits & 0x2) >> 1);
            showBitState("Eh Enable", (u32StartupClearBits & 0x4) >> 2);
            break;

        case OPERATION_FAMILY:
            printf("Family:           %c%c%c%c\n", 
                    in_u8pRspBuffer[23], in_u8pRspBuffer[24], in_u8pRspBuffer[25], in_u8pRspBuffer[26]); 
            //TPM_PT_VENDOR_STRING_3 and TPM_PT_VENDOR_STRING_4 are NULL for IFX SLB9665
            break;

        default:
            break;
    }
}


int 
TPMOperation::showInfo()
{
    int             s32Result = 0;
    UINT8           u8RspBuffer[TPM_MAX_RSP_BUFFER_SIZE];
    UINT32          u32RspSize;
    TPM_CMD_INPUT   cmdList;

    //define header 
    cmdList.u16Tag = TPM_ST_NO_SESSIONS;
    cmdList.u32CmdSize = 0x0; //left undefined first
    cmdList.u32CmdCode = TPM_CC_GET_CAPABILITY; //TPM_CC
    //define parameter 1
    cmdList.uParm1.u32Capability = TPM_CAP_TPM_PROPERTIES;
   
    if(getOperationType() == OPERATION_ALL)
    {
        cout << endl;
        s32Result = showAllInfo(cmdList, u8RspBuffer, TPM_MAX_RSP_BUFFER_SIZE, &u32RspSize);
    }
    else //show individual item  
    {
        s32Result = getSpecificInfo(cmdList, 
                                    getOperationType(), 
                                    u8RspBuffer, 
                                    TPM_MAX_RSP_BUFFER_SIZE, 
                                    &u32RspSize);
        if(s32Result == 0)
        {
            showData(getOperationType(), u8RspBuffer);        
        }
    }

    return s32Result;
}


int 
TPMOperation::getSpecificInfo(TPM_CMD_INPUT     in_cmdList,
                              OPERATION_TYPE    in_opType,
                              UINT8*            out_u8pRspBuffer,
                              UINT32            in_u32RspBufferSize,
                              UINT32*           out_u32pRspSize)
{
    int  s32Result = 0;

    //initialize
    *out_u32pRspSize = 0;
    memset(out_u8pRspBuffer, 0, TPM_MAX_RSP_BUFFER_SIZE);
    
    switch(in_opType)
    {
        case OPERATION_FW:
            in_cmdList.uParm2.u32Value = TPM_PT_FIRMWARE_VERSION_1; //Property 
            in_cmdList.uParm3.u32Value = 2; //PropertyCount = 2, (from 111 - 112)
            break;
        case OPERATION_VENDOR:
            in_cmdList.uParm2.u32Value = TPM_PT_MANUFACTURER;
            in_cmdList.uParm3.u32Value = 1; //PropertyCount = 1, (only 105)
            break;
        case OPERATION_CHIP:
            in_cmdList.uParm2.u32Value = TPM_PT_VENDOR_STRING_1; 
            in_cmdList.uParm3.u32Value = 4; //PropertyCount = 4, (from 106 - 109)
            break;
        case OPERATION_STATUS:
            in_cmdList.uParm2.u32Value = TPM_PT_PERMANENT; 
            in_cmdList.uParm3.u32Value = 2; //200 - 201
            break;
        case OPERATION_FAMILY:
            in_cmdList.uParm2.u32Value = TPM_PT_FAMILY_INDICATOR; //Property
            in_cmdList.uParm3.u32Value = 1; //PropertyCount = 1
            break;
        default:
            break;
    }
    s32Result = TxRxCmd(in_cmdList, out_u8pRspBuffer, in_u32RspBufferSize, out_u32pRspSize);

    return s32Result;
}


int
TPMOperation::doTpmSelfTest()
{
    int             s32Result = 0;
    UINT8           u8pRspBuffer[TPM_MAX_RSP_BUFFER_SIZE];
    UINT32          u32RspBufferSize;
    TPM_CMD_INPUT   cmdList;

    //define header 
    cmdList.u16Tag = TPM_ST_NO_SESSIONS;
    cmdList.u32CmdSize = 0x0; //left undefined first
    cmdList.u32CmdCode = TPM_CC_SELF_TEST; //TPM_CC

    cmdList.uParm1.u8YesNoData = 1; //SelfTest full 
   
    s32Result = TxRxCmd(cmdList, u8pRspBuffer, TPM_MAX_RSP_BUFFER_SIZE, &u32RspBufferSize);
    
    return s32Result;
}


int
TPMOperation::sendCommandManually()
{
    //int         s32Result = 0;
    //string      strParameter;
    UINT8       u8pRspBuffer[TPM_MAX_RSP_BUFFER_SIZE];
    UINT32      u32RspSize = 0;
    ErrorMsg*   pErrorMsg;
    //UINT32      u32TpmRC;
    UINT32      u32Index;

    /*
    in_classCommandLine.getParameter(&strParameter, 0);
    
    //show help information if need
    if(strParameter == HELP || strParameter == "")
    {
        showHelp();
        return 0;
    }

    s32Result = checkSyntax(in_classCommandLine);
    if(s32Result != 0)
    {
        showHelp();
        return s32Result;
    }
    */

    pErrorMsg = m_pTPMDAPI->sendCommandManually(m_strParameter[0].c_str(), 
                                                u8pRspBuffer, 
                                                TPM_MAX_RSP_BUFFER_SIZE, 
                                                &u32RspSize);
    if(pErrorMsg != NO_ERROR_MSG)
    {
        printError(TPM_DEBUG_LEVEL, pErrorMsg);
        return -1;
    }

    //u32TpmRC = (u8pRspBuffer[6] << 24) | (u8pRspBuffer[7] << 16) | (u8pRspBuffer[8] << 8) | u8pRspBuffer[9];
    //if(u32TpmRC != TPM_RC_SUCCESS)
    //{
    //    cout << "Command run fail" << endl;
    //    return -1;
    //}

    cout << "TPM response:" << endl;
    for(u32Index = 0; u32Index < u32RspSize; u32Index++)
    {
        if(u32Index == 0 || u32Index % 16 == 0)
        {
            printf("%03x: ",u32Index);
        }
        printf("%02x ", u8pRspBuffer[u32Index]);
        if((u32Index + 1) % 16 == 0 && u32Index != u32RspSize - 1)
        {
            printf("\n");
        }
    }
    printf("\n");

    return 0;
}


int
TPMOperation::executeOperation(StringParser in_classCommandLine)
{
    int s32Index;

    m_s32Result = -1;

    //get parameter amount
    in_classCommandLine.getParameterNumber(&m_s32ParameterAmount);
    //get parameter list 
    for(s32Index = 0; s32Index < m_s32ParameterAmount; s32Index++)
    {
        in_classCommandLine.getParameter(&m_strParameter[s32Index], s32Index);
    }

    in_classCommandLine.getParameter(&m_strParameter[0], 0);
    if(m_strParameter[0] == HELP)
    {
        showHelp();
        return 0;
    }

    //check Syntax 
    m_s32Result = checkSyntax(in_classCommandLine);
    if(m_s32Result != 0)
    {
        if(m_s32Result == -1)
        {
            cout << SYNTAX_ERROR_MSG << endl;
        }
        return m_s32Result;
    }
    
    switch(getOperationType())
    {
        case OPERATION_ALL:
        case OPERATION_FW:
        case OPERATION_VENDOR:
        case OPERATION_CHIP:
        case OPERATION_STATUS:
        case OPERATION_FAMILY:
            return showInfo();

        case OPERATION_SELFTEST:
            m_s32Result = doTpmSelfTest();
            break;

        case OPERATION_CMD:
            return sendCommandManually();

        default:
            break;
    }

    return m_s32Result;
}

int 
TPMOperation::checkSyntax(StringParser in_classCommandLine)
{
    int     s32Result = 0;
    char    tpmName[8];

    switch(getOperationType())
    {
        case OPERATION_ALL:
        case OPERATION_FW:
        case OPERATION_VENDOR:
        case OPERATION_STATUS:
        case OPERATION_FAMILY:
        case OPERATION_SELFTEST:
            if(m_s32ParameterAmount != 1)
            {
                s32Result = -1;
            }
            else 
            {
                sprintf(tpmName, "%s", m_strParameter[0].c_str());
                in_classCommandLine.convertToLowcase(tpmName);
                if(strcmp(tpmName, "tpm0") != 0)
                {
                    cout << "Invalid device name." << endl;
                    s32Result = 1;
                }
            }
            break;

        case OPERATION_CMD:
            if(m_s32ParameterAmount != 2)
            {
                s32Result = -1;
            }
            else 
            {
                sprintf(tpmName, "%s", m_strParameter[1].c_str());
                in_classCommandLine.convertToLowcase(tpmName);
                if(strcmp(tpmName, "tpm0") != 0)
                {
                    cout << "Invalid device name." << endl;
                    s32Result = 1;
                }
            }
            break;
        default:
            break;
    }

    return s32Result;
}

void
TPMOperation::showHelp()
{
    string strHelpContent;

    getHelp(&strHelpContent);
    cout << strHelpContent << endl;
    cout << "Parameters required:" << endl;

    switch(getOperationType())
    {
        case OPERATION_ALL:
        case OPERATION_FW:
        case OPERATION_VENDOR:
        case OPERATION_STATUS:
        case OPERATION_FAMILY:
        case OPERATION_SELFTEST:
            cout << "  P1: TPM device name." << endl;
            break;

        case OPERATION_CMD:
            cout << "  P1: cmd_file_name(used to define command), all characters of the file name should be lowercase." << endl;
            cout << "      example of command file:" << endl;
            cout << "             -------------------" << endl;
            cout << "             80 01" << endl;
            cout << "             00 00 00 0B" << endl;
            cout << "             00 00 01 43" << endl;
            cout << "             01" << endl;
            cout << "             -------------------" << endl;
            cout << "             The first three lines define command header." << endl;
            cout << "             Parameters defined begins at line 4" << endl;
            cout << "             Each line must end by \"\\n\" except for the last line." << endl;
            cout << "  P2: TPM device name." << endl;
            cout << endl;
            cout << "  example:" << endl;
            cout << "         lemur tpm utility cmd /mnt/joann/tpm_cmd tpm0" << endl;
            cout << "         ** Users need to define tpm command in file \"tpm_cmd\" first" << endl;
            break;
        default:
            break;
    }
}
