#include "USBCommand.hpp"

USBCommand::USBCommand(COMMAND_TYPE CommandType, string strHelpContent) : CommandBaseClass(CommandType, strHelpContent)
{
    //Base on command type to add different operations
    switch (CommandType)
    {
        case CMD_LIST:
            addOperation(new USBOperation(NO_OPRATION, "Not Necessary Parameter."));
            break;
        case CMD_SHOW:
            addOperation(new USBOperation(OPERATION_ALL, "Display USB All Information."));
            addOperation(new USBOperation(OPERATION_NODE, "Display USB Node Name on OS."));
            addOperation(new USBOperation(OPERATION_SN, "Display USB Storage Serial Number."));
            addOperation(new USBOperation(OPERATION_VENDOR, "Display USB Storage Manufacturer."));
            addOperation(new USBOperation(OPERATION_CAPACITY, "Display USB Capacity (size)."));
            addOperation(new USBOperation(OPERATION_LINK, "Display USB Link Status"));
            addOperation(new USBOperation(OPERATION_SPEED, "Display USB Storage Speed."));
            addOperation(new USBOperation(OPERATION_PHYSICAL_BLOCK_SIZE, "Display USB Physical Block Size."));
            addOperation(new USBOperation(OPERATION_LOGICAL_BLOCK_SIZE, "Display USB Logical Block Size."));
            addOperation(new USBOperation(OPERATION_SG_NODE, "Display USB SG Device Node Name."));
            break;
        case CMD_TEST:
            addOperation(new USBOperation(OPERATION_READPFMC, "USB Read Performance Test"));
            addOperation(new USBOperation(OPERATION_WRITEPFMC, "USB Write Performance Test"));
            addOperation(new USBOperation(OPERATION_WRC, "USB Read/Write Test"));
            break;
        //Add NEW command and operation here
        default:
            break;
    }
}

USBCommand::~USBCommand()
{

}

int
USBCommand::executeCommand(StringParser CommandLine)
{
    INT32 err = 0;
    INT32 OperationIndex;
    COMMAND_TYPE CommandType;
    string strParserOperationName;
    string OperationName;
    
    CommandLine.getOperation(&strParserOperationName);
    CommandLine.getCommandType(&CommandType);

    switch (CommandType)
    {
        case CMD_LIST:
            if ( strParserOperationName != "")
            {
                cout << "Not Necessary Operation Parameter." << endl;
               return -1;
            }
            break;
        case CMD_SHOW:
        case CMD_TEST:
            if (strParserOperationName == "help" || strParserOperationName == "")
            {
                listOperation(CommandLine);
                return -1;
            }
            break;
        default:
            cout << "Not supported command" << endl;
            return -1;
            break;
    }

    for (OperationIndex = 0; OperationIndex< m_operation_count; OperationIndex++)
    {
        m_operation[OperationIndex]->getName(&OperationName);
        //printf("m_operation[%d]=%s\n", OperationIndex, OperationName.c_str());
        if (OperationName == strParserOperationName)
        {
            err = m_operation[OperationIndex]->executeOperation(CommandLine);
            break;
        }

    }

    if (OperationIndex == m_operation_count)
    {
        cout << strParserOperationName << " operation not found" << endl;
        return -1;
    }

    return err;
}

