/****************************************************************************
*                                                                           *
* Copyright 2004-2011 NetApp Inc. All rights reserved. This file is         *
* confidential and a trade secret of NetApp Inc. The receipt or             *
* possession of this file does not convey any rights to reproduce or        *
* disclose its contents or to manufacture, use, or sell anything it may     *
* describe, in whole, or in part, without the specific written consent of   *
* NetApp Inc.                                                               *
*                                                                           *
****************************************************************************/
/****************************************************************************
*                                                                           *
* NAME            odmdefine.c                                               *
* SUMMARY                                                                   *
* VERSION         %version: %                                               *
* UPDATE DATE     %date_modified: %                                         *
* PROGRAMMER      %created_by:    Merck Hung %                              *
*                                                                           *
*        Copyright 2002-2011 NetApp Inc. All Rights Reserved.               *
*                                                                           *
* DESCRIPTION:                                                              *
*                                                                           *
****************************************************************************/

#include <linux/module.h>

#include <scsi/scsi.h>
#include <scsi/scsi_cmnd.h>
#include <scsi/scsi_dbg.h>
#include <scsi/scsi_device.h>
#include <scsi/scsi_driver.h>
#include <scsi/scsi_eh.h>
#include <scsi/scsi_host.h>
#include <scsi/scsi_ioctl.h>
#include <scsi/scsicam.h>

#include "odmdefine.h"

static unsigned int sas_disk_cnt[ MAX_SAS_HOSTS ] = { 0, 0, 0, 0, 0, 0, 0, 0, 0 };
static unsigned int ram_disk_cnt[ MAX_SAS_HOSTS ] = { 0, 0, 0, 0, 0, 0, 0, 0, 0 };
static unsigned int sata_disk_cnt = 0;
static unsigned int usb_disk_cnt = 0;
static unsigned int other_disk_cnt = 0;
static unsigned int fc_ram_disk_cnt = 0;
static unsigned int sas_init_flag = 0;
static unsigned int sas_idx_offset = 0;

static hostNoMatch_t sasHostNoMatchTbl[] = {
    { SCSI_HOST_SAS_0,    DISK_NAME_SAS_0,    &(sas_disk_cnt[ 0 ])    },
    { SCSI_HOST_SAS_1,    DISK_NAME_SAS_1,    &(sas_disk_cnt[ 1 ])    },
    { SCSI_HOST_SAS_2,    DISK_NAME_SAS_2,    &(sas_disk_cnt[ 2 ])    },
    { SCSI_HOST_SAS_3,    DISK_NAME_SAS_3,    &(sas_disk_cnt[ 3 ])    },
    { SCSI_HOST_SAS_4,    DISK_NAME_SAS_4,    &(sas_disk_cnt[ 4 ])    },
    { SCSI_HOST_SAS_5,    DISK_NAME_SAS_5,    &(sas_disk_cnt[ 5 ])    },
    { SCSI_HOST_SAS_6,    DISK_NAME_SAS_6,    &(sas_disk_cnt[ 6 ])    },
    { SCSI_HOST_SAS_7,    DISK_NAME_SAS_7,    &(sas_disk_cnt[ 7 ])    },
    { SCSI_HOST_SAS_8,    DISK_NAME_SAS_8,    &(sas_disk_cnt[ 8 ])    },
};

static hostNoMatch_t sataHostNoMatchTbl[] = {
    { SCSI_HOST_SATA_0,  DISK_NAME_SATA_0,   &sata_disk_cnt          },
    { SCSI_HOST_SATA_1,  DISK_NAME_SATA_1,   &sata_disk_cnt          },
};

static hostNoMatch_t usbHostNoMatchTbl[] = {
    { SCSI_HOST_USB,     DISK_NAME_USB,      &usb_disk_cnt           },
};

static hostNoMatch_t *hostNoMatchTbl[] = { sataHostNoMatchTbl, usbHostNoMatchTbl, sasHostNoMatchTbl };

static char *ramDiskNameWithHic[] = {  // depend on SAS controller host NO. in sasHostNoMatchTbl
    DISK_NAME_RAM_0,
    DISK_NAME_RAM_1,
    DISK_NAME_RAM_2,
    DISK_NAME_RAM_3,
    DISK_NAME_RAM_4,
    DISK_NAME_RAM_5,
    DISK_NAME_RAM_6,
    DISK_NAME_RAM_7,
    DISK_NAME_RAM_8    
};


int lookupODMDefinedName( struct scsi_device *sdp, char **name, int **cnt ) {

	int i;
	char **pRamdiskName;
	hostNoMatch_t *pHostNoMatch;

	pRamdiskName = ramDiskNameWithHic;

    if( !strncmp( sdp->vendor, FC_RAMDISK_VENDOR, strlen( FC_RAMDISK_VENDOR ) ) )
    {
        *name = DISK_NAME_FC;
        *cnt = &fc_ram_disk_cnt;
        return 0;
    }
    else if( strcmp(sdp->host->hostt->proc_name, SAS_PROC_NAME) == 0 )  // sas HDD entry
	{
        pHostNoMatch = hostNoMatchTbl[SCSI_SAS_TYPE];

        for( i = 0 ; i < MAX_SAS_HOSTS ; i++ )
        {
            if( sdp->host->unique_id == (pHostNoMatch + i)->hostNo )
            {
                if( !strncmp( sdp->vendor, RAMDISK_VENDOR, strlen( RAMDISK_VENDOR ) ) )
                {
                    *name = pRamdiskName[i];
                    *cnt = &(ram_disk_cnt[i]);
                }
                else
                {
                    *name = (pHostNoMatch + i)->prefixName;
                    *cnt = (pHostNoMatch + i)->counter;
                }
                break;
            }
        }
	}
	else if( strcmp(sdp->host->hostt->proc_name, SATA_PROC_NAME) == 0 )  // sata HDD entry
	{
	    pHostNoMatch = hostNoMatchTbl[SCSI_SATA_TYPE];

        for( i = 0 ; i < MAX_SATA_HOSTS ; i++ )
        {
            if( sdp->host->unique_id == (pHostNoMatch + i)->hostNo )
            {
                *name = (pHostNoMatch + i)->prefixName;
                *cnt = (pHostNoMatch + i)->counter;
                break;
            }
        }

        if( i >= MAX_SATA_HOSTS )
        {
            // Not found
            *name = DISK_NAME_OTHER;
            *cnt = &other_disk_cnt;
        }
	}
	else if( strcmp(sdp->host->hostt->proc_name, USB_PROC_NAME) == 0 )   // usb HDD entry
	{
	    pHostNoMatch = hostNoMatchTbl[SCSI_USB_TYPE];

        for( i = 0 ; i < MAX_USB_HOSTS ; i++ )
        {
            if( sdp->host->unique_id == (pHostNoMatch + i)->hostNo )
            {
                *name = (pHostNoMatch + i)->prefixName;
                *cnt = (pHostNoMatch + i)->counter;
                break;
            }
        }

        if( i >= MAX_USB_HOSTS )
        {
            // Not found
            *name = DISK_NAME_OTHER;
            *cnt = &other_disk_cnt;
        }
	}
	else
	{
	    // Not found
	    *name = DISK_NAME_OTHER;
	    *cnt = &other_disk_cnt;
	}

	return 0;
}


int odmFormatDiskName( char *prefix, int index, char *buf, int buflen ) {

    // Arapaho: workaround for SAS devices' index come after SATA/USB,
    //          which won't start from 0.
    if(!strncmp(prefix, "sas", 3))
    {
        if(!sas_init_flag)
        {
            sas_idx_offset = index;
            sas_init_flag = 1;
            printk("SAS flag set, index is %d\n", sas_idx_offset);
        }
        index -= sas_idx_offset;
    }
    snprintf( buf, buflen, "%s%d", prefix, index );
    return 0;
}

