/// @file       ErrorMsg.hpp
/// @brief      ErrorMsg class
/// @details    Provide base class for error message handling
/// @author     KenHu
/// @copyright  Copyright 2014 Hon Hai Technology Group. All Rights Reserved.

#ifndef ERROR_MSG_HPP
#define ERROR_MSG_HPP

#include <string>
#include "common.hpp"

using::std::string;

/// @def NO_ERROR_MSG        NULL
///      This definition means that there is no error occurs if API's return is a NULL pointer
#define NO_ERROR_MSG        NULL    

/// @class      ErrorMsg ErrorMsg.hpp "ErrorMsg.hpp"
/// @brief      Using link list structure to handle 
///             error code with error message
/// @param      m_u32ErrorCode is a 32bits error code
/// @param      m_strMessage is a string for error message
/// @param      m_pPrevious is a pointer to previous ErrorMsg for current ErrorMsg
/// @param      m_pNext is a pointer to next ErrorMsg for current ErrorMsg
class ErrorMsg
{
    public:
                            ErrorMsg(UINT32         in_u32ErrorCode, 
                                     const CHAR*    in_szMessage);                               
                            ErrorMsg(UINT32         in_u32ErrorCode, 
                                     const string   in_strMessage);
        virtual             ~ErrorMsg(void);

        void                appendToList(ErrorMsg* in_pNextErrorMsg);
        UINT32              getErrorCode(void);
        string              getDetailMessage(void);
        ErrorMsg*           getPreviousErrorMsg(void);
        ErrorMsg*           getNextErrorMsg(void);
        BOOL                isListHead(void);           
    protected:

    private:
        UINT32              m_u32ErrorCode;
        string              m_strMessage;
        ErrorMsg*           m_pPrevious;
        ErrorMsg*           m_pNext;
};

#endif
