#ifndef CPU_StabilityTest_H
#define CPU_StabilityTest_H

#include "BaseClassDAPI.hpp"
#include "error_code.hpp"
#include <cstddef>
#include <gmp.h>
#include <pthread.h>
#include <stdbool.h>
#include <stdlib.h>
#include <string.h>
#include <sys/timeb.h>
#include "ErrorMsg.hpp"

#define MAX_THREADS 64
#define PRINT(fmt, str) printf(fmt,str) //Debug mode, show process result
//#define PRINT(fmt, str)

typedef void *st_threadfunc_t;
typedef void *st_threadfuncarg_t;

typedef pthread_t st_thread_t;
typedef int st_threadret_t;

st_threadfunc_t borwein_initialize(st_threadfuncarg_t idx);
st_threadfunc_t borwein_calculate(st_threadfuncarg_t idx);
st_threadfunc_t borwein_finalize(st_threadfuncarg_t idx);
st_threadret_t st_thread_create(st_thread_t * th, st_threadfunc_t (*start_routine) (st_threadfuncarg_t), st_threadfuncarg_t arg);
void st_thread_join(int nthreads, st_thread_t * threads, st_threadret_t * rthreads);


typedef struct borwein_data
{
  mpf_t x1, x2, p1, p2, w1, w2, sqrx, sqr1x, x2plus1, w1plus1, x2divw1;
} bdata;



class StabilityTestObject : public BaseClassDAPI
{
public:
                            StabilityTestObject(STRING className);
            virtual         ~StabilityTestObject(void);
                 ErrorMsg*  PiCalculate(INT32 *test_result, int display, int loop);
protected:

private:
                 void       outxdigits(unsigned long long xdigits, char *str);
                 void       outtime(unsigned long long start, unsigned long long stop, char *str);
                 int        validate_checksum(int prec, char *pistr);
                 unsigned   pitimer();
                 ErrorMsg*  check_system_memory_available(int loop);

                 ErrorMsg*  m_errorCode;
                 int pass, errors, chksum_err, mode, first_error, start, stop;
                 int threads, algo;
                 unsigned long long xdigits, iter;
};

#endif

