#include "TemperatureDAPI.hpp"

TemperatureDAPI::TemperatureDAPI(STRING className) : BaseClassDAPI(className)
{

    m_errorCode =  getErrorMsg();
    if (m_errorCode != NO_ERROR_MSG)
    {
        addErrorMsg(ERROR_TEMPERATURE_DAPI_CONSTRUCTOR, NULL, m_errorCode);
    }

    if ((m_TemperatureDiver = new TemperatureDDAPI("Temperature_drv")) == NULL)
    {
        addErrorMsg( (ERRORCODE)(ERROR_TEMPERATURE_DAPI_CONSTRUCTOR | ERROR_DIAGNOSTIC_API_CONSTRUCTOR1_ERROR), "Create Temperature_DDAPI fail");
    }


   
}



TemperatureDAPI::~TemperatureDAPI()
{

    if (m_TemperatureDiver != NULL)
        delete m_TemperatureDiver;

}


ErrorMsg * TemperatureDAPI::displayTemperatureNameList(list<string> *pTemperatureList)
{
    //printf("%s\n", __func__);
	ErrorMsg*   pErrorMsg = NO_ERROR_MSG;

	pErrorMsg = m_TemperatureDiver->getTemperatureNameList(pTemperatureList);
    if (pErrorMsg != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_TEMPERATURE_DAPI_DISPLAY_TEMPERATURE_LIST, NULL, pErrorMsg);
    }

    return NO_ERROR_MSG;

}



ErrorMsg*   TemperatureDAPI::getDataByName(STRING WellKnownName,
                                           TEMPERATURE_DATA_ELEMENT TemperatureDataElement,
                                           list<string> *returnDataInfo)
{
	ErrorMsg*   pErrorMsg = NO_ERROR_MSG;
    switch(TemperatureDataElement)
    {
            case TEMPERATURE_DATA_ALL_INFO:
            	pErrorMsg = getTemperatureDataAllInfo(WellKnownName, returnDataInfo);
                if (pErrorMsg != NO_ERROR_MSG)
                {
                    return returnErrorMsg(ERROR_TEMPERATURE_DAPI_GET_DATA_BY_NAME, NULL, pErrorMsg);
                }
                break;
            case TEMPERATURE_DATA_CURRENT_TEMPERATURE:
            case TEMPERATURE_DATA_THRESHOLD_HIGH0:
            case TEMPERATURE_DATA_THRESHOLD_HIGH1:
            case TEMPERATURE_DATA_THRESHOLD_LOW0:
            case TEMPERATURE_DATA_THRESHOLD_LOW1:
            	pErrorMsg = m_TemperatureDiver->getDataByName(WellKnownName, TemperatureDataElement, returnDataInfo);
                if (pErrorMsg != NO_ERROR_MSG)
                {
                    return returnErrorMsg(ERROR_TEMPERATURE_DAPI_GET_DATA_BY_NAME, NULL, pErrorMsg);
                }
                break;
            default:
            	return returnErrorMsg(ERROR_TEMPERATURE_DAPI_GET_DATA_BY_NAME, NULL, m_errorCode);
                break;
    }
    return NO_ERROR_MSG;
}

ErrorMsg*   TemperatureDAPI::setDataByName(STRING WellKnownName,
										   TEMPERATURE_DATA_ELEMENT TemperatureDataElement,
										   string SetValue)
{
    switch(TemperatureDataElement)
    {
            case TEMPERATURE_DATA_ALL_INFO:
                //TemperatureDataElement=TEMPERATURE_DATA_ALL_INFO;
                break;
            case TEMPERATURE_DATA_CURRENT_TEMPERATURE:
            	break;
            case TEMPERATURE_DATA_THRESHOLD_HIGH0:
            case TEMPERATURE_DATA_THRESHOLD_HIGH1:
            case TEMPERATURE_DATA_THRESHOLD_LOW0:
            case TEMPERATURE_DATA_THRESHOLD_LOW1:
                 m_errorCode = m_TemperatureDiver->setDataByName(WellKnownName, TemperatureDataElement, SetValue);
                if (m_errorCode != NO_ERROR_MSG)
                {
                    return returnErrorMsg(ERROR_TEMPERATURE_DAPI_SET_DATA_BY_NAME, NULL, m_errorCode);
                }
                break;
            default:
            	return returnErrorMsg(ERROR_TEMPERATURE_DAPI_SET_DATA_BY_NAME, NULL, m_errorCode);
                break;
    }

    return NO_ERROR_MSG;
}

ErrorMsg* TemperatureDAPI::getTemperatureDataAllInfo(STRING WellKnownName, list<string> *returnDataInfo)
{
	list<string> resultTempInfo;
	string tempString;
	tempString.clear();
	ErrorMsg*   pErrorMsg = NO_ERROR_MSG;

	for (UINT32 index = 1; index < TEMPERATURE_DATA_ELEMENT_MAX_NUMBER; index++)
	{
		TEMPERATURE_DATA_ELEMENT DataElementCount = TEMPERATURE_DATA_ELEMENT(index);

		pErrorMsg = m_TemperatureDiver->getDataByName(WellKnownName, DataElementCount, &resultTempInfo);
		if (pErrorMsg != NO_ERROR_MSG)
		{
			return returnErrorMsg(ERROR_TEMPERATURE_DAPI_GET_DATA_BY_NAME, NULL, pErrorMsg);
		}
	}

	for (UINT32 index = 1; index < TEMPERATURE_DATA_ELEMENT_MAX_NUMBER; index++)
	{
		TEMPERATURE_DATA_ELEMENT DataElementCount = TEMPERATURE_DATA_ELEMENT(index);

		switch(DataElementCount)
		{
				//case TEMPERATURE_DATA_ALL_INFO:
				//	break;
				case TEMPERATURE_DATA_CURRENT_TEMPERATURE:
					tempString = "temperature: ";
					break;
				case TEMPERATURE_DATA_THRESHOLD_HIGH0:
					tempString = "threshold_hi0: ";
					break;
				case TEMPERATURE_DATA_THRESHOLD_LOW0:
					tempString = "threshold_low0: ";
					break;
				case TEMPERATURE_DATA_THRESHOLD_HIGH1:
					tempString = "threshold_hi1: ";
					break;
				case TEMPERATURE_DATA_THRESHOLD_LOW1:
					tempString = "threshold_low1: ";
					break;
				default:
					//return -1;
					break;
		}
		tempString.append(resultTempInfo.front());
		resultTempInfo.pop_front();
		returnDataInfo->push_back(tempString);
	}



    return NO_ERROR_MSG;
}





