#include "CHELSIS_T62100.hpp"



CHELSIS_T62100::CHELSIS_T62100(STRING nic_name):EthernetBaseClass(nic_name)
{
   Append_Node("/device/resource0");
   //printf("Chelsis T62100: %s\n",Get_Node().c_str());
   m_errorCode = NO_ERROR_MSG;
}



CHELSIS_T62100::~CHELSIS_T62100()
{

}



ErrorMsg* CHELSIS_T62100::Set_external_loopback()
{
    return NO_ERROR_MSG;
}



ErrorMsg* CHELSIS_T62100::Remove_external_loopback()
{
    return NO_ERROR_MSG;
}



int CHELSIS_T62100::read_PHY_reg(int reg, int page)
{
    return *MDIO_point & 0x0FFFF;
}



int CHELSIS_T62100::write_PHY_reg(int reg, int page , int data)
{
    return 0;
}



ErrorMsg* CHELSIS_T62100::Get_self_test_result(struct ethtool_test *test, TestType index, INT32 *test_result)
{
    return NO_ERROR_MSG;
}



ErrorMsg* CHELSIS_T62100::SetupSpeed(string nic, INT32 speed)
{
    char command_string[50];

    sprintf(command_string, "ifconfig %s mtu 9600\n", nic.c_str());
    system(command_string);

    return NO_ERROR_MSG;
}



ErrorMsg* CHELSIS_T62100::GetMAC(string ethernet_name, string *mac_address)
{

    return NO_ERROR_MSG;
}



ErrorMsg* CHELSIS_T62100::SetMAC(string ethernet_name, string *mac_address)
{
    return NO_ERROR_MSG;
}



ErrorMsg* CHELSIS_T62100::GetEEPROM(string ethernet_name, INT32 StartAddress, INT32 Length, INT32 *data)
{
    struct ethtool_eeprom *eeprom;
    struct ifreq          ifr;
           INT32          err, fd, i;

    errno = 0;

    if (ethernet_name.length() == 0)
    {
        return returnErrorMsg( (ERRORCODE)(ERROR_ETHERNET_DDAPI_GET_EEPROM | ERROR_ETHERNET_DDAPI_NO_SPECIFY_NIC), NULL, NULL);
    }

    fd = socket(AF_INET, SOCK_DGRAM, 0);
    if (fd < 0)
    {
        return returnErrorMsg( (ERRORCODE)(ERROR_ETHERNET_DDAPI_GET_EEPROM | ERROR_ETHERNET_DDAPI_SOCKET_OPEN_FAIL), NULL, NULL);
    }

    strncpy(ifr.ifr_name, ethernet_name.c_str(), sizeof(ifr.ifr_name));

    eeprom = (ethtool_eeprom *)calloc(1, sizeof(*eeprom) + Length);
    if (!eeprom)
    {
        free(eeprom);
        close(fd);
        return returnErrorMsg( (ERRORCODE)(ERROR_ETHERNET_DDAPI_GET_EEPROM | ERROR_ETHERNET_DDAPI_ALLOCAT_EEPROM_FAIL), NULL, NULL);
    }

    eeprom->cmd    = ETHTOOL_GEEPROM;
    eeprom->len    = Length;
    eeprom->offset = StartAddress;

    ifr.ifr_data = (__caddr_t)eeprom;

    err = ioctl(fd, SIOCETHTOOL, &ifr);
    if (err < 0)
    {
        //perror("Cannot do get EEPROM");
        free(eeprom);
        close(fd);
        return returnErrorMsg( (ERRORCODE)(ERROR_ETHERNET_DDAPI_GET_EEPROM | ERROR_ETHERNET_DDAPI_IOCTL_FAIL), strerror(errno), NULL);
    }

    for (i = 0; i < eeprom->len; i++)
    {
        data[i] = eeprom->data[i];
    }

    free(eeprom);
    close(fd);

    return NO_ERROR_MSG;
}



ErrorMsg* CHELSIS_T62100::SetEEPROM(string ethernet_name, INT32 StartAddress, INT32 *data, INT32 Length)
{
/*    int     domain, bus, device, func, result = 0;
    string  command;
    char    parameter[512];
    int     word_address = 0, remainder = 0, read_byte_data[2], word_data = 0, i;

    //Get XL710 PCI BDF
    m_errorCode = GetPCIBusDeviceFunction(ethernet_name, &domain, &bus, &device, &func);
    if (m_errorCode != NO_ERROR_MSG)
    {
        return returnErrorMsg((ERRORCODE)(ERROR_ETHERNET_DAPI_SETUP_SPEED), "XL710 get PCI BDF fail", m_errorCode);
    }

    for(i = 0; i < Length; i++)
    {
        //Transform address to wrod formate
        word_address = (StartAddress + i) / 2;
        remainder = (StartAddress + i) % 2;
        //printf("word_address:0x%02x   remainder:%d\n", word_address, remainder);

        //Get the word data
        GetEEPROM(ethernet_name, word_address * 2, 2, read_byte_data);
        //printf("read_byte_data[0]:0x%02x  read_byte_data[1]:0x%02x\n", read_byte_data[0], read_byte_data[1]);
        word_data = (read_byte_data[1] << 8) | read_byte_data[0];
        //printf("word_data:0x%04x\n", word_data);

        if(remainder == 0)
        {   //Write low byte
            word_data = word_data & 0xff00;
            word_data = word_data | data[i];
        }
        else
        {   //Write high byte
            word_data = word_data & 0x00ff;
            word_data = word_data | (data[i] << 8);
        }
        //printf("Modify word_data:0x%04x\n", word_data);

        //Ready to write word
        command.clear();
        command = "eeupdate64e";
        sprintf(parameter, " /bus=%d /dev=%d /fun=%d /WW 0x%x 0x%x >> /dev/null",bus, device, func, word_address, word_data);
        command.append(parameter);
        //printf("%s\n", command.c_str());
        result = system(command.c_str());
        if (result != 0)
        {
            return returnErrorMsg((ERRORCODE)(ERROR_ETHERNET_DRIVER_XL710_SET_EEPROM), "XL710 set EEPROM fail", m_errorCode);
        }
    }
//
    INT32  fd, err;
    struct ethtool_eeprom *eeprom;
    struct ifreq ifr;
    INT32  magic;
    size_t if_name_len = strlen(ethernet_name.c_str());

    errno = 0;

    if (if_name_len == 0)
    {
        return returnErrorMsg( (ERRORCODE)(ERROR_ETHERNET_DDAPI_SET_EEPROM | ERROR_ETHERNET_DDAPI_NO_SPECIFY_NIC), NULL, NULL);
    }


    fd = socket(AF_INET,SOCK_DGRAM,0);
    if (fd < 0)
    {
        return returnErrorMsg( (ERRORCODE)(ERROR_ETHERNET_DDAPI_SET_EEPROM | ERROR_ETHERNET_DDAPI_SOCKET_OPEN_FAIL), NULL, NULL);
    }

    strncpy(ifr.ifr_name, ethernet_name.c_str(), sizeof(ifr.ifr_name));

    eeprom = (ethtool_eeprom *)calloc(1, sizeof(*eeprom) + 1);
    if (!eeprom)
    {
        close(fd);
        return returnErrorMsg( (ERRORCODE)(ERROR_ETHERNET_DDAPI_SET_EEPROM | ERROR_ETHERNET_DDAPI_ALLOCAT_EEPROM_FAIL), NULL, NULL);
    }

    m_errorCode = GetEthernetMagic(ethernet_name.c_str(), &magic);

    if (m_errorCode != NO_ERROR_MSG)
    {
        return returnErrorMsg( (ERRORCODE)(ERROR_ETHERNET_DDAPI_SET_EEPROM), NULL, m_errorCode);
    }
    //printf("Magic: 0x%x\n",magic);

    eeprom->cmd = ETHTOOL_SEEPROM;
    eeprom->len = 1;
    eeprom->offset = StartAddress;
    eeprom->magic = magic; //Intel device_id:vend_id
    eeprom->data[0] = data;

    ifr.ifr_data = (__caddr_t)eeprom;

    err = ioctl(fd, SIOCETHTOOL, &ifr);
    if (err < 0)
    {
        close(fd);
        free(eeprom);
        return returnErrorMsg( (ERRORCODE)(ERROR_ETHERNET_DDAPI_SET_EEPROM | ERROR_ETHERNET_DDAPI_IOCTL_FAIL), strerror(errno), NULL);
    }

    close(fd);
    free(eeprom);
*/
    return NO_ERROR_MSG;
}



ErrorMsg* CHELSIS_T62100::Program(string ethernet_name, string filePath)
{
	  int     find = 0, result = 0;
    char    parameter[512];
    string  fileName;

    find = filePath.rfind("/");
    fileName = filePath.substr(find + 1);

    sprintf(parameter, "cp %s /lib/firmware/cxgb4", filePath.c_str());
    //printf("%s\n", parameter);
    result = system(parameter);
    if (result != 0)
    {
        return returnErrorMsg((ERRORCODE)(ERROR_ETHERNET_DAPI_SETUP_SPEED), "Chelsio T62100 cp file to /lib/firmware/cxgb4 fail");
    }

    sprintf(parameter, "cd /lib/firmware/; ethtool -f %s cxgb4/%s", ethernet_name.c_str(), fileName.c_str());
    //printf("%s\n", parameter);
    result = system(parameter);
    if (result != 0)
    {
        return returnErrorMsg((ERRORCODE)(ERROR_ETHERNET_DAPI_SETUP_SPEED), "Chelsio T62100 program fail");
    }

    return NO_ERROR_MSG;
}



ErrorMsg* CHELSIS_T62100::ReadSFP(string ethernet_name, INT32 DeviceAddress, INT32 StartAddress, INT32 Length, INT32 *data)
{
    return NO_ERROR_MSG;
}



ErrorMsg* CHELSIS_T62100::WriteSFP(string ethernet_name, INT32 DeviceAddress, INT32 StartAddress, INT32 data)
{
    return NO_ERROR_MSG;
}



ErrorMsg* CHELSIS_T62100::getLEDName(string nic, list<string> *LED)
{
    return NO_ERROR_MSG;
}



ErrorMsg* CHELSIS_T62100::setLEDStatus(string ethernet_name, string in_strLEDName, string in_strStatus)
{
    return NO_ERROR_MSG;
}



ErrorMsg* CHELSIS_T62100::getLEDStatus(string ethernet_name, string in_strLEDName, string *out_strpStatus)
{
    return NO_ERROR_MSG;
}


