#ifndef EthernetDDAPI_H
#define EthernetDDAPI_H

#include "BaseClassDDAPI.hpp"
#include <sys/socket.h>
#include <arpa/inet.h>
#include <stddef.h>
#include <stdlib.h>
#include <errno.h>
#include "EthernetBaseClass.hpp"
#include <iostream>
#include <fstream>
#include <string>
#include "INTEL_I210.hpp"
#include "INTEL_X552.hpp"
#include "INTEL_XL710.hpp"
#include "CHELSIS_T62100.hpp"
#include <linux/if_packet.h>
#include <linux/if_ether.h>
#include <sys/time.h>
#include <list>
#include <dirent.h>
#include <algorithm>
#include "PCIDDAPI.hpp"

using namespace std;
#define ETHTOOL_GDRVINFO   0x00000003 /* Get driver info. */
#define ETHTOOL_GEEPROM    0x0000000b /* Get EEPROM data */
#define ETHTOOL_SEEPROM    0x0000000c /* Set EEPROM data. */

#define ETHTOOL_GSSET_INFO 0x00000037 /* Get string set info */
#define EOPNOTSUPP         95         /* Operation not supported on transport endpoint */
#define ETHTOOL_GDRVINFO   0x00000003 /* Get driver info. */
#define SIOCETHTOOL        0x8946
#define ETHTOOL_TEST       0x0000001a /* execute NIC self-test. */

struct ethernet_data
{
    string name;
    string BDF;
    string controller;
    string ip;
    string driver;
};



class EthernetDDAPI : public EthernetBaseClass
{
public:
                                EthernetDDAPI(STRING className);
            virtual             ~EthernetDDAPI(void);

//                    ERRORCODE   RegRead(void);
//                    ERRORCODE   RegWrite(void);
                    ErrorMsg*   GetEthernetMAC(string ethernet_name, string *mac_address);
                    ErrorMsg*   GetEthernetNamesList(list<struct ethernet_data>*  ethernet_name_list);
                    CHAR *      LTrim(CHAR *string, CHAR junk);
                    ErrorMsg*   GetEthernetIP(CHAR *ethernet_name, UCHAR *ip);
                    ErrorMsg*   GetEthernetMask(CHAR *ethernet_name, UCHAR *mask);
                    ErrorMsg*   GetEthernetMTU(CHAR *ethernet_name, INT32 *mtu);
                    ErrorMsg*   GetEthernetEEPROM(CHAR *ethernet_name, UCHAR *data, INT32 data_counter);
                    ErrorMsg*   GetEthernetEEPROM(string ethernet_name, INT32 StartAddress, INT32 Length, INT32 *data);
                    ErrorMsg*   GetEthernetEEPROMSize(CHAR *ethernet_name, INT32 *data_counter);
                    ErrorMsg*   SendIoctl(string ethernet_name, TestType index, INT32 *test_result);
                    ErrorMsg*   SetEthernetEEPROM(CHAR *ethernet_name, INT32 offset, UCHAR data, INT32 Length);
                    ErrorMsg*   SetEthernetEEPROM(string ethernet_name, INT32 StartAddress, INT32 *data, INT32 Length);
                    ErrorMsg*   SetEthernetMAC(string ethernet_name, string *mac_address);
                    ErrorMsg*   SetupSpeed(string ethernet_name, int speed);
                    ErrorMsg*   Enable_external_loopback(string ethernet_name);
                    ErrorMsg*   Disable_external_loopback();
                    ErrorMsg*   Execute_external_loopback(string nic_1, string nic_2, float *throughput);
                    ErrorMsg*   get_vendor_ID(string *);
                    ErrorMsg*   get_device_ID(string *);
                    ErrorMsg*   get_device_bus_type(string *bus);
                    ErrorMsg*   get_receive_packet_number(string nic1_name, string nic2_name, long *transmit, long *receive_end);
                    ErrorMsg*   ConverWellKnowNameToRealName(string name, string *real_name);
                    ErrorMsg*   Program(string ethernet_name, string filePath);
                    ErrorMsg*   ReadSFP(string ethernet_name, INT32 DeviceAddress, INT32 StartAddress, INT32 Length, INT32 *data);
                    ErrorMsg*   WriteSFP(string ethernet_name, INT32 DeviceAddress, INT32 StartAddress, INT32 data);
                    ErrorMsg*   getLEDList(list<string> *out_pLEDList);
                    ErrorMsg*   setLEDStatus(string in_strLEDName, string in_strStatus);
                    ErrorMsg*   getLEDStatus(string in_strLEDName, string *out_strpStatus);
                    ErrorMsg*   getStatistics(string nic, string *statistics);
                    ErrorMsg*   clearPCIError(string nic);
                    ErrorMsg*   clearStatistics(string nic);

protected:

private:
                    ErrorMsg*   GetEthernetMagic(CHAR  *ethernet_name, INT32 *magic);


           struct ethtool_eeprom {
                     int   cmd;
                     int   magic;
                     int   offset; /* in bytes */
                     int   len; /* in bytes */
                     unsigned char  data[0];
           };

           /* for requesting NIC test and getting results*/
//           struct ethtool_test {
//                     unsigned int        cmd;            /* ETHTOOL_TEST */
//                     unsigned int        flags;          /* ETH_TEST_FL_xxx */
//                     unsigned int        reserved;
//                     unsigned int        len;            /* result length, in number of u64 elements */
//                     unsigned long long  data[0];
//           };


           /* for passing string sets for data tagging */
           struct ethtool_gstrings {
                     unsigned int    cmd;            /* ETHTOOL_GSTRINGS */
                     unsigned int    string_set;     /* string set id e.c. ETH_SS_TEST, etc*/
                     unsigned int    len;            /* number of strings in the string set */
                     unsigned char   data[0];
           };


           /* these strings are set to whatever the driver author decides... */
           struct ethtool_drvinfo {
                     unsigned int  cmd;
                     char          driver[32];     /* driver short name, "tulip", "eepro100" */
                     char          version[32];    /* driver version string */
                     char          fw_version[32]; /* firmware version string */
                     char          bus_info[32];   /* Bus info for this IF. */
                                                   /* For PCI devices, use pci_name(pci_dev). */
                     char          reserved1[32];
                     char          reserved2[12];
                                /*
                                 * Some struct members below are filled in
                                 * using ops->get_sset_count().  Obtaining
                                 * this info from ethtool_drvinfo is now
                                 * deprecated; Use ETHTOOL_GSSET_INFO
                                 * instead.
                                 */
                     unsigned int  n_priv_flags;   /* number of flags valid in ETHTOOL_GPFLAGS */
                     unsigned int  n_stats;        /* number of u64's from ETHTOOL_GSTATS */
                     unsigned int  testinfo_len;
                     unsigned int  eedump_len;     /* Size of data from ETHTOOL_GEEPROM (bytes) */
                     unsigned int  regdump_len;    /* Size of data from ETHTOOL_GREGS (bytes) */
           };


           struct ethtool_sset_info {
                     unsigned int        cmd;        /* ETHTOOL_GSSET_INFO */
                     unsigned int        reserved;
                     unsigned long long  sset_mask;  /* input: each bit selects an sset to query */
                                                     /* output: each bit a returned sset */
                     unsigned int        data[0];    /* ETH_SS_xxx count, in order, based on bits
                                                        in sset_mask.  One bit implies one
                                                        __u32, two bits implies two
                                                        __u32's, etc. */
           };

            ErrorMsg  *m_errorCode;
            string    m_ethernet_name;
};

#endif

