#include "DIMMTemperature.hpp"
#include "I2CDDAPI.hpp"

DIMMTemperature::DIMMTemperature(STRING in_szClassName) : I2CTemperature(in_szClassName)
{
    m_errorCode = NO_ERROR_MSG;
}



DIMMTemperature::~DIMMTemperature()
{

}



float
DIMMTemperature::ConverTemperature(int Temperature)
{
    int     Sign = 0, Integer_Temperature = 0;
    float   float_Temperature = 0.0;

    //Switch low byte and high byte
    Temperature = ((Temperature & 0x00FF) << 8) | ((Temperature & 0xFF00) >> 8);

    //Check the Sign bit
    if ((Temperature & 0x1000) != 0)
    {
        Sign = 1;
    }

    //Filter the unused 2 low bit number
    Temperature = (Temperature & 0x1FFC) >> 2;

    //negative number twos complement
    if (Sign == 1)
    {
        Temperature = (~Temperature& 0x07FF ) + 1;
    }

    //Filter the integer number
    Integer_Temperature = Temperature >> 2;

    //Check the first point number
    if ((Temperature & 0x01) != 0)
    {
        float_Temperature = float_Temperature + 0.25;
    }
    //Check the second point number
    if ((Temperature & 0x02) != 0)
    {
        float_Temperature = float_Temperature + 0.5;
    }

    if (Sign == 1)
    {
        float_Temperature = -1 * (Integer_Temperature + float_Temperature);
    }
    else
    {
        float_Temperature = Integer_Temperature + float_Temperature;
    }
    
    return float_Temperature;
}



ErrorMsg*
DIMMTemperature::GetCurrntTemperature(string wellknow_name, string *result)
{
    int       Temperature, count;
    char      TemperatureChar[10];
    float     float_Temperature = 0.0;
    I2CDDAPI  m_i2c_ddapi("I2C driver");


    m_errorCode = m_i2c_ddapi.read(wellknow_name.c_str(), CAT, CAT, &Temperature, &count);
    if (m_errorCode != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_I2C_DAPI_MUX_TEST, "Read Current Ambient temperature fail", m_errorCode);
    }

    float_Temperature = ConverTemperature(Temperature);
    sprintf(TemperatureChar, "%.2f", float_Temperature);

    *result = TemperatureChar;

    return NO_ERROR_MSG;
}



ErrorMsg*
DIMMTemperature::GetThresholdHigh(string wellknow_name, string *result)
{
    int       HighLimit, count;
    char      TemperatureChar[10];
    float     float_Temperature = 0.0;
    I2CDDAPI  m_i2c_ddapi("I2C driver");


    m_errorCode = m_i2c_ddapi.read(wellknow_name.c_str(), THL, THL, &HighLimit, &count);
    if (m_errorCode != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_I2C_DAPI_MUX_TEST, "Read Temperature High Limit fail", m_errorCode);
    }

    float_Temperature = ConverTemperature(HighLimit);
    sprintf(TemperatureChar, "%.2f", float_Temperature);

    *result = TemperatureChar;

    return NO_ERROR_MSG;
}



ErrorMsg*
DIMMTemperature::GetThresholdLow(string wellknow_name, string *result)
{
    int       LowLimit, count;
    char      TemperatureChar[10];
    float     float_Temperature = 0.0;
    I2CDDAPI  m_i2c_ddapi("I2C driver");


    m_errorCode = m_i2c_ddapi.read(wellknow_name.c_str(), TLL, TLL, &LowLimit, &count);
    if (m_errorCode != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_I2C_DAPI_MUX_TEST, "Read Temperature Low Limit fail", m_errorCode);
    }

    float_Temperature = ConverTemperature(LowLimit);
    sprintf(TemperatureChar, "%.2f", float_Temperature);

    *result = TemperatureChar;

    return NO_ERROR_MSG;
}



ErrorMsg*
DIMMTemperature::SetThresholdHigh(string wellknow_name, string data)
{
    I2CDDAPI  m_i2c_ddapi("I2C driver");
    int       int_data;

    return returnErrorMsg(ERROR_I2C_DAPI_MUX_TEST, "Write Temperature High Limit not support", NULL);
/*    sscanf(data.c_str(), "%d", &int_data);
    m_errorCode = m_i2c_ddapi.write(wellknow_name.c_str(), THL, &int_data, 1);
    if (m_errorCode != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_I2C_DAPI_MUX_TEST, "Write Temperature High Limit fail", m_errorCode);
    }

    return NO_ERROR_MSG;*/
}



ErrorMsg*
DIMMTemperature::SetThresholdLow(string wellknow_name, string data)
{
    I2CDDAPI  m_i2c_ddapi("I2C driver");
    int       int_data[2];

    return returnErrorMsg(ERROR_I2C_DAPI_MUX_TEST, "Write Temperature Low Limit not support", NULL);
/*    sscanf(data.c_str(), "%d", &int_data);
    m_errorCode = m_i2c_ddapi.write(wellknow_name.c_str(), TLL, int_data, 2);

    if (m_errorCode != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_I2C_DAPI_MUX_TEST, "Write Temperature Low Limit fail", m_errorCode);
    }

    return NO_ERROR_MSG;*/
}


