#include "I2CBus.hpp"



I2CBus :: I2CBus(STRING in_szClassName, string name, int ID, int frequency, string master) : BaseClassDDAPI(in_szClassName)
{
    m_name = name;
    m_ID = ID;
    m_frequency = frequency;
    m_master = master;
}



I2CBus :: ~I2CBus()
{

}



int I2CBus :: get_Bus_ID()
{
    return m_ID;
}



char * I2CBus :: get_Bus_name()
{
    return (char *)m_name.c_str();
}



int I2CBus :: get_frequency()
{
    return m_frequency;
}



string I2CBus :: get_master()
{
    return m_master;
}



ErrorMsg * I2CBus :: read(int Address, int StartRegister, int EndRegister, int ptotocol, int *data, int *count, I2C_DEVICE_TYPE device_type)
{
    char   filename[20]; //file name buffer
    int    result = 0;
    int    file ,i;
    UINT8  dataBuffer[8192];

    sprintf(filename, "/dev/i2c-%d", get_Bus_ID());
    //printf("Read BUS:0x%x address:0x%x Sregister:0x%x protocol:0x%x \n",m_ID, Address, StartRegister, ptotocol);
    file = open(filename,O_RDWR);
    if (file < 0) {
        //m_errorCode = returnErrorMsg(NO_ERROR, NULL, NULL);
        //printf("I2C open fail. %s\n",filename);
        return returnErrorMsg((ERRORCODE)(ERROR_I2C_DDAPI_I2CBUS_READ | ERROR_I2C_DDAPI_OPEN_NODE_FAIL), strerror(errno), NULL);
    }

    if (I2C_set_slave_address(file, Address) != 0)
    {
        //printf("I2C_set_slave_address fail.\n");
        return  returnErrorMsg((ERRORCODE)(ERROR_I2C_DDAPI_I2CBUS_READ | ERROR_I2C_DDAPI_SET_SLAVE_ADDRESS_FAIL), strerror(errno), NULL);
    }

    *count = 0;

    switch(ptotocol) {
        case I2C_DEVICE_PROTOCOL_SMBUS_BYTE:
            //printf("I2C_DEVICE_PROTOCOL_SMBUS_BYTE\n");
            result = I2C_smbus_read_byte(file);
            if(result == -1 )
            {
                //printf("Read: I2C_smbus_read_byte fail.\n");
                close(file);
                return returnErrorMsg((ERRORCODE)(ERROR_I2C_DDAPI_I2CBUS_READ | ERROR_I2C_DDAPI_READ_BYTE_FAIL), strerror(errno), NULL);
            }
            data[0] = result;
            *count = 1;
            break;

        case I2C_DEVICE_PROTOCOL_SMBUS_BYTE_DATA:
            //printf("I2C_DEVICE_PROTOCOL_SMBUS_BYTE_DATA\n");
            for(i=0; i< (EndRegister - StartRegister + 1); i++)
            {
                //DDR4 SPD need to select which pager to access
                if (device_type == SPD_DDR4)
                {
                    if ((StartRegister + i) < 0x100)
                    {
                        //Set up to page 0
                        I2C_set_slave_address(file, 0xAC);
                        I2C_smbus_read_byte_data(file, StartRegister + i);
                    }
                    else
                    {
                        //Set up to page 1
                        I2C_set_slave_address(file, 0xAE);
                        I2C_smbus_read_byte_data(file, StartRegister + i);
                    }

                    //Reset slave address to original
                    if (I2C_set_slave_address(file, Address) != 0)
                    {
                        //printf("I2C_set_slave_address fail.\n");
                        return  returnErrorMsg((ERRORCODE)(ERROR_I2C_DDAPI_I2CBUS_READ | ERROR_I2C_DDAPI_SET_SLAVE_ADDRESS_FAIL), strerror(errno), NULL);
                    }
                }

                result = I2C_smbus_read_byte_data(file, StartRegister + i);
                if(result == -1 )
                {
                    //printf("Read: I2C_smbus_read_byte_data fail.\n");
                    close(file);
                    return returnErrorMsg((ERRORCODE)(ERROR_I2C_DDAPI_I2CBUS_READ | ERROR_I2C_DDAPI_READ_BYTE_DATA_FAIL), strerror(errno), NULL);
                }
                data[i] = result;
                (*count)++;
            }
            break;

        case I2C_DEVICE_PROTOCOL_SMBUS_WORD_DATA:
            //printf("I2C_DEVICE_PROTOCOL_SMBUS_WORD\n");
            for(i = 0; i < (EndRegister - StartRegister + 1); i++)
            {
                result = I2C_smbus_read_word_data(file, StartRegister + i);
                if(result == -1 )
                {
                    //printf("Read: I2C_smbus_read_word_data fail.\n");
                    close(file);
                    return returnErrorMsg((ERRORCODE)(ERROR_I2C_DDAPI_I2CBUS_READ | ERROR_I2C_DDAPI_READ_WORD_DATA_FAIL), strerror(errno), NULL);
                }
                data[i] = result;
                (*count)++;
            }
            break;

        case I2C_DEVICE_PROTOCOL_SMBUS_BLOCK:
            //printf("I2C_DEVICE_PROTOCOL_SMBUS_BLOCK\n");
            result = I2C_smbus_read_block_data(file, StartRegister, data);
            if(result == -1 )
            {
                //printf("Read: I2C_smbus_read_block_data fail.\n");
                close(file);
                return returnErrorMsg((ERRORCODE)(ERROR_I2C_DDAPI_I2CBUS_READ | ERROR_I2C_DDAPI_READ_BLOCK_DATA_FAIL), strerror(errno), NULL);
            }
            *count = result;
            break;

        case I2C_DEVICE_PROTOCOL_SMBUS_PROCESS_CALL:
            printf("I2C_DEVICE_PROTOCOL_SMBUS_PROCESS_CALL\n");
            break;

        case I2C_DEVICE_PROTOCOL_I2C_BLOCK:
            //printf("I2C_DEVICE_PROTOCOL_I2C_BLOCK\n");

            //SMBus Specification Version 2.0(page 32):
            //A Block Read or Write is allowed to transfer a maximum of 32 data bytes.
            result= I2c_smbus_read_i2c_block_data(file, StartRegister, StartRegister+32 , data);
            if(result <= 0)
            {
                //printf("Read: I2c_smbus_read_i2c_block_data fail\n");
                close(file);
                return returnErrorMsg((ERRORCODE)(ERROR_I2C_DDAPI_I2CBUS_READ | ERROR_I2C_DDAPI_READ_I2C_BLOCK_DATA_FAIL), strerror(errno), NULL);
            }
            *count = result;
            break;
            
        case I2C_DEVICE_PROTOCOL_VPD_BLOCK:
            UINT8 register_address[2]; //store the VPD assign hight/low address(16 bit)
            //Hight_address = (startAddress >> 8) & 0x001f;
            //Low_address   = startAddress & 0x00ff;
            register_address[0] = (StartRegister >> 8) & 0x001f;
            register_address[1] = StartRegister & 0x00ff;
            //Print for debug
            //printf("startAddress=0x%x  Hight_address=0x%x  Low_address=0x%x\n",startAddress,Hight_address,Low_address);
            //printf("startAddress=0x%x  register_address[0]=0x%x  register_address[1]=0x%x\n",startAddress,register_address[0],register_address[1]);
            //printf("startAddress=%d  endAddress=%d  endAddress-startAddress=%d\n",startAddress,endAddress,endAddress-startAddress);

            result = I2C_access(file, 0, Address, 2, register_address); //write first which register will be read.
            if( result < 0 ) {
                close(file);
                //printf("deviceAddress %x, register_address %x\n", deviceAddress, register_address);
                return returnErrorMsg((ERRORCODE)(ERROR_I2C_DDAPI_I2CBUS_READ | ERROR_I2C_DDAPI_READ_I2C_BLOCK_DATA_FAIL), strerror(errno), NULL);
            }
            result = I2C_access(file, 1, Address, EndRegister - StartRegister + 1, dataBuffer); //read data to dataBuffer
            if( result < 0 ) {
                close(file);
                return returnErrorMsg((ERRORCODE)(ERROR_I2C_DDAPI_I2CBUS_READ | ERROR_I2C_DDAPI_READ_I2C_BLOCK_DATA_FAIL), strerror(errno), NULL);
            }

            for (i = 0; i < (EndRegister - StartRegister + 1); i++)
            {
                data[i] = dataBuffer[i];
            }
            
            *count = (EndRegister - StartRegister) + 1;
            break;

        case I2C_DEVICE_PROTOCOL_PMBUS_BLOCK:
            //printf("Read I2C_DEVICE_PROTOCOL_PMBUS_BLOCK\n");
            result= I2c_smbus_read_i2c_block_data(file, StartRegister, StartRegister+32 , data);
            if(result <= 0)
            {
                //printf("Read: I2c_smbus_read_i2c_block_data fail\n");
                close(file);
                return returnErrorMsg((ERRORCODE)(ERROR_I2C_DDAPI_I2CBUS_READ | ERROR_I2C_DDAPI_READ_I2C_BLOCK_DATA_FAIL), strerror(errno), NULL);
            }
            *count = result;
            break;
    }

    close(file);

    return NO_ERROR_MSG;
}



ErrorMsg * I2CBus :: write(int Address, int StartRegister, int ptotocol, int *data, int count, I2C_DEVICE_TYPE device_type)
{
    char   filename[20]; //file name buffer
    int    index;
    int    file;
    int    result = 0;


    sprintf(filename, "/dev/i2c-%d", get_Bus_ID());
    //printf("Write BUS:0x%x address:0x%x register:0x%x protocol:0x%x  Data:0x%x\n",m_ID, Address, Register, ptotocol, data);
    file = open(filename,O_RDWR);
    if (file < 0) {
        //m_errorCode = returnErrorMsg(NO_ERROR, NULL, NULL);
        //printf("I2C open fail. %s\n",filename);
        return returnErrorMsg((ERRORCODE)(ERROR_I2C_DDAPI_I2CBUS_WRITE | ERROR_I2C_DDAPI_OPEN_NODE_FAIL), strerror(errno), NULL);
    }

    if (I2C_set_slave_address(file, Address) != 0)
    {
        //printf("I2C_set_slave_address fail.\n");
        return  returnErrorMsg((ERRORCODE)(ERROR_I2C_DDAPI_I2CBUS_WRITE | ERROR_I2C_DDAPI_SET_SLAVE_ADDRESS_FAIL), strerror(errno), NULL);
    }

    switch(ptotocol) {
        case I2C_DEVICE_PROTOCOL_SMBUS_BYTE:
            //printf("I2C_DEVICE_PROTOCOL_SMBUS_BYTE\n");
            for (index = 0; index < count; index++)
            {
                if(I2C_smbus_write_byte(file,data[index]) != 0)
                {
                    //printf("I2C_smbus_write_byte fail.\n");
                    return returnErrorMsg((ERRORCODE)(ERROR_I2C_DDAPI_I2CBUS_WRITE | ERROR_I2C_DDAPI_WRITE_BYTE_FAIL), strerror(errno), NULL);
                }
            }
            break;

        case I2C_DEVICE_PROTOCOL_SMBUS_BYTE_DATA:
            //printf("I2C_DEVICE_PROTOCOL_SMBUS_BYTE_DATA\n");
            for (index = 0; index < count; index++)
            {
                //DDR4 SPD need to select which pager to access
                if (device_type == SPD_DDR4)
                {
                    if ((StartRegister + index) < 0x100)
                    {
                        //Set up to page 0
                        I2C_set_slave_address(file, 0x6C);
                        I2C_smbus_read_byte_data(file, StartRegister + index);
                    }
                    else
                    {
                        //Set up to page 0
                        I2C_set_slave_address(file, 0x6E);
                        I2C_smbus_read_byte_data(file, StartRegister + index);
                    }

                    //Reset slave address to original
                    if (I2C_set_slave_address(file, Address) != 0)
                    {
                        //printf("I2C_set_slave_address fail.\n");
                        return  returnErrorMsg((ERRORCODE)(ERROR_I2C_DDAPI_I2CBUS_READ | ERROR_I2C_DDAPI_SET_SLAVE_ADDRESS_FAIL), strerror(errno), NULL);
                    }
                }

                if (I2C_smbus_write_byte_data(file, StartRegister + index, data[index]) != 0)
                {
                    //printf("I2C_smbus_write_byte_data fail.\n");
                    return returnErrorMsg((ERRORCODE)(ERROR_I2C_DDAPI_I2CBUS_WRITE | ERROR_I2C_DDAPI_WRITE_BYTE_DATA_FAIL), strerror(errno), NULL);
                }
            }
            break;

        case I2C_DEVICE_PROTOCOL_SMBUS_WORD_DATA:
            //printf("I2C_DEVICE_PROTOCOL_SMBUS_WORD_DATA\n");
            int word_data;

            for (index = 0; index < (count/2); index++)
            {
                word_data = data[index*2 + 0]*0x100 + data[index*2 + 1];

                if (I2C_smbus_write_word_data(file, StartRegister + index, word_data) != 0)
                {
                    //printf("I2C_smbus_write_word_data fail.\n");
                    return returnErrorMsg((ERRORCODE)(ERROR_I2C_DDAPI_I2CBUS_WRITE | ERROR_I2C_DDAPI_WRITE_WORD_DATA_FAIL), strerror(errno), NULL);
                }
            }
            break;

        case I2C_DEVICE_PROTOCOL_SMBUS_BLOCK:
            //printf("I2C_DEVICE_PROTOCOL_SMBUS_BLOCK\n");
            if (I2C_smbus_write_block_data(file, StartRegister, count, data) == -1)
            {
                //printf("I2C_smbus_write_block_data fail.\n");
                return returnErrorMsg((ERRORCODE)(ERROR_I2C_DDAPI_I2CBUS_WRITE | ERROR_I2C_DDAPI_WRITE_BLOCK_DATA_FAIL), strerror(errno), NULL);
            }
            break;

        case I2C_DEVICE_PROTOCOL_SMBUS_PROCESS_CALL:
            //printf("I2C_DEVICE_PROTOCOL_SMBUS_PROCESS_CALL\n");
            break;

        case I2C_DEVICE_PROTOCOL_I2C_BLOCK:
            //printf("I2C_DEVICE_PROTOCOL_I2C_BLOCK\n");
            if (I2C_smbus_write_i2c_block_data(file, StartRegister, count, data) == -1)
            {
                //printf("I2C_smbus_write_i2c_block_data fail.\n");
                return returnErrorMsg((ERRORCODE)(ERROR_I2C_DDAPI_I2CBUS_WRITE | ERROR_I2C_DDAPI_WRITE_I2C_BLOCK_DATA_FAIL), strerror(errno), NULL);
            }
            break;

        case I2C_DEVICE_PROTOCOL_VPD_BLOCK:
            UINT8 arrange_dataBuffer[2+32]; //2 address byte + 32 data byte
            INT16 remind_data_lenth,send_lenth,offset;

            remind_data_lenth = count;
            send_lenth = 32;
            offset = 0;

            while(remind_data_lenth > 0)
            {
               arrange_dataBuffer[0] = ((StartRegister + offset) >> 8) & 0x001f; //Hight address
               arrange_dataBuffer[1] = (StartRegister + offset) & 0x00ff;        //Low address

               send_lenth = ((StartRegister + offset)/32+1)*32-1 - (StartRegister + offset) + 1; //dynamic calculate the send length
                         /*the end of 32 byte boundary address*/  /*start address to be write*/

               if( remind_data_lenth >= send_lenth ){
                  for(index=0; index<send_lenth; index++)
                     arrange_dataBuffer[index+2] = data[offset + index];
                  result = I2C_access(file, 0, Address, send_lenth+2, arrange_dataBuffer);
                  if( result < 0 ) {
                     close(file);
                     return returnErrorMsg((ERRORCODE)(ERROR_I2C_DDAPI_I2CBUS_WRITE | ERROR_I2C_DDAPI_WRITE_I2C_BLOCK_DATA_FAIL), strerror(errno), NULL);
                  }
                  offset += send_lenth;
               }


               if( remind_data_lenth < send_lenth ){
                  for(index=0; index<remind_data_lenth; index++)
                     arrange_dataBuffer[index+2] = data[offset + index];
                  result = I2C_access(file, 0, Address, remind_data_lenth+2, arrange_dataBuffer);
                  if( result < 0 ) {
                     close(file);
                    return returnErrorMsg((ERRORCODE)(ERROR_I2C_DDAPI_I2CBUS_WRITE | ERROR_I2C_DDAPI_WRITE_I2C_BLOCK_DATA_FAIL), strerror(errno), NULL);
                  }
               }

               remind_data_lenth = remind_data_lenth - send_lenth;
            }
            break;

        case I2C_DEVICE_PROTOCOL_PMBUS_BLOCK:
            //printf("I2C_DEVICE_PROTOCOL_PMBUS_BLOCK\n");
            if (I2C_smbus_write_block_data(file, StartRegister, count, data) == -1)
            {
                //printf("I2C_smbus_write_block_data fail.\n");
                return returnErrorMsg((ERRORCODE)(ERROR_I2C_DDAPI_I2CBUS_WRITE | ERROR_I2C_DDAPI_WRITE_BLOCK_DATA_FAIL), strerror(errno), NULL);
            }
            break;
    }

    close(file);

    return NO_ERROR_MSG;
}



ErrorMsg * I2CBus :: write(int Address, int StartRegister, int ptotocol, int *data, int count, int *numberRealWrite, I2C_DEVICE_TYPE device_type)
{
    char   filename[20]; //file name buffer
    int    index;
    int    file;
    int    result = 0;


    *numberRealWrite = 0;
    sprintf(filename, "/dev/i2c-%d", get_Bus_ID());
    //printf("Write BUS:0x%x address:0x%x register:0x%x protocol:0x%x  Data:0x%x\n",m_ID, Address, Register, ptotocol, data);
    file = open(filename,O_RDWR);
    if (file < 0) {
        //m_errorCode = returnErrorMsg(NO_ERROR, NULL, NULL);
        //printf("I2C open fail. %s\n",filename);
        return returnErrorMsg((ERRORCODE)(ERROR_I2C_DDAPI_I2CBUS_WRITE | ERROR_I2C_DDAPI_OPEN_NODE_FAIL), strerror(errno), NULL);
    }

    if (I2C_set_slave_address(file, Address) != 0)
    {
        //printf("I2C_set_slave_address fail.\n");
        return  returnErrorMsg((ERRORCODE)(ERROR_I2C_DDAPI_I2CBUS_WRITE | ERROR_I2C_DDAPI_SET_SLAVE_ADDRESS_FAIL), strerror(errno), NULL);
    }

    switch(ptotocol)
    {
        case I2C_DEVICE_PROTOCOL_SMBUS_BYTE:
            //printf("I2C_DEVICE_PROTOCOL_SMBUS_BYTE\n");
            for (index = 0; index < count; index++)
            {
                if(I2C_smbus_write_byte(file,data[index]) != 0)
                {
                    //printf("I2C_smbus_write_byte fail.\n");
                    return returnErrorMsg((ERRORCODE)(ERROR_I2C_DDAPI_I2CBUS_WRITE | ERROR_I2C_DDAPI_WRITE_BYTE_FAIL), strerror(errno), NULL);
                }
                (*numberRealWrite)++;
            }
            break;

        case I2C_DEVICE_PROTOCOL_SMBUS_BYTE_DATA:
            //printf("I2C_DEVICE_PROTOCOL_SMBUS_BYTE_DATA\n");
            for (index = 0; index < count; index++)
            {
                //DDR4 SPD need to select which pager to access
                if (device_type == SPD_DDR4)
                {
                    if ((StartRegister + index) < 0x100)
                    {
                        //Set up to page 0
                        I2C_set_slave_address(file, 0x6C);
                        I2C_smbus_read_byte_data(file, StartRegister + index);
                    }
                    else
                    {
                        //Set up to page 0
                        I2C_set_slave_address(file, 0x6E);
                        I2C_smbus_read_byte_data(file, StartRegister + index);
                    }

                    //Reset slave address to original
                    if (I2C_set_slave_address(file, Address) != 0)
                    {
                        //printf("I2C_set_slave_address fail.\n");
                        return  returnErrorMsg((ERRORCODE)(ERROR_I2C_DDAPI_I2CBUS_READ | ERROR_I2C_DDAPI_SET_SLAVE_ADDRESS_FAIL), strerror(errno), NULL);
                    }
                }

                if (I2C_smbus_write_byte_data(file, StartRegister + index, data[index]) != 0)
                {
                    //printf("I2C_smbus_write_byte_data fail.\n");
                    return returnErrorMsg((ERRORCODE)(ERROR_I2C_DDAPI_I2CBUS_WRITE | ERROR_I2C_DDAPI_WRITE_BYTE_DATA_FAIL), strerror(errno), NULL);
                }
                (*numberRealWrite)++;
            }
            break;

        case I2C_DEVICE_PROTOCOL_SMBUS_WORD_DATA:
            //printf("I2C_DEVICE_PROTOCOL_SMBUS_WORD_DATA\n");
            int word_data;

            for (index = 0; index < (count/2); index++)
            {
                word_data = data[index*2 + 0]*0x100 + data[index*2 + 1];
                if (I2C_smbus_write_word_data(file, StartRegister + index, word_data) != 0)
                {
                    //printf("I2C_smbus_write_word_data fail.\n");
                    return returnErrorMsg((ERRORCODE)(ERROR_I2C_DDAPI_I2CBUS_WRITE | ERROR_I2C_DDAPI_WRITE_WORD_DATA_FAIL), strerror(errno), NULL);
                }
                (*numberRealWrite)++;
            }
            break;

        case I2C_DEVICE_PROTOCOL_SMBUS_BLOCK:
            //printf("I2C_DEVICE_PROTOCOL_SMBUS_BLOCK\n");
            if (I2C_smbus_write_block_data(file, StartRegister, count, data) == -1)
            {
                //printf("I2C_smbus_write_block_data fail.\n");
                return returnErrorMsg((ERRORCODE)(ERROR_I2C_DDAPI_I2CBUS_WRITE | ERROR_I2C_DDAPI_WRITE_BLOCK_DATA_FAIL), strerror(errno), NULL);
            }
            (*numberRealWrite)++;
            break;

        case I2C_DEVICE_PROTOCOL_SMBUS_PROCESS_CALL:
            //printf("I2C_DEVICE_PROTOCOL_SMBUS_PROCESS_CALL\n");
            break;

        case I2C_DEVICE_PROTOCOL_I2C_BLOCK:
            //printf("I2C_DEVICE_PROTOCOL_I2C_BLOCK\n");
            if (I2C_smbus_write_i2c_block_data(file, StartRegister, count, data) == -1)
            {
                //printf("I2C_smbus_write_i2c_block_data fail.\n");
                return returnErrorMsg((ERRORCODE)(ERROR_I2C_DDAPI_I2CBUS_WRITE | ERROR_I2C_DDAPI_WRITE_I2C_BLOCK_DATA_FAIL), strerror(errno), NULL);
            }
            (*numberRealWrite)++;
            break;

        case I2C_DEVICE_PROTOCOL_VPD_BLOCK:
            UINT8 arrange_dataBuffer[2+32]; //2 address byte + 32 data byte
            INT16 remind_data_lenth,send_lenth,offset;

            remind_data_lenth = count;
            send_lenth = 32;
            offset = 0;

            while(remind_data_lenth > 0)
            {
               arrange_dataBuffer[0] = ((StartRegister + offset) >> 8) & 0x001f; //Hight address
               arrange_dataBuffer[1] = (StartRegister + offset) & 0x00ff;        //Low address

               send_lenth = ((StartRegister + offset)/32+1)*32-1 - (StartRegister + offset) + 1; //dynamic calculate the send length
                         /*the end of 32 byte boundary address*/  /*start address to be write*/

               if( remind_data_lenth >= send_lenth ){
                  for(index=0; index<send_lenth; index++)
                     arrange_dataBuffer[index+2] = data[offset + index];
                  result = I2C_access(file, 0, Address, send_lenth+2, arrange_dataBuffer);
                  if( result < 0 ) {
                     close(file);
                     return returnErrorMsg((ERRORCODE)(ERROR_I2C_DDAPI_I2CBUS_WRITE | ERROR_I2C_DDAPI_WRITE_I2C_BLOCK_DATA_FAIL), strerror(errno), NULL);
                  }
                  offset += send_lenth;
               }


               if( remind_data_lenth < send_lenth ){
                  for(index=0; index<remind_data_lenth; index++)
                     arrange_dataBuffer[index+2] = data[offset + index];
                  result = I2C_access(file, 0, Address, remind_data_lenth+2, arrange_dataBuffer);
                  if( result < 0 ) {
                     close(file);
                    return returnErrorMsg((ERRORCODE)(ERROR_I2C_DDAPI_I2CBUS_WRITE | ERROR_I2C_DDAPI_WRITE_I2C_BLOCK_DATA_FAIL), strerror(errno), NULL);
                  }
               }

               remind_data_lenth = remind_data_lenth - send_lenth;
            }
            *numberRealWrite = count;

            break;

        case I2C_DEVICE_PROTOCOL_PMBUS_BLOCK:
            //printf("I2C_DEVICE_PROTOCOL_PMBUS_BLOCK\n");
            if (I2C_smbus_write_block_data(file, StartRegister, count, data) == -1)
            {
                //printf("I2C_smbus_write_block_data fail.\n");
                return returnErrorMsg((ERRORCODE)(ERROR_I2C_DDAPI_I2CBUS_WRITE | ERROR_I2C_DDAPI_WRITE_BLOCK_DATA_FAIL), strerror(errno), NULL);
            }
            break;
    }

    close(file);

    return NO_ERROR_MSG;
}


INT16 I2CBus :: I2C_set_slave_address(int file, int addr)
{
    int res;
    
    //shift right 1 bit
    addr =addr >> 1;
    res = ioctl(file, I2C_SLAVE, addr);
    if (res != 0)
       return 1/*ERROR_I2C_BUS_MASTER0_SLAVE_ADDRESS_SET*/;
    else
       return 0;
}



INT16 I2CBus :: I2C_smbus_access(UINT16 file, UINT8  read_write, UINT16 command, UINT16 size, union i2c_smbus_data *data)
{
    struct i2c_smbus_ioctl_data args;
    int res;	

    args.read_write = read_write;
    args.command = command;
    args.size = size;
    args.data = data;
    res = ioctl(file,I2C_SMBUS,&args);

    return res;
}



INT16 I2CBus :: I2C_access(UINT16 file, UINT16 read_write, UINT16 slave_address, UINT16 len, UINT8  *data)
{
    struct i2c_msg  msg;
    struct i2c_rdwr_ioctl_data queue;
    int res;

    queue.msgs = &msg;
    queue.nmsgs = 1;

    queue.msgs->len = len;
    queue.msgs->addr = slave_address >> 1; //shift right 1 bit
    queue.msgs->flags = read_write;        /* 0:write  1:read */
    queue.msgs->buf = data;
    res = ioctl(file, I2C_RDWR, &queue);

    return res;
}



UINT16 I2CBus :: I2C_smbus_write_byte_data(UINT16 file, UINT16 command, UINT16 value)
{
    union i2c_smbus_data data;

    data.byte = value;

    return I2C_smbus_access(file,I2C_SMBUS_WRITE,command,
                            I2C_SMBUS_BYTE_DATA, &data);
}



INT16 I2CBus :: I2C_smbus_read_byte_data(UINT16 file, UINT16 command)
{
    union i2c_smbus_data data;

    if (I2C_smbus_access(file,I2C_SMBUS_READ,command,
                         I2C_SMBUS_BYTE_DATA,&data) == -1)
        return -1;
    else
        return 0x0FF & data.byte;
}



INT16 I2CBus :: I2C_smbus_read_block_data(UINT16 file, UINT16 command, INT32  *values)
{
    union i2c_smbus_data data;
    int   i;

    if (I2C_smbus_access(file,I2C_SMBUS_READ,command,
                         I2C_SMBUS_BLOCK_DATA,&data) == -1)
    {
        return -1;
    }
    else
    {
        for (i = 1; i <= data.block[0]; i++)
        {
            values[i-1] = data.block[i];
        }
        return data.block[0];
    }
}



INT16 I2CBus :: I2C_smbus_write_block_data(UINT16 file, UINT16 command, UINT16 length, INT32 *values)
{
    union i2c_smbus_data data;
    int   i;

    if (length > 32)
    {
        length = 32;
    }
    for (i = 1; i <= length; i++)
    {
        data.block[i] = values[i-1];
    }
    data.block[0] = length;

    return I2C_smbus_access(file,I2C_SMBUS_WRITE,command,
                            I2C_SMBUS_BLOCK_DATA, &data);
}



INT32 I2CBus :: I2C_smbus_read_word_data(UINT16 file, UINT16 command)
{
    union i2c_smbus_data data;

    if (I2C_smbus_access(file,I2C_SMBUS_READ,command,
                         I2C_SMBUS_WORD_DATA,&data) == -1)
    {
        return -1;
    }
    else
    {
        return 0x0FFFF & data.word;
    }
}



UINT16 I2CBus :: I2C_smbus_write_word_data(UINT16 file, UINT16 command, UINT16 value)
{
    union i2c_smbus_data data;

    data.word = value;

    return I2C_smbus_access(file,I2C_SMBUS_WRITE,command,
                            I2C_SMBUS_WORD_DATA, &data);
}



INT16 I2CBus :: I2c_smbus_read_i2c_block_data(UINT16 file, UINT16 command, UINT16 length, INT32 *values)
{
    union i2c_smbus_data data;
    int   i;

    if (length > 32)
    {
        length = 32;
    }

    data.block[0] = length;

    if (I2C_smbus_access(file,I2C_SMBUS_READ,command,
                         length == 32 ? I2C_SMBUS_I2C_BLOCK_BROKEN :
                         I2C_SMBUS_I2C_BLOCK_DATA,&data) == -1)
    {
        return -1;
    }
    else
    {
        for (i = 0; i <= data.block[1]; i++)
        {
            values[i] = data.block[i+2];
        }
        return data.block[1];
    }
}



UINT16 I2CBus :: I2C_smbus_write_i2c_block_data(UINT16 file, UINT16 command, UINT16 length, INT32 *values)
{
    union i2c_smbus_data data;
    int   i;

    if (length > 32)
    {
        length = 32;
    }
    for (i = 1; i <= length; i++)
    {
        data.block[i] = values[i-1];
    }
    data.block[0] = length;

    return I2C_smbus_access(file,I2C_SMBUS_WRITE,command,
                            I2C_SMBUS_I2C_BLOCK_BROKEN, &data);
}



INT16 I2CBus :: I2C_smbus_write_byte(UINT16 file, UINT8 value)
{
    return I2C_smbus_access(file,I2C_SMBUS_WRITE,value,
                            I2C_SMBUS_BYTE,NULL);
}



INT16 I2CBus :: I2C_smbus_read_byte(UINT16 file)
{
    union i2c_smbus_data data;

    if (I2C_smbus_access(file,I2C_SMBUS_READ,0,I2C_SMBUS_BYTE,&data) == -1)
    {
        return -1;
    }
    else
    {
        return 0x0FF & data.byte;
    }
}

