#ifndef I2CBus_H
#define I2CBus_H

#include <stdio.h>
#include <fcntl.h>
#include <unistd.h> //for close() API.
#include "BaseClassDDAPI.hpp"
#include "ErrorMsg.hpp"
#include "error_code.hpp"
#include "BaseClassAPI.hpp"
#include <string>
#include <linux/i2c-dev.h>
#include <linux/i2c.h>
#include <sys/ioctl.h>
#include <errno.h>

using namespace std;

// declare the I2C device protocol
enum I2C_DEVICE_PROTOCOL {
     I2C_DEVICE_PROTOCOL_SMBUS_BYTE = 0,
     I2C_DEVICE_PROTOCOL_SMBUS_BYTE_DATA,
     I2C_DEVICE_PROTOCOL_SMBUS_WORD_DATA,
     I2C_DEVICE_PROTOCOL_SMBUS_BLOCK,
     I2C_DEVICE_PROTOCOL_SMBUS_PROCESS_CALL,
     I2C_DEVICE_PROTOCOL_I2C_BLOCK,
     I2C_DEVICE_PROTOCOL_VPD_BLOCK,
     I2C_DEVICE_PROTOCOL_PMBUS_BLOCK,
};



typedef enum {
    GENERAL = 0,
    MUX,
    TEMPERATURE,
    BATTERY,
    SPD_DDR1,
    SPD_DDR2,
    SPD_DDR3,
    SPD_DDR4,
    VPD,
} I2C_DEVICE_TYPE;



class I2CBus : public BaseClassDDAPI
{
public:
                        I2CBus(STRING in_szClassName, string name, int ID, int speed, string master);
                        ~I2CBus();
            char     *  get_Bus_name();
            int         get_Bus_ID();
            int         get_frequency();
            string      get_master();
            ErrorMsg *  read(int Address, int StartRegister, int EndRegister, int ptotocol, int *data, int *count, I2C_DEVICE_TYPE type);
            ErrorMsg *  write(int Address, int StartRegister, int ptotocol, int *data, int count, I2C_DEVICE_TYPE type);
            ErrorMsg *  write(int Address, int StartRegister, int ptotocol, int *data, int count, int *numberRealWrite, I2C_DEVICE_TYPE type);
//            ErrorMsg    read_word();
//            ErrorMsg    write_word();
//            ErrorMsg    read_block();
//            ErrorMsg    write_block();


protected:


private:
            INT16       I2C_set_slave_address(int file, int addr);
            INT16       I2C_smbus_access(UINT16 file, UINT8 read_write, UINT16 command, UINT16 size, union i2c_smbus_data *data);
            UINT16      I2C_smbus_write_byte_data(UINT16 file, UINT16 command, UINT16 values);
            INT16       I2C_smbus_read_byte_data(UINT16 file, UINT16 command);
            INT16       I2C_smbus_read_block_data(UINT16 file, UINT16 command, INT32 *values);
            INT16       I2C_smbus_write_block_data(UINT16 file, UINT16 command, UINT16 length, INT32 *values);
            INT32       I2C_smbus_read_word_data(UINT16 file, UINT16 command);
            UINT16      I2C_smbus_write_word_data(UINT16 file, UINT16 command, UINT16 values);
            INT16       I2c_smbus_read_i2c_block_data(UINT16 file, UINT16 command, UINT16 length, INT32 *values);
            UINT16      I2C_smbus_write_i2c_block_data(UINT16 file, UINT16 command, UINT16 length, INT32 *values);
            INT16       I2C_smbus_write_byte(UINT16 file, UINT8 value);
            INT16       I2C_smbus_read_byte(UINT16 file);
            INT16       I2C_access(UINT16 file, UINT16 read_write, UINT16 slave_address, UINT16 len, UINT8  *data);

            ErrorMsg    *m_errorCode;
            string      m_name;
            int         m_ID;
            int         m_frequency;
            string      m_master;

};

#endif
