#include "MAX6657.hpp"
#include "I2CDDAPI.hpp"

MAX6657::MAX6657(STRING in_szClassName) : I2CTemperature(in_szClassName)
{
    m_errorCode = NO_ERROR_MSG;
}



MAX6657::~MAX6657()
{

}



ErrorMsg*
MAX6657::GetCurrntTemperature(string wellknow_name, string *result)
{
    int       Temperature, Extended_Temperature, count;
    char      TemperatureChar[10];
    I2CDDAPI  m_i2c_ddapi("I2C driver");


    m_errorCode = m_i2c_ddapi.read(wellknow_name.c_str(), RLTS, RLTS, &Temperature, &count);
    if (m_errorCode != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_I2C_DAPI_MUX_TEST, "Read Internal Temperature", m_errorCode);
    }

    m_errorCode = m_i2c_ddapi.read(wellknow_name.c_str(), RIET, RIET, &Extended_Temperature, &count);
    if (m_errorCode != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_I2C_DAPI_MUX_TEST, "Read Internal Extended Temperature fail", m_errorCode);
    }

    //printf("Temperature:%d    Extended_Temperature:0x%x\n", Temperature, Extended_Temperature);
    if ((Temperature & 0x80) !=0 )
    {
        Temperature = 0;
    }
    sprintf(TemperatureChar, "%d", Temperature);
    *result = TemperatureChar;

    return NO_ERROR_MSG;
}



ErrorMsg*
MAX6657::GetThresholdHigh(string wellknow_name, string *result)
{
    int       HighLimit, count;
    char      TemperatureChar[10];
    I2CDDAPI  m_i2c_ddapi("I2C driver");


    m_errorCode = m_i2c_ddapi.read(wellknow_name.c_str(), RLHN, RLHN, &HighLimit, &count);
    if (m_errorCode != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_I2C_DAPI_MUX_TEST, "Read Internal High Limit fail", m_errorCode);
    }

    //printf("High Limit:0x%x\n", HighLimit);
    if ((HighLimit & 0x80) !=0 )
    {
        HighLimit = 0;
    }
    sprintf(TemperatureChar, "%d", HighLimit);
    *result = TemperatureChar;

    return NO_ERROR_MSG;
}



ErrorMsg*
MAX6657::GetThresholdLow(string wellknow_name, string *result)
{
    int       LowLimit, count;
    char      TemperatureChar[10];
    I2CDDAPI  m_i2c_ddapi("I2C driver");


    m_errorCode = m_i2c_ddapi.read(wellknow_name.c_str(), RLLI, RLLI, &LowLimit, &count);
    if (m_errorCode != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_I2C_DAPI_MUX_TEST, "Read Internal Low Limit fail", m_errorCode);
    }

    //printf("Low Limit:%x \n", LowLimit);
    if ((LowLimit & 0x80) !=0 )
    {
        LowLimit = 0;
    }
    sprintf(TemperatureChar, "%d", LowLimit);
    *result = TemperatureChar;

    return NO_ERROR_MSG;
}



ErrorMsg*
MAX6657::SetThresholdHigh(string wellknow_name, string data)
{
    I2CDDAPI  m_i2c_ddapi("I2C driver");
    int       int_data;

    sscanf(data.c_str(), "%d", &int_data);
    m_errorCode = m_i2c_ddapi.write(wellknow_name.c_str(), WLHO, &int_data, 1);
    if (m_errorCode != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_I2C_DAPI_MUX_TEST, "Write Internal High Limit fail", m_errorCode);
    }

    return NO_ERROR_MSG;
}



ErrorMsg*
MAX6657::SetThresholdLow(string wellknow_name, string data)
{
    I2CDDAPI  m_i2c_ddapi("I2C driver");
    int       int_data;

    sscanf(data.c_str(), "%d", &int_data);
    m_errorCode = m_i2c_ddapi.write(wellknow_name.c_str(), WLLM, &int_data, 1);
    if (m_errorCode != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_I2C_DAPI_MUX_TEST, "Write Internal High Limit fail", m_errorCode);
    }

    return NO_ERROR_MSG;
}


