/// @file       FXNMCDrv.h
/// @brief      Provide IOCTL definitions of Foxconn memory controller driver 
/// @details    Provide low layer diagnostic and utility IOCTL commands for memory module
/// @author     KenHu
/// @copyright  Copyright 2014 Hon Hai Technology Group. All Rights Reserved.

#ifndef FXN_MC_DRV_H
#define FXN_MC_DRV_H

#include <common.hpp>

#define FXN_MC_DRV_NAME                     "fxn_mc_drv"

#define FXN_MC_DRV_MAGIC_NUMBER             '%'

#define FXN_MC_DRV_RESULT_STRING_LEN        512

typedef enum {
    GET_CORRECTED_ECC_COUNT_CMD_CODE = 0, 
    GET_UNCORRECTED_ECC_COUNT_CMD_CODE,  
    CLEAR_CORRECTED_ECC_STATUS_CMD_CODE,
    CLEAR_UNCORRECTED_ECC_STATUS_CMD_CODE
} FXN_MC_DRV_IOCTL_CMD_CODE;

typedef enum {
    MCH_BAR_16BIT,
    MCH_BAR_32BIT,
    MCH_BAR_64BIT,
    PCI_DEV_16BIT,
    PCI_DEV_32BIT,
    PCI_DEV_64BIT
} MC_REGISTER_TYPES;

typedef struct {
    MC_REGISTER_TYPES   eRegType;
    SYSTEM_ADDRESS_T    offset;
    UINT64              u64BitMask;
    UINT8               u32BitOffset;
    UINT8               u8BusNum;
    UINT8               u8DevNum;
    UINT8               u8FuncNum;
} mc_func_desc;

typedef struct {
    INT32               s32ID;
    INT32               s32ChannelID;
    INT32               s32StartRank;
    INT32               s32EndRank;
    PHYSICAL_ADDRESS_T  startAddress;
    PHYSICAL_ADDRESS_T  endAddress;
} ECCDIMMInformation;

#endif
