/// @file       NVMeIOCTL.hpp
/// @brief      NVMeIOCTL class definition
/// @details    Provide NVMe IOCTL class member and function definition
/// @version    1.0.0
/// @date       2014.02.25
/// @author     JerryLee
/// @copyright  Copyright 2014 Hon Hai Technology Group. All Rights Reserved.

#ifndef NVMeIOCTL_HPP_
#define NVMeIOCTL_HPP_

#include "IOCTL.hpp"
#include "common.hpp"
#include "nvme.hpp"

class NVMeIOCTL: public IOCTL
{
    public:
        NVMeIOCTL(STRING in_fileName);
        virtual ~NVMeIOCTL(void);

        enum IOCTL_OP_CODE
        {   UnspportIOCTLCommand = IOCTL::UnspportIOCTL,
            //EXECUTE_RESET_NVME_SUBSYS,
            EXECUTE_NVME_RESET,
            CONTROLLER_INFORMATION,
            NAMESPACE_INFORMATION,
            FIRMWARE_LOG,
            FIRMWARE_ACTIVATE,
            FIRMWARE_DOWNLOAD,
            SMART_LOG,
            ERROR_LOG,
            NAMESPACE_ID,
            WRITE_ZERO,
            FORMAT,
            GET_FEATURES,
            SET_FEATURES

        };

        ERRORCODE allocateIOCTLArgument(UINT8 IOCTLOperation);
        ERRORCODE setIOCTLOperationArgument(UINT8 IOCTLOperation, void* data);
        ERRORCODE executeIOCTL(UINT8 IOCTLOperation);
        ERRORCODE executeIOCTL(UINT8 IOCTLOperation, void* out_pIOCTLResult);

        protected:

        private:
        ERRORCODE nvme_submit_passthru(int ioctl_cmd, struct nvme_passthru_cmd *cmd);
        ERRORCODE nvme_submit_admin_passthru(struct nvme_passthru_cmd *cmd);
        ERRORCODE nvme_submit_io_passthru(struct nvme_passthru_cmd *cmd);
        ERRORCODE convertNVMEStatustoErrorcode(UINT32 status);

    };

#endif /* NVMeIOCTL_HPP_ */
