/// @file       NVSRAMDDAPI.hpp
/// @brief      NVSRAMDDAPI class
/// @details    The header file of NVSRAM DDAPI
/// @version    1.0.0
/// @date       2015.11.19
/// @author     Jerry Lee
/// @copyright  Copyright 2014 Hon Hai Technology Group. All Rights Reserved.
#ifndef NVSRAMDDAPI_H
#define NVSRAMDDAPI_H

#include "BaseClassDDAPI.hpp"
#include "PCIDDAPI.hpp"
#include "MemoryDDAPI.hpp"

#define NVSRAM_BASE_ADDRESS 0x60f6000000ull
#define NVSRAM_CONFIG_ELEMENT_SIZE 64
#define DFW_API_TEST_PATTERN_BYTE_ONE   0x55
#define DFW_API_TEST_PATTERN_BYTE_TWO   0xAA
#define DFW_API_TEST_PATTERN_BYTE_THREE 0xFF
#define DFW_API_TEST_PATTERN_BYTE_FOUR  0x00

// declare the NVSRAM lock value
enum NVSRAM_LOCK_VALUE {
    NVSRAM_NO_LOCK_VALUE = 0x00,
    NVSRAM_ENTIRE_LOCK_VALUE = 0x80
};

// NVSRAM information definitions
typedef struct NVSRAM_RESOURCE_INFO {
        UINT8 logicalID;
        string BusDeviceFunction;
        PHYSICAL_ADDRESS_T BaseAddress;
        UINT32 BARSize;
        UINT8 elementSize;
} NvsramInfo;

class NVSRAMDDAPI: public BaseClassDDAPI
{
    public:
        NVSRAMDDAPI(STRING className);
        virtual ~NVSRAMDDAPI( void );

        ErrorMsg* getDeviceList(vector<NVSRAM_RESOURCE_INFO>* returnedList);
        ErrorMsg* getBaseAddress(UINT8 logicalID, PHYSICAL_ADDRESS_T* returnBaseAddress);
        ErrorMsg* getResourceSize(UINT8 logicalID, UINT32* returnResourceSize);
        ErrorMsg* getElementSize(UINT8 logicalID, UINT8* returnElementSize);
        ErrorMsg* recallNVSRAM(void);
        ErrorMsg* storeNVSRAM(void);

    protected:

    private:
        PCIDDAPI* pciDDAPI;
        MemoryDDAPI* memDDAPI;
        vector<NvsramInfo> nvsramList;
        UINT64 nvsramBARSize;
        PHYSICAL_ADDRESS_T nvsramBaseAddress;

        ErrorMsg* addNVSRAMDevice(void);
};

#endif // NVSRAMDDAPI_H

