/// @file       PMBusPSU.hpp
/// @brief      PMBusPSU class
/// @details    Provide command definitions and utilities for PMBus devices
/// @author     KenHu
/// @copyright  Copyright 2015-2016 Hon Hai Technology Group. All Rights Reserved.

#ifndef PMBUS_PSU_HPP
#define PMBUS_PSU_HPP

#include "BaseClassPSUDDAPI.hpp"
#include "I2CDAPI.hpp"

#define PMBUS_CMD_STORE_DEFAULT_CODE    0x13
#define PMBUS_CMD_OT_FAULT_LIMIT        0x4f
#define PMBUS_CMD_OT_WARN_LIMIT         0x51
#define PMBUS_CMD_UT_WARN_LIMIT         0x52
#define PMBUS_CMD_UT_FAULT_LIMIT        0x53
#define PMBUS_CMD_FAN_COMMAND_1         0x3b
#define PMBUS_CMD_FAN_COMMAND_2         0x3c
#define PMBUS_CMD_FAN_COMMAND_3         0x3e
#define PMBUS_CMD_FAN_COMMAND_4         0x3f
#define PMBUS_CMD_OT_FAULT_LIMIT        0x4f
#define PMBUS_CMD_OT_FAULT_RESPONSE     0x50
#define PMBUS_CMD_OT_WARN_LIMIT         0x51
#define PMBUS_CMD_UT_WARN_LIMIT         0x52
#define PMBUS_CMD_UT_FAULT_LIMIT        0x53
#define PMBUS_CMD_UT_FAULT_RESPONSE     0x54
#define PMBUS_CMD_READ_TEMPERATURE_1    0x8d
#define PMBUS_CMD_READ_TEMPERATURE_2    0x8e
#define PMBUS_CMD_READ_TEMPERATURE_3    0x8f
#define PMBUS_CMD_READ_FAN_SPEED_1      0x90
#define PMBUS_CMD_READ_FAN_SPEED_2      0x91
#define PMBUS_CMD_READ_FAN_SPEED_3      0x92
#define PMBUS_CMD_READ_FAN_SPEED_4      0x93
#define PMBUS_CMD_READ_POUT             0x96
#define PMBUS_CMD_READ_PIN              0x97
#define PMBUS_CMD_PMBUS_REVISION        0x98
#define PMBUS_CMD_MFR_ID                0x99
#define PMBUS_CMD_MFR_REVISION          0x9b
#define PMBUS_CMD_MFR_SERIAL            0x9e
#define PMBUS_CMD_MFR_POUT_MAX          0xa7
#define PMBUS_CMD_UNDEFINE =            0xff

#define PMBUS_DEFAULT_BLOCK_DATA_AMOUNT 32

#define TWO_COMPLEMENT(x,y)     (((x) & (1 << ((y)-1)))? (int)((x) - (1 << (y))) : (x))
#define N_VALUE(code)           TWO_COMPLEMENT((((code) & 0xF800) >> 11), 5)
#define Y_VALUE(code)           TWO_COMPLEMENT(((code) & 0x7FF), 11)
#define LINEAR_VALUE(y,n)       (UINT16)(((y) & 0x7FF) | ((n) << 11))

#ifndef FAN1_NAME
#define FAN1_NAME   NULL
#endif
#ifndef FAN2_NAME
#define FAN2_NAME   NULL
#endif
#ifndef FAN3_NAME
#define FAN3_NAME   NULL
#endif
#ifndef FAN4_NAME
#define FAN4_NAME   NULL
#endif

#ifndef TEMP1_NAME
#define TEMP1_NAME   NULL
#endif
#ifndef TEMP2_NAME
#define TEMP2_NAME   NULL
#endif
#ifndef TEMP3_NAME
#define TEMP3_NAME   NULL
#endif

#ifndef OT_FAULT_LIMIT_NAME
#define OT_FAULT_LIMIT_NAME NULL
#endif
#ifndef OT_WARN_LIMIT_NAME
#define OT_WARN_LIMIT_NAME  NULL
#endif
#ifndef UT_FAULT_LIMIT_NAME
#define UT_FAULT_LIMIT_NAME NULL
#endif
#ifndef UT_WARN_LIMIT_NAME
#define UT_WARN_LIMIT_NAME  NULL
#endif

class PMBusPSU : public BaseClassPSUDDAPI
{
public:
                        PMBusPSU(void);
    virtual            ~PMBusPSU(void);

    ErrorMsg*           getFanList(list<string> *out_pList);
    ErrorMsg*           getFanSpeed(string  in_strDeviceName,
                                    INT32   *out_s32pValue);
    ErrorMsg*           getMaxPower(string *out_strpValue);
    ErrorMsg*           getTemperature(string   in_strDeviceName,
                                       FLOAT    *out_fpValue);
    ErrorMsg*           getTemperatureList(list<string> *out_pList);
    ErrorMsg*           getTemperatureThreshold(string  in_strDeviceName,
                                                FLOAT   *out_fpValue);
    ErrorMsg*           getTemperatureThresholdList(list<string> *out_pList);
    ErrorMsg*           getType(string *out_strpValue);

    ErrorMsg*           isPresent(BOOL *out_bpPresent);

    ErrorMsg*           setFanSpeed(string  in_strDeviceName,
                                    INT32   in_s32Value);
    ErrorMsg*           setTemperatureThreshold(string   in_strDeviceName,
                                                FLOAT    in_fValue);
protected:
    UINT16              transformHumanToLinear(FLOAT in_fValue);
    FLOAT               transformLinearToHuman(UINT16 in_u16Data);

    I2CDAPI             *m_pI2CDAPI;
private:
};

#endif

