#include "Temperature_CPU.hpp"



Temperature_CPU :: Temperature_CPU(STRING  ClassName) : TemperatureGeneric(ClassName)
{
	m_ProcessorDdapi = new ProcessorDDAPI("TEMP_CPU");
	if(m_ProcessorDdapi != NULL)
	{
		//m_PSUDdapi->generateSATAList();
	}
}



Temperature_CPU :: ~Temperature_CPU()
{
    if(m_ProcessorDdapi != NULL)
        delete m_ProcessorDdapi;
}

ErrorMsg*  Temperature_CPU :: getTemperatureNameList(list<string> *pTemperatureList)
{
    //printf("%s\n", __PRETTY_FUNCTION__);;
    ErrorMsg*   pErrorMsg = NO_ERROR_MSG;
    INT32	CPU_number;

    string tempString;
    tempString.clear();

    pErrorMsg = m_ProcessorDdapi->GetCPUPhysicalNumber(&CPU_number);

    if(pErrorMsg == NO_ERROR_MSG)
    {
        for (int i = 0; i < CPU_number; i++)
        {
        	tempString="Processor";
        	tempString.append(to_string(i));
            pTemperatureList->push_back(tempString);
            tempString.clear();
        }
    }
    else
    {
    	//printError(error_code);
    	//return -1;
    }



    return pErrorMsg;
}


ErrorMsg*  Temperature_CPU :: getDataByName(STRING WellKnownName,
										    TEMPERATURE_DATA_ELEMENT TemperatureDataElement,
                                            list<string> *returnDataInfo)
{

    //ErrorMsg*   pErrorMsg = NO_ERROR_MSG;

	switch(TemperatureDataElement)
	{
		case TEMPERATURE_DATA_ALL_INFO:
			break;

		case TEMPERATURE_DATA_CURRENT_TEMPERATURE:
			getCurrentTemperatureList(WellKnownName, returnDataInfo);
			//pErrorMsg = getCurrentTemperatureList(WellKnownName, returnDataInfo);
			break;

		case TEMPERATURE_DATA_THRESHOLD_HIGH0:
			getThresholdHigh0List(WellKnownName, returnDataInfo);
			//pErrorMsg = getThresholdHigh0List(WellKnownName, returnDataInfo);
			break;

		case TEMPERATURE_DATA_THRESHOLD_HIGH1:
			getThresholdHigh1List(WellKnownName, returnDataInfo);
			//pErrorMsg = getThresholdHigh1List(WellKnownName, returnDataInfo);
			break;

		case TEMPERATURE_DATA_THRESHOLD_LOW0:
			getThresholdLow0List(WellKnownName, returnDataInfo);
			//pErrorMsg = getThresholdLow0List(WellKnownName, returnDataInfo);
			break;

		case TEMPERATURE_DATA_THRESHOLD_LOW1:
			getThresholdLow1List(WellKnownName, returnDataInfo);
			//pErrorMsg = getThresholdLow1List(WellKnownName, returnDataInfo);
			break;

		default:
			//return -1;
			break;
	}

    return NO_ERROR_MSG;
}

ErrorMsg*  Temperature_CPU :: setDataByName(STRING WellKnownName,
                         TEMPERATURE_DATA_ELEMENT TemperatureDataElement,
						 string SetValue)
{
    //ErrorMsg*   pErrorMsg = NO_ERROR_MSG;

	switch(TemperatureDataElement)
	{
		case TEMPERATURE_DATA_ALL_INFO:
			break;

		case TEMPERATURE_DATA_CURRENT_TEMPERATURE:

			break;

		case TEMPERATURE_DATA_THRESHOLD_HIGH0:
			setThresholdHigh0List(WellKnownName, SetValue);
			//pErrorMsg = setThresholdHigh0List(WellKnownName, SetValue);
			break;

		case TEMPERATURE_DATA_THRESHOLD_HIGH1:
			setThresholdHigh1List(WellKnownName, SetValue);
			//pErrorMsg = setThresholdHigh1List(WellKnownName, SetValue);
			break;

		case TEMPERATURE_DATA_THRESHOLD_LOW0:
			setThresholdLow0List(WellKnownName, SetValue);
			//pErrorMsg = setThresholdLow0List(WellKnownName, SetValue);
			break;

		case TEMPERATURE_DATA_THRESHOLD_LOW1:
			setThresholdLow1List(WellKnownName, SetValue);
			//pErrorMsg = setThresholdLow1List(WellKnownName, SetValue);
			break;

		default:
			//return -1;
			break;
	}

    return NO_ERROR_MSG;
}






ErrorMsg* Temperature_CPU :: getCurrentTemperatureList(STRING WellKnownName, list<string> *pTemperatureList)
{
    //printf("%s\n", __PRETTY_FUNCTION__);

	ERRORCODE errorCode = NO_ERROR;

    ErrorMsg*   pErrorMsg = NO_ERROR_MSG;
    string tempString;
    tempString.clear();

    //printf("WellKnownName=%s\n", WellKnownName);

	pErrorMsg = m_ProcessorDdapi->getTemperatureDataByID(WellKnownName, CPU_CURRENT_TEMPERATURE, &tempString);
    //pErrorMsg = m_I2CDdapi->getTemperatureDataByID("Tmpe01", CURRENT_TEMPERATURE, &tempString);

	//printf("I2C : %s\n", tempString.c_str());
	if (pErrorMsg != NO_ERROR_MSG)
    {
		return returnErrorMsg((ERRORCODE)(ERROR_TEMPERATURE_DDAPI_GET_CURRENT_TEMPERATURE | errorCode), NULL, pErrorMsg);
    }
    else
    {
    	pTemperatureList->push_back(tempString);
    }

    return NO_ERROR_MSG;
}


ErrorMsg*   Temperature_CPU :: getThresholdHigh0List(STRING WellKnownName,
                                                     list<string> *returnDataInfo)
{
    //printf("%s\n", __PRETTY_FUNCTION__);;
    ErrorMsg*   pErrorMsg = NO_ERROR_MSG;
    string tempString;
    tempString.clear();

    //printf("WellKnownName=%s\n", WellKnownName);

	pErrorMsg = m_ProcessorDdapi->getTemperatureDataByID(WellKnownName, CPU_THRESHOLD_HIGH, &tempString);
    //pErrorMsg = m_I2CDdapi->getTemperatureDataByID("Tmpe01", CURRENT_TEMPERATURE, &tempString);

	//printf("I2C : %s\n", tempString.c_str());
	if (pErrorMsg != NO_ERROR_MSG)
    {
        //return returnErrorMsg(ERROR_TEMPERATURE_DAPI_DISPLAY_TEMPERATURE_LIST, NULL, m_errorCode);
    }
    else
    {
    	returnDataInfo->push_back(tempString);
    }

    return NO_ERROR_MSG;
}


ErrorMsg*   Temperature_CPU :: getThresholdLow0List(STRING WellKnownName,
                                                    list<string> *returnDataInfo)
{
    //printf("%s\n", __PRETTY_FUNCTION__);;
    ErrorMsg*   pErrorMsg = NO_ERROR_MSG;
    string tempString;
    tempString.clear();

    //printf("WellKnownName=%s\n", WellKnownName);

	pErrorMsg = m_ProcessorDdapi->getTemperatureDataByID(WellKnownName, CPU_THRESHOLD_LOW, &tempString);
    //pErrorMsg = m_I2CDdapi->getTemperatureDataByID("Tmpe01", CURRENT_TEMPERATURE, &tempString);

	//printf("I2C : %s\n", tempString.c_str());
	if (pErrorMsg != NO_ERROR_MSG)
    {
        //return returnErrorMsg(ERROR_TEMPERATURE_DAPI_DISPLAY_TEMPERATURE_LIST, NULL, m_errorCode);
    }
    else
    {
    	returnDataInfo->push_back(tempString);
    }

    return NO_ERROR_MSG;
}





ErrorMsg*   Temperature_CPU :: getThresholdHigh1List(STRING WellKnownName,
                                                     list<string> *returnDataInfo)
{
    //printf("%s\n", __PRETTY_FUNCTION__);
/*

    ErrorMsg*   pErrorMsg = NO_ERROR_MSG;
    string tempString;
    tempString.clear();

    pErrorMsg = m_PSUDdapi->getDataByName(WellKnownName,  eDriveTemperature, &tempString);

    //printf("tempString=%s\n", tempString.c_str());
    if (pErrorMsg != NO_ERROR_MSG)
    {
        //return returnErrorMsg(ERROR_TEMPERATURE_DAPI_DISPLAY_TEMPERATURE_LIST, NULL, m_errorCode);
    }

    returnDataInfo->push_back(tempString);
*/

    return NO_ERROR_MSG;
}


ErrorMsg*   Temperature_CPU :: getThresholdLow1List(STRING WellKnownName,
                                                    list<string> *returnDataInfo)
{
    //printf("%s\n", __PRETTY_FUNCTION__);

/*

    ErrorMsg*   pErrorMsg = NO_ERROR_MSG;
    string tempString;
    tempString.clear();

    pErrorMsg = m_PSUDdapi->getDataByName(WellKnownName,  eDriveTemperature, &tempString);

    //printf("tempString=%s\n", tempString.c_str());
    if (pErrorMsg != NO_ERROR_MSG)
    {
        //return returnErrorMsg(ERROR_TEMPERATURE_DAPI_DISPLAY_TEMPERATURE_LIST, NULL, m_errorCode);
    }

    returnDataInfo->push_back(tempString);
*/

    return NO_ERROR_MSG;
}
ErrorMsg*   Temperature_CPU :: setThresholdHigh0List(STRING WellKnownName, string SetValue)
{
    //printf("%s\n", __PRETTY_FUNCTION__);;
    ErrorMsg*   pErrorMsg = NO_ERROR_MSG;
    string tempString;
    tempString.clear();

	pErrorMsg = m_ProcessorDdapi->setTemperatureDataByID(WellKnownName, CPU_THRESHOLD_HIGH, SetValue);
    //pErrorMsg = m_I2CDdapi->getTemperatureDataByID("Tmpe01", CURRENT_TEMPERATURE, &tempString);

	//printf("I2C : %s\n", tempString.c_str());
	if (pErrorMsg != NO_ERROR_MSG)
    {
        //return returnErrorMsg(ERROR_TEMPERATURE_DAPI_DISPLAY_TEMPERATURE_LIST, NULL, m_errorCode);
    }

    return NO_ERROR_MSG;
}

ErrorMsg*   Temperature_CPU :: setThresholdLow0List(STRING WellKnownName, string SetValue)
{
    ErrorMsg*   pErrorMsg = NO_ERROR_MSG;
    string tempString;
    tempString.clear();

    //printf("WellKnownName=%s\n", WellKnownName);

	pErrorMsg = m_ProcessorDdapi->setTemperatureDataByID(WellKnownName, CPU_THRESHOLD_LOW, SetValue);
    //pErrorMsg = m_I2CDdapi->getTemperatureDataByID("Tmpe01", CURRENT_TEMPERATURE, &tempString);

	//printf("I2C : %s\n", tempString.c_str());
	if (pErrorMsg != NO_ERROR_MSG)
    {
        //return returnErrorMsg(ERROR_TEMPERATURE_DAPI_DISPLAY_TEMPERATURE_LIST, NULL, m_errorCode);
    }

    return NO_ERROR_MSG;
}
ErrorMsg*   Temperature_CPU :: setThresholdHigh1List(STRING WellKnownName, string SetValue)
{
    return NO_ERROR_MSG;
}
ErrorMsg*   Temperature_CPU :: setThresholdLow1List(STRING WellKnownName, string SetValue)
{
    return NO_ERROR_MSG;
}

