#include "Temperature_SAS.hpp"



Temperature_SAS :: Temperature_SAS(STRING  ClassName) : TemperatureGeneric(ClassName)
{
	m_SASDdapi = new SASDDAPI("TEMP_SAS");
	if(m_SASDdapi != NULL)
	{

	}

}



Temperature_SAS :: ~Temperature_SAS()
{
	if( m_SASDdapi != NULL)
		delete m_SASDdapi;
}

ErrorMsg*  Temperature_SAS :: getTemperatureNameList(list<string> *pTemperatureList)
{
    //printf("%s\n", __PRETTY_FUNCTION__);;
    ErrorMsg* errorMsg = NULL;

    string tempString;
    tempString.clear();

    vector<SAS_CONTROLLER_INFO> returnList;
    errorMsg = m_SASDdapi->getControllerList(&returnList);

    if (errorMsg == NO_ERROR_MSG)
    {
    	for(vector<SAS_CONTROLLER_INFO>::iterator iter = returnList.begin(); iter != returnList.end(); ++iter)
    	{
    		//printf("Arrange: %s", iter->wellKnowName.c_str());
    		//cout << iter->wellKnowName << "(" << iter->DomindBusDeviceFunction << ")" << endl;
    		pTemperatureList->push_back(iter->wellKnowName);
    	}
    }


    return errorMsg;
}




ErrorMsg*  Temperature_SAS :: getDataByName(STRING WellKnownName,
                                               TEMPERATURE_DATA_ELEMENT TemperatureDataElement,
                                               list<string> *returnDataInfo)
{
    //printf("%s\n", __PRETTY_FUNCTION__);


    ErrorMsg*   pErrorMsg = NO_ERROR_MSG;
/*

    printf("%s\n", __func__);
    printf("WellKnownName:%s\n", WellKnownName);
    printf("TemperatureDataElement:%d\n", TemperatureDataElement);
*/

    switch(TemperatureDataElement)
    {
        case TEMPERATURE_DATA_ALL_INFO:
            //TemperatureDataElement=TEMPERATURE_DATA_ALL_INFO;
            break;
        case TEMPERATURE_DATA_CURRENT_TEMPERATURE:
            pErrorMsg = getCurrentTemperatureList(WellKnownName, returnDataInfo);
            break;

        case TEMPERATURE_DATA_THRESHOLD_HIGH0:
            pErrorMsg = getThresholdHigh0List(WellKnownName, returnDataInfo);
            break;

        case TEMPERATURE_DATA_THRESHOLD_HIGH1:
            pErrorMsg = getThresholdHigh1List(WellKnownName, returnDataInfo);
            break;

        case TEMPERATURE_DATA_THRESHOLD_LOW0:
            pErrorMsg = getThresholdLow0List(WellKnownName, returnDataInfo);
            break;

        case TEMPERATURE_DATA_THRESHOLD_LOW1:
            pErrorMsg = getThresholdLow1List(WellKnownName, returnDataInfo);
            break;

        default:
            //return -1;
            break;
    }

    return pErrorMsg;

}





ErrorMsg* Temperature_SAS :: getCurrentTemperatureList(STRING WellKnownName,
                                                        list<string> *returnDataInfo)
{
    ErrorMsg* pErrorMsg = NO_ERROR_MSG;
    ERRORCODE errorCode = NO_ERROR;
    string tempString;
    string tempWellKnowName;
    tempString.clear();
    tempWellKnowName.clear();
    UINT8 controllerID = 0xff;
    UINT32 returnTemperature;

    vector<SAS_CONTROLLER_INFO> returnList;
    pErrorMsg = m_SASDdapi->getControllerList(&returnList);

	if (pErrorMsg == NO_ERROR_MSG)
    {
		for(vector<SAS_CONTROLLER_INFO>::iterator iter = returnList.begin(); iter != returnList.end(); ++iter)
       	{
			tempWellKnowName=iter->wellKnowName;
			transform(tempWellKnowName.begin(), tempWellKnowName.end(), tempWellKnowName.begin(),(int (*)(int))tolower);
       		if(strcmp(tempWellKnowName.c_str(), WellKnownName) == 0)
       		{
       			controllerID = iter - returnList.begin();
       			break;
       		}
       	}
	}
	if(controllerID == 0xff)
	{
		//below error need modify
		return returnErrorMsg(ERRORCODE(ERROR_TEMPERATURE_DAPI_GET_CURRENT_TEMPERATURE), NULL, pErrorMsg);

	}

    m_SASDdapi->setControllerID(controllerID);

    pErrorMsg = m_SASDdapi->doIOCTLCommand(SASIOCTL::CURRENT_TEMP, &returnTemperature);

    //printf("returnTemperature=%d\n", returnTemperature);
    //pErrorMsg = m_SASDdapi->getDataByName(WellKnownName,  eDriveTemperature, &tempString);

    //printf("tempString=%s\n", tempString.c_str());
    if (pErrorMsg != NO_ERROR_MSG)
    {
    	return returnErrorMsg((ERRORCODE)(ERROR_TEMPERATURE_DDAPI_GET_CURRENT_TEMPERATURE | errorCode), NULL, pErrorMsg);
    }
    else
    {
    	returnDataInfo->push_back(std::to_string(returnTemperature));
    }

    return NO_ERROR_MSG;
}



ErrorMsg* Temperature_SAS :: getThresholdHigh0List(STRING WellKnownName,
												   list<string> *returnDataInfo)
{

    return NO_ERROR_MSG;
}


ErrorMsg*   Temperature_SAS :: getThresholdLow0List(STRING WellKnownName,
                                                    list<string> *returnDataInfo)
{

    return NO_ERROR_MSG;
}





ErrorMsg*   Temperature_SAS :: getThresholdHigh1List(STRING WellKnownName,
                                                     list<string> *returnDataInfo)
{

    return NO_ERROR_MSG;
}


ErrorMsg*   Temperature_SAS :: getThresholdLow1List(STRING WellKnownName,
                                                    list<string> *returnDataInfo)
{

    return NO_ERROR_MSG;
}



