#include "TPMDDAPI.hpp"

TPMDDAPI::TPMDDAPI(STRING className) : BaseClassDDAPI(className)
{

}

TPMDDAPI::~TPMDDAPI()
{

}


ERRORCODE
TPMDDAPI::openDevice()
{
    //cout << "TPMDDAPI::openDevice" << endl;//check 

    m_tpmDevice = open(TPM_DEV, O_RDWR);
    if(m_tpmDevice < 0)
    {
        return ERROR_OPEN_FILE;
    }

    return NO_ERROR;
}

void
TPMDDAPI::closeDevice()
{
    //cout << "TPMDDAPI::closeDevice" << endl;//check 

    if(m_tpmDevice != 0)
    {
        close(m_tpmDevice);
        m_tpmDevice = -1;
    }
}


ErrorMsg*
TPMDDAPI::commandInterface(UINT8*   in_u8pCmdBuffer,
                           UINT32   in_u32CmdBufferSize,
                           UINT8*   out_u8pRspBuffer,
                           UINT32   in_u32RspBufferSize,
                           UINT32*  out_u32pRspSize) 
{
    int     s32AccessSize = 0;

#if 0
    //check 
    cout << "TPMDDAPI::commandInterface, check cmdList:" << endl;
    for(UINT32 u32Index = 0; u32Index < in_u32CmdBufferSize; u32Index++)
    {
        printf("in_u8pCmdBuffer[%d] - 0x%02x\n", u32Index, in_u8pCmdBuffer[u32Index]);
    }
#endif 

    m_eErrorCode = openDevice();
    if(m_eErrorCode != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_TPM_DDAPI_ACCESS_CMD, "Can't open device", NULL);  
    }

    //Send request data to TPM 
    s32AccessSize = write(m_tpmDevice, in_u8pCmdBuffer, in_u32CmdBufferSize);
    if(s32AccessSize == ERR_COMMUNICATION || in_u32CmdBufferSize != (UINT32)s32AccessSize)
    {
        closeDevice();
        return returnErrorMsg(ERROR_TPM_DDAPI_ACCESS_CMD, "Send command fail", NULL);
    }

    //Read the TPM response header
    s32AccessSize = read(m_tpmDevice, out_u8pRspBuffer, in_u32RspBufferSize);
    if(s32AccessSize == ERR_COMMUNICATION)
    {
        closeDevice();
        return returnErrorMsg(ERROR_TPM_DDAPI_ACCESS_CMD, "Get response fail", NULL);
    }
    *out_u32pRspSize = (UINT32)s32AccessSize;

#if 0
    //check
    cout << "------------------------------------------" << endl;
    cout << "TPMDDAPI::commandInterface, check rspList:" << endl;
    for(UINT32  u32Index = 0; u32Index < *out_u32pRspSize; u32Index++)
    {
        printf("out_u8pRspBuffer[%d] - 0x%02x\n", u32Index, out_u8pRspBuffer[u32Index]);
    }
    cout << endl << endl;
#endif 

    closeDevice();
    return NO_ERROR_MSG;
}


ErrorMsg*
TPMDDAPI::getCmdSize(TPM_CMD_INPUT  in_cmdList,
                     UINT32*        out_u32pCmdSize)
{
    switch(in_cmdList.u32CmdCode)
    {
        case TPM_CC_SELF_TEST:
            *out_u32pCmdSize = TPM_HEADER_SIZE + 1;
            break;

        case TPM_CC_GET_CAPABILITY:
            *out_u32pCmdSize = TPM_HEADER_SIZE + 12;
            break;

        default:
            break;                        
    }

    return NO_ERROR_MSG;
}


ErrorMsg*
TPMDDAPI::createCmdList(TPM_CMD_INPUT     in_cmdList,
                        UINT8*            out_u8pCmdList)
{
    UINT32  u32Temp;
    
    //header - tag
    out_u8pCmdList[0] = (in_cmdList.u16Tag & 0xff00) >> 8;
    out_u8pCmdList[1] = (in_cmdList.u16Tag & 0xff);
    //header - command size
    converDWordToBytes(in_cmdList.u32CmdSize, out_u8pCmdList + 2);
    //header - command code 
    converDWordToBytes(in_cmdList.u32CmdCode, out_u8pCmdList + 6);

    //parameters (if any)
    switch(in_cmdList.u32CmdCode)
    {
        case TPM_CC_SELF_TEST:
            out_u8pCmdList[10] = in_cmdList.uParm1.u8YesNoData;
            break;

        case TPM_CC_GET_CAPABILITY:
            //capability (TPM_CAP)
            u32Temp = in_cmdList.uParm1.u32Capability;
            converDWordToBytes(u32Temp, out_u8pCmdList + 10);
            //property 
            u32Temp = in_cmdList.uParm2.u32Value;
            converDWordToBytes(u32Temp, out_u8pCmdList + 14);
            //propertyCount
            u32Temp = in_cmdList.uParm3.u32Value;
            converDWordToBytes(u32Temp, out_u8pCmdList + 18);

        default:
            break;

    }

    return NO_ERROR_MSG;
}


void 
TPMDDAPI::converDWordToBytes(UINT32     in_u32Data,
                             UINT8*     out_u8pBuffer)
{
    //printf("TPMDDAPI::converDWordToBytes, in_u32Data - 0x%08x\n", in_u32Data);//check 

    out_u8pBuffer[0] = (in_u32Data & 0xff000000) >> 24;
    out_u8pBuffer[1] = (in_u32Data & 0x00ff0000) >> 16;
    out_u8pBuffer[2] = (in_u32Data & 0x0000ff00) >> 8;
    out_u8pBuffer[3] = (in_u32Data & 0xff);

    //check 
    /*
    UINT32 u32Index;
    for(u32Index = 0; u32Index < 4; u32Index++)
    {
        printf("TPMDDAPI::converDWordToBytes, out_u8pBuffer[%d] - 0x%02x\n", u32Index, out_u8pBuffer[u32Index]);
    }
    */
}

#if 0
void
TPMDDAPI::convertStringToDWord(char*    in_string,
                               UINT32*  out_u32pData)
{
    UINT8   u8Buffer[4];
    char    temp[4];

    sprintf(temp, "%c%c%c%c", in_string[0], in_string[1], in_string);
    sprintf(temp, "%c%c", in_string[0], in_string[1]);
    u8Buffer[0] = strtol(temp, NULL, 16);
    sprintf(temp, "%c%c", in_string[3], in_string[4]);
    u8Buffer[1] = strtol(temp, NULL, 16);
    sprintf(temp, "%c%c", in_string[6], in_string[7]);
    u8Buffer[2] = strtol(temp, NULL, 16);
    sprintf(temp, "%c%c", in_string[9], in_string[10]);
    u8Buffer[3] = strtol(temp, NULL, 16);

    //check 
    /*
    int s32Index;
    for(s32Index = 0; s32Index < 4; s32Index++)
    {
        printf("u8Buffer[%d] - 0x%02x\n", s32Index, u8Buffer[s32Index]);
    }
    */

    *out_u32pData = (u8Buffer[0] << 24) | (u8Buffer[1] << 16) | (u8Buffer[2] << 8) | u8Buffer[3];
}
#endif


ErrorMsg*
TPMDDAPI::sendCommandManually(STRING    in_cmdFileName,
                              UINT8*    out_u8pRspBuffer,
                              UINT32    in_u32RspBufferSize,
                              UINT32*   out_u32pRspSize)
{
    FILE*   cmdFile;
    bool    bBufferAllocated = FALSE;
    char    temp[8];
    char    line[TPM_MAX_STRING_LENGTH];
    int     s32LineNum = 0;
    UINT8*  u8pCmdBuffer;
    UINT8   u8CmdTagBuffer[2];
    UINT8   u8Temp;
    UINT32  u32Index;
    UINT32  u32CmdBufferSize;
    UINT32  u32StringLength;
    UINT32  u32CmdBufferIndex = 0;

    //printf("in_cmdFileName: %s\n", in_cmdFileName);//check
    cmdFile = fopen(in_cmdFileName, "r");
    if(!cmdFile)
    {
        return returnErrorMsg(ERROR_TPM_DDAPI_SEND_CMD_MANUALLY, "fail to open file", NULL);
    }

    u32CmdBufferIndex = 2; //position 0 and 1 are for Tag 
    while(fgets(line, sizeof(line), cmdFile) != NULL)
    {
        u32StringLength = strlen(line);

        if(s32LineNum == 0) //TPMI_ST_COMMAND_TAG line
        {
            if(u32StringLength - 1 != 5) //check tag size 
            {
                fclose(cmdFile);
                return returnErrorMsg(ERROR_TPM_DDAPI_SEND_CMD_MANUALLY, "incorrect command length", NULL);
            }
            sprintf(temp,"%c%c", line[0],line[1]);
            u8Temp = strtol(temp, NULL, 16);
            u8CmdTagBuffer[0] = u8Temp;
            sprintf(temp,"%c%c", line[3],line[4]);
            u8Temp = strtol(temp, NULL, 16);
            u8CmdTagBuffer[1] = u8Temp;
        }
        
        if(s32LineNum == 1)
        {
            bBufferAllocated = TRUE;
            sprintf(temp, "%c%c%c%c%c%c%c%c", line[0],line[1],line[3],line[4],line[6],line[7],line[9],line[10]);
            u32CmdBufferSize = strtol(temp, NULL, 16);
            u8pCmdBuffer = (UINT8*)malloc(sizeof(UINT8) * u32CmdBufferSize);
        }
        
        if(s32LineNum >= 1) 
        {
            for(u32Index = 0; u32Index < u32StringLength; u32Index += 3)
            {
                if(u32Index > u32StringLength)
                {
                    break;
                }
                sprintf(temp,"%c%c", line[u32Index], line[u32Index + 1]);
                u8Temp = strtol(temp, NULL, 16);
                u8pCmdBuffer[u32CmdBufferIndex] = u8Temp;
                u32CmdBufferIndex++;
            }
        }
        s32LineNum++;
    }//while 

    fclose(cmdFile);

    if(bBufferAllocated && u32CmdBufferSize > 2)
    {
        //tag 
        u8pCmdBuffer[0] = u8CmdTagBuffer[0];
        u8pCmdBuffer[1] = u8CmdTagBuffer[1];

        //send command 
        *out_u32pRspSize = 0;
        m_pErrorMsg = commandInterface(u8pCmdBuffer, u32CmdBufferSize, out_u8pRspBuffer, in_u32RspBufferSize, out_u32pRspSize);
        
        free(u8pCmdBuffer);
    }

    if(m_pErrorMsg != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_TPM_DDAPI_SEND_CMD_MANUALLY, NULL, m_pErrorMsg);
    }
    return m_pErrorMsg;
}
