#ifndef USB_DDAPI_H
#define USB_DDAPI_H

#include "BaseClassDDAPI.hpp"
//#include "USBDevice.hpp"

//#include "USBSwitch.hpp"
//#include "USBBus.hpp"
#include <sys/stat.h>

#include <dirent.h>
#include <unistd.h>

#include <fcntl.h>
#include <sys/time.h>
#include "CommonFunctions.hpp"
//#include <algorithm>
#include <list>
#include "SCSIFactory.hpp"
#include <errno.h>
#include <sys/ioctl.h>


using namespace std;

#define DEFAULT_USB_TEST_SIZE   20
#define DEFAULT_USB_BLOCK_SIZE  512l
#define MAX_USB_DEVICE_NO       256
#define USB_DEVICE_NAME_LENGTH  60
#define USB_ID_UNDEFINED        0xFFFF

#define USB_SYSTEM_BUS_PATH     "/sys/bus/usb/devices/"
#define USB_SYSTEM_BLPCK_PATH   "/sys/block/"

#define MEGA_BYTES_SIZE         (1024*1024)
#define USB_BLOCK_SIZE          (512)
#define UINT_MAX                (2147483647 * 2U + 1U)




typedef enum
{
    USB_DATA_ALL_INFO=0,
    USB_DATA_NODE_INFO,
    USB_DATA_SN_INFO,
    USB_DATA_VENDOR_INFO,
    USB_DATA_CAPACITY_INFO,
    USB_DATA_LINK_INFO,
    USB_DATA_SPEED_INFO,
	USB_DATA_PHYSICAL_BLOCK_SIZE_INFO,
	USB_DATA_LOGICAL_BLOCK_SIZE_INFO,
	USB_DATA_SG_NODE_INFO,


    MAX_DATA_ELEMENT_NUMBER,
}USB_DATA_ELEMENT;

typedef enum
{
    USB_AUDIO=1,
    USB_COMMUMICATION,
    USB_HID,

    USB_PHYSICAL=5,
    USB_IMAGE,
    USB_PRINTER,
    USB_STORAGE,
    USB_HUB,
    USB_CDC_DATA,
    USB_SMART_CARD,

    USB_VENDOR_SPEC=255,
}USB_INTERFACE_TYPE;

class USBDDAPI : public BaseClassDDAPI
{
public:
                     USBDDAPI(STRING className);
                    ~USBDDAPI();

                     ErrorMsg*  getUSBDevicesList(list<string> *pUSBList);
                     ErrorMsg*  getIndexByName(STRING WellKnownName, UINT32 *Index);
                     ErrorMsg*  getElementByIndex(UINT32 Index,
                                            USB_DATA_ELEMENT USBDataElement,
                                            list<string> *returnDataInfo);

                     ErrorMsg*   PREAD(UINT32  fileDescription,
                                       UINT32* buffer,
                                       UINT32  logicalSector,
                                       UINT32  testSize);

                     ErrorMsg*   PWRITE(UINT32  fileDescription,
                                        UINT32* buffer,
                                        UINT32  logicalSector,
                                        UINT32  testSize);

                     ErrorMsg*   SCSIRead(UINT32  fileDescription,
                                          UINT8  *cdbP,
                                          UINT8  *buffer,
                                          UINT32  bufferSize);

                     ErrorMsg*   SCSIWrite(UINT32  fileDescription,
                                           UINT8  *cdbP,
                                           UINT8  *buffer,
                                           UINT32  bufferSize);


protected:
                     ErrorMsg* m_errorCode;

private:             typedef struct UsbConfiguration{
                             UINT32 Index;
                             CHAR   WellKnownName[(USB_DEVICE_NAME_LENGTH + 1)];
                             CHAR   DeviceNode[(USB_DEVICE_NAME_LENGTH + 1)];
                             CHAR   BusPortAddress[(USB_DEVICE_NAME_LENGTH + 1)];
                             //CHAR   USBDeviceType[(USB_DEVICE_NAME_LENGTH + 1)];
                     }USB_CONFIGURATION;

                     typedef struct ElementMapping{
                             USB_DATA_ELEMENT   DateElement;
                             CHAR               FileName[(USB_DEVICE_NAME_LENGTH + 1)];
                     }USB_ELEMENT_MAP;


                     list<string> USBDeviceBusPortList;
                     UINT8     m_senseBuff[SCSI_SENSE_BUFFERSIZE];
                     USB_CONFIGURATION USBConfig[MAX_USB_DEVICE_NO];
                     USB_ELEMENT_MAP   USBElementMap[MAX_DATA_ELEMENT_NUMBER];

                     ErrorMsg*  getUSBSysBusAddress(UINT32   Index,
                                                    string* returnDataInfo);

                     ErrorMsg*  searchUSBInterfaceAddress(list<string> *returnDataInfo);

                     ErrorMsg*  confirmUSBDeviceType(list<string> *USBInterfaceAddressList,
                                                     USB_CONFIGURATION *returnUSBConfiguration);
                     ErrorMsg*  getUSBDeviceInterfaceElement(string InterfaceAddress,
                                                             string strType,
                                                             string *returnString);


                     ErrorMsg*  addUSBDeviceEntry(UINT32 Index,
                                                  string WellKnownName,
                                                  string DeviceNode,
                                                  string BusPortAddress,
                                                  USB_CONFIGURATION *returnUSBConfiguration);





                     ErrorMsg*  searchUSBStorageDeviceNode(string BusPortAddress, string* DeviceNodeList);

                     // For Show node
                     ErrorMsg*  getUSBDeviceNode(UINT32 Index,
                                                 list<string>* returnDataInfo);

                     // For Show SN
                     ErrorMsg*  getUSBInfoFromSysBus(UINT32   Index,
                                                     USB_DATA_ELEMENT   DataElement,
                                                     list<string>* returnDataInfo);
                     ErrorMsg*  getUSBInfoFromSysBlock(UINT32   Index,
                                                       USB_DATA_ELEMENT   DataElement,
                                                       list<string>* returnDataInfo);
                     ErrorMsg*  getUSBSGDeviceNode(UINT32   Index,
                    		 	 	 	 	 	 USB_DATA_ELEMENT   DataElement,
												 list<string>* returnDataInfo);

};


#endif

