/*
 * PMC-Sierra SPC 8001 SAS/SATA based host adapters driver
 *
 * Copyright (c) 2008-2009 USI Co., Ltd.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions, and the following disclaimer,
 *    without modification.
 * 2. Redistributions in binary form must reproduce at minimum a disclaimer
 *    substantially similar to the "NO WARRANTY" disclaimer below
 *    ("Disclaimer") and any redistribution must be conditioned upon
 *    including a substantially similar Disclaimer requirement for further
 *    binary redistribution.
 * 3. Neither the names of the above-listed copyright holders nor the names
 *    of any contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * Alternatively, this software may be distributed under the terms of the
 * GNU General Public License ("GPL") version 2 as published by the Free
 * Software Foundation.
 *
 * NO WARRANTY
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR CONTRIBUTORS BE LIABLE FOR SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 */
#include <linux/slab.h>
#include <linux/ata.h>
#include <linux/types.h>
#include <scsi/libsas.h>
#include "pm8001_sas.h"
#include "pm80xx_hwi.h"
#include "pm8001_hwi.h"
#include "pm8001_chips.h"
#include "pm8001_ctl.h"

#define SMP_DIRECT 1
#define SMP_INDIRECT 2


int pm80xx_bar4_shift(struct pm8001_hba_info *pm8001_ha, u32 shiftValue)
{
    u32 regVal;
    unsigned long start;

    pm8001_cw32(pm8001_ha, 0, MEMBASE_II_SHIFT_REGISTER, shiftValue);
   /* confirm the setting is written */
    start = jiffies + HZ; /* 1 sec */
    do {
        regVal = pm8001_cr32(pm8001_ha, 0, MEMBASE_II_SHIFT_REGISTER);
    } while ((regVal != shiftValue) && time_before(jiffies, start));

    if (regVal != shiftValue) {
        PM8001_FAIL_DBG(pm8001_ha,
            pm8001_printk("TIMEOUT:MEMBASE_II_SHIFT_REGISTER"
            " = 0x%x\n", regVal));
        return -1;
    }
    return 0;
}



void pm80xx_pci_mem_copy(struct pm8001_hba_info  *pm8001_ha, u32 soffset, const void *dst,
                       u32 dw_count, u32 busBaseNumber)
{
    u32 index, val,offset;
    u32 *dst1;
    dst1 = (u32 *)dst;

    for (index= 0; index < dw_count; index+=4,dst1++)
    {
      offset = (soffset + index / 4);
      if(offset < (64 * 1024))
      {
        val = pm8001_cr32(pm8001_ha, busBaseNumber, offset);
        *dst1 =  cpu_to_le32(val);
      }
    }
    return;
}

ssize_t pm80xx_get_fatal_dump(struct device *cdev,
    struct device_attribute *attr, char *buf)
{
    struct Scsi_Host *shost = class_to_shost(cdev);
    struct sas_ha_struct *sha = SHOST_TO_SAS_HA(shost);
    struct pm8001_hba_info *pm8001_ha = sha->lldd_ha;
    void __iomem *fatal_table_address = pm8001_ha->fatal_tbl_addr;
    u32 *temp;
    u32 status = 1;
    u32 accum_len;
    u32 regVal;
    unsigned long start;
    u8 *direct_data;
    char* fatal_error_data = buf;
    u32 index;

    pm8001_ha->forensic_info.dataBuf.direct_data = buf;
    if(pm8001_ha->chip_id == chip_8001)
    {
      pm8001_ha->forensic_info.dataBuf.direct_data += sprintf(pm8001_ha->forensic_info.dataBuf.direct_data, "Not supported for SPC controller");
      return((char*)pm8001_ha->forensic_info.dataBuf.direct_data - (char*)buf);
    }
    if(pm8001_ha->forensic_info.dataBuf.directOffset==0)
    {
      PM8001_IO_DBG(pm8001_ha,
          pm8001_printk("forensic_info TYPE_NON_FATAL......................\n"));
      direct_data = (u8 * )fatal_error_data;
      pm8001_ha->forensic_info.DataType = TYPE_NON_FATAL;
      pm8001_ha->forensic_info.dataBuf.directLen = SYSFS_OFFSET;
      pm8001_ha->forensic_info.dataBuf.directOffset = 0; 
      pm8001_ha->forensic_info.dataBuf.readLen = 0;  

      pm8001_ha->forensic_info.dataBuf.direct_data = direct_data;
      PM8001_IO_DBG(pm8001_ha,pm8001_printk("ossaHwCB:status1 %d readLen 0x%x directLen 0x%x directOffset 0x%x\n",
        status,
      pm8001_ha->forensic_info.dataBuf.readLen,
      pm8001_ha->forensic_info.dataBuf.directLen,
      pm8001_ha->forensic_info.dataBuf.directOffset));
    }

    if(pm8001_ha->forensic_info.dataBuf.directOffset == 0)
    {
      /* start to get data */
      /*
         Program the MEMBASE II Shifting Register with 0x00.
        */
       pm8001_cw32(pm8001_ha, 0, MEMBASE_II_SHIFT_REGISTER, pm8001_ha->fatal_forensic_shift_offset);
       pm8001_ha->forensic_last_offset =0;
       pm8001_ha->forensic_fatal_step = 0;
       pm8001_ha->fatal_bar_loc = 0; 
    }
      /* Read until accum_len is retrived */
    accum_len = pm8001_mr32(fatal_table_address, MPI_FATAL_EDUMP_TABLE_ACCUM_LEN);
    PM8001_IO_DBG(pm8001_ha,pm8001_printk("get_fatal_spcv: accum_len 0x%x\n", accum_len)); 
    if(accum_len == 0xFFFFFFFF)
    {
      PM8001_IO_DBG(pm8001_ha,pm8001_printk("get_fatal_spcv: Possible PCI issue 0x%x not expected\n", accum_len)); 
      return(status);
    }
    if(accum_len == 0 || accum_len >= 0x100000)
    {
      pm8001_ha->forensic_info.dataBuf.direct_data += sprintf(pm8001_ha->forensic_info.dataBuf.direct_data, "%08x ", 0xFFFFFFFF);
      return((char*)pm8001_ha->forensic_info.dataBuf.direct_data - (char*)buf);
    }
    temp = (u32*)pm8001_ha->memoryMap.region[FORENSIC_MEM].virt_ptr;
    if(pm8001_ha->forensic_fatal_step == 0) 
    {
      moreData:
      if(pm8001_ha->forensic_info.dataBuf.direct_data )
      {
          /* Data is in bar, copy to host memory */
        pm80xx_pci_mem_copy(pm8001_ha, pm8001_ha->fatal_bar_loc, pm8001_ha->memoryMap.region[FORENSIC_MEM].virt_ptr, pm8001_ha->forensic_info.dataBuf.directLen ,1 );
      }
      pm8001_ha->fatal_bar_loc += pm8001_ha->forensic_info.dataBuf.directLen;
      pm8001_ha->forensic_info.dataBuf.directOffset += pm8001_ha->forensic_info.dataBuf.directLen;
      pm8001_ha->forensic_last_offset   += pm8001_ha->forensic_info.dataBuf.directLen;
      pm8001_ha->forensic_info.dataBuf.readLen = pm8001_ha->forensic_info.dataBuf.directLen;

      if(pm8001_ha->forensic_last_offset  >= accum_len)
      {

        pm8001_ha->forensic_info.dataBuf.direct_data += sprintf(pm8001_ha->forensic_info.dataBuf.direct_data, "%08x ", 3);
        for(index=0; index<(SYSFS_OFFSET/4); index++)
        {
          pm8001_ha->forensic_info.dataBuf.direct_data += sprintf(pm8001_ha->forensic_info.dataBuf.direct_data, "%08x ", *(temp+index));
        }

        pm8001_ha->fatal_bar_loc = 0; 
        pm8001_ha->forensic_fatal_step = 1;
        pm8001_ha->fatal_forensic_shift_offset = 0;
        pm8001_ha->forensic_last_offset = 0;
        status = 0;
        return((char*)pm8001_ha->forensic_info.dataBuf.direct_data - (char*)buf);
      }
      if(pm8001_ha->fatal_bar_loc < (64*1024))
      {
        pm8001_ha->forensic_info.dataBuf.direct_data += sprintf(pm8001_ha->forensic_info.dataBuf.direct_data, "%08x ", 2);
        for(index=0; index<(SYSFS_OFFSET/4); index++)
        {
          pm8001_ha->forensic_info.dataBuf.direct_data += sprintf(pm8001_ha->forensic_info.dataBuf.direct_data, "%08x ", *(temp+index));
        }
        status = 0;
        return((char*)pm8001_ha->forensic_info.dataBuf.direct_data - (char*)buf);
      }

      /*
         Increment the MEMBASE II Shifting Register value by 0x100.
       */
      pm8001_ha->forensic_info.dataBuf.direct_data += sprintf(pm8001_ha->forensic_info.dataBuf.direct_data, "%08x ", 2);
      for(index=0; index<256; index++)
      {
        pm8001_ha->forensic_info.dataBuf.direct_data += sprintf(pm8001_ha->forensic_info.dataBuf.direct_data, "%08x ", *(temp+index));
      }
      pm8001_ha->fatal_forensic_shift_offset += 0x100;
      pm8001_cw32(pm8001_ha, 0, MEMBASE_II_SHIFT_REGISTER,  pm8001_ha->fatal_forensic_shift_offset);
      pm8001_ha->fatal_bar_loc = 0;
      status = 0;
      return((char*)pm8001_ha->forensic_info.dataBuf.direct_data - (char*)buf);
    }  
    if(pm8001_ha->forensic_fatal_step == 1)
    {
      pm8001_ha->fatal_forensic_shift_offset = 0; /* location in 64k region */
      /*
         Read 64K of the debug data.
       */
      pm8001_cw32(pm8001_ha, 0, MEMBASE_II_SHIFT_REGISTER,  pm8001_ha->fatal_forensic_shift_offset);
      pm8001_mw32(fatal_table_address, MPI_FATAL_EDUMP_TABLE_HANDSHAKE, MPI_FATAL_EDUMP_HANDSHAKE_RDY);

      /* Poll FDDHSHK  until clear  */
      start = jiffies + (2 * HZ); /* 2 sec */

      do {
        regVal = pm8001_mr32(fatal_table_address, MPI_FATAL_EDUMP_TABLE_HANDSHAKE);
      } while ((regVal) && time_before(jiffies, start));

      if (regVal != 0) {
        PM8001_FAIL_DBG(pm8001_ha,
              pm8001_printk("TIMEOUT:MEMBASE_II_SHIFT_REGISTER"
                " = 0x%x\n", regVal));
        return -1;
      }

      /*
         Read the next 64K of the debug data.
       */
      pm8001_ha->forensic_fatal_step = 0;
      if(  pm8001_mr32(fatal_table_address, MPI_FATAL_EDUMP_TABLE_STATUS) != MPI_FATAL_EDUMP_TABLE_STAT_NF_SUCCESS_DONE )
      { 
        pm8001_mw32(fatal_table_address, MPI_FATAL_EDUMP_TABLE_HANDSHAKE, 0);
        goto moreData;
      }
      else
      {
        pm8001_ha->forensic_info.dataBuf.direct_data += sprintf(pm8001_ha->forensic_info.dataBuf.direct_data, "%08x ", 4);
        pm8001_ha->forensic_info.dataBuf.readLen = 0xFFFFFFFF;
        pm8001_ha->forensic_info.dataBuf.directLen =  0;
        pm8001_ha->forensic_info.dataBuf.directOffset = 0;
        pm8001_ha->forensic_info.dataBuf.readLen = 0;
        status = 0;
      }
    }

    return((char*)pm8001_ha->forensic_info.dataBuf.direct_data - (char*)buf);
}
/**
 * read_main_config_table - read the configure table and save it.
 * @pm8001_ha: our hba card information
 */
 #if (LINUX_VERSION_CODE < KERNEL_VERSION(3,8,0) ) 
static void __devinit 
#else
static void 
#endif
read_main_config_table(struct pm8001_hba_info *pm8001_ha)
{
    void __iomem *address = pm8001_ha->main_cfg_tbl_addr;

    pm8001_ha->main_cfg_tbl.pm80xx_tbl.signature            = pm8001_mr32(address, MAIN_SIGNATURE_OFFSET);
    pm8001_ha->main_cfg_tbl.pm80xx_tbl.interface_rev        = pm8001_mr32(address, MAIN_INTERFACE_REVISION);
    pm8001_ha->main_cfg_tbl.pm80xx_tbl.firmware_rev         = pm8001_mr32(address, MAIN_FW_REVISION);
    pm8001_ha->main_cfg_tbl.pm80xx_tbl.max_out_io           = pm8001_mr32(address, MAIN_MAX_OUTSTANDING_IO_OFFSET);
    pm8001_ha->main_cfg_tbl.pm80xx_tbl.max_sgl              = pm8001_mr32(address, MAIN_MAX_SGL_OFFSET);
    pm8001_ha->main_cfg_tbl.pm80xx_tbl.ctrl_cap_flag        = pm8001_mr32(address, MAIN_CNTRL_CAP_OFFSET);
    pm8001_ha->main_cfg_tbl.pm80xx_tbl.gst_offset           = pm8001_mr32(address, MAIN_GST_OFFSET);
    pm8001_ha->main_cfg_tbl.pm80xx_tbl.inbound_queue_offset = pm8001_mr32(address, MAIN_IBQ_OFFSET);
    pm8001_ha->main_cfg_tbl.pm80xx_tbl.outbound_queue_offset = pm8001_mr32(address, MAIN_OBQ_OFFSET);

    /* read Error Dump Offset and Length */
    pm8001_ha->main_cfg_tbl.pm80xx_tbl.fatal_err_dump_offset0 =
        pm8001_mr32(address, MAIN_FATAL_ERROR_RDUMP0_OFFSET);
    pm8001_ha->main_cfg_tbl.pm80xx_tbl.fatal_err_dump_length0 =
        pm8001_mr32(address, MAIN_FATAL_ERROR_RDUMP0_LENGTH);
    pm8001_ha->main_cfg_tbl.pm80xx_tbl.fatal_err_dump_offset1 =
        pm8001_mr32(address, MAIN_FATAL_ERROR_RDUMP1_OFFSET);
    pm8001_ha->main_cfg_tbl.pm80xx_tbl.fatal_err_dump_length1 =
        pm8001_mr32(address, MAIN_FATAL_ERROR_RDUMP1_LENGTH);

    /* read GPIO LED settings from the configuration table */
    pm8001_ha->main_cfg_tbl.pm80xx_tbl.gpio_led_mapping =
        pm8001_mr32(address, MAIN_GPIO_LED_FLAGS_OFFSET);

        /* read analog Setting offset from the configuration table */
    pm8001_ha->main_cfg_tbl.pm80xx_tbl.analog_setup_table_offset =
        pm8001_mr32(address, MAIN_ANALOG_SETUP_OFFSET);

    pm8001_ha->main_cfg_tbl.pm80xx_tbl.int_vec_table_offset =
        pm8001_mr32(address, MAIN_INT_VECTOR_TABLE_OFFSET);
    pm8001_ha->main_cfg_tbl.pm80xx_tbl.phy_attr_table_offset =
        pm8001_mr32(address, MAIN_SAS_PHY_ATTR_TABLE_OFFSET);

    PM8001_DEV_DBG(pm8001_ha,
            pm8001_printk("Main cfg table; sign:%x interface rev:%x fw rev:%x \n",
             pm8001_ha->main_cfg_tbl.pm80xx_tbl.signature, 
                         pm8001_ha->main_cfg_tbl.pm80xx_tbl.interface_rev,
             pm8001_ha->main_cfg_tbl.pm80xx_tbl.firmware_rev));

    PM8001_DEV_DBG(pm8001_ha,
            pm8001_printk("table offset; gst:%x " 
            "iq:%x oq:%x int vec:%x phy attr:%x \n",
             pm8001_ha->main_cfg_tbl.pm80xx_tbl.gst_offset, 
                         pm8001_ha->main_cfg_tbl.pm80xx_tbl.inbound_queue_offset,
             pm8001_ha->main_cfg_tbl.pm80xx_tbl.outbound_queue_offset, 
                         pm8001_ha->main_cfg_tbl.pm80xx_tbl.int_vec_table_offset,
                 pm8001_ha->main_cfg_tbl.pm80xx_tbl.phy_attr_table_offset)); 

}

/**
 * read_general_status_table - read the general status table and save it.
 * @pm8001_ha: our hba card information
 */
 #if (LINUX_VERSION_CODE < KERNEL_VERSION(3,8,0) ) 
static void __devinit
#else
static void 
#endif
read_general_status_table(struct pm8001_hba_info *pm8001_ha)
{
    void __iomem *address = pm8001_ha->general_stat_tbl_addr;
    pm8001_ha->gs_tbl.pm80xx_tbl.gst_len_mpistate       = pm8001_mr32(address, GST_GSTLEN_MPIS_OFFSET);
    pm8001_ha->gs_tbl.pm80xx_tbl.iq_freeze_state0       = pm8001_mr32(address, GST_IQ_FREEZE_STATE0_OFFSET);
    pm8001_ha->gs_tbl.pm80xx_tbl.iq_freeze_state1       = pm8001_mr32(address, GST_IQ_FREEZE_STATE1_OFFSET);
    pm8001_ha->gs_tbl.pm80xx_tbl.msgu_tcnt              = pm8001_mr32(address, GST_MSGUTCNT_OFFSET);
    pm8001_ha->gs_tbl.pm80xx_tbl.iop_tcnt               = pm8001_mr32(address, GST_IOPTCNT_OFFSET);
    pm8001_ha->gs_tbl.pm80xx_tbl.gpio_input_val         = pm8001_mr32(address, GST_GPIO_INPUT_VAL);
    pm8001_ha->gs_tbl.pm80xx_tbl.recover_err_info[0]    = pm8001_mr32(address, GST_RERRINFO_OFFSET0);
    pm8001_ha->gs_tbl.pm80xx_tbl.recover_err_info[1]    = pm8001_mr32(address, GST_RERRINFO_OFFSET1);
    pm8001_ha->gs_tbl.pm80xx_tbl.recover_err_info[2]    = pm8001_mr32(address, GST_RERRINFO_OFFSET2);
    pm8001_ha->gs_tbl.pm80xx_tbl.recover_err_info[3]    = pm8001_mr32(address, GST_RERRINFO_OFFSET3);
    pm8001_ha->gs_tbl.pm80xx_tbl.recover_err_info[4]    = pm8001_mr32(address, GST_RERRINFO_OFFSET4);
    pm8001_ha->gs_tbl.pm80xx_tbl.recover_err_info[5]    = pm8001_mr32(address, GST_RERRINFO_OFFSET5);
    pm8001_ha->gs_tbl.pm80xx_tbl.recover_err_info[6]    = pm8001_mr32(address, GST_RERRINFO_OFFSET6);
    pm8001_ha->gs_tbl.pm80xx_tbl.recover_err_info[7]    = pm8001_mr32(address, GST_RERRINFO_OFFSET7);
}
/**
 * read_phy_attr_table - read the phy attribute table and save it.
 * @pm8001_ha: our hba card information
 */
 #if (LINUX_VERSION_CODE < KERNEL_VERSION(3,8,0) ) 
static void __devinit
#else
static void 
#endif
read_phy_attr_table(struct pm8001_hba_info *pm8001_ha)
{
    void __iomem *address = pm8001_ha->pspa_q_tbl_addr;
    pm8001_ha->phy_attr_table.phystart1_16[0] = pm8001_mr32(address,  PSPA_PHYSTATE0_OFFSET); 
    pm8001_ha->phy_attr_table.phystart1_16[1] = pm8001_mr32(address,  PSPA_PHYSTATE1_OFFSET); 
    pm8001_ha->phy_attr_table.phystart1_16[2] = pm8001_mr32(address,  PSPA_PHYSTATE2_OFFSET);
    pm8001_ha->phy_attr_table.phystart1_16[3] = pm8001_mr32(address,  PSPA_PHYSTATE3_OFFSET);
    pm8001_ha->phy_attr_table.phystart1_16[4] = pm8001_mr32(address,  PSPA_PHYSTATE4_OFFSET);
    pm8001_ha->phy_attr_table.phystart1_16[5] = pm8001_mr32(address,  PSPA_PHYSTATE5_OFFSET);
    pm8001_ha->phy_attr_table.phystart1_16[6] = pm8001_mr32(address,  PSPA_PHYSTATE6_OFFSET);
    pm8001_ha->phy_attr_table.phystart1_16[7] = pm8001_mr32(address,  PSPA_PHYSTATE7_OFFSET);
    pm8001_ha->phy_attr_table.phystart1_16[8] = pm8001_mr32(address,  PSPA_PHYSTATE8_OFFSET);
    pm8001_ha->phy_attr_table.phystart1_16[9] = pm8001_mr32(address,  PSPA_PHYSTATE9_OFFSET);
    pm8001_ha->phy_attr_table.phystart1_16[10] = pm8001_mr32(address,  PSPA_PHYSTATE10_OFFSET);
    pm8001_ha->phy_attr_table.phystart1_16[11] = pm8001_mr32(address,  PSPA_PHYSTATE11_OFFSET);
    pm8001_ha->phy_attr_table.phystart1_16[12] = pm8001_mr32(address,  PSPA_PHYSTATE12_OFFSET);
    pm8001_ha->phy_attr_table.phystart1_16[13] = pm8001_mr32(address,  PSPA_PHYSTATE13_OFFSET);
    pm8001_ha->phy_attr_table.phystart1_16[14] = pm8001_mr32(address,  PSPA_PHYSTATE14_OFFSET);
    pm8001_ha->phy_attr_table.phystart1_16[15] = pm8001_mr32(address,  PSPA_PHYSTATE15_OFFSET);

    pm8001_ha->phy_attr_table.outbound_hw_event_pid1_16[0] = pm8001_mr32(address,  PSPA_OB_HW_EVENT_PID0_OFFSET); 
    pm8001_ha->phy_attr_table.outbound_hw_event_pid1_16[1] = pm8001_mr32(address,  PSPA_OB_HW_EVENT_PID1_OFFSET);
    pm8001_ha->phy_attr_table.outbound_hw_event_pid1_16[2] = pm8001_mr32(address,  PSPA_OB_HW_EVENT_PID2_OFFSET);
    pm8001_ha->phy_attr_table.outbound_hw_event_pid1_16[3] = pm8001_mr32(address,  PSPA_OB_HW_EVENT_PID3_OFFSET);
    pm8001_ha->phy_attr_table.outbound_hw_event_pid1_16[4] = pm8001_mr32(address,  PSPA_OB_HW_EVENT_PID4_OFFSET);
    pm8001_ha->phy_attr_table.outbound_hw_event_pid1_16[5] = pm8001_mr32(address,  PSPA_OB_HW_EVENT_PID5_OFFSET);
    pm8001_ha->phy_attr_table.outbound_hw_event_pid1_16[6] = pm8001_mr32(address,  PSPA_OB_HW_EVENT_PID6_OFFSET);
    pm8001_ha->phy_attr_table.outbound_hw_event_pid1_16[7] = pm8001_mr32(address,  PSPA_OB_HW_EVENT_PID7_OFFSET);
    pm8001_ha->phy_attr_table.outbound_hw_event_pid1_16[8] = pm8001_mr32(address,  PSPA_OB_HW_EVENT_PID8_OFFSET);
    pm8001_ha->phy_attr_table.outbound_hw_event_pid1_16[9] = pm8001_mr32(address,  PSPA_OB_HW_EVENT_PID9_OFFSET);
    pm8001_ha->phy_attr_table.outbound_hw_event_pid1_16[10] = pm8001_mr32(address,  PSPA_OB_HW_EVENT_PID10_OFFSET);
    pm8001_ha->phy_attr_table.outbound_hw_event_pid1_16[11] = pm8001_mr32(address,  PSPA_OB_HW_EVENT_PID11_OFFSET);
    pm8001_ha->phy_attr_table.outbound_hw_event_pid1_16[12] = pm8001_mr32(address,  PSPA_OB_HW_EVENT_PID12_OFFSET);
    pm8001_ha->phy_attr_table.outbound_hw_event_pid1_16[13] = pm8001_mr32(address,  PSPA_OB_HW_EVENT_PID13_OFFSET);
    pm8001_ha->phy_attr_table.outbound_hw_event_pid1_16[14] = pm8001_mr32(address,  PSPA_OB_HW_EVENT_PID14_OFFSET);
    pm8001_ha->phy_attr_table.outbound_hw_event_pid1_16[15] = pm8001_mr32(address,  PSPA_OB_HW_EVENT_PID15_OFFSET);

}

/**
 * read_inbnd_queue_table - read the inbound queue table and save it.
 * @pm8001_ha: our hba card information
 */
#if (LINUX_VERSION_CODE < KERNEL_VERSION(3,8,0) ) 
static void __devinit
#else
static void 
#endif
read_inbnd_queue_table(struct pm8001_hba_info *pm8001_ha)
{
    int i;
    void __iomem *address = pm8001_ha->inbnd_q_tbl_addr;
    for (i = 0; i < PM8001_MAX_INB_NUM; i++) {
        u32 offset = i * 0x20;
        pm8001_ha->inbnd_q_tbl[i].pi_pci_bar =
              get_pci_bar_index(pm8001_mr32(address, (offset + IB_PIPCI_BAR)));
        pm8001_ha->inbnd_q_tbl[i].pi_offset =
            pm8001_mr32(address, (offset + IB_PIPCI_BAR_OFFSET));
    }
}

/**
 * read_outbnd_queue_table - read the outbound queue table and save it.
 * @pm8001_ha: our hba card information
 */
#if (LINUX_VERSION_CODE < KERNEL_VERSION(3,8,0) ) 
static void __devinit
#else
static void 
#endif
read_outbnd_queue_table(struct pm8001_hba_info *pm8001_ha)
{
    int i;
    void __iomem *address = pm8001_ha->outbnd_q_tbl_addr;
    for (i = 0; i < PM8001_MAX_OUTB_NUM; i++) {
        u32 offset = i * 0x24;
        pm8001_ha->outbnd_q_tbl[i].ci_pci_bar =
              get_pci_bar_index(pm8001_mr32(address, (offset + OB_CIPCI_BAR)));
        pm8001_ha->outbnd_q_tbl[i].ci_offset =
            pm8001_mr32(address, (offset + OB_CIPCI_BAR_OFFSET));
    }
}


/**
 * init_default_table_values - init the default table.
 * @pm8001_ha: our hba card information
 */
#if (LINUX_VERSION_CODE < KERNEL_VERSION(3,8,0) ) 
static void __devinit
#else
static void 
#endif
init_default_table_values(struct pm8001_hba_info *pm8001_ha)
{
    int i;
    u32 offsetib, offsetob;
    void __iomem *addressib = pm8001_ha->inbnd_q_tbl_addr;
    void __iomem *addressob = pm8001_ha->outbnd_q_tbl_addr;

    pm8001_ha->main_cfg_tbl.pm80xx_tbl.upper_event_log_addr     =
        pm8001_ha->memoryMap.region[AAP1].phys_addr_hi;
    pm8001_ha->main_cfg_tbl.pm80xx_tbl.lower_event_log_addr     =
        pm8001_ha->memoryMap.region[AAP1].phys_addr_lo;
    pm8001_ha->main_cfg_tbl.pm80xx_tbl.event_log_size   = PM8001_EVENT_LOG_SIZE;
    pm8001_ha->main_cfg_tbl.pm80xx_tbl.event_log_severity       = 0x03;

    pm8001_ha->main_cfg_tbl.pm80xx_tbl.upper_pcs_event_log_addr =
        pm8001_ha->memoryMap.region[IOP].phys_addr_hi;
    pm8001_ha->main_cfg_tbl.pm80xx_tbl.lower_pcs_event_log_addr =
        pm8001_ha->memoryMap.region[IOP].phys_addr_lo;
    pm8001_ha->main_cfg_tbl.pm80xx_tbl.pcs_event_log_size   = PM8001_EVENT_LOG_SIZE;
    pm8001_ha->main_cfg_tbl.pm80xx_tbl.pcs_event_log_severity   = 0x03;
    /* SGPIO configuration enabled. 0x1 << 2 */
    pm8001_ha->main_cfg_tbl.pm80xx_tbl.fatal_err_interrupt      =
        0x01 | MAIN_CFG_SGPIO_ENABLE;

    /* NCQ CHANGE STARTS */
        /* Disable CRC */
    pm8001_ha->main_cfg_tbl.pm80xx_tbl.crc_core_dump = (0x1 << 16); //disable end-to-end CRC checking
    /* NCQ CHANGE ENDS */

    for (i = 0; i < PM8001_MAX_INB_NUM; i++) {
        pm8001_ha->inbnd_q_tbl[i].element_pri_size_cnt  =
            0x00000100 | (0x00000080 << 16) | (0x00<<30);
        pm8001_ha->inbnd_q_tbl[i].upper_base_addr   =
            pm8001_ha->memoryMap.region[IB + i].phys_addr_hi;
        pm8001_ha->inbnd_q_tbl[i].lower_base_addr   =
        pm8001_ha->memoryMap.region[IB + i].phys_addr_lo;
        pm8001_ha->inbnd_q_tbl[i].base_virt     =
            (u8 *)pm8001_ha->memoryMap.region[IB + i].virt_ptr;
        pm8001_ha->inbnd_q_tbl[i].total_length      =
            pm8001_ha->memoryMap.region[IB + i].total_len;
        pm8001_ha->inbnd_q_tbl[i].ci_upper_base_addr    =
            pm8001_ha->memoryMap.region[CI + i].phys_addr_hi;
        pm8001_ha->inbnd_q_tbl[i].ci_lower_base_addr    =
            pm8001_ha->memoryMap.region[CI + i].phys_addr_lo;
        pm8001_ha->inbnd_q_tbl[i].ci_virt       =
            pm8001_ha->memoryMap.region[CI + i].virt_ptr;
        offsetib = i * 0x20;
        pm8001_ha->inbnd_q_tbl[i].pi_pci_bar        =
            get_pci_bar_index(pm8001_mr32(addressib,
                (offsetib + 0x14)));
        pm8001_ha->inbnd_q_tbl[i].pi_offset     =
            pm8001_mr32(addressib, (offsetib + 0x18));
        pm8001_ha->inbnd_q_tbl[i].producer_idx      = 0;
        pm8001_ha->inbnd_q_tbl[i].consumer_index    = 0;
        PM8001_DEV_DBG(pm8001_ha,
                pm8001_printk("IQ %d pi_bar 0x%x pi_offset 0x%x \n", i, 
                  pm8001_ha->inbnd_q_tbl[i].pi_pci_bar,
                  pm8001_ha->inbnd_q_tbl[i].pi_offset));
    }
    for (i = 0; i < PM8001_MAX_OUTB_NUM; i++) {
        pm8001_ha->outbnd_q_tbl[i].element_size_cnt =
            256 | (128 << 16) | (1<<30);
        pm8001_ha->outbnd_q_tbl[i].upper_base_addr  =
            pm8001_ha->memoryMap.region[OB + i].phys_addr_hi;
        pm8001_ha->outbnd_q_tbl[i].lower_base_addr  =
            pm8001_ha->memoryMap.region[OB + i].phys_addr_lo;
        pm8001_ha->outbnd_q_tbl[i].base_virt        =
            (u8 *)pm8001_ha->memoryMap.region[OB + i].virt_ptr;
        pm8001_ha->outbnd_q_tbl[i].total_length     =
            pm8001_ha->memoryMap.region[OB + i].total_len;
        pm8001_ha->outbnd_q_tbl[i].pi_upper_base_addr   =
            pm8001_ha->memoryMap.region[PI + i].phys_addr_hi;
        pm8001_ha->outbnd_q_tbl[i].pi_lower_base_addr   =
            pm8001_ha->memoryMap.region[PI + i].phys_addr_lo;
        pm8001_ha->outbnd_q_tbl[i].interrup_vec_cnt_delay = (i << 24); /* interrupt vector based on oq */
        pm8001_ha->outbnd_q_tbl[i].pi_virt      =
            pm8001_ha->memoryMap.region[PI + i].virt_ptr;
        offsetob = i * 0x24;
        pm8001_ha->outbnd_q_tbl[i].ci_pci_bar       =
            get_pci_bar_index(pm8001_mr32(addressob,
            offsetob + 0x14));
        pm8001_ha->outbnd_q_tbl[i].ci_offset        =
            pm8001_mr32(addressob, (offsetob + 0x18));
        pm8001_ha->outbnd_q_tbl[i].consumer_idx     = 0;
        pm8001_ha->outbnd_q_tbl[i].producer_index   = 0;

        PM8001_DEV_DBG(pm8001_ha,
                pm8001_printk("OQ %d vec:0x%x, ci_bar 0x%x ci_offset 0x%x \n", i,
                pm8001_ha->outbnd_q_tbl[i].interrup_vec_cnt_delay,
                  pm8001_ha->outbnd_q_tbl[i].ci_pci_bar,
                  pm8001_ha->outbnd_q_tbl[i].ci_offset));
    }
}

/**
 * pm80xx_update_main_config_table - update the main default table to the HBA.
 * @pm8001_ha: our hba card information
 */
#if (LINUX_VERSION_CODE < KERNEL_VERSION(3,8,0) ) 
void __devinit
#else
void 
#endif
pm80xx_update_main_config_table(struct pm8001_hba_info *pm8001_ha)
{
    void __iomem *address = pm8001_ha->main_cfg_tbl_addr;
    pm8001_mw32(address, MAIN_IQNPPD_HPPD_OFFSET,pm8001_ha->main_cfg_tbl.pm80xx_tbl.inbound_q_nppd_hppd);
    pm8001_mw32(address, MAIN_EVENT_LOG_ADDR_HI,
        pm8001_ha->main_cfg_tbl.pm80xx_tbl.upper_event_log_addr);
    pm8001_mw32(address, MAIN_EVENT_LOG_ADDR_LO,
        pm8001_ha->main_cfg_tbl.pm80xx_tbl.lower_event_log_addr);
    pm8001_mw32(address, MAIN_EVENT_LOG_BUFF_SIZE, pm8001_ha->main_cfg_tbl.pm80xx_tbl.event_log_size);
    pm8001_mw32(address, MAIN_EVENT_LOG_OPTION, pm8001_ha->main_cfg_tbl.pm80xx_tbl.event_log_severity);
    pm8001_mw32(address, MAIN_PCS_EVENT_LOG_ADDR_HI,
        pm8001_ha->main_cfg_tbl.pm80xx_tbl.upper_pcs_event_log_addr);
    pm8001_mw32(address, MAIN_PCS_EVENT_LOG_ADDR_LO,
        pm8001_ha->main_cfg_tbl.pm80xx_tbl.lower_pcs_event_log_addr);
    pm8001_mw32(address, MAIN_PCS_EVENT_LOG_BUFF_SIZE, pm8001_ha->main_cfg_tbl.pm80xx_tbl.pcs_event_log_size);
    pm8001_mw32(address, MAIN_PCS_EVENT_LOG_OPTION,
        pm8001_ha->main_cfg_tbl.pm80xx_tbl.pcs_event_log_severity);
    pm8001_mw32(address, MAIN_FATAL_ERROR_INTERRUPT,
        pm8001_ha->main_cfg_tbl.pm80xx_tbl.fatal_err_interrupt);

    /* NCQ CHANGE STARTS */
    pm8001_mw32(address, MAIN_EVENT_CRC_CHECK,
        pm8001_ha->main_cfg_tbl.pm80xx_tbl.crc_core_dump);
    /* NCQ CHANGE ENDS */

    /* SPCv specific */
        pm8001_ha->main_cfg_tbl.pm80xx_tbl.gpio_led_mapping &= 0xCFFFFFFF;
        pm8001_ha->main_cfg_tbl.pm80xx_tbl.gpio_led_mapping |= 0x20000000; //Set GPIOLED to 0x2 for LED indicator
        pm8001_mw32(address, MAIN_GPIO_LED_FLAGS_OFFSET, pm8001_ha->main_cfg_tbl.pm80xx_tbl.gpio_led_mapping);

    PM8001_DEV_DBG(pm8001_ha,
        pm8001_printk("Programming DW 0x21 in main cfg table with 0x%x \n",  pm8001_mr32(address, MAIN_GPIO_LED_FLAGS_OFFSET)));

        pm8001_mw32(address, MAIN_PORT_RECOVERY_TIMER,
            pm8001_ha->main_cfg_tbl.pm80xx_tbl.port_recovery_timer);
    pm8001_mw32(address, MAIN_INT_REASSERTION_DELAY,
            pm8001_ha->main_cfg_tbl.pm80xx_tbl.interrupt_reassertion_delay);
}

/**
 * update_inbnd_queue_table - update the inbound queue table to the HBA.
 * @pm8001_ha: our hba card information
 */
 #if (LINUX_VERSION_CODE < KERNEL_VERSION(3,8,0) )
static void __devinit
 #else
static void
#endif
update_inbnd_queue_table(struct pm8001_hba_info *pm8001_ha, int number)
{
    void __iomem *address = pm8001_ha->inbnd_q_tbl_addr;
    u16 offset = number * 0x20;
    pm8001_mw32(address, offset + IB_PROPERITY_OFFSET,
        pm8001_ha->inbnd_q_tbl[number].element_pri_size_cnt);
    pm8001_mw32(address, offset + IB_BASE_ADDR_HI_OFFSET,
        pm8001_ha->inbnd_q_tbl[number].upper_base_addr);
    pm8001_mw32(address, offset + IB_BASE_ADDR_LO_OFFSET,
        pm8001_ha->inbnd_q_tbl[number].lower_base_addr);
    pm8001_mw32(address, offset + IB_CI_BASE_ADDR_HI_OFFSET,
        pm8001_ha->inbnd_q_tbl[number].ci_upper_base_addr);
    pm8001_mw32(address, offset + IB_CI_BASE_ADDR_LO_OFFSET,
        pm8001_ha->inbnd_q_tbl[number].ci_lower_base_addr);

    PM8001_DEV_DBG(pm8001_ha,
            pm8001_printk("IQ %d: Element pri size 0x%x "
            "Q upr base addr 0x%x "
            "Q lwr base addr 0x%x \n",
            number, 
            pm8001_ha->inbnd_q_tbl[number].element_pri_size_cnt,
            pm8001_ha->inbnd_q_tbl[number].upper_base_addr,
            pm8001_ha->inbnd_q_tbl[number].lower_base_addr));

    PM8001_DEV_DBG(pm8001_ha,
            pm8001_printk("CI upper base addr 0x%x "
            "CI lower base addr 0x%x \n",
            pm8001_ha->inbnd_q_tbl[number].ci_upper_base_addr,
            pm8001_ha->inbnd_q_tbl[number].ci_lower_base_addr));

}

/**
 * update_outbnd_queue_table - update the outbound queue table to the HBA.
 * @pm8001_ha: our hba card information
 */
#if (LINUX_VERSION_CODE < KERNEL_VERSION(3,8,0) )
static void __devinit
#else
static void 
#endif
update_outbnd_queue_table(struct pm8001_hba_info *pm8001_ha, int number)
{
    void __iomem *address = pm8001_ha->outbnd_q_tbl_addr;
    u16 offset = number * 0x24;
    pm8001_mw32(address, offset + OB_PROPERITY_OFFSET,
        pm8001_ha->outbnd_q_tbl[number].element_size_cnt);
    pm8001_mw32(address, offset + OB_BASE_ADDR_HI_OFFSET,
        pm8001_ha->outbnd_q_tbl[number].upper_base_addr);
    pm8001_mw32(address, offset + OB_BASE_ADDR_LO_OFFSET,
        pm8001_ha->outbnd_q_tbl[number].lower_base_addr);
    pm8001_mw32(address, offset + OB_PI_BASE_ADDR_HI_OFFSET,
        pm8001_ha->outbnd_q_tbl[number].pi_upper_base_addr);
    pm8001_mw32(address, offset + OB_PI_BASE_ADDR_LO_OFFSET,
        pm8001_ha->outbnd_q_tbl[number].pi_lower_base_addr);
    pm8001_mw32(address, offset + OB_INTERRUPT_COALES_OFFSET,
        pm8001_ha->outbnd_q_tbl[number].interrup_vec_cnt_delay);

    PM8001_DEV_DBG(pm8001_ha,
            pm8001_printk("OQ %d: Element pri size 0x%x "
             "Q upr base addr 0x%x "
             "Q lwr base addr 0x%x \n",
             number,
             pm8001_ha->outbnd_q_tbl[number].element_size_cnt,
             pm8001_ha->outbnd_q_tbl[number].upper_base_addr,
             pm8001_ha->outbnd_q_tbl[number].lower_base_addr));

    PM8001_DEV_DBG(pm8001_ha,
            pm8001_printk("PI upper base addr 0x%x "
             "PI lower base addr 0x%x "
             "Interrupt coales offset 0x%x \n",
             pm8001_ha->outbnd_q_tbl[number].pi_upper_base_addr,
             pm8001_ha->outbnd_q_tbl[number].pi_lower_base_addr,
             pm8001_ha->outbnd_q_tbl[number].interrup_vec_cnt_delay));
}

/**
 * mpi_init_check - check firmware initialization status.
 * @pm8001_ha: our hba card information
 */
static int mpi_init_check(struct pm8001_hba_info *pm8001_ha)
{
    u32 max_wait_count;
    u32 value;
    u32 gst_len_mpistate;

    /* Write bit0=1 to Inbound DoorBell Register to tell the SPC FW the
    table is updated */
    pm8001_cw32(pm8001_ha, 0, MSGU_IBDB_SET, SPCv_MSGU_CFG_TABLE_UPDATE);
    if(ISSPCV_12G(pm8001_ha->pdev)) {
    /* wait until Inbound DoorBell Clear Register toggled */
        max_wait_count = 4 * 1000 * 1000;/* 4 sec */
    } else {
        max_wait_count = 2 * 1000 * 1000;/* 2 sec */
    }
    do {
        udelay(1);
        value = pm8001_cr32(pm8001_ha, 0, MSGU_IBDB_SET);
        value &= SPCv_MSGU_CFG_TABLE_UPDATE;
    } while ((value != 0) && (--max_wait_count));

    if (!max_wait_count)
        return -1;
    /* check the MPI-State for initialization upto 100ms*/
    max_wait_count = 100 * 1000;/* 100 msec */
    do {
        udelay(1);
        gst_len_mpistate =
            pm8001_mr32(pm8001_ha->general_stat_tbl_addr,
                    GST_GSTLEN_MPIS_OFFSET);
    } while ((GST_MPI_STATE_INIT != (gst_len_mpistate & GST_MPI_STATE_MASK)) && (--max_wait_count));
    if (!max_wait_count)
        return -1;

    /* check MPI Initialization error */
    gst_len_mpistate = gst_len_mpistate >> 16;
    if (0x0000 != gst_len_mpistate)
        return -1;

    return 0;
}

/**
 * check_fw_ready - The LLDD check if the FW is ready, if not, return error.
 * @pm8001_ha: our hba card information
 */
static int check_fw_ready(struct pm8001_hba_info *pm8001_ha)
{
    volatile u32 value;
    u32 max_wait_count;
    u32 max_wait_time;
    int ret = 0;

    /* reset / PCIe ready */
    max_wait_time = max_wait_count = 100 * 1000;    /* 100 milli sec */
        do
        {
           udelay(1);
       value = pm8001_cr32(pm8001_ha, 0, MSGU_SCRATCH_PAD_1);
        } while ((value == 0xFFFFFFFF) && (--max_wait_count));

    /* check ila status */
    max_wait_time = max_wait_count = 1000 * 1000;   /* 1000 milli sec */
        do
        {
           udelay(1);
       value = pm8001_cr32(pm8001_ha, 0, MSGU_SCRATCH_PAD_1);
        }
    while (((value & SCRATCH_PAD_ILA_READY) != SCRATCH_PAD_ILA_READY) && (--max_wait_count));
    if(!max_wait_count)
        ret = -1;
    else
    {
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk(" ila ready status in %d millisec \n", (max_wait_time - max_wait_count)));
    }

    /* check RAAE status */
    max_wait_time = max_wait_count = 1800 * 1000;   /* 1800 milli sec */
        do
        {
           udelay(1);
       value = pm8001_cr32(pm8001_ha, 0, MSGU_SCRATCH_PAD_1);
        }
        while (((value & SCRATCH_PAD_RAAE_READY) != SCRATCH_PAD_RAAE_READY) && (--max_wait_count));
    if(!max_wait_count)
        ret = -1;
    else
    {
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk(" raae ready status in %d millisec \n", (max_wait_time - max_wait_count)));
    }

    /* check iop0 status */
    max_wait_time = max_wait_count = 600 * 1000;    /* 600 milli sec */
        do
        {
           udelay(1);
       value = pm8001_cr32(pm8001_ha, 0, MSGU_SCRATCH_PAD_1);
        }
    while (((value & SCRATCH_PAD_IOP0_READY) != SCRATCH_PAD_IOP0_READY) && (--max_wait_count));
    if(!max_wait_count)
        ret = -1;
    else
    {
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk(" iop0 ready status in %d millisec \n", (max_wait_time - max_wait_count)));
    }

    /* check iop1 status only for 16 port controllers */
    if ((pm8001_ha->chip_id != chip_8008) && (pm8001_ha->chip_id != chip_8009))
    {
        max_wait_time = max_wait_count = 200 * 1000;    /* 200 milli sec */
            do
            {
                udelay(1);
            value = pm8001_cr32(pm8001_ha, 0, MSGU_SCRATCH_PAD_1);
            }
        while (((value & SCRATCH_PAD_IOP1_READY) != SCRATCH_PAD_IOP1_READY) && (--max_wait_count));
        if(!max_wait_count)
            ret = -1;
        else
        {
            PM8001_MSG_DBG(pm8001_ha,
                pm8001_printk(" iop1 ready status in %d millisec \n", (max_wait_time - max_wait_count)));
        }
    }

    return ret;
}

static void init_pci_device_addresses(struct pm8001_hba_info *pm8001_ha)
{
    void __iomem *base_addr;
    u32 value;
    u32 offset;
    u32 pcibar;
    u32 pcilogic;

    value = pm8001_cr32(pm8001_ha, 0, MSGU_SCRATCH_PAD_0);  
    offset = value & 0x03FFFFFF; /* scratch pad 0 TBL address */

    PM8001_DEV_DBG(pm8001_ha,
        pm8001_printk("Scratchpad 0 Offset: 0x%x value 0x%x \n", offset, value));
    pcilogic = (value & 0xFC000000) >> 26;
    pcibar = get_pci_bar_index(pcilogic);
    PM8001_INIT_DBG(pm8001_ha,
        pm8001_printk("Scratchpad 0 PCI BAR: %d\n", pcibar));
    pm8001_ha->main_cfg_tbl_addr = base_addr =
        pm8001_ha->io_mem[pcibar].memvirtaddr + offset;
    pm8001_ha->general_stat_tbl_addr =
        base_addr + (pm8001_cr32(pm8001_ha, pcibar, offset + 0x18) & 0xFFFFFF);
    pm8001_ha->inbnd_q_tbl_addr =
        base_addr + (pm8001_cr32(pm8001_ha, pcibar, offset + 0x1C) & 0xFFFFFF);
    pm8001_ha->outbnd_q_tbl_addr =
        base_addr + (pm8001_cr32(pm8001_ha, pcibar, offset + 0x20) & 0xFFFFFF);
    pm8001_ha->ivt_tbl_addr = 
        base_addr + (pm8001_cr32(pm8001_ha, pcibar, offset + 0x8C) & 0xFFFFFF);
    pm8001_ha->pspa_q_tbl_addr = 
        base_addr + (pm8001_cr32(pm8001_ha, pcibar, offset + 0x90) & 0xFFFFFF);
    pm8001_ha->fatal_tbl_addr = 
        base_addr + (pm8001_cr32(pm8001_ha, pcibar, offset + 0xA0) & 0xFFFFFF);

        PM8001_INIT_DBG(pm8001_ha, 
            pm8001_printk("GST OFFSET 0x%x \n", 
            pm8001_cr32(pm8001_ha, pcibar, offset + 0x18))); 
        PM8001_INIT_DBG(pm8001_ha, 
            pm8001_printk("INBND OFFSET 0x%x \n", 
            pm8001_cr32(pm8001_ha, pcibar, offset + 0x1C))); 
        PM8001_INIT_DBG(pm8001_ha, 
            pm8001_printk("OBND OFFSET 0x%x \n", 
            pm8001_cr32(pm8001_ha, pcibar, offset + 0x20))); 
        PM8001_INIT_DBG(pm8001_ha, 
            pm8001_printk("IVT OFFSET 0x%x \n", 
            pm8001_cr32(pm8001_ha, pcibar, offset + 0x8C))); 
        PM8001_INIT_DBG(pm8001_ha, 
            pm8001_printk("PSPA OFFSET 0x%x \n", 
            pm8001_cr32(pm8001_ha, pcibar, offset + 0x90)));
        PM8001_INIT_DBG(pm8001_ha, 
            pm8001_printk("addr of main cfg %p general status %p \n",
            pm8001_ha->main_cfg_tbl_addr, 
            pm8001_ha->general_stat_tbl_addr)); 
        PM8001_INIT_DBG(pm8001_ha, 
            pm8001_printk("addr of inbnd %p obnd %p \n",
            pm8001_ha->inbnd_q_tbl_addr, 
            pm8001_ha->outbnd_q_tbl_addr));
        PM8001_INIT_DBG(pm8001_ha, 
            pm8001_printk("addr of pspa %p ivt %p \n",
            pm8001_ha->pspa_q_tbl_addr, 
            pm8001_ha->ivt_tbl_addr));
}

/**
 * pm80xx_set_thermal_config - support the thermal configuration
 * @pm8001_ha: our hba card information.
 */
int
pm80xx_set_thermal_config(struct pm8001_hba_info *pm8001_ha)
{
    struct set_ctrl_cfg_req payload;
    struct inbound_queue_table *circularQ;
    int rc;
    u32 tag,opCode=0;
        u32 opc = OPC_INB_SET_CONTROLLER_CONFIG;

        memset(&payload, 0, sizeof(struct set_ctrl_cfg_req));
    rc = pm8001_tag_alloc(pm8001_ha, &tag);
    if (rc)
                return -1;

    circularQ = &pm8001_ha->inbnd_q_tbl[0];
        payload.tag = cpu_to_le32(tag);
        if((pm8001_ha->chip_id == chip_8074) || 
            (pm8001_ha->chip_id == chip_8076)) {
        opCode = THERMAL_OP_CODE_8H;
    } else {
        opCode = THERMAL_OP_CODE;
    }
    payload.cfg_pg[0] = (THERMAL_LOG_ENABLE << 9) | (THERMAL_ENABLE << 8) | opCode;
    payload.cfg_pg[1] = (LTEMPHIL << 24) | (RTEMPHIL << 8);

    PM8001_DEV_DBG(pm8001_ha,
            pm8001_printk("Setting up thermal configuration. cfg_pg 0 0x%x cfg_pg 1 0x%x \n", payload.cfg_pg[0], payload.cfg_pg[1]));

    rc = pm8001_mpi_build_cmd(pm8001_ha, circularQ, opc, &payload, 0);
        return rc;

}

/**
 * pm80xx_set_sas_protocol_timer_config - support the SAS Protocol Timer configuration page
 * @pm8001_ha: our hba card information.
 */
static int
pm80xx_set_sas_protocol_timer_config(struct pm8001_hba_info *pm8001_ha)
{
    struct set_ctrl_cfg_req payload;
    struct inbound_queue_table *circularQ;
    SASProtocolTimerConfig_t SASConfigPage;
    int rc;
    u32 tag;
    u32 opc = OPC_INB_SET_CONTROLLER_CONFIG;

        memset(&payload, 0, sizeof(struct set_ctrl_cfg_req));
    memset(&SASConfigPage, 0, sizeof(SASProtocolTimerConfig_t));

    rc = pm8001_tag_alloc(pm8001_ha, &tag);

    if (rc)
            return -1;

    circularQ = &pm8001_ha->inbnd_q_tbl[0];
        payload.tag = cpu_to_le32(tag);

        SASConfigPage.pageCode        =  SAS_PROTOCOL_TIMER_CONFIG_PAGE;
        SASConfigPage.MST_MSI         =  3 << 15; /* enables both MCT for SSP target and initiator */
        SASConfigPage.STP_SSP_MCT_TMO =  (STP_MCT_TMO << 16) | SSP_MCT_TMO; /* default of 3200 us for STP and SSP maximum connection time */
        SASConfigPage.STP_FRM_TMO     = (SAS_MAX_OPEN_TIME << 24) | (SMP_MAX_CONN_TIMER << 16) | STP_FRM_TIMER; /* MAX_OPEN_TIME, SMP_MAX_CONN_TIMER, STP frame timeout */
        SASConfigPage.STP_IDLE_TMO    =  STP_IDLE_TIME;
    
    if (SASConfigPage.STP_IDLE_TMO > 0x3FFFFFF)
        {
            SASConfigPage.STP_IDLE_TMO = 0x3FFFFFF;
        }
    
    SASConfigPage.OPNRJT_RTRY_INTVL =         (SAS_MFD << 16)          | SAS_OPNRJT_RTRY_INTVL; /* Multi Data Fetach enabled and 2 us for Open Reject Retry interval */
        SASConfigPage.Data_Cmd_OPNRJT_RTRY_TMO =  (SAS_DOPNRJT_RTRY_TMO << 16) | SAS_COPNRJT_RTRY_TMO; /* 128 us for ORR Timeout for DATA phase and 32 us for ORR Timeout for command phase */
        SASConfigPage.Data_Cmd_OPNRJT_RTRY_THR =  (SAS_DOPNRJT_RTRY_THR << 16) | SAS_COPNRJT_RTRY_THR; /* 16 for ORR backoff threshold for DATA phase and 1024 for ORR backoff threshold for command phase */
        SASConfigPage.MAX_AIP =  SAS_MAX_AIP; /* MAX AIP. Default is  0x200000 */

    PM8001_INIT_DBG(pm8001_ha, pm8001_printk("SASConfigPage.pageCode 0x%08x\n",SASConfigPage.pageCode));
    PM8001_INIT_DBG(pm8001_ha, pm8001_printk("SASConfigPage.MST_MSI 0x%08x\n",SASConfigPage.MST_MSI));
    PM8001_INIT_DBG(pm8001_ha, pm8001_printk("SASConfigPage.STP_SSP_MCT_TMO 0x%08x\n",SASConfigPage.STP_SSP_MCT_TMO));
    PM8001_INIT_DBG(pm8001_ha, pm8001_printk("SASConfigPage.STP_FRM_TMO 0x%08x\n",SASConfigPage.STP_FRM_TMO));
    PM8001_INIT_DBG(pm8001_ha, pm8001_printk("SASConfigPage.STP_IDLE_TMO 0x%08x\n",SASConfigPage.STP_IDLE_TMO));
    PM8001_INIT_DBG(pm8001_ha, pm8001_printk("SASConfigPage.OPNRJT_RTRY_INTVL 0x%08x\n",SASConfigPage.OPNRJT_RTRY_INTVL));
    PM8001_INIT_DBG(pm8001_ha, pm8001_printk("SASConfigPage.Data_Cmd_OPNRJT_RTRY_TMO 0x%08x\n",SASConfigPage.Data_Cmd_OPNRJT_RTRY_TMO));
    PM8001_INIT_DBG(pm8001_ha, pm8001_printk("SASConfigPage.Data_Cmd_OPNRJT_RTRY_THR 0x%08x\n",SASConfigPage.Data_Cmd_OPNRJT_RTRY_THR));
    PM8001_INIT_DBG(pm8001_ha, pm8001_printk("SASConfigPage.MAX_AIP 0x%08x\n",SASConfigPage.MAX_AIP));

    memcpy(&payload.cfg_pg, &SASConfigPage, sizeof(SASProtocolTimerConfig_t));

    rc = pm8001_mpi_build_cmd(pm8001_ha, circularQ, opc, &payload, 0);

    return rc;
}

/**
 * pm80xx_get_encrypt_info - Check for encryption 
 * @pm8001_ha: our hba card information.
 */
static int
pm80xx_get_encrypt_info(struct pm8001_hba_info *pm8001_ha)
{
    u32 scratch3_value;
    int ret;

    /* Read encryption status from SCRATCH PAD 3 */
    scratch3_value = pm8001_cr32(pm8001_ha, 0, MSGU_SCRATCH_PAD_3);

        if((scratch3_value & SCRATCH_PAD3_ENC_MASK) == SCRATCH_PAD3_ENC_READY ) 
        {
          if( scratch3_value & SCRATCH_PAD3_XTS_ENABLED)
          {
            pm8001_ha->encrypt_info.cipher_mode = CIPHER_MODE_XTS;
          }
          if( (scratch3_value & SCRATCH_PAD3_SM_MASK ) == SCRATCH_PAD3_SMF_ENABLED )
          {
            pm8001_ha->encrypt_info.sec_mode = SEC_MODE_SMF;
          }
          if( (scratch3_value & SCRATCH_PAD3_SM_MASK ) == SCRATCH_PAD3_SMA_ENABLED)
          {
            pm8001_ha->encrypt_info.sec_mode = SEC_MODE_SMA;
          }
          if( (scratch3_value & SCRATCH_PAD3_SM_MASK ) == SCRATCH_PAD3_SMB_ENABLED )
          {
            pm8001_ha->encrypt_info.sec_mode = SEC_MODE_SMB;
          }
          pm8001_ha->encrypt_info.status = 0;
          PM8001_INIT_DBG(pm8001_ha, pm8001_printk("Encryption: SCRATCH_PAD3_ENC_READY 0x%08X. Cipher mode 0x%x Sec mode 0x%x status 0x%x \n", scratch3_value, pm8001_ha->encrypt_info.cipher_mode, pm8001_ha->encrypt_info.sec_mode, pm8001_ha->encrypt_info.status ));
      ret = 0;
        } 
        else if((scratch3_value & SCRATCH_PAD3_ENC_READY) == SCRATCH_PAD3_ENC_DISABLED) /* 0 */
        {
          PM8001_INIT_DBG(pm8001_ha, pm8001_printk("Encryption: SCRATCH_PAD3_ENC_DISABLED 0x%08X\n", scratch3_value ));
          pm8001_ha->encrypt_info.status = 0xFFFFFFFF;
          pm8001_ha->encrypt_info.cipher_mode = 0;
          pm8001_ha->encrypt_info.sec_mode = 0;
      return 0;
        }
        else if((scratch3_value & SCRATCH_PAD3_ENC_MASK ) == SCRATCH_PAD3_ENC_DIS_ERR) /* 1 */
        {
          pm8001_ha->encrypt_info.status = (scratch3_value & SCRATCH_PAD3_ERR_CODE ) >> 16;
          if( scratch3_value & SCRATCH_PAD3_XTS_ENABLED)
          {
            pm8001_ha->encrypt_info.cipher_mode = CIPHER_MODE_XTS;
          }
          if( (scratch3_value & SCRATCH_PAD3_SM_MASK ) == SCRATCH_PAD3_SMF_ENABLED )
          {
            pm8001_ha->encrypt_info.sec_mode = SEC_MODE_SMF;
          }
          if( (scratch3_value & SCRATCH_PAD3_SM_MASK ) == SCRATCH_PAD3_SMA_ENABLED)
          {
            pm8001_ha->encrypt_info.sec_mode = SEC_MODE_SMA;
          }
          if( (scratch3_value & SCRATCH_PAD3_SM_MASK ) == SCRATCH_PAD3_SMB_ENABLED )
          {
            pm8001_ha->encrypt_info.sec_mode = SEC_MODE_SMB;
          }
          PM8001_INIT_DBG(pm8001_ha, pm8001_printk("Encryption: SCRATCH_PAD3_DIS_ERR 0x%08X. Cipher mode 0x%x sec mode 0x%x status 0x%x \n", scratch3_value, pm8001_ha->encrypt_info.cipher_mode, pm8001_ha->encrypt_info.sec_mode, pm8001_ha->encrypt_info.status ));
      ret = -1;
        }
        else if((scratch3_value & SCRATCH_PAD3_ENC_MASK ) == SCRATCH_PAD3_ENC_ENA_ERR) /* 2 */
        {

          pm8001_ha->encrypt_info.status = (scratch3_value & SCRATCH_PAD3_ERR_CODE ) >> 16;
          if( scratch3_value & SCRATCH_PAD3_XTS_ENABLED)
          {
            pm8001_ha->encrypt_info.cipher_mode = CIPHER_MODE_XTS;
          }
          if( (scratch3_value & SCRATCH_PAD3_SM_MASK ) == SCRATCH_PAD3_SMF_ENABLED )
          {
            pm8001_ha->encrypt_info.sec_mode = SEC_MODE_SMF;
          }
          if( (scratch3_value & SCRATCH_PAD3_SM_MASK ) == SCRATCH_PAD3_SMA_ENABLED)
          {
            pm8001_ha->encrypt_info.sec_mode = SEC_MODE_SMA;
          }
          if( (scratch3_value & SCRATCH_PAD3_SM_MASK ) == SCRATCH_PAD3_SMB_ENABLED )
          {
            pm8001_ha->encrypt_info.sec_mode = SEC_MODE_SMB;
          }

          PM8001_INIT_DBG(pm8001_ha, pm8001_printk("Encryption: SCRATCH_PAD3_ENA_ERR 0x%08X. Cipher mode 0x%x sec mode 0x%x status 0x%x \n", scratch3_value, pm8001_ha->encrypt_info.cipher_mode, pm8001_ha->encrypt_info.sec_mode, pm8001_ha->encrypt_info.status ));
      ret = -1;
        }
    return ret;
}

/**
 * pm80xx_encrypt_update - update flash with encryption informtion
 * @pm8001_ha: our hba card information.
 */
static int pm80xx_encrypt_update(struct pm8001_hba_info *pm8001_ha)
{
    struct kek_mgmt_req payload;
    struct inbound_queue_table *circularQ;
    int rc;
    u32 tag;
        u32 opc = OPC_INB_KEK_MANAGEMENT;

        memset(&payload, 0, sizeof(struct kek_mgmt_req));
    rc = pm8001_tag_alloc(pm8001_ha, &tag);
    if (rc)
                return -1;

    circularQ = &pm8001_ha->inbnd_q_tbl[0];
        payload.tag = cpu_to_le32(tag);
    /* Currently only one key is used. New KEK index is 1. Current KEK index is 1. Store KEK to NVRAM is 1. */
    payload.new_curidx_ksop = ((1 << 24) | (1 << 16) | (1 << 8) | KEK_MGMT_SUBOP_KEYCARDUPDATE);

    PM8001_DEV_DBG(pm8001_ha, pm8001_printk("Saving Encryption info to flash. payload 0x%x \n", payload.new_curidx_ksop));

    rc = pm8001_mpi_build_cmd(pm8001_ha, circularQ, opc, &payload, 0);

        return rc;
}

/**
 * pm8001_chip_init - the main init function that initialize whole PM8001 chip.
 * @pm8001_ha: our hba card information
 */
#if (LINUX_VERSION_CODE < KERNEL_VERSION(3,8,0) )
static int __devinit 
#else
static int 
#endif
pm80xx_chip_init(struct pm8001_hba_info *pm8001_ha)
{
    int ret;
    u8 i=0;

    /* check the firmware status */
    if (-1 == check_fw_ready(pm8001_ha)) {
        PM8001_FAIL_DBG(pm8001_ha,
            pm8001_printk("Firmware is not ready!\n"));
        return -EBUSY;
    }

    /* Initialize pci space address eg: mpi offset */
    init_pci_device_addresses(pm8001_ha);
    init_default_table_values(pm8001_ha);
    read_main_config_table(pm8001_ha);
    read_general_status_table(pm8001_ha);
    read_inbnd_queue_table(pm8001_ha);
    read_outbnd_queue_table(pm8001_ha);
    read_phy_attr_table(pm8001_ha); 

    /* update main config table ,inbound table and outbound table */
    pm80xx_update_main_config_table(pm8001_ha);
    for (i = 0; i < PM8001_MAX_INB_NUM; i++) {
        update_inbnd_queue_table(pm8001_ha, i);
    }
    for (i = 0; i < PM8001_MAX_OUTB_NUM; i++) {
        update_outbnd_queue_table(pm8001_ha, i);
    }

    /* notify firmware update finished and check initialization status */
    if (0 == mpi_init_check(pm8001_ha)) {
        PM8001_INIT_DBG(pm8001_ha,
            pm8001_printk("MPI initialize successful!\n"));
    } else
        return -EBUSY;

    /* send SAS protocol timer configuration page to FW */
    ret = pm80xx_set_sas_protocol_timer_config(pm8001_ha);

    /* Check for encryption */
    if (pm8001_ha->chip->encrypt)
    {
            PM8001_INIT_DBG(pm8001_ha, pm8001_printk("Checking for encryption \n"));
        ret = pm80xx_get_encrypt_info(pm8001_ha);
        if (ret == -1)
        {
                PM8001_INIT_DBG(pm8001_ha, pm8001_printk("Encryption error !! \n"));
            if (pm8001_ha->encrypt_info.status == 0x81)
            {
                PM8001_INIT_DBG(pm8001_ha, pm8001_printk("Encryption enabled with error. Saving encryption key to flash \n"));
                pm80xx_encrypt_update(pm8001_ha);
            }
        }
    }
    return 0;
}


static int mpi_uninit_check(struct pm8001_hba_info *pm8001_ha)
{
    u32 max_wait_count;
    u32 value;
    u32 gst_len_mpistate;
    init_pci_device_addresses(pm8001_ha);
    /* Write bit1=1 to Inbound DoorBell Register to tell the SPC FW the
    table is stop */
    pm8001_cw32(pm8001_ha, 0, MSGU_IBDB_SET, SPCv_MSGU_CFG_TABLE_RESET);

    /* wait until Inbound DoorBell Clear Register toggled */
    if(ISSPCV_12G(pm8001_ha->pdev)) {
        max_wait_count = 4 * 1000 * 1000;/* 3 sec; perviosuly implementation was 1 sec; changed based on spcv pm */
    } else {
        max_wait_count = 2 * 1000 * 1000;/* 2 sec */
    }

    do {
        udelay(1);
        value = pm8001_cr32(pm8001_ha, 0, MSGU_IBDB_SET);
        value &= SPCv_MSGU_CFG_TABLE_RESET;
    } while ((value != 0) && (--max_wait_count));

    if (!max_wait_count) {
        PM8001_FAIL_DBG(pm8001_ha,
            pm8001_printk("TIMEOUT:IBDB value/=%x\n", value));
        return -1;
    }

    /* check the MPI-State for termination in progress */
    /* wait until Inbound DoorBell Clear Register toggled */
    max_wait_count = 2 * 1000 * 1000;  /* 2 sec; updated from 1 sec based on pm80xx functionality */
    do {
        udelay(1);
        gst_len_mpistate =
            pm8001_mr32(pm8001_ha->general_stat_tbl_addr,
            GST_GSTLEN_MPIS_OFFSET);
        if (GST_MPI_STATE_UNINIT ==
            (gst_len_mpistate & GST_MPI_STATE_MASK))
            break;
    } while (--max_wait_count);
    if (!max_wait_count) {
        PM8001_FAIL_DBG(pm8001_ha,
            pm8001_printk(" TIME OUT MPI State = 0x%x\n",
                gst_len_mpistate & GST_MPI_STATE_MASK));
        return -1;
    }
       
    return 0;
}


#if 0
static void pm80xx_chip_save_restore_config(struct pm8001_hba_info *pm8001_ha, int save_restore)
{
	u32	i, value;

	if(save_restore == 1) {
		/*
		 * Save the PCI config space (standard only)
		 */
		for (i = 0; i < PCI_EXT_CFG_START/4; ++i) {
			 pci_read_config_dword(pm8001_ha->pdev, i * 4, &value);
			 pm8001_ha->config_save[i] = value;
		}
	}
	else {

		/*
		 * Restore the PCI config space (standard only)
		 */
		for (i = 0; i < PCI_EXT_CFG_START/4; ++i) {
			pci_read_config_dword(pm8001_ha->pdev, i * 4, &value);
			  if (value != pm8001_ha->config_save[i]) {
				  PM8001_MSG_DBG(pm8001_ha,
				            pm8001_printk("Restore PCI config[%d] with value : 0x%x \n", i, pm8001_ha->config_save[i]));
				  pci_write_config_dword(pm8001_ha->pdev, i * 4, pm8001_ha->config_save[i]);
			  }
		}
	}
}
#endif /* 0 */

/**
 * pm8001_chip_soft_rst - soft reset the PM8001 chip, so that the clear all
 * the FW register status to the originated status.
 * @pm8001_ha: our hba card information
 */

static int
pm80xx_chip_soft_rst(struct pm8001_hba_info *pm8001_ha)
{
    volatile u32 regval;
    u32 iButton0, iButton1;
    volatile u32 bootloader_state;

    PM8001_INIT_DBG(pm8001_ha,
            pm8001_printk("Adapter soft reset \n"));
#if 0 //reyaz
    /* Check if MPI is in ready state to reset */
    if (mpi_uninit_check(pm8001_ha) != 0) {
        PM8001_FAIL_DBG(pm8001_ha,
            pm8001_printk("MPI state is not ready\n"));
        return -1;
    }
#endif
    /* checked for reset register normal state; 0x0 */
    regval = pm8001_cr32(pm8001_ha, 0, SPC_REG_SOFT_RESET);
    PM8001_INIT_DBG(pm8001_ha,
            pm8001_printk("reset register before write : 0x%x \n", regval));

#if 0 //Enable this for hard reset
    /*
     * Hard reset
     */
    pm80xx_chip_save_restore_config(pm8001_ha,  1);
    regval &= ~0x00000003;
    regval |= 0x03;
    pm8001_cw32(pm8001_ha, 0, SPC_REG_SOFT_RESET, regval);
    mdelay(3000);
    pm80xx_chip_save_restore_config(pm8001_ha,  0);
    /*
     * Soft reset
     */
    regval &= ~0x00000003;
    regval |= SPCv_NORMAL_RESET_VALUE;
    pm8001_cw32(pm8001_ha, 0, SPC_REG_SOFT_RESET, regval);
    mdelay(500);
#else
    /*
     * Original code
     */
    pm8001_cw32(pm8001_ha, 0, SPC_REG_SOFT_RESET, SPCv_NORMAL_RESET_VALUE);
    mdelay(1000);
#endif

    /* Check if MPI is in ready state to reset */
    if (mpi_uninit_check(pm8001_ha) != 0) {
        PM8001_FAIL_DBG(pm8001_ha,
            pm8001_printk("MPI state is not ready, force soft reset again\n"));
		pm8001_cw32(pm8001_ha, 0, SPC_REG_SOFT_RESET, SPCv_NORMAL_RESET_VALUE);
		mdelay(500);
		/* Check if MPI is in ready state to reset */
    	if (mpi_uninit_check(pm8001_ha) != 0) {
        	PM8001_FAIL_DBG(pm8001_ha,
            	pm8001_printk("MPI state is not ready\n"));
        	return -1;
    	}
	}

    regval = pm8001_cr32(pm8001_ha, 0, SPC_REG_SOFT_RESET);
    PM8001_INIT_DBG(pm8001_ha,
            pm8001_printk("reset register after write 0x%x \n", regval));
    mdelay(2000);

    if((regval & SPCv_SOFT_RESET_READ_MASK) == SPCv_SOFT_RESET_NORMAL_RESET_OCCURED)
    {
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk(" soft reset successful [regval: 0x%x]\n", regval));
    }
    else
    {
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk(" soft reset failed [regval: 0x%x]\n", regval));

        /* check whether bootloader is successfully executed or in HDA mode */
        bootloader_state = pm8001_cr32(pm8001_ha, 0, MSGU_SCRATCH_PAD_1) & SCRATCH_PAD1_BOOTSTATE_MASK;

            if(bootloader_state == SCRATCH_PAD1_BOOTSTATE_HDA_SEEPROM)
            {
            PM8001_MSG_DBG(pm8001_ha,
                pm8001_printk("Bootloader state - HDA mode SEEPROM \n"));
            } 
            else if(bootloader_state ==  SCRATCH_PAD1_BOOTSTATE_HDA_BOOTSTRAP)
            {
            PM8001_MSG_DBG(pm8001_ha,
                pm8001_printk("Bootloader state - HDA mode Bootstrap Pin \n"));
            }
            else if(bootloader_state == SCRATCH_PAD1_BOOTSTATE_HDA_SOFTRESET )
            {
            PM8001_MSG_DBG(pm8001_ha,
                pm8001_printk("Bootloader state - HDA mode soft reset \n"));
            }
            else if(bootloader_state == SCRATCH_PAD1_BOOTSTATE_CRIT_ERROR )
            {
            PM8001_MSG_DBG(pm8001_ha,
                pm8001_printk("Bootloader state - HDA mode critical error\n"));
            }
        return -EBUSY;
        }

        /* check the firmware status after reset */
    if (-1 == check_fw_ready(pm8001_ha)) {
        PM8001_FAIL_DBG(pm8001_ha,
            pm8001_printk("Firmware is not ready!\n"));
        if(pm8001_ha->pdev->subsystem_vendor != 0x9005) {
            iButton0 = pm8001_cr32(pm8001_ha, 0, MSGU_HOST_SCRATCH_PAD_6);
            iButton1 = pm8001_cr32(pm8001_ha,0,MSGU_HOST_SCRATCH_PAD_7);
            if(iButton0 == 0 && iButton1 == 0)
            {

                PM8001_INIT_DBG(pm8001_ha,
                        pm8001_printk("iButton Feature is not Available!!!\n"));
                return -EBUSY;
            }
            if(iButton0 == 0xdeadbeef && iButton1 == 0xdeadbeef) 
            {

                PM8001_INIT_DBG(pm8001_ha,
                        pm8001_printk("CRC Check for iButton Feature Failed!!!\n"));
                return -EBUSY;
            }
        }
    }
        PM8001_INIT_DBG(pm8001_ha,
            pm8001_printk("SPCv soft reset Complete\n"));
    return 0;
}

static void pm80xx_hw_chip_rst(struct pm8001_hba_info *pm8001_ha)
{
     u32 i;

    PM8001_INIT_DBG(pm8001_ha,
        pm8001_printk("chip reset start\n"));

    /* do SPCv chip reset. */
    pm8001_cw32(pm8001_ha, 0, SPC_REG_SOFT_RESET, 0x11);
    PM8001_INIT_DBG(pm8001_ha,
        pm8001_printk("SPC soft reset Complete\n"));

    /* Check this ..whether delay is required or no */
    /* delay 10 usec */
    udelay(10);

    /* wait for 20 msec until the firmware gets reloaded */
    i = 20;
    do {
        mdelay(1);
    } while ((--i) != 0);

    PM8001_INIT_DBG(pm8001_ha,
        pm8001_printk("chip reset finished\n"));
}

/**
 * pm8001_chip_interrupt_enable - enable PM8001 chip interrupt
 * @pm8001_ha: our hba card information
 */
static void
pm80xx_chip_intx_interrupt_enable(struct pm8001_hba_info *pm8001_ha)
{
    pm8001_cw32(pm8001_ha, 0, MSGU_ODMR, ODMR_CLEAR_ALL);
    pm8001_cw32(pm8001_ha, 0, MSGU_ODCR, ODCR_CLEAR_ALL);
}

 /**
  * pm8001_chip_intx_interrupt_disable- disable PM8001 chip interrupt
  * @pm8001_ha: our hba card information
  */
static void
pm80xx_chip_intx_interrupt_disable(struct pm8001_hba_info *pm8001_ha)
{
    pm8001_cw32(pm8001_ha, 0, MSGU_ODMR_CLR, ODMR_MASK_ALL);
}

/**
 * pm8001_chip_interrupt_enable - enable PM8001 chip interrupt
 * @pm8001_ha: our hba card information
 */
static void
pm80xx_chip_interrupt_enable(struct pm8001_hba_info *pm8001_ha, u8 vec)
{
#ifdef PM8001_USE_MSIX
    u32 mask;
    if(vec <32) {
        mask = (u32)(1 << vec);
        pm8001_cw32(pm8001_ha, 0, MSGU_ODMR_CLR, (u32)(mask & 0xFFFFFFFF));
    } else {
        mask = (u32)(1 << (vec -32));
        pm8001_cw32(pm8001_ha, 0, MSGU_ODMR_CLR_U, (u32)(mask & 0xFFFFFFFF));
    }
    return;
#endif
    pm80xx_chip_intx_interrupt_enable(pm8001_ha);

}

/**
 * pm8001_chip_interrupt_disable- disable PM8001 chip interrupt
 * @pm8001_ha: our hba card information
 */
static void
pm80xx_chip_interrupt_disable(struct pm8001_hba_info *pm8001_ha, u8 vec)
{
#ifdef PM8001_USE_MSIX
    u32 mask;
    if (vec == 0xFF) {
        mask = 0xFFFFFFFF;
        pm8001_cw32(pm8001_ha, 0, MSGU_ODMR, (u32)(mask & 0xFFFFFFFF));
        pm8001_cw32(pm8001_ha, 0, MSGU_ODMR_U, (u32)(mask & 0xFFFFFFFF));
    } else if( vec < 32) {
        mask = (u32)(1 << vec);
        pm8001_cw32(pm8001_ha, 0, MSGU_ODMR, (u32)(mask & 0xFFFFFFFF));
    } else {
        mask = (u32)(1 << (vec -32));
        pm8001_cw32(pm8001_ha, 0, MSGU_ODMR_U, (u32)(mask & 0xFFFFFFFF));
    }
    return;
#endif
    pm80xx_chip_intx_interrupt_disable(pm8001_ha);
}

/* NCQ CHANGE STARTS */
#define PM8001_TASK_TIMEOUT 20

static void pm80xx_send_abort_all(struct pm8001_hba_info *pm8001_ha, 
        struct pm8001_device *pm8001_ha_dev)
{
    int res;
    u32 ccb_tag;
    struct pm8001_ccb_info *ccb;
    struct sas_task *task = NULL;
    struct task_abort_req task_abort;
    struct inbound_queue_table *circularQ;
    u32 opc = OPC_INB_SATA_ABORT;
    int ret;
    

    if (!pm8001_ha_dev)
    {
        PM8001_FAIL_DBG(pm8001_ha, pm8001_printk("dev is NULL !!! RETURNING !!! \n"));
        return;
    }

#if defined PM8001_RHEL63 || (LINUX_VERSION_CODE == KERNEL_VERSION(3, 0, 76))  || \
    (LINUX_VERSION_CODE >= KERNEL_VERSION(3, 6, 0))
    task = sas_alloc_slow_task(GFP_KERNEL);
#elif LINUX_VERSION_CODE < KERNEL_VERSION(3, 2, 0)
    task = pm8001_alloc_task();
#else
    task = sas_alloc_task(GFP_KERNEL);
#endif
    if (!task)
    {
        PM8001_FAIL_DBG(pm8001_ha, pm8001_printk("cannot allocated task !!!\n"));
        return;
    }
    
    task->task_done = pm8001_task_done;

    res = pm8001_tag_alloc(pm8001_ha, &ccb_tag);
    if (res)
        return;
    
    PM8001_FAIL_DBG(pm8001_ha, 
        pm8001_printk("Executing abort task task %p device %p tag %x \n", task, pm8001_ha_dev, ccb_tag));
    
    ccb = &pm8001_ha->ccb_info[ccb_tag];
    ccb->device = pm8001_ha_dev;
    ccb->ccb_tag = ccb_tag;
    ccb->task = task;

    circularQ = &pm8001_ha->inbnd_q_tbl[0];

    memset(&task_abort, 0, sizeof(task_abort));
    task_abort.abort_all = cpu_to_le32(1);
    task_abort.device_id = cpu_to_le32(pm8001_ha_dev->device_id);
    task_abort.tag = cpu_to_le32(ccb_tag);
        
    PM8001_FAIL_DBG(pm8001_ha, 
        pm8001_printk("ABORT ALL dev id %x tag %x \n", task_abort.device_id, task_abort.tag));

    ret = pm8001_mpi_build_cmd(pm8001_ha, circularQ, opc, &task_abort, 0);

    PM8001_FAIL_DBG(pm8001_ha, 
        pm8001_printk("Executing abort task end\n"));

}

// This routine sends read log command following NCQ
static void pm80xx_send_read_log(struct pm8001_hba_info *pm8001_ha, 
        struct pm8001_device *pm8001_ha_dev)
{
    struct sata_start_req sata_cmd;
    int res;
    u32 ccb_tag;
    struct pm8001_ccb_info *ccb;
    struct sas_task *task = NULL;
    struct host_to_dev_fis fis;
    struct domain_device *dev;
    struct inbound_queue_table *circularQ;
    u32 opc = OPC_INB_SATA_HOST_OPSTART;
    
#if defined PM8001_RHEL63 || (LINUX_VERSION_CODE == KERNEL_VERSION(3, 0, 76))  || \
    (LINUX_VERSION_CODE >= KERNEL_VERSION(3, 6, 0))
    task = sas_alloc_slow_task(GFP_KERNEL);
#elif LINUX_VERSION_CODE < KERNEL_VERSION(3, 2, 0)
    task = pm8001_alloc_task();
#else
    task = sas_alloc_task(GFP_KERNEL);
#endif
    if (!task)
    {
        PM8001_FAIL_DBG(pm8001_ha, 
            pm8001_printk("cannot allocate task !!!\n"));
        return;
    }
    task->task_done = pm8001_task_done;

    res = pm8001_tag_alloc(pm8001_ha, &ccb_tag);
    if (res)
    {
        PM8001_FAIL_DBG(pm8001_ha, 
            pm8001_printk("cannot allocate tag !!!\n"));
        return;
    }

    PM8001_FAIL_DBG(pm8001_ha, 
        pm8001_printk("Executing send read log. task %p device %p tag %x \n", task, pm8001_ha_dev, ccb_tag));

    //allocate domain device by ourselves as libsas is not going to provide any
    dev = kzalloc(sizeof(struct domain_device), GFP_KERNEL);
    if (!dev)
    {
        PM8001_FAIL_DBG(pm8001_ha, 
            pm8001_printk("DOMAIN DEVICE CANNOT BE ALLOCATED !! \n"));
    }
    else
    {
        task->dev = dev;
        task->dev->lldd_dev = pm8001_ha_dev;
    }   
        
    
    ccb = &pm8001_ha->ccb_info[ccb_tag];
    ccb->device = pm8001_ha_dev;
    ccb->ccb_tag = ccb_tag;
    ccb->task = task;
    pm8001_ha_dev->id |= NCQ_READ_LOG_FLAG; // set this flag to indicate read log
    pm8001_ha_dev->id |= NCQ_2ND_RLE_FLAG;  // set this flag to guard against 2nd RLE. Workaround 
                        // till FW fix is available. 

    memset(&sata_cmd, 0, sizeof(sata_cmd));
    circularQ = &pm8001_ha->inbnd_q_tbl[0];

    // construct FIS
    memset(&fis, 0, sizeof(struct host_to_dev_fis));
    fis.fis_type = 0x27;
    fis.flags = 0x80;
    fis.command = ATA_CMD_READ_LOG_EXT;
    fis.lbal = 0x10; /* page number */
    fis.sector_count = 0x1;

    sata_cmd.tag = cpu_to_le32(ccb_tag);
    sata_cmd.device_id = cpu_to_le32(pm8001_ha_dev->device_id);
    sata_cmd.ncqtag_atap_dir_m_dad |= ((0x1 << 7) | (0x5 << 9)); //read from controller. PIO is set.
    memcpy(&sata_cmd.sata_fis, &fis, sizeof(struct host_to_dev_fis));  //copy the fis

    PM8001_FAIL_DBG(pm8001_ha, pm8001_printk("READ LOG SATA CMD tag %x device id %x length %d dir_dad %x \n", sata_cmd.tag, sata_cmd.device_id, sata_cmd.data_len, sata_cmd.ncqtag_atap_dir_m_dad));
    PM8001_FAIL_DBG(pm8001_ha, pm8001_printk("READ LOG SATA FIS flags %x cmd %x lbal %x \n", sata_cmd.sata_fis.flags, sata_cmd.sata_fis.command, sata_cmd.sata_fis.lbal));
    res = pm8001_mpi_build_cmd(pm8001_ha, circularQ, opc, &sata_cmd, 0);

    PM8001_FAIL_DBG(pm8001_ha, pm8001_printk("Executing read log end\n"));

}
/* NCQ CHANGE ENDS */

/**
 * mpi_ssp_completion- process the event that FW response to the SSP request.
 * @pm8001_ha: our hba card information
 * @piomb: the message contents of this outbound message.
 *
 * When FW has completed a ssp request for example a IO request, after it has
 * filled the SG data with the data, it will trigger this event represent
 * that he has finished the job,please check the coresponding buffer.
 * So we will tell the caller who maybe waiting the result to tell upper layer
 * that the task has been finished.
 */
static void
mpi_ssp_completion(struct pm8001_hba_info *pm8001_ha , void *piomb)
{
    struct sas_task *t;
    struct pm8001_ccb_info *ccb;
    unsigned long flags;
    u32 status;
    u32 param;
    u32 tag;
    struct ssp_completion_resp *psspPayload;
    struct task_status_struct *ts;
    struct ssp_response_iu *iu;
    struct pm8001_device *pm8001_dev;
    psspPayload = (struct ssp_completion_resp *)(piomb + 4);
    status = le32_to_cpu(psspPayload->status);
    tag = le32_to_cpu(psspPayload->tag);
    ccb = &pm8001_ha->ccb_info[tag];

    if(ccb->task == NULL)
    {
        if (ccb->n_elem == 0xffffffff) {
            param = le32_to_cpu(psspPayload->param);    
            PM8001_INIT_DBG(pm8001_ha,
                pm8001_printk("status is %x,tag is %x\n",status,tag));
            pm8001_ha->tmf_resp.status = status;
            if(param) {
                    pm8001_ha->tmf_resp.resp_code = psspPayload->ssp_resp_iu.resp_data[3];
            }
            complete(pm8001_ha->ioctl_completion);
            pm8001_dev = ccb->device;
            ccb->n_elem = 0;
            ccb->ccb_tag = 0XFFFFFFFF;
            ccb->device = NULL;
            pm8001_tag_free(pm8001_ha,tag);
            return;
        }
        if (ccb->n_elem == 0) {
            complete(pm8001_ha->ioctl_completion);
            pm8001_ha->ioctl_completion = NULL;
            pm8001_tag_free(pm8001_ha,tag);
            return;
        }
    }
    if ((status == IO_ABORTED) && ccb->open_retry) {
        /* Being completed by another */
        ccb->open_retry = 0;
        return;
    }
    pm8001_dev = ccb->device;
    param = le32_to_cpu(psspPayload->param);
    t = ccb->task;

    if (status && status != IO_UNDERFLOW)
        PM8001_FAIL_DBG(pm8001_ha,
            pm8001_printk("sas IO status 0x%x\n", status));
    if (unlikely(!t || !t->lldd_task || !t->dev))
        return;
    ts = &t->task_status;
    PM8001_DEV_DBG(pm8001_ha,
        pm8001_printk("tag::0x%x, status::0x%x task::0x%p \n", tag, status, t));
    /*
     * SSP/SMP/SATA IO Completion Status values  checking for Printing SAS Adddress in case of IO Failure
     */
    if((status != IO_SUCCESS ) && (status != IO_OVERFLOW ) && (status != IO_UNDERFLOW))
    {
    PM8001_FAIL_DBG(pm8001_ha,
                pm8001_printk("SAS Address of IO Failure Drive:%016llx", SAS_ADDR(t->dev->sas_addr)-1));        // for SAS_ADDR on IO FAILURE
    }
    switch (status) {
    case IO_SUCCESS:
        PM8001_IO_DBG(pm8001_ha, pm8001_printk("IO_SUCCESS"
            ",param = 0x%x\n", param));
        if (param == 0) {
            ts->resp = SAS_TASK_COMPLETE;
            ts->stat = SAM_STAT_GOOD;
        } else {
            ts->resp = SAS_TASK_COMPLETE;
            ts->stat = SAS_PROTO_RESPONSE;
            ts->residual = param;
            iu = &psspPayload->ssp_resp_iu;
            sas_ssp_task_response(pm8001_ha->dev, t, iu);
        }
        if (pm8001_dev)
            pm8001_dev->running_req--;
        break;
    case IO_ABORTED:
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("IO_ABORTED IOMB Tag\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_ABORTED_TASK;
        break;
    case IO_UNDERFLOW:
        /* SSP Completion with error */
        PM8001_IO_DBG(pm8001_ha, pm8001_printk("IO_UNDERFLOW"
            ",param = 0x%x\n", param));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_DATA_UNDERRUN;
        ts->residual = param;
        if (pm8001_dev)
            pm8001_dev->running_req--;
        break;
    case IO_NO_DEVICE:
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("%s: IO_NO_DEVICE\n", __func__));
        ts->resp = SAS_TASK_UNDELIVERED;
        ts->stat = SAS_PHY_DOWN;
        break;
    case IO_XFER_ERROR_BREAK:
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("IO_XFER_ERROR_BREAK\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_OPEN_REJECT;
        /* Force the midlayer to retry */
        ts->open_rej_reason = SAS_OREJ_RSVD_RETRY;
        break;
    case IO_XFER_ERROR_PHY_NOT_READY:
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("IO_XFER_ERROR_PHY_NOT_READY\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_OPEN_REJECT;
        ts->open_rej_reason = SAS_OREJ_RSVD_RETRY;
        break;
    case IO_OPEN_CNX_ERROR_PROTOCOL_NOT_SUPPORTED:
        PM8001_IO_DBG(pm8001_ha,
        pm8001_printk("IO_OPEN_CNX_ERROR_PROTOCOL_NOT_SUPPORTED\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_OPEN_REJECT;
        ts->open_rej_reason = SAS_OREJ_EPROTO;
        break;
    case IO_OPEN_CNX_ERROR_ZONE_VIOLATION:
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("IO_OPEN_CNX_ERROR_ZONE_VIOLATION\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_OPEN_REJECT;
        ts->open_rej_reason = SAS_OREJ_UNKNOWN;
        break;
    case IO_OPEN_CNX_ERROR_BREAK:
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("IO_OPEN_CNX_ERROR_BREAK\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_OPEN_REJECT;
        ts->open_rej_reason = SAS_OREJ_RSVD_RETRY;
        break;
    case IO_OPEN_CNX_ERROR_IT_NEXUS_LOSS:
    case IO_XFER_OPEN_RETRY_BACKOFF_THRESHOLD_REACHED:
    case IO_OPEN_CNX_ERROR_IT_NEXUS_LOSS_OPEN_TMO:
    case IO_OPEN_CNX_ERROR_IT_NEXUS_LOSS_NO_DEST:
    case IO_OPEN_CNX_ERROR_IT_NEXUS_LOSS_OPEN_COLLIDE:
    case IO_OPEN_CNX_ERROR_IT_NEXUS_LOSS_PATHWAY_BLOCKED:
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("%s: IO_OPEN_CNX_ERROR_IT_NEXUS_LOSS (0x%x)\n", __func__, status));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_OPEN_REJECT;
        ts->open_rej_reason = SAS_OREJ_UNKNOWN;
        //if (!t->uldd_task)
            pm8001_handle_event(pm8001_ha,
                pm8001_dev,
                IO_OPEN_CNX_ERROR_IT_NEXUS_LOSS);
        break;
    case IO_OPEN_CNX_ERROR_BAD_DESTINATION:
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("%s: IO_OPEN_CNX_ERROR_BAD_DESTINATION\n", __func__));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_OPEN_REJECT;
        ts->open_rej_reason = SAS_OREJ_BAD_DEST;
        break;
    case IO_OPEN_CNX_ERROR_CONNECTION_RATE_NOT_SUPPORTED:
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("IO_OPEN_CNX_ERROR_CONNECTION_RATE_"
            "NOT_SUPPORTED\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_OPEN_REJECT;
        ts->open_rej_reason = SAS_OREJ_CONN_RATE;
        break;
    case IO_OPEN_CNX_ERROR_WRONG_DESTINATION:
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("IO_OPEN_CNX_ERROR_WRONG_DESTINATION\n"));
        ts->resp = SAS_TASK_UNDELIVERED;
        ts->stat = SAS_OPEN_REJECT;
        ts->open_rej_reason = SAS_OREJ_WRONG_DEST;
        break;
    case IO_XFER_ERROR_NAK_RECEIVED:
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("IO_XFER_ERROR_NAK_RECEIVED\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_OPEN_REJECT;
        ts->open_rej_reason = SAS_OREJ_RSVD_RETRY;
        break;
    case IO_XFER_ERROR_ACK_NAK_TIMEOUT:
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("IO_XFER_ERROR_ACK_NAK_TIMEOUT\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_NAK_R_ERR;
        break;
    case IO_XFER_ERROR_DMA:
        PM8001_IO_DBG(pm8001_ha,
        pm8001_printk("IO_XFER_ERROR_DMA\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_OPEN_REJECT;
        break;
    case IO_XFER_OPEN_RETRY_TIMEOUT:
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("IO_XFER_OPEN_RETRY_TIMEOUT\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_OPEN_REJECT;
        ts->open_rej_reason = SAS_OREJ_RSVD_RETRY;
        break;
    case IO_XFER_ERROR_OFFSET_MISMATCH:
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("IO_XFER_ERROR_OFFSET_MISMATCH\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_OPEN_REJECT;
        break;
    case IO_PORT_IN_RESET:
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("IO_PORT_IN_RESET\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_OPEN_REJECT;
        break;
    case IO_DS_NON_OPERATIONAL:
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("%s: IO_DS_NON_OPERATIONAL\n", __func__));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_OPEN_REJECT;
        //if (!t->uldd_task)
            pm8001_handle_event(pm8001_ha,
                pm8001_dev,
                IO_DS_NON_OPERATIONAL);
        break;
    case IO_DS_IN_RECOVERY:
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("IO_DS_IN_RECOVERY\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_OPEN_REJECT;
        break;
    case IO_TM_TAG_NOT_FOUND:
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("IO_TM_TAG_NOT_FOUND\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_OPEN_REJECT;
        break;
    case IO_SSP_EXT_IU_ZERO_LEN_ERROR:
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("IO_SSP_EXT_IU_ZERO_LEN_ERROR\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_OPEN_REJECT;
        break;
    case IO_OPEN_CNX_ERROR_HW_RESOURCE_BUSY:
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("IO_OPEN_CNX_ERROR_HW_RESOURCE_BUSY\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_OPEN_REJECT;
        ts->open_rej_reason = SAS_OREJ_RSVD_RETRY;
        break;
    default:
        PM8001_DEVIO_DBG(pm8001_ha,
            pm8001_printk(" Unknown status 0x%x \n", status));
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("Unknown status 0x%x\n", status));
        /* not allowed case. Therefore, return failed status */
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_OPEN_REJECT;
        break;
    }
    PM8001_IO_DBG(pm8001_ha,
        pm8001_printk("scsi_status = 0x%x \n ",
        psspPayload->ssp_resp_iu.status));
    spin_lock_irqsave(&t->task_state_lock, flags);
    t->task_state_flags &= ~SAS_TASK_STATE_PENDING;
    t->task_state_flags &= ~SAS_TASK_AT_INITIATOR;
    t->task_state_flags |= SAS_TASK_STATE_DONE;
    if (unlikely((t->task_state_flags & SAS_TASK_STATE_ABORTED))) {
        spin_unlock_irqrestore(&t->task_state_lock, flags);
        PM8001_FAIL_DBG(pm8001_ha, pm8001_printk("task 0x%p done with"
            " io_status 0x%x resp 0x%x "
            "stat 0x%x but aborted by upper layer!\n",
            t, status, ts->resp, ts->stat));
        pm8001_ccb_task_free(pm8001_ha, t, ccb, tag);
    } else {
        spin_unlock_irqrestore(&t->task_state_lock, flags);
        pm8001_ccb_task_free(pm8001_ha, t, ccb, tag);
        mb();/* in order to force CPU ordering */
        t->task_done(t);
    }
}

/*See the comments for mpi_ssp_completion */
static void mpi_ssp_event(struct pm8001_hba_info *pm8001_ha , void *piomb)
{
    struct sas_task *t;
    unsigned long flags;
    struct task_status_struct *ts;
    struct pm8001_ccb_info *ccb;
    struct pm8001_device *pm8001_dev;
    struct ssp_event_resp *psspPayload =
        (struct ssp_event_resp *)(piomb + 4);
    u32 event = le32_to_cpu(psspPayload->event);
    u32 tag = le32_to_cpu(psspPayload->tag);
    u32 port_id = le32_to_cpu(psspPayload->port_id);
    u32 dev_id = le32_to_cpu(psspPayload->device_id);

    ccb = &pm8001_ha->ccb_info[tag];
    t = ccb->task;
    pm8001_dev = ccb->device;
    if (event)
        PM8001_FAIL_DBG(pm8001_ha,
            pm8001_printk("SSP EVENT 0x%x\n", event));
    if (unlikely(!t || !t->lldd_task || !t->dev))
        return;
    ts = &t->task_status;
    PM8001_IOERR_DBG(pm8001_ha,
        pm8001_printk("port_id:0x%x, device_id:0x%x, tag:0x%x, event:0x%x\n",
                port_id, dev_id, tag, event));
    switch (event) {
    case IO_OVERFLOW:
        PM8001_IO_DBG(pm8001_ha, pm8001_printk("IO_UNDERFLOW\n");)
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_DATA_OVERRUN;
        ts->residual = 0;
        if (pm8001_dev)
            pm8001_dev->running_req--;
        break;
    case IO_XFER_ERROR_BREAK:
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("IO_XFER_ERROR_BREAK\n"));
        pm8001_handle_event(pm8001_ha, t, IO_XFER_ERROR_BREAK);
        return;
    case IO_XFER_ERROR_PHY_NOT_READY:
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("IO_XFER_ERROR_PHY_NOT_READY\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_OPEN_REJECT;
        ts->open_rej_reason = SAS_OREJ_RSVD_RETRY;
        break;
    case IO_OPEN_CNX_ERROR_PROTOCOL_NOT_SUPPORTED:
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("IO_OPEN_CNX_ERROR_PROTOCOL_NOT"
            "_SUPPORTED\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_OPEN_REJECT;
        ts->open_rej_reason = SAS_OREJ_EPROTO;
        break;
    case IO_OPEN_CNX_ERROR_ZONE_VIOLATION:
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("IO_OPEN_CNX_ERROR_ZONE_VIOLATION\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_OPEN_REJECT;
        ts->open_rej_reason = SAS_OREJ_UNKNOWN;
        break;
    case IO_OPEN_CNX_ERROR_BREAK:
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("IO_OPEN_CNX_ERROR_BREAK\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_OPEN_REJECT;
        ts->open_rej_reason = SAS_OREJ_RSVD_RETRY;
        break;
    case IO_OPEN_CNX_ERROR_IT_NEXUS_LOSS:
    case IO_XFER_OPEN_RETRY_BACKOFF_THRESHOLD_REACHED:
    case IO_OPEN_CNX_ERROR_IT_NEXUS_LOSS_OPEN_TMO:
    case IO_OPEN_CNX_ERROR_IT_NEXUS_LOSS_NO_DEST:
    case IO_OPEN_CNX_ERROR_IT_NEXUS_LOSS_OPEN_COLLIDE:
    case IO_OPEN_CNX_ERROR_IT_NEXUS_LOSS_PATHWAY_BLOCKED:
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("%s: IO_OPEN_CNX_ERROR_IT_NEXUS_LOSS (0x%x)\n", __func__, event));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_OPEN_REJECT;
        ts->open_rej_reason = SAS_OREJ_UNKNOWN;
        //if (!t->uldd_task)
            pm8001_handle_event(pm8001_ha,
                pm8001_dev,
                IO_OPEN_CNX_ERROR_IT_NEXUS_LOSS);
        break;
    case IO_OPEN_CNX_ERROR_BAD_DESTINATION:
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("%s: IO_OPEN_CNX_ERROR_BAD_DESTINATION\n", __func__));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_OPEN_REJECT;
        ts->open_rej_reason = SAS_OREJ_BAD_DEST;
        break;
    case IO_OPEN_CNX_ERROR_CONNECTION_RATE_NOT_SUPPORTED:
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("IO_OPEN_CNX_ERROR_CONNECTION_RATE_"
            "NOT_SUPPORTED\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_OPEN_REJECT;
        ts->open_rej_reason = SAS_OREJ_CONN_RATE;
        break;
    case IO_OPEN_CNX_ERROR_WRONG_DESTINATION:
        PM8001_IO_DBG(pm8001_ha,
               pm8001_printk("IO_OPEN_CNX_ERROR_WRONG_DESTINATION\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_OPEN_REJECT;
        ts->open_rej_reason = SAS_OREJ_WRONG_DEST;
        break;
    case IO_XFER_ERROR_NAK_RECEIVED:
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("IO_XFER_ERROR_NAK_RECEIVED\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_OPEN_REJECT;
        ts->open_rej_reason = SAS_OREJ_RSVD_RETRY;
        break;
    case IO_XFER_ERROR_ACK_NAK_TIMEOUT:
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("IO_XFER_ERROR_ACK_NAK_TIMEOUT\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_NAK_R_ERR;
        break;
    case IO_XFER_OPEN_RETRY_TIMEOUT:
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("IO_XFER_OPEN_RETRY_TIMEOUT\n"));
        pm8001_handle_event(pm8001_ha, t, IO_XFER_OPEN_RETRY_TIMEOUT);
        return;
    case IO_XFER_ERROR_UNEXPECTED_PHASE:
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("IO_XFER_ERROR_UNEXPECTED_PHASE\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_DATA_OVERRUN;
        break;
    case IO_XFER_ERROR_XFER_RDY_OVERRUN:
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("IO_XFER_ERROR_XFER_RDY_OVERRUN\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_DATA_OVERRUN;
        break;
    case IO_XFER_ERROR_XFER_RDY_NOT_EXPECTED:
        PM8001_IO_DBG(pm8001_ha,
               pm8001_printk("IO_XFER_ERROR_XFER_RDY_NOT_EXPECTED\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_DATA_OVERRUN;
        break;
    case IO_XFER_ERROR_CMD_ISSUE_ACK_NAK_TIMEOUT:
        PM8001_IO_DBG(pm8001_ha,
        pm8001_printk("IO_XFER_ERROR_CMD_ISSUE_ACK_NAK_TIMEOUT\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_DATA_OVERRUN;
        break;
    case IO_XFER_ERROR_OFFSET_MISMATCH:
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("IO_XFER_ERROR_OFFSET_MISMATCH\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_DATA_OVERRUN;
        break;
    case IO_XFER_ERROR_XFER_ZERO_DATA_LEN:
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("IO_XFER_ERROR_XFER_ZERO_DATA_LEN\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_DATA_OVERRUN;
        break;
    case IO_XFER_ERROR_INTERNAL_CRC_ERROR:
        PM8001_IOERR_DBG(pm8001_ha,
            pm8001_printk("IO_XFR_ERROR_INTERNAL_CRC_ERROR\n"));
        /* TBC: used default set values */
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_DATA_OVERRUN;
        break;
    case IO_XFER_CMD_FRAME_ISSUED:
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("  IO_XFER_CMD_FRAME_ISSUED\n"));
        return;
    default:
        PM8001_DEVIO_DBG(pm8001_ha,
            pm8001_printk(" Unknown status 0x%x \n", event));
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("Unknown status 0x%x\n", event));
        /* not allowed case. Therefore, return failed status */
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_DATA_OVERRUN;
        break;
    }
    spin_lock_irqsave(&t->task_state_lock, flags);
    t->task_state_flags &= ~SAS_TASK_STATE_PENDING;
    t->task_state_flags &= ~SAS_TASK_AT_INITIATOR;
    t->task_state_flags |= SAS_TASK_STATE_DONE;
    if (unlikely((t->task_state_flags & SAS_TASK_STATE_ABORTED))) {
        spin_unlock_irqrestore(&t->task_state_lock, flags);
        PM8001_FAIL_DBG(pm8001_ha, pm8001_printk("task 0x%p done with"
            " event 0x%x resp 0x%x "
            "stat 0x%x but aborted by upper layer!\n",
            t, event, ts->resp, ts->stat));
        pm8001_ccb_task_free(pm8001_ha, t, ccb, tag);
    } else {
        spin_unlock_irqrestore(&t->task_state_lock, flags);
        pm8001_ccb_task_free(pm8001_ha, t, ccb, tag);
        mb();/* in order to force CPU ordering */
        t->task_done(t);
    }
}

/*See the comments for mpi_ssp_completion */
static void
mpi_sata_completion(struct pm8001_hba_info *pm8001_ha, void *piomb)
{
    int i ,j;
    struct sas_task *t;
    struct pm8001_ccb_info *ccb;
    u32 param;
    u32 status;
    u32 tag;
    struct sata_completion_resp *psataPayload;
    struct task_status_struct *ts;
    struct ata_task_resp *resp ;
    u32 *sata_resp;
    u8 sata_addr_low[4];        // for SAS_ADDR on IO FAILURE
    u32 temp_sata_addr_low;     // for SAS_ADDR on IO FAILURE
    u8 sata_addr_hi[4];     // for SAS_ADDR on IO FAILURE
    u32 temp_sata_addr_hi;      // for SAS_ADDR on IO FAILURE
    struct pm8001_device *pm8001_dev;
    unsigned long flags=0;

    psataPayload = (struct sata_completion_resp *)(piomb + 4);
    status = le32_to_cpu(psataPayload->status);
    tag = le32_to_cpu(psataPayload->tag);

    if (!tag)
    {
        PM8001_FAIL_DBG(pm8001_ha, 
            pm8001_printk("tag null !!! returning \n"));
        return;
    }
    ccb = &pm8001_ha->ccb_info[tag];
    param = le32_to_cpu(psataPayload->param);
    if (ccb)
    {
        t = ccb->task;
        pm8001_dev = ccb->device;
    }
    else
    {
        PM8001_FAIL_DBG(pm8001_ha, 
            pm8001_printk("No CCB !!!. returning \n"));
        return;
    }

    if (t && t->dev && (t->dev->lldd_dev))
    {   
        
    //PM8001_FAIL_DBG(pm8001_ha, 
    //  pm8001_printk("SATA IO STATUS 0x%x task %x device %x tag %x t->dev %x lldd_dev %x \n", status, t, pm8001_dev, tag, t->dev, t->dev->lldd_dev));
       
        pm8001_dev = t->dev->lldd_dev;
    }
    else
    {
    PM8001_FAIL_DBG(pm8001_ha, 
        pm8001_printk("SATA IO STATUS 0x%x task %p device %p tag %x \n", status, t, pm8001_dev, tag));
    }
    
    if (status)
        PM8001_FAIL_DBG(pm8001_ha,
            pm8001_printk("SATA IO STATUS 0x%x task %p \n", status, t));

    /* if it is NCQ handling and response is for abort_all with status set to IO_ABORTED. */ 
    if (status == IO_ABORTED && (pm8001_dev && (pm8001_dev->id & NCQ_ABORT_ALL_FLAG)))
    {
        PM8001_FAIL_DBG(pm8001_ha, 
            pm8001_printk("SATA IO ABORTED 0x%x AS PART OF NCQ HANDLING for device id %x \n", status, pm8001_dev->id));
        //return;
    }
    // If it is NCQ handling and response is for read_log and required task attributes are not set
    // Note: task can be NULL for NCQ error command 
    if ((pm8001_dev && !(pm8001_dev->id & NCQ_READ_LOG_FLAG)) && unlikely(!t || !t->lldd_task || !t->dev))
    {
        PM8001_FAIL_DBG(pm8001_ha,
            pm8001_printk("task or dev null\n"));
        return;
    }

    if (t)  
        ts = &t->task_status;
    else
    {
        PM8001_FAIL_DBG(pm8001_ha,
            pm8001_printk("task null returning \n"));
        return;
    }   
    
    if (!ts)
    {
        PM8001_FAIL_DBG(pm8001_ha,
            pm8001_printk("ts null returning \n"));
        return;
    }   
    if (unlikely(status))
    {
        PM8001_IOERR_DBG(pm8001_ha,
            pm8001_printk("status:0x%x, tag:0x%x, task::0x%p\n", status, tag, t));
#ifdef EVENT_LOG_LEVEL2
        print_event_log(pm8001_ha);
#endif
    }

//  PM8001_DEV_DBG(pm8001_ha,
//      pm8001_printk("tag::%d, status::0x%x, task::0x%p\n", tag, status, t));
    
    /*
     * SSP/SMP/SATA IO Completion Status values  checking for Printing SAS Adddress in case of IO Failure
     */

    if((status != IO_SUCCESS ) && (status != IO_OVERFLOW ) && (status != IO_UNDERFLOW))
    {
        if (!((t->dev->parent) && (DEV_IS_EXPANDER(t->dev->parent->dev_type))))
        {
            for(i=0,j=4;i<=3&&j<=7 ;i++,j++)
            {
                sata_addr_low[i]=pm8001_ha->sas_addr[j];
            }   
            for(i=0,j=0;i<=3&&j<=3;i++,j++)
            {
                sata_addr_hi[i]=pm8001_ha->sas_addr[j];
            }   
    
            memcpy(&temp_sata_addr_low,sata_addr_low,sizeof(sata_addr_low));
            memcpy(&temp_sata_addr_hi,sata_addr_hi,sizeof(sata_addr_hi));
    
            temp_sata_addr_hi = (((temp_sata_addr_hi>>24)&0xff) | ((temp_sata_addr_hi<<8)&0xff0000) | ((temp_sata_addr_hi>>8)&0xff00) | ((temp_sata_addr_hi<<24)&0xff000000));
            temp_sata_addr_low = ((((temp_sata_addr_low>>24)&0xff) | ((temp_sata_addr_low<<8)&0xff0000) | ((temp_sata_addr_low>>8)&0xff00) | ((temp_sata_addr_low<<24)&0xff000000))+ pm8001_dev->attached_phy + 0x10);
    
            PM8001_FAIL_DBG(pm8001_ha,
                pm8001_printk("SAS Address of IO Failure Drive:%08x%08x", temp_sata_addr_hi,temp_sata_addr_low));       // for SAS_ADDR on IO FAILURE

        }
        else 
        {
            for(i=0,j=4;i<=3&&j<=7;i++,j++)
            {
                sata_addr_low[i]=t->dev->sas_addr[j];
            }   
    
            for(i=0,j=0;i<=3&&j<=3;i++,j++)
            {
                sata_addr_hi[i]=t->dev->sas_addr[j];
            }   
       
            temp_sata_addr_low = (sata_addr_low[0]<<24)|(sata_addr_low[1]<<16)|(sata_addr_low[2]<<8)|(sata_addr_low[3]);
            temp_sata_addr_hi =  (sata_addr_hi[0]<<24)|(sata_addr_hi[1]<<16)|(sata_addr_hi[2]<<8)|(sata_addr_hi[3]);

            PM8001_FAIL_DBG(pm8001_ha,
                pm8001_printk("SAS Address of IO Failure Drive:%08x%08x", temp_sata_addr_hi,temp_sata_addr_low));       // for SAS_ADDR on IO FAILURE
        }
    }
    switch (status) {
    case IO_SUCCESS:
        PM8001_IO_DBG(pm8001_ha, pm8001_printk("IO_SUCCESS\n"));
        if (param == 0) {
            ts->resp = SAS_TASK_COMPLETE;
            ts->stat = SAM_STAT_GOOD;
            // check if response is for SEND READ LOG   
            if (pm8001_dev && (pm8001_dev->id & NCQ_READ_LOG_FLAG))
            {
                PM8001_FAIL_DBG(pm8001_ha, 
                    pm8001_printk("NCQ READ LOG RESP RECEIVED. task %p device id %x tag %x \n", t, pm8001_dev->device_id, tag));
                /* free associated task allocated in send_read_log */
                PM8001_FAIL_DBG(pm8001_ha, pm8001_printk("Sending NCQ ABORT ALL. Freeing read log task %p tag %x !!! \n", t, tag));

                /* set new bit for abort_all */
                pm8001_dev->id |= NCQ_ABORT_ALL_FLAG;
                /* clear bit for read log */
                pm8001_dev->id = pm8001_dev->id & 0x7FFFFFFF;
                    pm80xx_send_abort_all(pm8001_ha, pm8001_dev);
                
                /* alraedy lock is held. Just free the tag */
                pm8001_tag_free(pm8001_ha, tag);    
#if LINUX_VERSION_CODE < KERNEL_VERSION(3, 2, 0) && \
        !defined (PM8001_RHEL63) && (LINUX_VERSION_CODE != KERNEL_VERSION(3, 0, 76))  && \
    (LINUX_VERSION_CODE >= KERNEL_VERSION(3, 6, 0))
                pm8001_free_task(t);
#else
                sas_free_task(t);
#endif
                return;
            }
        } else {
            u8 len;
            ts->resp = SAS_TASK_COMPLETE;
            ts->stat = SAS_PROTO_RESPONSE;
            ts->residual = param;
            PM8001_IO_DBG(pm8001_ha,
                pm8001_printk("SAS_PROTO_RESPONSE len = %d\n",
                param));
            sata_resp = &psataPayload->sata_resp[0];
            resp = (struct ata_task_resp *)ts->buf;
            if (t->ata_task.dma_xfer == 0 &&
            t->data_dir == PCI_DMA_FROMDEVICE) {
                len = sizeof(struct pio_setup_fis);
                PM8001_IO_DBG(pm8001_ha,
                pm8001_printk("PIO read len = %d\n", len));
            } else if (t->ata_task.use_ncq) {
                len = sizeof(struct set_dev_bits_fis);
                PM8001_IO_DBG(pm8001_ha,
                    pm8001_printk("FPDMA len = %d\n", len));
            } else {
                len = sizeof(struct dev_to_host_fis);
                PM8001_IO_DBG(pm8001_ha,
                pm8001_printk("other len = %d\n", len));
            }
            if (SAS_STATUS_BUF_SIZE >= sizeof(*resp)) {
                resp->frame_len = len;
                memcpy(&resp->ending_fis[0], sata_resp, len);
                ts->buf_valid_size = sizeof(*resp);
            } else
                PM8001_IO_DBG(pm8001_ha,
                    pm8001_printk("response to large\n"));
        }
        if (pm8001_dev)
            pm8001_dev->running_req--;
        break;
    case IO_ABORTED:
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("IO_ABORTED IOMB Tag\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_ABORTED_TASK;
        if (pm8001_dev)
            pm8001_dev->running_req--;
        break;
        /* following cases are to do cases */
    case IO_UNDERFLOW:
        /* SATA Completion with error */
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("IO_UNDERFLOW param = %d\n", param));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_DATA_UNDERRUN;
        ts->residual =  param;
        if (pm8001_dev)
            pm8001_dev->running_req--;
        break;
    case IO_NO_DEVICE:
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("i%s: IO_NO_DEVICE\n", __func__));
        ts->resp = SAS_TASK_UNDELIVERED;
        ts->stat = SAS_PHY_DOWN;
        break;
    case IO_XFER_ERROR_BREAK:
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("IO_XFER_ERROR_BREAK\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_INTERRUPTED;
        break;
    case IO_XFER_ERROR_PHY_NOT_READY:
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("IO_XFER_ERROR_PHY_NOT_READY\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_OPEN_REJECT;
        ts->open_rej_reason = SAS_OREJ_RSVD_RETRY;
        break;
    case IO_OPEN_CNX_ERROR_PROTOCOL_NOT_SUPPORTED:
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("IO_OPEN_CNX_ERROR_PROTOCOL_NOT"
            "_SUPPORTED\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_OPEN_REJECT;
        ts->open_rej_reason = SAS_OREJ_EPROTO;
        break;
    case IO_OPEN_CNX_ERROR_ZONE_VIOLATION:
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("IO_OPEN_CNX_ERROR_ZONE_VIOLATION\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_OPEN_REJECT;
        ts->open_rej_reason = SAS_OREJ_UNKNOWN;
        break;
    case IO_OPEN_CNX_ERROR_BREAK:
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("IO_OPEN_CNX_ERROR_BREAK\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_OPEN_REJECT;
        ts->open_rej_reason = SAS_OREJ_RSVD_CONT0;
        break;
    case IO_OPEN_CNX_ERROR_IT_NEXUS_LOSS:
    case IO_XFER_OPEN_RETRY_BACKOFF_THRESHOLD_REACHED:
    case IO_OPEN_CNX_ERROR_IT_NEXUS_LOSS_OPEN_TMO:
    case IO_OPEN_CNX_ERROR_IT_NEXUS_LOSS_NO_DEST:
    case IO_OPEN_CNX_ERROR_IT_NEXUS_LOSS_OPEN_COLLIDE:
    case IO_OPEN_CNX_ERROR_IT_NEXUS_LOSS_PATHWAY_BLOCKED:
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("%s: IO_OPEN_CNX_ERROR_IT_NEXUS_LOSS(ox%x)\n", __func__, status));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_DEV_NO_RESPONSE;
        //if (!t->uldd_task) 
        {
            pm8001_handle_event(pm8001_ha,
                pm8001_dev,
                IO_OPEN_CNX_ERROR_IT_NEXUS_LOSS);
            ts->resp = SAS_TASK_UNDELIVERED;
            ts->stat = SAS_QUEUE_FULL;
            pm8001_ccb_task_free(pm8001_ha, t, ccb, tag);
            mb();/*in order to force CPU ordering*/
            spin_unlock_irqrestore(&pm8001_ha->lock, pm8001_ha->lock_flags);
            t->task_done(t);
            spin_lock_irqsave(&pm8001_ha->lock, pm8001_ha->lock_flags);
            return;
        }
        break;
    case IO_OPEN_CNX_ERROR_BAD_DESTINATION:
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("%s: IO_OPEN_CNX_ERROR_BAD_DESTINATION\n", __func__));
        ts->resp = SAS_TASK_UNDELIVERED;
        ts->stat = SAS_OPEN_REJECT;
        ts->open_rej_reason = SAS_OREJ_BAD_DEST;
        //if (!t->uldd_task) 
        {
            pm8001_handle_event(pm8001_ha,
                pm8001_dev,
                IO_OPEN_CNX_ERROR_IT_NEXUS_LOSS);
            ts->resp = SAS_TASK_UNDELIVERED;
            ts->stat = SAS_QUEUE_FULL;
            pm8001_ccb_task_free(pm8001_ha, t, ccb, tag);
            mb();/*ditto*/
            spin_unlock_irqrestore(&pm8001_ha->lock, pm8001_ha->lock_flags);
            t->task_done(t);
            spin_lock_irqsave(&pm8001_ha->lock, pm8001_ha->lock_flags);
            return;
        }
        break;
    case IO_OPEN_CNX_ERROR_CONNECTION_RATE_NOT_SUPPORTED:
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("IO_OPEN_CNX_ERROR_CONNECTION_RATE_"
            "NOT_SUPPORTED\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_OPEN_REJECT;
        ts->open_rej_reason = SAS_OREJ_CONN_RATE;
        break;
    case IO_OPEN_CNX_ERROR_STP_RESOURCES_BUSY:
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("IO_OPEN_CNX_ERROR_STP_RESOURCES"
            "_BUSY\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_DEV_NO_RESPONSE;
        //if (!t->uldd_task) 
        {
            pm8001_handle_event(pm8001_ha,
                pm8001_dev,
                IO_OPEN_CNX_ERROR_STP_RESOURCES_BUSY);
            ts->resp = SAS_TASK_UNDELIVERED;
            ts->stat = SAS_QUEUE_FULL;
            pm8001_ccb_task_free(pm8001_ha, t, ccb, tag);
            mb();/* ditto*/
            spin_unlock_irqrestore(&pm8001_ha->lock, pm8001_ha->lock_flags);
            t->task_done(t);
            spin_lock_irqsave(&pm8001_ha->lock, pm8001_ha->lock_flags);
            return;
        }
        break;
    case IO_OPEN_CNX_ERROR_WRONG_DESTINATION:
        PM8001_IO_DBG(pm8001_ha,
               pm8001_printk("IO_OPEN_CNX_ERROR_WRONG_DESTINATION\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_OPEN_REJECT;
        ts->open_rej_reason = SAS_OREJ_WRONG_DEST;
        break;
    case IO_XFER_ERROR_NAK_RECEIVED:
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("IO_XFER_ERROR_NAK_RECEIVED\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_NAK_R_ERR;
        break;
    case IO_XFER_ERROR_ACK_NAK_TIMEOUT:
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("IO_XFER_ERROR_ACK_NAK_TIMEOUT\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_NAK_R_ERR;
        break;
    case IO_XFER_ERROR_DMA:
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("IO_XFER_ERROR_DMA\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_ABORTED_TASK;
        break;
    case IO_XFER_ERROR_SATA_LINK_TIMEOUT:
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("IO_XFER_ERROR_SATA_LINK_TIMEOUT\n"));
        ts->resp = SAS_TASK_UNDELIVERED;
        ts->stat = SAS_DEV_NO_RESPONSE;
        break;
    case IO_XFER_ERROR_REJECTED_NCQ_MODE:
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("IO_XFER_ERROR_REJECTED_NCQ_MODE\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_DATA_UNDERRUN;
        break;
    case IO_XFER_OPEN_RETRY_TIMEOUT:
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("IO_XFER_OPEN_RETRY_TIMEOUT\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_OPEN_TO;
        break;
    case IO_PORT_IN_RESET:
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("IO_PORT_IN_RESET\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_DEV_NO_RESPONSE;
        break;
    case IO_DS_NON_OPERATIONAL:
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("%s: IO_DS_NON_OPERATIONAL\n", __func__));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_DEV_NO_RESPONSE;
        //if (!t->uldd_task) 
        {
            pm8001_handle_event(pm8001_ha, pm8001_dev,
                    IO_DS_NON_OPERATIONAL);
            ts->resp = SAS_TASK_UNDELIVERED;
            ts->stat = SAS_QUEUE_FULL;
            pm8001_ccb_task_free(pm8001_ha, t, ccb, tag);
            mb();/*ditto*/
            spin_unlock_irqrestore(&pm8001_ha->lock, pm8001_ha->lock_flags);
            t->task_done(t);
            spin_lock_irqsave(&pm8001_ha->lock, pm8001_ha->lock_flags);
            return;
        }
        break;
    case IO_DS_IN_RECOVERY:
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("  IO_DS_IN_RECOVERY\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_DEV_NO_RESPONSE;
        break;
    case IO_DS_IN_ERROR:
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("IO_DS_IN_ERROR\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_DEV_NO_RESPONSE;
        //if (!t->uldd_task) 
        {
            pm8001_handle_event(pm8001_ha, pm8001_dev,
                    IO_DS_IN_ERROR);
            ts->resp = SAS_TASK_UNDELIVERED;
            ts->stat = SAS_QUEUE_FULL;
            pm8001_ccb_task_free(pm8001_ha, t, ccb, tag);
            mb();/*ditto*/
            spin_unlock_irqrestore(&pm8001_ha->lock, pm8001_ha->lock_flags);
            t->task_done(t);
            spin_lock_irqsave(&pm8001_ha->lock, pm8001_ha->lock_flags);
            return;
        }
        break;
    case IO_OPEN_CNX_ERROR_HW_RESOURCE_BUSY:
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("IO_OPEN_CNX_ERROR_HW_RESOURCE_BUSY\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_OPEN_REJECT;
        ts->open_rej_reason = SAS_OREJ_RSVD_RETRY;
    default:
        PM8001_DEVIO_DBG(pm8001_ha,
            pm8001_printk(" Unknown status 0x%x \n", status));
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("Unknown status 0x%x\n", status));
        /* not allowed case. Therefore, return failed status */
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_DEV_NO_RESPONSE;
        break;
    }
    spin_lock_irqsave(&t->task_state_lock, flags);
    t->task_state_flags &= ~SAS_TASK_STATE_PENDING;
    t->task_state_flags &= ~SAS_TASK_AT_INITIATOR;
    t->task_state_flags |= SAS_TASK_STATE_DONE;
    if (unlikely((t->task_state_flags & SAS_TASK_STATE_ABORTED))) {
        spin_unlock_irqrestore(&t->task_state_lock, flags);
        PM8001_FAIL_DBG(pm8001_ha,
            pm8001_printk("task 0x%p done with io_status 0x%x"
            " resp 0x%x stat 0x%x but aborted by upper layer!\n",
            t, status, ts->resp, ts->stat));
        pm8001_ccb_task_free(pm8001_ha, t, ccb, tag);
    } else if (t->uldd_task) {
        spin_unlock_irqrestore(&t->task_state_lock, flags);
        pm8001_ccb_task_free(pm8001_ha, t, ccb, tag);
        mb();/* ditto */
        spin_unlock_irqrestore(&pm8001_ha->lock, pm8001_ha->lock_flags);
        t->task_done(t);
        spin_lock_irqsave(&pm8001_ha->lock, pm8001_ha->lock_flags);
    } else if (!t->uldd_task) {
        PM8001_FAIL_DBG(pm8001_ha, pm8001_printk("task with no uldd task \n"));
        spin_unlock_irqrestore(&t->task_state_lock, flags);
        pm8001_ccb_task_free(pm8001_ha, t, ccb, tag);
        mb();/*ditto*/
        spin_unlock_irqrestore(&pm8001_ha->lock, pm8001_ha->lock_flags);
        t->task_done(t);
        spin_lock_irqsave(&pm8001_ha->lock, pm8001_ha->lock_flags);
    }
}

/*See the comments for mpi_ssp_completion */
static void mpi_sata_event(struct pm8001_hba_info *pm8001_ha , void *piomb)
{
    struct sas_task *t = 0;
    struct task_status_struct *ts;
    struct pm8001_ccb_info *ccb;
    struct pm8001_device *pm8001_dev = 0;
    struct sata_event_resp *psataPayload =
        (struct sata_event_resp *)(piomb + 4);
    u32 event = le32_to_cpu(psataPayload->event);
    u32 tag = le32_to_cpu(psataPayload->tag);
    u32 port_id = le32_to_cpu(psataPayload->port_id);
    u32 dev_id = le32_to_cpu(psataPayload->device_id);
    unsigned long flags=0;

    ccb = &pm8001_ha->ccb_info[tag];

    if (ccb)
    {
        t = ccb->task;
        pm8001_dev = ccb->device;
    }
    else
    {
        PM8001_FAIL_DBG(pm8001_ha, 
            pm8001_printk("No CCB !!!. returning \n"));
    }   
    if (event)
        PM8001_FAIL_DBG(pm8001_ha,
            pm8001_printk("SATA EVENT 0x%x\n", event));

    /* NCQ CHANGE STARTS */
    //Added for NCQ as tag might be invalid here.
    if (event == IO_XFER_ERROR_ABORTED_NCQ_MODE)
    {
        /* find device using device id */ 
        PM8001_FAIL_DBG(pm8001_ha, 
            pm8001_printk("NCQ ERROR received for device %x tag %x \n", dev_id, tag));
        pm8001_dev = pm8001_find_dev(pm8001_ha, dev_id);
    
        /* send read log extension */   
        if (pm8001_dev)
        {
            PM8001_FAIL_DBG(pm8001_ha, 
                pm8001_printk("SENDING READ LOG to device id %x device %p \n", dev_id, pm8001_dev));
            pm80xx_send_read_log(pm8001_ha, pm8001_dev);
        }
        return;
    }
    /* NCQ CHANGE ENDS */

    if (unlikely(!t || !t->lldd_task || !t->dev)) {
        PM8001_FAIL_DBG(pm8001_ha,
            pm8001_printk("task or dev null\n"));
        return;
    }

    ts = &t->task_status;
    PM8001_IOERR_DBG(pm8001_ha,
        pm8001_printk("port_id:0x%x, device_id:0x%x, tag:0x%x, event:0x%x\n",
                port_id, dev_id, tag, event));
    switch (event) {
    case IO_OVERFLOW:
        PM8001_FAIL_DBG(pm8001_ha, pm8001_printk("IO_UNDERFLOW\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_DATA_OVERRUN;
        ts->residual = 0;
        if (pm8001_dev)
            pm8001_dev->running_req--;
        break;
    case IO_XFER_ERROR_BREAK:
        PM8001_FAIL_DBG(pm8001_ha,
            pm8001_printk("IO_XFER_ERROR_BREAK\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_INTERRUPTED;
        break;
    case IO_XFER_ERROR_PHY_NOT_READY:
        PM8001_FAIL_DBG(pm8001_ha,
            pm8001_printk("IO_XFER_ERROR_PHY_NOT_READY\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_OPEN_REJECT;
        ts->open_rej_reason = SAS_OREJ_RSVD_RETRY;
        break;
    case IO_OPEN_CNX_ERROR_PROTOCOL_NOT_SUPPORTED:
        PM8001_FAIL_DBG(pm8001_ha,
            pm8001_printk("IO_OPEN_CNX_ERROR_PROTOCOL_NOT"
            "_SUPPORTED\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_OPEN_REJECT;
        ts->open_rej_reason = SAS_OREJ_EPROTO;
        break;
    case IO_OPEN_CNX_ERROR_ZONE_VIOLATION:
        PM8001_FAIL_DBG(pm8001_ha,
            pm8001_printk("IO_OPEN_CNX_ERROR_ZONE_VIOLATION\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_OPEN_REJECT;
        ts->open_rej_reason = SAS_OREJ_UNKNOWN;
        break;
    case IO_OPEN_CNX_ERROR_BREAK:
        PM8001_FAIL_DBG(pm8001_ha,
            pm8001_printk("IO_OPEN_CNX_ERROR_BREAK\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_OPEN_REJECT;
        ts->open_rej_reason = SAS_OREJ_RSVD_CONT0;
        break;
    case IO_OPEN_CNX_ERROR_IT_NEXUS_LOSS:
    case IO_XFER_OPEN_RETRY_BACKOFF_THRESHOLD_REACHED:
    case IO_OPEN_CNX_ERROR_IT_NEXUS_LOSS_OPEN_TMO:
    case IO_OPEN_CNX_ERROR_IT_NEXUS_LOSS_NO_DEST:
    case IO_OPEN_CNX_ERROR_IT_NEXUS_LOSS_OPEN_COLLIDE:
    case IO_OPEN_CNX_ERROR_IT_NEXUS_LOSS_PATHWAY_BLOCKED:
        PM8001_FAIL_DBG(pm8001_ha,
            pm8001_printk("%s: IO_OPEN_CNX_ERROR_IT_NEXUS_LOSS(0x%x)\n", __func__, event));
        ts->resp = SAS_TASK_UNDELIVERED;
        ts->stat = SAS_DEV_NO_RESPONSE;
        //if (!t->uldd_task) 
        {
            pm8001_handle_event(pm8001_ha,
                pm8001_dev,
                IO_OPEN_CNX_ERROR_IT_NEXUS_LOSS);
            ts->resp = SAS_TASK_COMPLETE;
            ts->stat = SAS_QUEUE_FULL;
            pm8001_ccb_task_free(pm8001_ha, t, ccb, tag);
            mb();/*ditto*/
            spin_unlock_irqrestore(&pm8001_ha->lock, pm8001_ha->lock_flags);
            t->task_done(t);
            spin_lock_irqsave(&pm8001_ha->lock, pm8001_ha->lock_flags);
            return;
        }
        break;
    case IO_OPEN_CNX_ERROR_BAD_DESTINATION:
        PM8001_FAIL_DBG(pm8001_ha,
            pm8001_printk("%s: IO_OPEN_CNX_ERROR_BAD_DESTINATION\n", __func__));
        ts->resp = SAS_TASK_UNDELIVERED;
        ts->stat = SAS_OPEN_REJECT;
        ts->open_rej_reason = SAS_OREJ_BAD_DEST;
        break;
    case IO_OPEN_CNX_ERROR_CONNECTION_RATE_NOT_SUPPORTED:
        PM8001_FAIL_DBG(pm8001_ha,
            pm8001_printk("IO_OPEN_CNX_ERROR_CONNECTION_RATE_"
            "NOT_SUPPORTED\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_OPEN_REJECT;
        ts->open_rej_reason = SAS_OREJ_CONN_RATE;
        break;
    case IO_OPEN_CNX_ERROR_WRONG_DESTINATION:
        PM8001_FAIL_DBG(pm8001_ha,
               pm8001_printk("IO_OPEN_CNX_ERROR_WRONG_DESTINATION\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_OPEN_REJECT;
        ts->open_rej_reason = SAS_OREJ_WRONG_DEST;
        break;
    case IO_XFER_ERROR_NAK_RECEIVED:
        PM8001_FAIL_DBG(pm8001_ha,
            pm8001_printk("IO_XFER_ERROR_NAK_RECEIVED\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_NAK_R_ERR;
        break;
    case IO_XFER_ERROR_PEER_ABORTED:
        PM8001_FAIL_DBG(pm8001_ha,
            pm8001_printk("IO_XFER_ERROR_PEER_ABORTED\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_NAK_R_ERR;
        break;
    case IO_XFER_ERROR_REJECTED_NCQ_MODE:
        PM8001_FAIL_DBG(pm8001_ha,
            pm8001_printk("IO_XFER_ERROR_REJECTED_NCQ_MODE\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_DATA_UNDERRUN;
        break;
    /* Note: we will not hit the below case statement as NCQ is handled before the switch/case */
    case IO_XFER_ERROR_ABORTED_NCQ_MODE:
        PM8001_FAIL_DBG(pm8001_ha,
            pm8001_printk("IO_XFER_ERROR_ABORTED_NCQ_MODE\n")); 
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_OPEN_TO;
        break;
    case IO_XFER_OPEN_RETRY_TIMEOUT:
        PM8001_FAIL_DBG(pm8001_ha,
            pm8001_printk("IO_XFER_OPEN_RETRY_TIMEOUT\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_OPEN_TO;
        break;
    case IO_XFER_ERROR_UNEXPECTED_PHASE:
        PM8001_FAIL_DBG(pm8001_ha,
            pm8001_printk("IO_XFER_ERROR_UNEXPECTED_PHASE\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_OPEN_TO;
        break;
    case IO_XFER_ERROR_XFER_RDY_OVERRUN:
        PM8001_FAIL_DBG(pm8001_ha,
            pm8001_printk("IO_XFER_ERROR_XFER_RDY_OVERRUN\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_OPEN_TO;
        break;
    case IO_XFER_ERROR_XFER_RDY_NOT_EXPECTED:
        PM8001_FAIL_DBG(pm8001_ha,
               pm8001_printk("IO_XFER_ERROR_XFER_RDY_NOT_EXPECTED\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_OPEN_TO;
        break;
    case IO_XFER_ERROR_OFFSET_MISMATCH:
        PM8001_FAIL_DBG(pm8001_ha,
            pm8001_printk("IO_XFER_ERROR_OFFSET_MISMATCH\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_OPEN_TO;
        break;
    case IO_XFER_ERROR_XFER_ZERO_DATA_LEN:
        PM8001_FAIL_DBG(pm8001_ha,
            pm8001_printk("IO_XFER_ERROR_XFER_ZERO_DATA_LEN\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_OPEN_TO;
        break;
    case IO_XFER_CMD_FRAME_ISSUED:
        PM8001_FAIL_DBG(pm8001_ha,
            pm8001_printk("IO_XFER_CMD_FRAME_ISSUED\n"));
        break;
    case IO_XFER_PIO_SETUP_ERROR:
        PM8001_FAIL_DBG(pm8001_ha,
            pm8001_printk("IO_XFER_PIO_SETUP_ERROR\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_OPEN_TO;
        break;
    case IO_XFER_ERROR_INTERNAL_CRC_ERROR:
        PM8001_FAIL_DBG(pm8001_ha,
            pm8001_printk("IO_XFR_ERROR_INTERNAL_CRC_ERROR\n"));
        /* TBC: used default set values */
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_OPEN_TO;
        break;
    case IO_XFER_DMA_ACTIVATE_TIMEOUT:
        PM8001_FAIL_DBG(pm8001_ha,
            pm8001_printk("IO_XFR_DMA_ACTIVATE_TIMEOUT\n"));
        /* TBC: used default set values */
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_OPEN_TO;
        break;
    default:
        PM8001_FAIL_DBG(pm8001_ha,
            pm8001_printk(" Unknown status 0x%x \n", event));
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("Unknown status 0x%x\n", event));
        /* not allowed case. Therefore, return failed status */
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_OPEN_TO;
        break;
    }
    spin_lock_irqsave(&t->task_state_lock, flags);
    t->task_state_flags &= ~SAS_TASK_STATE_PENDING;
    t->task_state_flags &= ~SAS_TASK_AT_INITIATOR;
    t->task_state_flags |= SAS_TASK_STATE_DONE;
    if (unlikely((t->task_state_flags & SAS_TASK_STATE_ABORTED))) {
        spin_unlock_irqrestore(&t->task_state_lock, flags);
        PM8001_FAIL_DBG(pm8001_ha,
            pm8001_printk("task 0x%p done with io_status 0x%x"
            " resp 0x%x stat 0x%x but aborted by upper layer!\n",
            t, event, ts->resp, ts->stat));
        pm8001_ccb_task_free(pm8001_ha, t, ccb, tag);
    } else if (t->uldd_task) {
        spin_unlock_irqrestore(&t->task_state_lock, flags);
        pm8001_ccb_task_free(pm8001_ha, t, ccb, tag);
        mb();/* ditto */
        spin_unlock_irqrestore(&pm8001_ha->lock, pm8001_ha->lock_flags);
        t->task_done(t);
        spin_lock_irqsave(&pm8001_ha->lock, pm8001_ha->lock_flags);
    } else if (!t->uldd_task) {
        spin_unlock_irqrestore(&t->task_state_lock, flags);
        pm8001_ccb_task_free(pm8001_ha, t, ccb, tag);
        mb();/*ditto*/
        spin_unlock_irqrestore(&pm8001_ha->lock, pm8001_ha->lock_flags);
        t->task_done(t);
        spin_lock_irqsave(&pm8001_ha->lock, pm8001_ha->lock_flags);
    }
}


/*See the comments for mpi_ssp_completion */
static void
mpi_smp_completion(struct pm8001_hba_info *pm8001_ha, void *piomb)
{
    u32 param, i;
    struct sas_task *t;
    struct pm8001_ccb_info *ccb;
    unsigned long flags;
    u32 status;
    u32 tag;
    struct smp_completion_resp *psmpPayload;
    struct task_status_struct *ts;
    struct pm8001_device *pm8001_dev;
    char *pdma_respaddr=NULL;

    psmpPayload = (struct smp_completion_resp *)(piomb + 4);
    status = le32_to_cpu(psmpPayload->status);
    tag = le32_to_cpu(psmpPayload->tag);

    ccb = &pm8001_ha->ccb_info[tag];
    param = le32_to_cpu(psmpPayload->param);
    if(ccb->task == NULL)
    {
      complete(pm8001_ha->smp_ioctl_completion);
      pm8001_ha->smp_ioctl_completion = NULL;
          clear_bit(tag, (void*)pm8001_ha->tags);
      return;
    }
    t = ccb->task;
    ts = &t->task_status;
    pm8001_dev = ccb->device;
    if (status)
        PM8001_FAIL_DBG(pm8001_ha,
            pm8001_printk("smp IO status 0x%x\n", status));
    if (unlikely(!t || !t->lldd_task || !t->dev))
        return;

    PM8001_DEV_DBG(pm8001_ha,
        pm8001_printk("tag::0x%x status::0x%x \n", tag, status));
    
    
    switch (status) {

    case IO_SUCCESS:
        PM8001_IO_DBG(pm8001_ha, pm8001_printk("IO_SUCCESS\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAM_STAT_GOOD;
        if (pm8001_dev)
            pm8001_dev->running_req--;
        if (pm8001_ha->smp_exp_mode == SMP_DIRECT)
        {   
            PM8001_IO_DBG(pm8001_ha,
                pm8001_printk("DIRECT RESPONSE Length:%d\n", param));
            pdma_respaddr = (char *)(phys_to_virt(cpu_to_le64((u64)sg_dma_address(&t->smp_task.smp_resp))));
            for(i=0; i<param; i++)
            {
                *(pdma_respaddr+i) = psmpPayload->_r_a[i];          
                PM8001_IO_DBG(pm8001_ha,
                    pm8001_printk("SMP Byte%d DMA data 0x%x, psmp 0x%x \n", 
                    i, *(pdma_respaddr+i), psmpPayload->_r_a[i]));
            }
        }
        break;
    case IO_ABORTED:
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("IO_ABORTED IOMB\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_ABORTED_TASK;
        if (pm8001_dev)
            pm8001_dev->running_req--;
        break;
    case IO_OVERFLOW:
        PM8001_IO_DBG(pm8001_ha, pm8001_printk("IO_UNDERFLOW\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_DATA_OVERRUN;
        ts->residual = 0;
        if (pm8001_dev)
            pm8001_dev->running_req--;
        break;
    case IO_NO_DEVICE:
        PM8001_IO_DBG(pm8001_ha, pm8001_printk("%s: IO_NO_DEVICE\n", __func__));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_PHY_DOWN;
        break;
    case IO_ERROR_HW_TIMEOUT:
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("IO_ERROR_HW_TIMEOUT\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAM_STAT_BUSY;
        break;
    case IO_XFER_ERROR_BREAK:
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("IO_XFER_ERROR_BREAK\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAM_STAT_BUSY;
        break;
    case IO_XFER_ERROR_PHY_NOT_READY:
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("IO_XFER_ERROR_PHY_NOT_READY\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAM_STAT_BUSY;
        break;
    case IO_OPEN_CNX_ERROR_PROTOCOL_NOT_SUPPORTED:
        PM8001_IO_DBG(pm8001_ha,
        pm8001_printk("IO_OPEN_CNX_ERROR_PROTOCOL_NOT_SUPPORTED\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_OPEN_REJECT;
        ts->open_rej_reason = SAS_OREJ_UNKNOWN;
        break;
    case IO_OPEN_CNX_ERROR_ZONE_VIOLATION:
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("IO_OPEN_CNX_ERROR_ZONE_VIOLATION\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_OPEN_REJECT;
        ts->open_rej_reason = SAS_OREJ_UNKNOWN;
        break;
    case IO_OPEN_CNX_ERROR_BREAK:
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("IO_OPEN_CNX_ERROR_BREAK\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_OPEN_REJECT;
        ts->open_rej_reason = SAS_OREJ_RSVD_CONT0;
        break;
    case IO_OPEN_CNX_ERROR_IT_NEXUS_LOSS:
    case IO_XFER_OPEN_RETRY_BACKOFF_THRESHOLD_REACHED:
    case IO_OPEN_CNX_ERROR_IT_NEXUS_LOSS_OPEN_TMO:
    case IO_OPEN_CNX_ERROR_IT_NEXUS_LOSS_NO_DEST:
    case IO_OPEN_CNX_ERROR_IT_NEXUS_LOSS_OPEN_COLLIDE:
    case IO_OPEN_CNX_ERROR_IT_NEXUS_LOSS_PATHWAY_BLOCKED:
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("%s: IO_OPEN_CNX_ERROR_IT_NEXUS_LOSS(0x%x)\n", __func__, status));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_OPEN_REJECT;
        ts->open_rej_reason = SAS_OREJ_UNKNOWN;
        //pm8001_dev->device_state = DEVICE_NON_OPERATIONAL;
        printk("%s:%d calling event handle for status 0x%x\n", __func__, __LINE__, status);
        pm8001_handle_event(pm8001_ha,
                pm8001_dev,
                IO_OPEN_CNX_ERROR_IT_NEXUS_LOSS);
        break;
    case IO_OPEN_CNX_ERROR_BAD_DESTINATION:
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("%s: IO_OPEN_CNX_ERROR_BAD_DESTINATION\n", __func__));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_OPEN_REJECT;
        ts->open_rej_reason = SAS_OREJ_BAD_DEST;
        break;
    case IO_OPEN_CNX_ERROR_CONNECTION_RATE_NOT_SUPPORTED:
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("IO_OPEN_CNX_ERROR_CONNECTION_RATE_"
            "NOT_SUPPORTED\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_OPEN_REJECT;
        ts->open_rej_reason = SAS_OREJ_CONN_RATE;
        break;
    case IO_OPEN_CNX_ERROR_WRONG_DESTINATION:
        PM8001_IO_DBG(pm8001_ha,
               pm8001_printk("IO_OPEN_CNX_ERROR_WRONG_DESTINATION\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_OPEN_REJECT;
        ts->open_rej_reason = SAS_OREJ_WRONG_DEST;
        break;
    case IO_XFER_ERROR_RX_FRAME:
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("IO_XFER_ERROR_RX_FRAME\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_DEV_NO_RESPONSE;
        break;
    case IO_XFER_OPEN_RETRY_TIMEOUT:
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("IO_XFER_OPEN_RETRY_TIMEOUT\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_OPEN_REJECT;
        ts->open_rej_reason = SAS_OREJ_RSVD_RETRY;
        break;
    case IO_ERROR_INTERNAL_SMP_RESOURCE:
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("IO_ERROR_INTERNAL_SMP_RESOURCE\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_QUEUE_FULL;
        break;
    case IO_PORT_IN_RESET:
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("IO_PORT_IN_RESET\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_OPEN_REJECT;
        ts->open_rej_reason = SAS_OREJ_RSVD_RETRY;
        break;
    case IO_DS_NON_OPERATIONAL:
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("%s: IO_DS_NON_OPERATIONAL\n", __func__));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_DEV_NO_RESPONSE;
        break;
    case IO_DS_IN_RECOVERY:
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("IO_DS_IN_RECOVERY\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_OPEN_REJECT;
        ts->open_rej_reason = SAS_OREJ_RSVD_RETRY;
        break;
    case IO_OPEN_CNX_ERROR_HW_RESOURCE_BUSY:
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("IO_OPEN_CNX_ERROR_HW_RESOURCE_BUSY\n"));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_OPEN_REJECT;
        ts->open_rej_reason = SAS_OREJ_RSVD_RETRY;
        break;
    default:
        PM8001_DEVIO_DBG(pm8001_ha,
            pm8001_printk(" Unknown status 0x%x \n", status));
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("Unknown status 0x%x\n", status));
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAS_DEV_NO_RESPONSE;
        /* not allowed case. Therefore, return failed status */
        break;
    }
    spin_lock_irqsave(&t->task_state_lock, flags);
    t->task_state_flags &= ~SAS_TASK_STATE_PENDING;
    t->task_state_flags &= ~SAS_TASK_AT_INITIATOR;
    t->task_state_flags |= SAS_TASK_STATE_DONE;
    if (unlikely((t->task_state_flags & SAS_TASK_STATE_ABORTED))) {
        spin_unlock_irqrestore(&t->task_state_lock, flags);
        PM8001_FAIL_DBG(pm8001_ha, pm8001_printk("task 0x%p done with"
            " io_status 0x%x resp 0x%x "
            "stat 0x%x but aborted by upper layer!\n",
            t, status, ts->resp, ts->stat));
        pm8001_ccb_task_free(pm8001_ha, t, ccb, tag);
    } else {
        spin_unlock_irqrestore(&t->task_state_lock, flags);
        pm8001_ccb_task_free(pm8001_ha, t, ccb, tag);
        mb();/* in order to force CPU ordering */
        t->task_done(t);
    }
}

/**
 * pm80xx_hw_event_ack_req- For PM8001,some events need to acknowage to FW.
 * @pm8001_ha: our hba card information
 * @Qnum: the outbound queue message number.
 * @SEA: source of event to ack
 * @port_id: port id.
 * @phyId: phy id.
 * @param0: parameter 0.
 * @param1: parameter 1.
 */
static void pm80xx_hw_event_ack_req(struct pm8001_hba_info *pm8001_ha,
    u32 Qnum, u32 SEA, u32 port_id, u32 phyId, u32 param0, u32 param1)
{
    struct hw_event_ack_req  payload;
    u32 opc = OPC_INB_SAS_HW_EVENT_ACK;

    struct inbound_queue_table *circularQ;

    memset((u8 *)&payload, 0, sizeof(payload));
    circularQ = &pm8001_ha->inbnd_q_tbl[Qnum];
    payload.tag = cpu_to_le32(1);
    payload.phyid_sea_portid = cpu_to_le32(((SEA & 0xFFFF) << 8) |
        ((phyId & 0xFF) << 24) | (port_id & 0xFF));
    payload.param0 = cpu_to_le32(param0);
    payload.param1 = cpu_to_le32(param1);
    pm8001_mpi_build_cmd(pm8001_ha, circularQ, opc, &payload, 0);
}

static int pm80xx_chip_phy_ctl_req(struct pm8001_hba_info *pm8001_ha,
    u32 phyId, u32 phy_op);

/**
 * hw_event_sas_phy_up -FW tells me a SAS phy up event.
 * @pm8001_ha: our hba card information
 * @piomb: IO message buffer
 */
static void
hw_event_sas_phy_up(struct pm8001_hba_info *pm8001_ha, void *piomb)
{
    struct hw_event_resp *pPayload =
        (struct hw_event_resp *)(piomb + 4);
    u32 lr_status_evt_portid =
        le32_to_cpu(pPayload->lr_status_evt_portid);
    u32 phyid_npip_portstate = le32_to_cpu(pPayload->phyid_npip_portstate);

    u8 link_rate =
        (u8)((lr_status_evt_portid & 0xF0000000) >> 28);
    u8 port_id = (u8)(lr_status_evt_portid & 0x000000FF);
    u8 phy_id =
        (u8)((phyid_npip_portstate & 0xFF0000) >> 16);
    u8 portstate = (u8)(phyid_npip_portstate & 0x0000000F);
    
        struct pm8001_port *port = &pm8001_ha->port[port_id];
#ifndef DIAG_MODE
    struct sas_ha_struct *sas_ha = pm8001_ha->sas;
#endif
    struct pm8001_phy *phy = &pm8001_ha->phy[phy_id];
    unsigned long flags;
    u8 deviceType = pPayload->sas_identify.dev_type;
    port->port_state =  portstate;
    phy->phy_state = 0x2;
    PM8001_MSG_DBG(pm8001_ha, 
        pm8001_printk("portid:%d; phyid:%d; linkrate:%d; portstate:%x; devicetype:%x\n",
                        port_id, phy_id, link_rate, portstate, deviceType));

   switch (deviceType) {
    case SAS_PHY_UNUSED:
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk("device type no device.\n"));
        break;
    case SAS_END_DEVICE:
        PM8001_MSG_DBG(pm8001_ha, pm8001_printk("end device.\n"));
        port->port_attached = 1;
        pm80xx_chip_phy_ctl_req(pm8001_ha, phy_id,
            PHY_NOTIFY_ENABLE_SPINUP);
#ifndef DIAG_MODE
        pm8001_get_lrate_mode(phy, link_rate);
#endif
        break;
    case SAS_EDGE_EXPANDER_DEVICE:
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk("expander device.\n"));
        port->port_attached = 1;
#ifndef DIAG_MODE
        pm8001_get_lrate_mode(phy, link_rate);
#endif
        break;
    case SAS_FANOUT_EXPANDER_DEVICE:
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk("fanout expander device.\n"));
        port->port_attached = 1;
#ifndef DIAG_MODE
        pm8001_get_lrate_mode(phy, link_rate);
#endif
        break;
    default:
        PM8001_DEVIO_DBG(pm8001_ha,
            pm8001_printk(" Unknown device type 0x%x \n", deviceType));
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk("unknown device type(%x)\n", deviceType));
        break;
    }
    phy->phy_type |= PORT_TYPE_SAS;
    phy->identify.device_type = deviceType;
    phy->phy_attached = 1;
    if (phy->identify.device_type == SAS_END_DEVICE)
        phy->identify.target_port_protocols = SAS_PROTOCOL_SSP;
    else if (phy->identify.device_type != SAS_PHY_UNUSED)
        phy->identify.target_port_protocols = SAS_PROTOCOL_SMP;
    phy->sas_phy.oob_mode = SAS_OOB_MODE;
#ifndef DIAG_MODE
    sas_ha->notify_phy_event(&phy->sas_phy, PHYE_OOB_DONE);
#endif
    spin_lock_irqsave(&phy->sas_phy.frame_rcvd_lock, flags);
    memcpy(phy->frame_rcvd, &pPayload->sas_identify,
        sizeof(struct sas_identify_frame)-4);
    phy->frame_rcvd_size = sizeof(struct sas_identify_frame) - 4;
    pm8001_get_attached_sas_addr(phy, phy->sas_phy.attached_sas_addr);
    spin_unlock_irqrestore(&phy->sas_phy.frame_rcvd_lock, flags);
    if (pm8001_ha->flags == PM8001F_RUN_TIME)
        mdelay(200);/*delay a moment to wait disk to spinup*/
    pm8001_bytes_dmaed(pm8001_ha, phy_id);
}

/**
 * hw_event_sata_phy_up -FW tells me a SATA phy up event.
 * @pm8001_ha: our hba card information
 * @piomb: IO message buffer
 */
static void
hw_event_sata_phy_up(struct pm8001_hba_info *pm8001_ha, void *piomb)
{
    struct hw_event_resp *pPayload =
        (struct hw_event_resp *)(piomb + 4);
    u32 phyid_npip_portstate = le32_to_cpu(pPayload->phyid_npip_portstate);
    u32 lr_status_evt_portid =
        le32_to_cpu(pPayload->lr_status_evt_portid);
    u8 link_rate =
        (u8)((lr_status_evt_portid & 0xF0000000) >> 28);
    u8 port_id = (u8)(lr_status_evt_portid & 0x000000FF);
    u8 phy_id =
        (u8)((phyid_npip_portstate & 0xFF0000) >> 16);

    u8 portstate = (u8)(phyid_npip_portstate & 0x0000000F);

    struct pm8001_port *port = &pm8001_ha->port[port_id];
    struct sas_ha_struct *sas_ha = pm8001_ha->sas;
    struct pm8001_phy *phy = &pm8001_ha->phy[phy_id];
    unsigned long flags;
    PM8001_DEVIO_DBG(pm8001_ha, pm8001_printk(
        "port id = %d, phy id = %d link_rate %d portstate 0x%x\n", 
                port_id, phy_id, link_rate, portstate));

    port->port_state =  portstate;
    port->port_attached = 1;
    pm8001_get_lrate_mode(phy, link_rate);
    phy->phy_type |= PORT_TYPE_SATA;
    phy->phy_attached = 1;
    phy->phy_state = 0x2;
    phy->sas_phy.oob_mode = SATA_OOB_MODE;
    sas_ha->notify_phy_event(&phy->sas_phy, PHYE_OOB_DONE);
    spin_lock_irqsave(&phy->sas_phy.frame_rcvd_lock, flags);
    memcpy(phy->frame_rcvd, ((u8 *)&pPayload->sata_fis - 4),
        sizeof(struct dev_to_host_fis));
    phy->frame_rcvd_size = sizeof(struct dev_to_host_fis);
    phy->identify.target_port_protocols = SAS_PROTOCOL_SATA;
    #if (LINUX_VERSION_CODE < KERNEL_VERSION(3,10,0) )
    phy->identify.device_type = SATA_DEV;
    #else
    phy->identify.device_type = SAS_SATA_DEV;
    #endif
    pm8001_get_attached_sas_addr(phy, phy->sas_phy.attached_sas_addr);
    spin_unlock_irqrestore(&phy->sas_phy.frame_rcvd_lock, flags);
    pm8001_bytes_dmaed(pm8001_ha, phy_id);
}

/**
 * hw_event_phy_down -we should notify the libsas the phy is down.
 * @pm8001_ha: our hba card information
 * @piomb: IO message buffer
 */
static void
hw_event_phy_down(struct pm8001_hba_info *pm8001_ha, void *piomb)
{
    struct hw_event_resp *pPayload =
        (struct hw_event_resp *)(piomb + 4);

    u32 lr_status_evt_portid =
        le32_to_cpu(pPayload->lr_status_evt_portid);
    u8 port_id = (u8)(lr_status_evt_portid & 0x000000FF);
    u32 phyid_npip_portstate = le32_to_cpu(pPayload->phyid_npip_portstate);
    u8 phy_id =
        (u8)((phyid_npip_portstate & 0xFF0000) >> 16);
    u8 portstate = (u8)(phyid_npip_portstate & 0x0000000F);

    struct pm8001_port *port = &pm8001_ha->port[port_id];
    struct pm8001_phy *phy = &pm8001_ha->phy[phy_id];
    port->port_state =  portstate;
    phy->phy_type = 0;
    phy->identify.device_type = 0;
    phy->phy_attached = 0;
#ifndef DIAG_MODE
    memset(&phy->dev_sas_addr, 0, SAS_ADDR_SIZE);
#endif
    switch (portstate) {
    case PORT_VALID:
        break;
    case PORT_INVALID:
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk(" PortInvalid portID %d\n", port_id));
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk(" Last phy Down and port invalid\n"));
        port->port_attached = 0;
        pm80xx_hw_event_ack_req(pm8001_ha, 0, HW_EVENT_PHY_DOWN,
            port_id, phy_id, 0, 0);
        break;
    case PORT_IN_RESET:
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk(" Port In Reset portID %d\n", port_id));
        break;
    case PORT_NOT_ESTABLISHED:
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk(" phy Down and PORT_NOT_ESTABLISHED\n"));
        port->port_attached = 0;
        break;
    case PORT_LOSTCOMM:
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk(" phy Down and PORT_LOSTCOMM\n"));
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk(" Last phy Down and port invalid\n"));
        port->port_attached = 0;
        pm80xx_hw_event_ack_req(pm8001_ha, 0, HW_EVENT_PHY_DOWN,
            port_id, phy_id, 0, 0);
        break;
    default:
        PM8001_DEVIO_DBG(pm8001_ha,
            pm8001_printk(" phy Down and(default) = 0x%x\n", portstate));
        port->port_attached = 0;
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk(" phy Down and(default) = 0x%x\n",
            portstate));
        break;

    }
}

static int mpi_phy_start_resp(struct pm8001_hba_info *pm8001_ha, void* piomb)
{
    struct phy_start_resp *pPayload =
        (struct phy_start_resp*)(piomb + 4);
    u32 status =
        le32_to_cpu(pPayload->status);
    u32 phy_id =
        le32_to_cpu(pPayload->phyid);
    struct pm8001_phy *phy = &pm8001_ha->phy[phy_id];

    PM8001_INIT_DBG(pm8001_ha,
        pm8001_printk ("phy start resp status:0x%x, phyid:0x%x\n", status, phy_id));
    phy->phy_started = 1;
    if (status == 0) {
        phy->phy_state = 1;
        if (pm8001_ha->flags == PM8001F_RUN_TIME && phy->enable_completion)
        complete(phy->enable_completion);
    }
    if (pm8001_ha->rescan_host.scan_status == 1) {
                pm8001_ha->rescan_host.phys_scanned++;
                //if (pm8001_ha->rescan_host.phys_scanned == pm8001_ha->chip->n_phy - 1)
		    if (pm8001_ha->rescan_host.phys_scanned == pm8001_ha->rescan_host.phys_count)
                        complete(pm8001_ha->scan_completion);
    }

    return 0;

}

/**
 * mpi_thermal_hw_event -The hw event has come.
 * @pm8001_ha: our hba card information
 * @piomb: IO message buffer
 */
static int mpi_thermal_hw_event(struct pm8001_hba_info *pm8001_ha, void* piomb)
{
    struct thermal_hw_event *pPayload =
        (struct thermal_hw_event *)(piomb + 4);

    u32 thermal_event = le32_to_cpu(pPayload->thermal_event);
    u32 rht_lht = le32_to_cpu(pPayload->rht_lht);

    if (thermal_event & 0x40)
    {
        PM8001_FAIL_DBG(pm8001_ha, pm8001_printk("Thermal Event: Local high temperature is violated !!\n"));
        PM8001_FAIL_DBG(pm8001_ha, pm8001_printk("Thermal Event: Measured local high temperature is %d \n", ((rht_lht & 0xFF00) >> 8)));
    }
    if (thermal_event & 0x10)
    {
        PM8001_FAIL_DBG(pm8001_ha, pm8001_printk("Thermal Event: Remote high temperature is violated !!\n"));
        PM8001_FAIL_DBG(pm8001_ha, pm8001_printk("Thermal Event: Measured remote high temperature is %d \n", ((rht_lht & 0xFF000000) >> 24)));
    }
    return 0;
}
    
/**
 * mpi_hw_event -The hw event has come.
 * @pm8001_ha: our hba card information
 * @piomb: IO message buffer
 */
static int mpi_hw_event(struct pm8001_hba_info *pm8001_ha, void* piomb)
{
#if 0
    unsigned long flags;
#endif
    struct hw_event_resp *pPayload =
        (struct hw_event_resp *)(piomb + 4);
    u32 lr_status_evt_portid =
        le32_to_cpu(pPayload->lr_status_evt_portid);
    u32 phyid_npip_portstate = le32_to_cpu(pPayload->phyid_npip_portstate);
    u8 port_id = (u8)(lr_status_evt_portid & 0x000000FF);
    u8 phy_id =
        (u8)((phyid_npip_portstate & 0xFF0000) >> 16);
    u16 eventType =
        (u16)((lr_status_evt_portid & 0x00FFFF00) >> 8);
    u8 status =
        (u8)((lr_status_evt_portid & 0x0F000000) >> 24);

    struct pm8001_phy *phy = &pm8001_ha->phy[phy_id];
#if 0
    struct sas_ha_struct *sas_ha = pm8001_ha->sas;
    struct asd_sas_phy *sas_phy = sas_ha->sas_phy[phy_id];
#endif
    PM8001_DEV_DBG(pm8001_ha,
        pm8001_printk("portid:%d phyid:%d event:0x%x status:0x%x \n", 
                               port_id, phy_id, eventType, status));

    switch (eventType) {

    case HW_EVENT_SAS_PHY_UP:
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk("HW_EVENT_SAS_PHY_UP \n"));
        if(pm8001_ha->reset_in_progress){
            return 0;
        }
        hw_event_sas_phy_up(pm8001_ha, piomb);
        break;
    case HW_EVENT_SATA_PHY_UP:
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk("HW_EVENT_SATA_PHY_UP\n"));
        if(pm8001_ha->reset_in_progress){
            return 0;
        }
    
        hw_event_sata_phy_up(pm8001_ha, piomb);
        break;
    case HW_EVENT_SATA_SPINUP_HOLD:
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk("HW_EVENT_SATA_SPINUP_HOLD\n"));
#if 0
        sas_ha->notify_phy_event(&phy->sas_phy, PHYE_SPINUP_HOLD);
#endif
        break;
    case HW_EVENT_PHY_DOWN:
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk("HW_EVENT_PHY_DOWN\n"));
#if 0
        sas_ha->notify_phy_event(&phy->sas_phy, PHYE_LOSS_OF_SIGNAL);
#endif
        phy->phy_attached = 0;
        phy->phy_state = 0;
        hw_event_phy_down(pm8001_ha, piomb);
        break;
    case HW_EVENT_PORT_INVALID:
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk("HW_EVENT_PORT_INVALID\n"));
#if 0
        sas_phy_disconnected(sas_phy);
        phy->phy_attached = 0;
        sas_ha->notify_port_event(sas_phy, PORTE_LINK_RESET_ERR);
#endif
        break;
    /* the broadcast change primitive received, tell the LIBSAS this event
    to revalidate the sas domain*/
    case HW_EVENT_BROADCAST_CHANGE:
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk("HW_EVENT_BROADCAST_CHANGE\n"));
        pm80xx_hw_event_ack_req(pm8001_ha, 0, HW_EVENT_BROADCAST_CHANGE,
            port_id, phy_id, 1, 0);
        if(pm8001_ha->reset_in_progress){
            return 0;
        }
#if 0
        spin_lock_irqsave(&sas_phy->sas_prim_lock, flags);
        sas_phy->sas_prim = HW_EVENT_BROADCAST_CHANGE;
        spin_unlock_irqrestore(&sas_phy->sas_prim_lock, flags);
        sas_ha->notify_port_event(sas_phy, PORTE_BROADCAST_RCVD);
#endif
        break;
    case HW_EVENT_PHY_ERROR:
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk("HW_EVENT_PHY_ERROR\n"));
#if 0
        sas_phy_disconnected(&phy->sas_phy);
        phy->phy_attached = 0;
        sas_ha->notify_phy_event(&phy->sas_phy, PHYE_OOB_ERROR);
#endif
        break;
    case HW_EVENT_BROADCAST_EXP:
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk("HW_EVENT_BROADCAST_EXP\n"));
#if 0
        spin_lock_irqsave(&sas_phy->sas_prim_lock, flags);
        sas_phy->sas_prim = HW_EVENT_BROADCAST_EXP;
        spin_unlock_irqrestore(&sas_phy->sas_prim_lock, flags);
        sas_ha->notify_port_event(sas_phy, PORTE_BROADCAST_RCVD);
#endif
        break;
    case HW_EVENT_LINK_ERR_INVALID_DWORD:
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk("HW_EVENT_LINK_ERR_INVALID_DWORD\n"));
        pm80xx_hw_event_ack_req(pm8001_ha, 0,
            HW_EVENT_LINK_ERR_INVALID_DWORD, port_id, phy_id, 0, 0);
#if 0
        sas_phy_disconnected(sas_phy);
        phy->phy_attached = 0;
        sas_ha->notify_port_event(sas_phy, PORTE_LINK_RESET_ERR);
#endif
        break;
    case HW_EVENT_LINK_ERR_DISPARITY_ERROR:
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk("HW_EVENT_LINK_ERR_DISPARITY_ERROR\n"));
        pm80xx_hw_event_ack_req(pm8001_ha, 0,
            HW_EVENT_LINK_ERR_DISPARITY_ERROR,
            port_id, phy_id, 0, 0);
#if 0
        sas_phy_disconnected(sas_phy);
        phy->phy_attached = 0;
        sas_ha->notify_port_event(sas_phy, PORTE_LINK_RESET_ERR);
#endif
        break;
    case HW_EVENT_LINK_ERR_CODE_VIOLATION:
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk("HW_EVENT_LINK_ERR_CODE_VIOLATION\n"));
        pm80xx_hw_event_ack_req(pm8001_ha, 0,
            HW_EVENT_LINK_ERR_CODE_VIOLATION,
            port_id, phy_id, 0, 0);
#if 0
        sas_phy_disconnected(sas_phy);
        phy->phy_attached = 0;
        sas_ha->notify_port_event(sas_phy, PORTE_LINK_RESET_ERR);
#endif
        break;
    case HW_EVENT_LINK_ERR_LOSS_OF_DWORD_SYNCH:
        PM8001_MSG_DBG(pm8001_ha,
              pm8001_printk("HW_EVENT_LINK_ERR_LOSS_OF_DWORD_SYNCH\n"));
        pm80xx_hw_event_ack_req(pm8001_ha, 0,
            HW_EVENT_LINK_ERR_LOSS_OF_DWORD_SYNCH,
            port_id, phy_id, 0, 0);
#if 0
        sas_phy_disconnected(sas_phy);
        phy->phy_attached = 0;
        sas_ha->notify_port_event(sas_phy, PORTE_LINK_RESET_ERR);
#endif
        break;
    case HW_EVENT_MALFUNCTION:
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk("HW_EVENT_MALFUNCTION\n"));
        break;
    case HW_EVENT_BROADCAST_SES:
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk("HW_EVENT_BROADCAST_SES\n"));
        if(pm8001_ha->reset_in_progress){
            return 0;
        }
#if 0
        spin_lock_irqsave(&sas_phy->sas_prim_lock, flags);
        sas_phy->sas_prim = HW_EVENT_BROADCAST_SES;
        spin_unlock_irqrestore(&sas_phy->sas_prim_lock, flags);
        sas_ha->notify_port_event(sas_phy, PORTE_BROADCAST_RCVD);
#endif
        break;
    case HW_EVENT_INBOUND_CRC_ERROR:
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk("HW_EVENT_INBOUND_CRC_ERROR\n"));
        pm80xx_hw_event_ack_req(pm8001_ha, 0,
            HW_EVENT_INBOUND_CRC_ERROR,
            port_id, phy_id, 0, 0);
        break;
    case HW_EVENT_HARD_RESET_RECEIVED:
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk("HW_EVENT_HARD_RESET_RECEIVED\n"));
#if 0
        sas_ha->notify_port_event(sas_phy, PORTE_HARD_RESET);
#endif
        break;
    case HW_EVENT_ID_FRAME_TIMEOUT:
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk("HW_EVENT_ID_FRAME_TIMEOUT\n"));
#if 0
        sas_phy_disconnected(sas_phy);
        phy->phy_attached = 0;
        sas_ha->notify_port_event(sas_phy, PORTE_LINK_RESET_ERR);
#endif
        break;
    case HW_EVENT_LINK_ERR_PHY_RESET_FAILED:
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk("HW_EVENT_LINK_ERR_PHY_RESET_FAILED\n"));
        pm80xx_hw_event_ack_req(pm8001_ha, 0,
            HW_EVENT_LINK_ERR_PHY_RESET_FAILED,
            port_id, phy_id, 0, 0);
#if 0
        sas_phy_disconnected(sas_phy);
        phy->phy_attached = 0;
        sas_ha->notify_port_event(sas_phy, PORTE_LINK_RESET_ERR);
#endif
        break;
    case HW_EVENT_PORT_RESET_TIMER_TMO:
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk("HW_EVENT_PORT_RESET_TIMER_TMO\n"));
#if 0
        sas_phy_disconnected(sas_phy);
        phy->phy_attached = 0;
        sas_ha->notify_port_event(sas_phy, PORTE_LINK_RESET_ERR);
#endif
        break;
    case HW_EVENT_PORT_RECOVERY_TIMER_TMO:
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk("HW_EVENT_PORT_RECOVERY_TIMER_TMO\n"));
        pm80xx_hw_event_ack_req(pm8001_ha, 0,
            HW_EVENT_PORT_RECOVERY_TIMER_TMO,
            port_id, phy_id, 0, 0);
#if 0
        sas_phy_disconnected(sas_phy);
        phy->phy_attached = 0;
        sas_ha->notify_port_event(sas_phy, PORTE_LINK_RESET_ERR);
#endif
        break;
    case HW_EVENT_PORT_RECOVER:
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk("HW_EVENT_PORT_RECOVER\n"));
        break;
    case HW_EVENT_PORT_RESET_COMPLETE:
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk("HW_EVENT_PORT_RESET_COMPLETE\n"));
        break;
    case EVENT_BROADCAST_ASYNCH_EVENT:
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk("EVENT_BROADCAST_ASYNCH_EVENT\n"));
        break;
    default:
        PM8001_DEVIO_DBG(pm8001_ha,
            pm8001_printk("Unknown event type = 0x%x\n", eventType));
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk("Unknown event type = 0x%x\n", eventType));
        break;
    }
    return 0;
}

/**
 * mpi_phy_stop_resp - SPCv specific
 * @pm8001_ha: our hba card information
 * @piomb: IO message buffer
 */
static int mpi_phy_stop_resp(struct pm8001_hba_info *pm8001_ha, void* piomb)
{
    struct phy_stop_resp *pPayload =
        (struct phy_stop_resp *)(piomb + 4);
    u32 status =
        le32_to_cpu(pPayload->status);
    u32 phyid =
        le32_to_cpu(pPayload->phyid);
    struct pm8001_phy *phy = &pm8001_ha->phy[phyid & 0xFF];
    PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk("phy:0x%x status:0x%x\n",
                    phyid & 0xFF, status));
    if (status == 0) {
        phy->phy_state = 0;
    }
    phy->phy_started = 0;
    return 0;
}

/**
 * mpi_set_controller_config_resp - SPCv specific
 * @pm8001_ha: our hba card information
 * @piomb: IO message buffer
 */
static int mpi_set_controller_config_resp(struct pm8001_hba_info *pm8001_ha, void* piomb)
{
    struct set_ctrl_cfg_resp *pPayload = (struct set_ctrl_cfg_resp *)(piomb + 4);
    u32 status = le32_to_cpu(pPayload->status);
    u32 err_qlfr_pgcd = le32_to_cpu(pPayload->err_qlfr_pgcd);

    PM8001_MSG_DBG(pm8001_ha, pm8001_printk("SET CONTROLLER RESP: status 0x%x qlfr_pgcd 0x%x \n", status, err_qlfr_pgcd));
    
    return 0;
}

/**
 * mpi_get_controller_config_resp - SPCv specific
 * @pm8001_ha: our hba card information
 * @piomb: IO message buffer
 */
static int mpi_get_controller_config_resp(struct pm8001_hba_info *pm8001_ha, void* piomb)
{
    PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk(" pm80xx_addition_functionality \n"));
    
    return 0;
}

/**
 * mpi_get_phy_profile_resp - SPCv specific
 * @pm8001_ha: our hba card information
 * @piomb: IO message buffer
 */
static int mpi_get_phy_profile_resp(struct pm8001_hba_info *pm8001_ha, void* piomb)
{
	int ret;
	u32 tag, ppc_phyid, page_code,phy_id;
	struct phy_profile *phy_profile, *phy_prof;
	struct phy_errcnt *phy_err, *phy_err_cnt;
	struct get_phy_profile_resp *pPayload = 
				(struct get_phy_profile_resp*)(piomb + 4);
    u32 status = le32_to_cpu(pPayload->status);
    ppc_phyid  = le32_to_cpu(pPayload->ppc_phyid);       
    page_code  = (u8)((ppc_phyid & 0xFF00) >> 8);
    phy_id      = (u8)(ppc_phyid & 0xFF);
    
    PM8001_MSG_DBG(pm8001_ha,
        pm8001_printk(" pm80xx_addition_functionality \n"));

    if (status){        
         /* status is FAILED */
        PM8001_FAIL_DBG(pm8001_ha, pm8001_printk("mpiGetPhyProfileReq \
            failed  with status 0x%08X \n", status));

        return -1;
    }

	tag = le32_to_cpu(pPayload->tag);

    spin_lock(&pm8001_ha->ioctl_lock);
    if(pm8001_ha->ioctl_completion != NULL || pm8001_ha->phy[phy_id].enable_completion !=NULL)
    {
        if(pm8001_ha->ioctl_completion != NULL){
                ret = del_timer(&pm8001_ha->ioctl_timer);
                if (ret)
                    PM8001_MSG_DBG(pm8001_ha,
                                   pm8001_printk("The timer was still in use... \n"));

                pm8001_ha->ioctl_timer_expired = 0;
        }
        if (page_code == SAS_PHY_ERR_COUNTERS_PAGE) {
            phy_err = (struct phy_errcnt*)&pm8001_ha->phy_profile_resp;
            phy_err_cnt = (struct phy_errcnt*)pPayload->ppc_specific_rsp;
            phy_err->InvalidDword = le32_to_cpu(phy_err_cnt->InvalidDword);
            phy_err->runningDisparityError = le32_to_cpu(phy_err_cnt->runningDisparityError);
            phy_err->LossOfSyncDW = le32_to_cpu(phy_err_cnt->LossOfSyncDW);
            phy_err->phyResetProblem = le32_to_cpu(phy_err_cnt->phyResetProblem);
        }
        if (page_code == SAS_PHY_GENERAL_STATUS_PAGE) {
            phy_profile = (struct phy_profile*)&pm8001_ha->phy_profile_resp;
            phy_prof = (struct phy_profile *)pPayload->ppc_specific_rsp;
            phy_profile->phy_id = le32_to_cpu(phy_prof->phy_id);
            phy_profile->phys = le32_to_cpu(phy_prof->phys);
            phy_profile->plr = le32_to_cpu(phy_prof->plr);
            phy_profile->nlr = le32_to_cpu(phy_prof->nlr);
            phy_profile->port_id = le32_to_cpu(phy_prof->port_id);
            phy_profile->prts = le32_to_cpu(phy_prof->prts);
        }

        if(pm8001_ha->ioctl_completion != NULL){
            complete(pm8001_ha->ioctl_completion);
            pm8001_ha->ioctl_completion = NULL;
        }

        if(pm8001_ha->phy[phy_id].enable_completion !=NULL){
            complete(pm8001_ha->phy[phy_id].enable_completion);
            pm8001_ha->phy[phy_id].enable_completion = NULL;
        }

    }
    spin_unlock(&pm8001_ha->ioctl_lock);
    pm8001_tag_free(pm8001_ha, tag);
    return 0;	
}

/**
 * mpi_flash_op_ext_resp - SPCv specific
 * @pm8001_ha: our hba card information
 * @piomb: IO message buffer
 */
static int mpi_flash_op_ext_resp(struct pm8001_hba_info *pm8001_ha, void* piomb)
{
    PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk(" pm80xx_addition_functionality \n"));
    
    return 0;
}
// Delray phy settings
/**
 * mpi_set_phy_profile_resp - SPCv specific
 * @pm8001_ha: our hba card information
 * @piomb: IO message buffer
 */
static int mpi_set_phy_profile_resp(struct pm8001_hba_info *pm8001_ha, void* piomb)
{
        u8 page_code; 
        struct set_phy_profile_resp *pPayload = (struct set_phy_profile_resp *)(piomb + 4);       
        u32 ppc_phyid = le32_to_cpu(pPayload->ppc_phyid);
        u32 status = le32_to_cpu(pPayload->status);       
        page_code = (u8)((ppc_phyid & 0xFF00) >> 8);
        PM8001_INIT_DBG(pm8001_ha,pm8001_printk("ppc value is %x \n",page_code));

        if (status){        
             /* status is FAILED */
            PM8001_INIT_DBG(pm8001_ha,pm8001_printk("mpiSetPhyProfileRsp failed  with status 0x%08X \n", status));
            return -1;
        }
        else{
            PM8001_INIT_DBG(pm8001_ha,pm8001_printk("mpiSetPhyProfileRsp: SUCCESS type 0x%X\n",page_code ));
            if(page_code == SAS_PHY_ANALOG_SETTINGS_PAGE){
                    PM8001_INIT_DBG(pm8001_ha,pm8001_printk("mpiSetPhyProfileRsp: AGSA_SAS_PHY_ANALOG_SETTINGS_PAGE 0x%X\n",page_code));
                }
                else{
                    PM8001_INIT_DBG(pm8001_ha,pm8001_printk("Invalid ppc 0x%X\n",page_code));
                    return -1;
                }

        }
        return 0;

}

/**
 * mpi_kek_management_resp - SPCv specific
 * @pm8001_ha: our hba card information
 * @piomb: IO message buffer
 */
static int mpi_kek_management_resp(struct pm8001_hba_info *pm8001_ha, void* piomb)
{
    struct kek_mgmt_resp *pPayload = (struct kek_mgmt_resp *)(piomb + 4);

    u32 status = le32_to_cpu(pPayload->status);
    u32 kidx_new_curr_ksop = le32_to_cpu(pPayload->kidx_new_curr_ksop);
    u32 err_qlfr = le32_to_cpu(pPayload->err_qlfr);
    
    PM8001_MSG_DBG(pm8001_ha, pm8001_printk(" KEK MGMT RESP. Status 0x%x idx_ksop 0x%x err_qlfr 0x%x \n", status, kidx_new_curr_ksop, err_qlfr));
    
    return 0;
}

/**
 * mpi_dek_management_resp - SPCv specific
 * @pm8001_ha: our hba card information
 * @piomb: IO message buffer
 */
static int mpi_dek_management_resp(struct pm8001_hba_info *pm8001_ha, void* piomb)
{
    PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk(" pm80xx_addition_functionality \n"));
    
    return 0;
}

/**
 * ssp_coalesced_comp_resp - SPCv specific
 * @pm8001_ha: our hba card information
 * @piomb: IO message buffer
 */
static int ssp_coalesced_comp_resp(struct pm8001_hba_info *pm8001_ha, void* piomb)
{
    PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk(" pm80xx_addition_functionality \n"));
    
    return 0;
}

static int mpi_gpio_resp(struct pm8001_hba_info *pm8001_ha, void *piomb)
{
    int ret;
    u32 tag;
    struct gpio_ioctl_resp *pgpio_resp;
    struct gpio_resp *ppayload = (struct gpio_resp *)(piomb + 4);

    tag = le32_to_cpu(ppayload->tag);
    spin_lock(&pm8001_ha->ioctl_lock);
    if(pm8001_ha->ioctl_completion != NULL)
    {
        ret = del_timer(&pm8001_ha->ioctl_timer);
        if (ret)
            PM8001_MSG_DBG(pm8001_ha, 
                pm8001_printk("The timer was still in use... \n"));

        pm8001_ha->ioctl_timer_expired = 0;
        pgpio_resp = &pm8001_ha->gpio_resp; 
        pgpio_resp->gpio_rd_val = le32_to_cpu(ppayload->gpio_rd_val);
        pgpio_resp->gpio_in_enabled = le32_to_cpu(ppayload->gpio_in_enabled);
        pgpio_resp->gpio_pinsetup1 = le32_to_cpu(ppayload->gpio_pinsetup1);
        pgpio_resp->gpio_pinsetup2 = le32_to_cpu(ppayload->gpio_pinsetup2);
        pgpio_resp->gpio_evt_change = le32_to_cpu(ppayload->gpio_evt_change);
        pgpio_resp->gpio_evt_rise = le32_to_cpu(ppayload->gpio_evt_rise);
        pgpio_resp->gpio_evt_fall = le32_to_cpu(ppayload->gpio_evt_fall);

        complete(pm8001_ha->ioctl_completion);
        pm8001_ha->ioctl_completion = NULL;
    }
    spin_unlock(&pm8001_ha->ioctl_lock);
    pm8001_tag_free(pm8001_ha, tag);
    return 0;
}

static int mpi_gpio_event(struct pm8001_hba_info *pm8001_ha, void *piomb)
{
    u32 gpio_event = 0;
    struct gpio_event *ppayload = (struct gpio_event *)(piomb + 4);
    gpio_event = le32_to_cpu(ppayload->gpio_event);
    PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk("GPIO event: 0x%X\n", gpio_event));
    pm8001_ha->gpio_event_occured = 1;
    wake_up_interruptible(&pm8001_ha->pollq);
    return 0;
}

/**
 * process_one_iomb - process one outbound Queue memory block
 * @pm8001_ha: our hba card information
 * @piomb: IO message buffer
 */
static void process_one_iomb(struct pm8001_hba_info *pm8001_ha, void *piomb)
{
    u32 pHeader = (u32)*(u32 *)piomb;
    u16 opc = (u16)(pHeader & 0xFFF);

    PM8001_MSG_DBG(pm8001_ha, printk(KERN_ERR "%s: Process One IOMB, opc - 0x%x\n", __func__, opc));
    switch (opc) {
    case OPC_OUB_ECHO:
        PM8001_MSG_DBG(pm8001_ha, pm8001_printk("OPC_OUB_ECHO\n"));
        break;
    case OPC_OUB_HW_EVENT:
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk("OPC_OUB_HW_EVENT\n"));
        mpi_hw_event(pm8001_ha, piomb);
        break;
    case OPC_OUB_THERM_HW_EVENT:
        PM8001_MSG_DBG(pm8001_ha, pm8001_printk("OPC_OUB_THERMAL_EVENT\n"));
        mpi_thermal_hw_event(pm8001_ha, piomb);
        break;
    case OPC_OUB_SSP_COMP:
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk("OPC_OUB_SSP_COMP\n"));
        mpi_ssp_completion(pm8001_ha, piomb);
        break;
    case OPC_OUB_SMP_COMP:
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk("OPC_OUB_SMP_COMP\n"));
        mpi_smp_completion(pm8001_ha, piomb);
        break;
    case OPC_OUB_LOCAL_PHY_CNTRL:
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk("OPC_OUB_LOCAL_PHY_CNTRL\n"));
        pm8001_mpi_local_phy_ctl(pm8001_ha, piomb);
        break;
    case OPC_OUB_DEV_REGIST:
                PM8001_MSG_DBG(pm8001_ha,
                        pm8001_printk("OPC_OUB_DEV_REGIST\n"));
                pm8001_mpi_reg_resp(pm8001_ha, piomb);
                break;
    case OPC_OUB_DEREG_DEV:
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk("unresgister the deviece\n"));
        pm8001_mpi_dereg_resp(pm8001_ha, piomb);
        break;
    case OPC_OUB_GET_DEV_HANDLE:
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk("OPC_OUB_GET_DEV_HANDLE\n"));
        break;
    case OPC_OUB_SATA_COMP:
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk("OPC_OUB_SATA_COMP\n"));
        mpi_sata_completion(pm8001_ha, piomb);
        break;
    case OPC_OUB_SATA_EVENT:
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk("OPC_OUB_SATA_EVENT\n"));
        mpi_sata_event(pm8001_ha, piomb);
        break;
    case OPC_OUB_SSP_EVENT:
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk("OPC_OUB_SSP_EVENT\n"));
        mpi_ssp_event(pm8001_ha, piomb);
        break;
    case OPC_OUB_DEV_HANDLE_ARRIV:
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk("OPC_OUB_DEV_HANDLE_ARRIV\n"));
        /*This is for target*/
        break;
    case OPC_OUB_SSP_RECV_EVENT:
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk("OPC_OUB_SSP_RECV_EVENT\n"));
        /*This is for target*/
        break;
    case OPC_OUB_FW_FLASH_UPDATE:
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk("OPC_OUB_FW_FLASH_UPDATE\n"));
        pm8001_mpi_fw_flash_update_resp(pm8001_ha, piomb);
        break;
    case OPC_OUB_GPIO_RESPONSE:
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk("OPC_OUB_GPIO_RESPONSE\n"));
        mpi_gpio_resp(pm8001_ha, piomb);
        break;
    case OPC_OUB_GPIO_EVENT:
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk("OPC_OUB_GPIO_EVENT\n"));
        mpi_gpio_event(pm8001_ha, piomb);
        break;
    case OPC_OUB_GENERAL_EVENT:
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk("OPC_OUB_GENERAL_EVENT\n"));
        pm8001_mpi_general_event(pm8001_ha, piomb);
        break;
    case OPC_OUB_SSP_ABORT_RSP:
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk("OPC_OUB_SSP_ABORT_RSP\n"));
        pm8001_mpi_task_abort_resp(pm8001_ha, piomb);
        break;
    case OPC_OUB_SATA_ABORT_RSP:
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk("OPC_OUB_SATA_ABORT_RSP\n"));
        pm8001_mpi_task_abort_resp(pm8001_ha, piomb);
        break;
    case OPC_OUB_SAS_DIAG_MODE_START_END:
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk("OPC_OUB_SAS_DIAG_MODE_START_END\n"));
        pm8001_mpi_diag_start_end_response(pm8001_ha, piomb);
        break;
    case OPC_OUB_SAS_DIAG_EXECUTE:
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk("OPC_OUB_SAS_DIAG_EXECUTE\n"));
        pm8001_mpi_diag_exec_response(pm8001_ha, piomb);
        break;
    case OPC_OUB_GET_TIME_STAMP:
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk("OPC_OUB_GET_TIME_STAMP\n"));
        break;
    case OPC_OUB_SAS_HW_EVENT_ACK:
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk("OPC_OUB_SAS_HW_EVENT_ACK\n"));
        break;
    case OPC_OUB_PORT_CONTROL:
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk("OPC_OUB_PORT_CONTROL\n"));
        break;
    case OPC_OUB_SMP_ABORT_RSP:
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk("OPC_OUB_SMP_ABORT_RSP\n"));
        pm8001_mpi_task_abort_resp(pm8001_ha, piomb);
        break;
    case OPC_OUB_GET_NVMD_DATA:
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk("OPC_OUB_GET_NVMD_DATA\n"));
        pm8001_mpi_get_nvmd_resp(pm8001_ha, piomb);
        break;
    case OPC_OUB_SET_NVMD_DATA:
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk("OPC_OUB_SET_NVMD_DATA\n"));
        pm8001_mpi_set_nvmd_resp(pm8001_ha, piomb);
        break;
    case OPC_OUB_DEVICE_HANDLE_REMOVAL:
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk("OPC_OUB_DEVICE_HANDLE_REMOVAL\n"));
        break;
    case OPC_OUB_SET_DEVICE_STATE:
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk("OPC_OUB_SET_DEVICE_STATE\n"));
        pm8001_mpi_set_dev_state_resp(pm8001_ha, piomb);
        break;
    case OPC_OUB_GET_DEVICE_STATE:
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk("OPC_OUB_GET_DEVICE_STATE\n"));
        break;
    case OPC_OUB_SET_DEV_INFO:
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk("OPC_OUB_SET_DEV_INFO\n"));
        break;
    /* spcv specifc commands */
    case OPC_OUB_PHY_START_RESP:
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk("OPC_OUB_PHY_START_RESP opcode:%x \n", opc));
        mpi_phy_start_resp(pm8001_ha, piomb);
        break;
    case OPC_OUB_PHY_STOP_RESP:
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk("OPC_OUB_PHY_STOP_RESP opcode:%x \n", opc));
        mpi_phy_stop_resp(pm8001_ha, piomb);
        break;
    case OPC_OUB_SET_CONTROLLER_CONFIG:
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk("OPC_OUB_SET_CONTROLLER_CONFIG opcode:%x \n", opc));
        mpi_set_controller_config_resp(pm8001_ha, piomb);
        break;
    case OPC_OUB_GET_CONTROLLER_CONFIG:
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk("OPC_OUB_GET_CONTROLLER_CONFIG opcode:%x \n", opc));
        mpi_get_controller_config_resp(pm8001_ha, piomb);
        break;
    case OPC_OUB_GET_PHY_PROFILE:
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk("OPC_OUB_GET_PHY_PROFILE opcode:%x \n", opc));
        mpi_get_phy_profile_resp(pm8001_ha, piomb);
        break;
    case OPC_OUB_FLASH_OP_EXT:
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk("OPC_OUB_FLASH_OP_EXT opcode:%x \n", opc));
        mpi_flash_op_ext_resp(pm8001_ha, piomb);
        break;
    case OPC_OUB_SET_PHY_PROFILE:
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk("OPC_OUB_SET_PHY_PROFILE opcode:%x \n", opc));
        mpi_set_phy_profile_resp(pm8001_ha, piomb);
        break;
    case OPC_OUB_KEK_MANAGEMENT_RESP:
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk("OPC_OUB_KEK_MANAGEMENT_RESP opcode:%x \n", opc));
        mpi_kek_management_resp(pm8001_ha, piomb);
        break;
    case OPC_OUB_DEK_MANAGEMENT_RESP:
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk("OPC_OUB_DEK_MANAGEMENT_RESP opcode:%x \n", opc));
        mpi_dek_management_resp(pm8001_ha, piomb);
        break;
    case OPC_OUB_SSP_COALESCED_COMP_RESP:
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk("OPC_OUB_SSP_COALESCED_COMP_RESP opcode:%x \n", opc));
        ssp_coalesced_comp_resp(pm8001_ha, piomb);
        break;
    case OPC_OUB_SGPIO_RESP:
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk("OPC_OUB_SGPIO RESPONSE opcode: %x\n", opc));
        pm8001_sgpio_resp(pm8001_ha, piomb);
        break;
    default:
        PM8001_DEVIO_DBG(pm8001_ha,
            pm8001_printk("Unknown outbound Queue IOMB OPC = 0x%x\n", opc));
        PM8001_MSG_DBG(pm8001_ha,
            pm8001_printk("Unknown outbound Queue IOMB OPC = 0x%x\n",
            opc));
        break;
    }
}

static int process_oq(struct pm8001_hba_info *pm8001_ha, u8 vec)
{
    struct outbound_queue_table *circularQ;
    void *pMsg1 = NULL;
    u8 uninitialized_var(bc);
    u32 ret = MPI_IO_STATUS_FAIL;

    spin_lock_irqsave(&pm8001_ha->lock, pm8001_ha->lock_flags);
    circularQ = &pm8001_ha->outbnd_q_tbl[vec];
    do {
        ret = pm8001_mpi_msg_consume(pm8001_ha, circularQ, &pMsg1, &bc);
        if (MPI_IO_STATUS_SUCCESS == ret) {
            /* process the outbound message */
            process_one_iomb(pm8001_ha, (void *)(pMsg1 - 4));
            /* free the message from the outbound circular buffer */
            pm8001_mpi_msg_free_set(pm8001_ha, pMsg1, circularQ, bc);
      	//printk(KERN_ERR "%s: Processed outbound\n", __func__);
        }
        if (MPI_IO_STATUS_BUSY == ret) {
        	//printk(KERN_ERR "%s: MPI busy status\n", __func__);
            /* Update the producer index from SPC */
            circularQ->producer_index = cpu_to_le32(pm8001_read_32(circularQ->pi_virt));
            if (le32_to_cpu(circularQ->producer_index) == circularQ->consumer_idx) {
              	//printk(KERN_ERR "%s: leaving Outbound processing\n", __func__);
                /* OQ is empty */
                break;
            }
        }
    } while (1);
    spin_unlock_irqrestore(&pm8001_ha->lock, pm8001_ha->lock_flags);
    return ret;
}

/* PCI_DMA_... to our direction translation. */
static const u8 data_dir_flags[] = {
    [PCI_DMA_BIDIRECTIONAL] = DATA_DIR_BYRECIPIENT,/* UNSPECIFIED */
    [PCI_DMA_TODEVICE]  = DATA_DIR_OUT,/* OUTBOUND */
    [PCI_DMA_FROMDEVICE]    = DATA_DIR_IN,/* INBOUND */
    [PCI_DMA_NONE]      = DATA_DIR_NONE,/* NO TRANSFER */
};

static void build_smp_cmd(u32 deviceID, __le32 hTag, struct smp_req *psmp_cmd, int mode, int length)
{
    psmp_cmd->tag = hTag;
    psmp_cmd->device_id = cpu_to_le32(deviceID);
    if (mode == SMP_DIRECT)
    {
        length = length - 4; //subtracr crc 
        psmp_cmd->len_ip_ir = cpu_to_le32(length << 16);    
    }
    else
    {
        psmp_cmd->len_ip_ir = cpu_to_le32(1|(1 << 1));
    }   
}

/**
 * pm8001_chip_smp_req - send a SMP task to FW
 * @pm8001_ha: our hba card information.
 * @ccb: the ccb information this request used.
 */
static int pm80xx_chip_smp_req(struct pm8001_hba_info *pm8001_ha,
    struct pm8001_ccb_info *ccb)
{
    int elem, rc;
    struct sas_task *task = ccb->task;
    struct domain_device *dev = task->dev;
    struct pm8001_device *pm8001_dev = dev->lldd_dev;
    struct scatterlist *sg_req, *sg_resp;
    u32 req_len, resp_len;
    struct smp_req smp_cmd;
    u32 opc;
    struct inbound_queue_table *circularQ;
    char *preq_dma_addr=NULL;       
    __le64 tmp_addr;
    u32 i, length;

    memset(&smp_cmd, 0, sizeof(smp_cmd));
    /*
     * DMA-map SMP request, response buffers
     */
    sg_req = &task->smp_task.smp_req;
    elem = dma_map_sg(pm8001_ha->dev, sg_req, 1, PCI_DMA_TODEVICE);
    if (!elem)
        return -ENOMEM;
    req_len = sg_dma_len(sg_req);

    sg_resp = &task->smp_task.smp_resp;
    elem = dma_map_sg(pm8001_ha->dev, sg_resp, 1, PCI_DMA_FROMDEVICE);
    if (!elem) {
        rc = -ENOMEM;
        goto err_out;
    }
    resp_len = sg_dma_len(sg_resp);
    /* must be in dwords */
    if ((req_len & 0x3) || (resp_len & 0x3)) {
        rc = -EINVAL;
        goto err_out_2;
    }

    opc = OPC_INB_SMP_REQUEST;
    circularQ = &pm8001_ha->inbnd_q_tbl[0];
    smp_cmd.tag = cpu_to_le32(ccb->ccb_tag);

    length = sg_req->length;
    PM8001_IO_DBG(pm8001_ha,
        pm8001_printk("SMP Frame Length %d \n", sg_req->length));
    if(!(length - 8))
        pm8001_ha->smp_exp_mode=SMP_DIRECT;
    else
        pm8001_ha->smp_exp_mode=SMP_INDIRECT;



    tmp_addr = cpu_to_le64((u64)sg_dma_address(&task->smp_task.smp_req));
        preq_dma_addr = (char *)phys_to_virt(tmp_addr);
        
    /* INDIRECT MODE command settings. Use DMA */
    if (pm8001_ha->smp_exp_mode == SMP_INDIRECT)
    {
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("SMP REQUEST INDIRECT MODE\n"));  
        //for SPCv indirect mode. Place the top 4 bytes of SMP Request header here.
        for (i = 0;i < 4;i++)
        {       
            smp_cmd.smp_req16[i] = *(preq_dma_addr + i);
        }
        smp_cmd.long_smp_req.long_req_addr =
                cpu_to_le64((u64)sg_dma_address(&task->smp_task.smp_req) + 4); //exclude top 4 bytes for SMP req header
        smp_cmd.long_smp_req.long_req_size =
                cpu_to_le32((u32)sg_dma_len(&task->smp_task.smp_req)-8); //exclude 4 bytes for SMP req header and CRC
        smp_cmd.long_smp_req.long_resp_addr =
                cpu_to_le64((u64)sg_dma_address(&task->smp_task.smp_resp));
        smp_cmd.long_smp_req.long_resp_size =
                cpu_to_le32((u32)sg_dma_len(&task->smp_task.smp_resp)-4);
    }
    /* DIRECT MODE */
    else
    {
        smp_cmd.long_smp_req.long_req_addr =
            cpu_to_le64((u64)sg_dma_address(&task->smp_task.smp_req));
        smp_cmd.long_smp_req.long_req_size =
            cpu_to_le32((u32)sg_dma_len(&task->smp_task.smp_req)-4);
        smp_cmd.long_smp_req.long_resp_addr =
            cpu_to_le64((u64)sg_dma_address(&task->smp_task.smp_resp));
        smp_cmd.long_smp_req.long_resp_size =
            cpu_to_le32((u32)sg_dma_len(&task->smp_task.smp_resp)-4);
    }
    if (pm8001_ha->smp_exp_mode == SMP_DIRECT)
    {
        PM8001_IO_DBG(pm8001_ha,
            pm8001_printk("SMP REQUEST DIRECT MODE\n"));
        for (i = 0; i<length ;i++)  
            if(i<16)
            {
                    smp_cmd.smp_req16[i] = *(preq_dma_addr+i); 
                PM8001_IO_DBG(pm8001_ha,
                    pm8001_printk("Byte[%d]:%x (DMA data:%x)\n", 
                    i, smp_cmd.smp_req16[i], *(preq_dma_addr)));
            }           
            else
            {
                    smp_cmd.smp_req[i] = *(preq_dma_addr+i); 
                PM8001_IO_DBG(pm8001_ha,
                    pm8001_printk("Byte[%d]:%x (DMA data:%x)\n", 
                    i, smp_cmd.smp_req[i], *(preq_dma_addr)));
            }           
    }
    
    build_smp_cmd(pm8001_dev->device_id, smp_cmd.tag, &smp_cmd, pm8001_ha->smp_exp_mode, length);
    pm8001_mpi_build_cmd(pm8001_ha, circularQ, opc, (u32 *)&smp_cmd, 0);
    return 0;

err_out_2:
    dma_unmap_sg(pm8001_ha->dev, &ccb->task->smp_task.smp_resp, 1,
            PCI_DMA_FROMDEVICE);
err_out:
    dma_unmap_sg(pm8001_ha->dev, &ccb->task->smp_task.smp_req, 1,
            PCI_DMA_TODEVICE);
    return rc;
}

/**
 * pm80xx_chip_smputil_req - send a SMP task to FW
 * @pm8001_ha: our hba card information.
 */
int pm80xx_chip_smputil_req(struct pm8001_hba_info *pm8001_ha, int req_len, int resp_len, u8* smp_frame, u8* resp_buffer, u8* sas_address)
{
    struct pm8001_device *pm8001_dev;
    struct smp_req smp_cmd;
    u32 opc, i;
    struct inbound_queue_table *circularQ;
    int ret;
        u8* req_buffer;
    struct pm8001_ccb_info *ccb;
    struct mpi_mem smp_req_buffer;
    struct mpi_mem smp_resp_buffer;
    
    DECLARE_COMPLETION_ONSTACK(completion);
    
   
    pm8001_ha->smp_ioctl_completion = &completion;
    memset(&smp_cmd, 0, sizeof(smp_cmd));

        pm8001_dev = pm8001_find_dev_by_sas_address(pm8001_ha, sas_address);
    
    if(pm8001_dev == NULL)
        {
        return -1;
        }
    opc = OPC_INB_SMP_REQUEST;
    circularQ = &pm8001_ha->inbnd_q_tbl[0];
    ret = pm8001_tag_alloc(pm8001_ha, &smp_cmd.tag);
        if (ret)
          return -1;
    ccb = &pm8001_ha->ccb_info[smp_cmd.tag];
    ccb->ccb_tag = smp_cmd.tag;
    ccb->n_elem = 0xffffffff;
    ccb->device = pm8001_dev;
    ccb->task = NULL;
    smp_cmd.tag = cpu_to_le32(smp_cmd.tag);

    if(!(req_len - 8))
        pm8001_ha->smp_exp_mode=SMP_DIRECT;
    else
        pm8001_ha->smp_exp_mode=SMP_INDIRECT;

    smp_req_buffer.total_len = req_len;
        smp_req_buffer.alignment = req_len;
    
    if (pm8001_mem_alloc(pm8001_ha->pdev,
        &smp_req_buffer.virt_ptr,
        &smp_req_buffer.phys_addr,
        &smp_req_buffer.phys_addr_hi,
        &smp_req_buffer.phys_addr_lo,
        smp_req_buffer.total_len,
        smp_req_buffer.alignment) != 0) {
                PM8001_FAIL_DBG(pm8001_ha,
                    pm8001_printk("Mem alloc failed\n"
                    ));
        return -1;
    }

    smp_resp_buffer.total_len = resp_len;
        smp_resp_buffer.alignment = resp_len;
    
    if (pm8001_mem_alloc(pm8001_ha->pdev,
        &smp_resp_buffer.virt_ptr,
        &smp_resp_buffer.phys_addr,
        &smp_resp_buffer.phys_addr_hi,
        &smp_resp_buffer.phys_addr_lo,
        smp_resp_buffer.total_len,
        smp_resp_buffer.alignment) != 0) {
                PM8001_FAIL_DBG(pm8001_ha,
                    pm8001_printk("Mem alloc failed\n"
                    ));
        return -1;
    }

    for(i=0;i<4;i++)
    {
       smp_cmd.smp_req16[i] = smp_frame[i];
    }

        req_buffer = (u8*)smp_req_buffer.virt_ptr;

        if (pm8001_ha->smp_exp_mode == SMP_INDIRECT)
    {
            for(i=0;i<(req_len-8);i++)\
        {
                req_buffer[i] = smp_frame[i+4];
            }
        }
        else
    {
           for(i=0;i<req_len;i++)
       {
               req_buffer[i] = smp_frame[i];
           }
        }
                
        if (pm8001_ha->smp_exp_mode == SMP_INDIRECT)
        {
            smp_cmd.long_smp_req.long_req_addr = smp_req_buffer.phys_addr; 
            
        smp_cmd.long_smp_req.long_req_size = req_len - 8;
            
        smp_cmd.long_smp_req.long_resp_addr = smp_resp_buffer.phys_addr;
            
        smp_cmd.long_smp_req.long_resp_size = resp_len - 4;
        }     

        else 
        {
            smp_cmd.long_smp_req.long_req_addr = smp_req_buffer.phys_addr;

            smp_cmd.long_smp_req.long_req_size = req_len - 4;

            smp_cmd.long_smp_req.long_resp_addr = smp_resp_buffer.phys_addr;

            smp_cmd.long_smp_req.long_resp_size = resp_len - 4;
        }
    if (pm8001_ha->smp_exp_mode == SMP_DIRECT)
    {
            for (i = 0; i<req_len ;i++)
            {   
            if(i<16)
            {
                    smp_cmd.smp_req16[i] =  smp_frame[i]; 
            }           
            else
            {
                smp_cmd.smp_req[i] = smp_frame[i]; 
            }           
            }
    }
            
    build_smp_cmd(pm8001_dev->device_id, smp_cmd.tag, &smp_cmd, pm8001_ha->smp_exp_mode, req_len);
        smp_cmd.len_ip_ir = smp_cmd.len_ip_ir | 1;

    pm8001_mpi_build_cmd(pm8001_ha, circularQ, opc, (u32 *)&smp_cmd, 0);
    
    wait_for_completion(&completion);

        memcpy(resp_buffer, smp_resp_buffer.virt_ptr, resp_len);
    
        if (smp_req_buffer.virt_ptr != NULL) 
    {
        pci_free_consistent(pm8001_ha->pdev,
         (smp_req_buffer.total_len +
          smp_req_buffer.alignment),
         smp_req_buffer.virt_ptr,
         smp_req_buffer.phys_addr);
    }

        if (smp_resp_buffer.virt_ptr != NULL)
    {
        pci_free_consistent(pm8001_ha->pdev,
         (smp_resp_buffer.total_len +
          smp_resp_buffer.alignment),
         smp_resp_buffer.virt_ptr,
         smp_resp_buffer.phys_addr);
    }

    return 0;
}
static int check_enc_sas_cmd(struct sas_task *task)
{
    #if ( defined(PM8001_RHEL63) || (LINUX_VERSION_CODE < KERNEL_VERSION(3,11,0))) 
    if ((task->ssp_task.cdb[0] == READ_10)
            || (task->ssp_task.cdb[0] == WRITE_10)
            || (task->ssp_task.cdb[0] == WRITE_VERIFY)
            || (task->ssp_task.cdb[0] == READ_16)
            || (task->ssp_task.cdb[0] == WRITE_16))
    #else   
    if ((task->ssp_task.cmd->cmnd[0] == READ_10)
        || (task->ssp_task.cmd->cmnd[0] == WRITE_10)
            || (task->ssp_task.cmd->cmnd[0] == WRITE_VERIFY)
        || (task->ssp_task.cmd->cmnd[0] == READ_16)
        || (task->ssp_task.cmd->cmnd[0] == WRITE_16))
    #endif
    return 1;
    else
        return 0;
}

static int check_enc_sat_cmd(struct sas_task *task)
{
    int ret = 0;
    switch(task->ata_task.fis.command)
    {
        case SAT_READ_FPDMA_QUEUED:
        case SAT_WRITE_FPDMA_QUEUED:
        case SAT_READ_DMA_EXT:
        case SAT_WRITE_DMA_EXT:
        case SAT_READ_DMA:
        case SAT_WRITE_DMA:
        case SAT_READ_SECTORS:
        case SAT_WRITE_SECTORS:
        case SAT_READ_SECTORS_EXT:
        case SAT_WRITE_SECTORS_EXT:
            ret = 1;
            break;
        default:
            ret = 0;
            break;
    }
    return ret;
}

int pm80xx_chip_ssp_util_req(struct pm8001_hba_info *pm8001_ha,
    struct domain_device *dev,u8 *resp, u8 *cdb, u32 len)
{
    struct pm8001_ccb_info *ccb;
    struct pm8001_device *pm8001_dev = dev->lldd_dev;
    struct ssp_ini_io_start_req ssp_cmd;
    u8 LUN[8];
    struct mpi_mem ssp_resp_buffer;
    struct inbound_queue_table *circularQ;
        u32 opc = OPC_INB_SSPINIIOSTART, ret;


    DECLARE_COMPLETION_ONSTACK(completion);

    pm8001_ha->ioctl_completion = &completion;

    memset(LUN,0,8);
    memset(&ssp_cmd, 0, sizeof(ssp_cmd));
    ret = pm8001_tag_alloc(pm8001_ha, &ssp_cmd.tag);

    ccb = &pm8001_ha->ccb_info[ssp_cmd.tag];
        ccb->ccb_tag = ssp_cmd.tag;
        ccb->n_elem = 0;
        ccb->device = pm8001_dev;
        ccb->task = NULL;

    if (ret)
          return -1;
    LUN[1] = 0;/*default lun 0*/
    memcpy(ssp_cmd.ssp_iu.lun, LUN, 8);
    /* data address domain added for spcv; set to 0 by host, used internally by controller */
    ssp_cmd.dad_dir_m_tlr = cpu_to_le32(data_dir_flags[2] << 8 | 0x0);/*0 for
            //SAS 1.1 and SAS 2.0 compatible TLR*/

    ssp_cmd.data_len = cpu_to_le32(len);
    ssp_cmd.device_id = cpu_to_le32(pm8001_dev->device_id);
    ssp_cmd.tag = cpu_to_le32(ssp_cmd.tag);
    memcpy(ssp_cmd.ssp_iu.cdb, cdb, 16);
    //q_index = (u32) (pm8001_dev->id & 0x00ffffff) % PM8001_MAX_INB_NUM;
    circularQ = &pm8001_ha->inbnd_q_tbl[0];

    ssp_resp_buffer.total_len = len;
        ssp_resp_buffer.alignment = len;
    
    if (pm8001_mem_alloc(pm8001_ha->pdev,
        &ssp_resp_buffer.virt_ptr,
        &ssp_resp_buffer.phys_addr,
        &ssp_resp_buffer.phys_addr_hi,
        &ssp_resp_buffer.phys_addr_lo,
        ssp_resp_buffer.total_len,
        ssp_resp_buffer.alignment) != 0) {
                PM8001_FAIL_DBG(pm8001_ha,
                    pm8001_printk("Mem alloc failed\n"
                    ));
        return -1;
    }
    
    ssp_cmd.addr_low = cpu_to_le32(lower_32_bits(ssp_resp_buffer.phys_addr));
    ssp_cmd.addr_high = cpu_to_le32(upper_32_bits(ssp_resp_buffer.phys_addr));
    ssp_cmd.len = cpu_to_le32(len);
    ssp_cmd.esgl = 0;

    //q_index = (u32) (pm8001_dev->id & 0x00ffffff) % PM8001_MAX_OUTB_NUM;
    ret = pm8001_mpi_build_cmd(pm8001_ha, circularQ, opc, &ssp_cmd, 0);
    wait_for_completion(&completion);

    memcpy(resp, ssp_resp_buffer.virt_ptr, len);
    

        if (ssp_resp_buffer.virt_ptr != NULL)
    {
        pci_free_consistent(pm8001_ha->pdev,
         (ssp_resp_buffer.total_len +
          ssp_resp_buffer.alignment),
         ssp_resp_buffer.virt_ptr,
         ssp_resp_buffer.phys_addr);
    }
        
    return ret;
}
/**
 * pm80xx_chip_ssp_io_req - send a SSP task to FW
 * @pm8001_ha: our hba card information.
 * @ccb: the ccb information this request used.
 */
static int pm80xx_chip_ssp_io_req(struct pm8001_hba_info *pm8001_ha,
    struct pm8001_ccb_info *ccb)
{
    struct sas_task *task = ccb->task;
    struct domain_device *dev = task->dev;
    struct pm8001_device *pm8001_dev = dev->lldd_dev;
    struct ssp_ini_io_start_req ssp_cmd;
    u32 tag = ccb->ccb_tag;
    int ret;
    u64 phys_addr, start_addr, end_addr;
    u32 end_addr_high, end_addr_low;
    struct inbound_queue_table *circularQ;
        u32 opc = OPC_INB_SSPINIIOSTART;
    u32 q_index;
    memset(&ssp_cmd, 0, sizeof(ssp_cmd));
    memcpy(ssp_cmd.ssp_iu.lun, task->ssp_task.LUN, 8);
    /* data address domain added for spcv; set to 0 by host, used internally by controller */
    ssp_cmd.dad_dir_m_tlr = cpu_to_le32(data_dir_flags[task->data_dir] << 8 | 0x0);/*0 for
            SAS 1.1 and SAS 2.0 compatible TLR*/

    ssp_cmd.data_len = cpu_to_le32(task->total_xfer_len);
    ssp_cmd.device_id = cpu_to_le32(pm8001_dev->device_id);
    ssp_cmd.tag = cpu_to_le32(tag);
    if (task->ssp_task.enable_first_burst)
        ssp_cmd.ssp_iu.efb_prio_attr |= 0x80;
    ssp_cmd.ssp_iu.efb_prio_attr |= (task->ssp_task.task_prio << 3);
    ssp_cmd.ssp_iu.efb_prio_attr |= (task->ssp_task.task_attr & 7);
    #if ( defined(PM8001_RHEL63) || (LINUX_VERSION_CODE < KERNEL_VERSION(3,11,0)))  
    memcpy(ssp_cmd.ssp_iu.cdb, task->ssp_task.cdb, 16);
    #else
    memcpy(ssp_cmd.ssp_iu.cdb, task->ssp_task.cmd->cmnd, 16);
    #endif
    
    //q_index = (u32) (pm8001_dev->id & 0x00ffffff) % PM8001_MAX_INB_NUM;
    q_index = (u32) (get_cpu()) % (pm8001_ha->max_q_num);
    put_cpu();
    circularQ = &pm8001_ha->inbnd_q_tbl[q_index];

    /* Check if encryption is set */
    if (pm8001_ha->chip->encrypt && !(pm8001_ha->encrypt_info.status) && check_enc_sas_cmd(task))
    {
        #if ( defined(PM8001_RHEL63) || (LINUX_VERSION_CODE < KERNEL_VERSION(3,11,0))) 
        PM8001_IO_DBG(pm8001_ha, pm8001_printk("Encryption enabled. Sending Encrypt SAS command 0x%x \n", task->ssp_task.cdb[0]));
        #else
        PM8001_IO_DBG(pm8001_ha, pm8001_printk("Encryption enabled. Sending Encrypt SAS command 0x%x \n", task->ssp_task.cmd->cmnd[0]));
        #endif  
        opc = OPC_INB_SSP_INI_DIF_ENC_IO;
        /* enable encryption */
        ssp_cmd.dad_dir_m_tlr = cpu_to_le32((data_dir_flags[task->data_dir] << 8) | 0x20 | 0x0);/*0 for
            SAS 1.1 and SAS 2.0 compatible TLR*/
        
        /* fill in PRD (scatter/gather) table, if any */
        if (task->num_scatter > 1) {
            pm8001_chip_make_sg(task->scatter, ccb->n_elem, ccb->buf_prd);
            phys_addr = ccb->ccb_dma_handle +
                offsetof(struct pm8001_ccb_info, buf_prd[0]);
            ssp_cmd.enc_addr_low = cpu_to_le32(lower_32_bits(phys_addr));
            ssp_cmd.enc_addr_high = cpu_to_le32(upper_32_bits(phys_addr));
            ssp_cmd.enc_esgl = cpu_to_le32(1<<31);
        } else if (task->num_scatter == 1) {
            u64 dma_addr = sg_dma_address(task->scatter);
            ssp_cmd.enc_addr_low = cpu_to_le32(lower_32_bits(dma_addr));
            ssp_cmd.enc_addr_high = cpu_to_le32(upper_32_bits(dma_addr));
            ssp_cmd.enc_len = cpu_to_le32(task->total_xfer_len);
            ssp_cmd.enc_esgl = 0;

            /* Check 4G Boundary */
            start_addr = cpu_to_le64(dma_addr);
            end_addr = (start_addr + ssp_cmd.enc_len) - 1;
            end_addr_low = cpu_to_le32(lower_32_bits(end_addr));
            end_addr_high = cpu_to_le32(upper_32_bits(end_addr));
            if ( end_addr_high != ssp_cmd.enc_addr_high ) {
                PM8001_FAIL_DBG(pm8001_ha, pm8001_printk("The sg list address start_addr=0x%016llx data_len=0x%x end_addr_high=0x%08x end_addr_low=0x%08x has crossed 4G boundary\n", start_addr, ssp_cmd.enc_len, end_addr_high, end_addr_low));                
                pm8001_chip_make_sg(task->scatter, 1, ccb->buf_prd);
                phys_addr = ccb->ccb_dma_handle +
                    offsetof(struct pm8001_ccb_info, buf_prd[0]);
                ssp_cmd.enc_addr_low = cpu_to_le32(lower_32_bits(phys_addr));
                ssp_cmd.enc_addr_high = cpu_to_le32(upper_32_bits(phys_addr));
                ssp_cmd.enc_esgl = cpu_to_le32(1<<31);
            }

        } else if (task->num_scatter == 0) {
            ssp_cmd.enc_addr_low = 0;
            ssp_cmd.enc_addr_high = 0;
            ssp_cmd.enc_len = cpu_to_le32(task->total_xfer_len);
            ssp_cmd.enc_esgl = 0;
        }
        ssp_cmd.key_cmode = 0x6 << 4; /* XTS mode. All other fields are 0 */
        /* set tweak values. Should be the start lba */
        #if ( defined(PM8001_RHEL63) || (LINUX_VERSION_CODE < KERNEL_VERSION(3,11,0)))  
        ssp_cmd.twk_val0 = cpu_to_le32((task->ssp_task.cdb[2] << 24) | (task->ssp_task.cdb[3] << 16) | (task->ssp_task.cdb[4] << 8) | (task->ssp_task.cdb[5]));
        #else
        ssp_cmd.twk_val0 = cpu_to_le32((task->ssp_task.cmd->cmnd[2] << 24) | (task->ssp_task.cmd->cmnd[3] << 16) | (task->ssp_task.cmd->cmnd[4] << 8) | (task->ssp_task.cmd->cmnd[5]));
        #endif      
        if ((ssp_cmd.ssp_iu.cdb[0] == READ_16) || (ssp_cmd.ssp_iu.cdb[0] == WRITE_16))
        {
                    PM8001_MSG_DBG(pm8001_ha, pm8001_printk("Encryption for 3TB. cdb[0] %x \n", ssp_cmd.ssp_iu.cdb[0]));                
            #if ( defined(PM8001_RHEL63) || (LINUX_VERSION_CODE < KERNEL_VERSION(3,11,0)))
            ssp_cmd.twk_val0 = cpu_to_le32((task->ssp_task.cdb[6] << 24) | (task->ssp_task.cdb[7] << 16) | (task->ssp_task.cdb[8] << 8) | (task->ssp_task.cdb[9]));
            ssp_cmd.twk_val1 = cpu_to_le32((task->ssp_task.cdb[2] << 24) | (task->ssp_task.cdb[3] << 16) | (task->ssp_task.cdb[4] << 8) | (task->ssp_task.cdb[5]));
            #else
            ssp_cmd.twk_val0 = cpu_to_le32((task->ssp_task.cmd->cmnd[6] << 24) | (task->ssp_task.cmd->cmnd[7] << 16) | (task->ssp_task.cmd->cmnd[8] << 8) | (task->ssp_task.cmd->cmnd[9]));
            ssp_cmd.twk_val1 = cpu_to_le32((task->ssp_task.cmd->cmnd[2] << 24) | (task->ssp_task.cmd->cmnd[3] << 16) | (task->ssp_task.cmd->cmnd[4] << 8) | (task->ssp_task.cmd->cmnd[5]));
            #endif
        }
    }
    else
    {
        #if ( defined(PM8001_RHEL63) || (LINUX_VERSION_CODE < KERNEL_VERSION(3,11,0)))
        PM8001_IO_DBG(pm8001_ha, pm8001_printk("Sending Normal SAS command 0x%x inb q %p \n", task->ssp_task.cdb[0], inb));
        #else       
        PM8001_IO_DBG(pm8001_ha, pm8001_printk("Sending Normal SAS command 0x%x inb q %p \n", task->ssp_task.cmd->cmnd[0], inb));
        #endif
        /* fill in PRD (scatter/gather) table, if any */
        if (task->num_scatter > 1) {
            pm8001_chip_make_sg(task->scatter, ccb->n_elem, ccb->buf_prd);
            phys_addr = ccb->ccb_dma_handle +
                offsetof(struct pm8001_ccb_info, buf_prd[0]);
            ssp_cmd.addr_low = cpu_to_le32(lower_32_bits(phys_addr));
            ssp_cmd.addr_high = cpu_to_le32(upper_32_bits(phys_addr));
            ssp_cmd.esgl = cpu_to_le32(1<<31);
        } else if (task->num_scatter == 1) {
            u64 dma_addr = sg_dma_address(task->scatter);
            ssp_cmd.addr_low = cpu_to_le32(lower_32_bits(dma_addr));
            ssp_cmd.addr_high = cpu_to_le32(upper_32_bits(dma_addr));
            ssp_cmd.len = cpu_to_le32(task->total_xfer_len);
            ssp_cmd.esgl = 0;

            /* Check 4G Boundary */
            start_addr = cpu_to_le64(dma_addr);
            end_addr = (start_addr + ssp_cmd.len) - 1;
            end_addr_low = cpu_to_le32(lower_32_bits(end_addr));
            end_addr_high = cpu_to_le32(upper_32_bits(end_addr));
            if ( end_addr_high != ssp_cmd.addr_high ) {
                PM8001_FAIL_DBG(pm8001_ha, pm8001_printk("The sg list address start_addr=0x%016llx data_len=0x%x end_addr_high=0x%08x end_addr_low=0x%08x has crossed 4G boundary\n", start_addr, ssp_cmd.len, end_addr_high, end_addr_low));                 
                pm8001_chip_make_sg(task->scatter, 1, ccb->buf_prd);
                phys_addr = ccb->ccb_dma_handle +
                    offsetof(struct pm8001_ccb_info, buf_prd[0]);
                ssp_cmd.addr_low = cpu_to_le32(lower_32_bits(phys_addr));
                ssp_cmd.addr_high = cpu_to_le32(upper_32_bits(phys_addr));
                ssp_cmd.esgl = cpu_to_le32(1<<31);
            }

        } else if (task->num_scatter == 0) {
            ssp_cmd.addr_low = 0;
            ssp_cmd.addr_high = 0;
            ssp_cmd.len = cpu_to_le32(task->total_xfer_len);
            ssp_cmd.esgl = 0;
        }
    }
//  PM8001_DEV_DBG(pm8001_ha, pm8001_printk("SAS task:0x%p IQ %d OQ %d OPC %x tag %d \n", task, inb, outb, opc, ssp_cmd.tag));
//  q_index = (u32) (pm8001_dev->id & 0x00ffffff) % PM8001_MAX_OUTB_NUM;
    ret = pm8001_mpi_build_cmd(pm8001_ha, circularQ, opc, &ssp_cmd, q_index);

        
    return ret;
}

static int pm80xx_chip_sata_req(struct pm8001_hba_info *pm8001_ha,
    struct pm8001_ccb_info *ccb)
{
    struct sas_task *task = ccb->task;
    struct task_status_struct *ts;
    struct domain_device *dev = task->dev;
    struct pm8001_device *pm8001_ha_dev = dev->lldd_dev;
    u32 tag = ccb->ccb_tag;
    int ret;
        struct sata_start_req sata_cmd;
    u32 hdr_tag, ncg_tag = 0;
    u64 phys_addr, start_addr, end_addr;
    u32 end_addr_high, end_addr_low;
    u32 ATAP = 0x0;
    u32 dir;
    struct inbound_queue_table *circularQ;
    unsigned long flags;
    u32 opc = OPC_INB_SATA_HOST_OPSTART;
    u32 q_index;

#if 1 // for NCQ IO abort in progress  
    //HANDLE NCQ and return
    if (pm8001_ha_dev && ((pm8001_ha_dev->id & NCQ_READ_LOG_FLAG) || (pm8001_ha_dev->id & NCQ_ABORT_ALL_FLAG)))
    {
        PM8001_FAIL_DBG(pm8001_ha, pm8001_printk("STOPPING IO FOR DEVICE %x task %p \n", pm8001_ha_dev->id, task));
    
        ts = &task->task_status;
        spin_lock_irqsave(&task->task_state_lock, flags);
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAM_STAT_BUSY;
        task->task_state_flags &= ~SAS_TASK_STATE_PENDING;
        task->task_state_flags &= ~SAS_TASK_AT_INITIATOR;
        task->task_state_flags |= SAS_TASK_STATE_DONE;
        if (task->uldd_task) {
        PM8001_FAIL_DBG(pm8001_ha, pm8001_printk("task with uldd task \n"));
            
        spin_unlock_irqrestore(&task->task_state_lock, flags);
        pm8001_ccb_task_free(pm8001_ha, task, ccb, tag);
        mb();/* ditto */
        spin_unlock_irqrestore(&pm8001_ha->lock, pm8001_ha->lock_flags);
        task->task_done(task);
        spin_lock_irqsave(&pm8001_ha->lock, pm8001_ha->lock_flags);
        return 0;
    } else if (!task->uldd_task) {
        PM8001_FAIL_DBG(pm8001_ha, pm8001_printk("task with no uldd task \n"));
        spin_unlock_irqrestore(&task->task_state_lock, flags);
        pm8001_ccb_task_free(pm8001_ha, task, ccb, tag);
        mb();/*ditto*/
        spin_unlock_irqrestore(&pm8001_ha->lock, pm8001_ha->lock_flags);
        task->task_done(task);
        spin_lock_irqsave(&pm8001_ha->lock, pm8001_ha->lock_flags);
        return 0;
    }
    }
#endif      
    memset(&sata_cmd, 0, sizeof(sata_cmd));
    q_index = (u32) (get_cpu()) % (pm8001_ha->max_q_num);       
    put_cpu();
    circularQ = &pm8001_ha->inbnd_q_tbl[q_index];
    if (task->data_dir == PCI_DMA_NONE) {
        ATAP = 0x04;  /* no data*/
        PM8001_IO_DBG(pm8001_ha, pm8001_printk("no data\n"));
    } else if (likely(!task->ata_task.device_control_reg_update)) {
        if (task->ata_task.dma_xfer) {
            ATAP = 0x06; /* DMA */
            PM8001_IO_DBG(pm8001_ha, pm8001_printk("DMA\n"));
        } else {
            ATAP = 0x05; /* PIO*/
            PM8001_IO_DBG(pm8001_ha, pm8001_printk("PIO\n"));
        }
        if (task->ata_task.use_ncq &&
            dev->sata_dev.class != ATA_DEV_ATAPI) {
            ATAP = 0x07; /* FPDMA */
            PM8001_IO_DBG(pm8001_ha, pm8001_printk("FPDMA\n"));
        }
    }
    if (task->ata_task.use_ncq && pm8001_get_ncq_tag(task, &hdr_tag))
    {
        /* NCQ CHANGE STARTS - TAKEN FROM MVSAS */
        task->ata_task.fis.sector_count |= (u8) (hdr_tag << 3);
        /* NCQ CHANGE ENDS */   
        ncg_tag = hdr_tag;
    }
    dir = data_dir_flags[task->data_dir] << 8;
    sata_cmd.tag = cpu_to_le32(tag);
    sata_cmd.device_id = cpu_to_le32(pm8001_ha_dev->device_id);
    sata_cmd.data_len = cpu_to_le32(task->total_xfer_len);
        
    sata_cmd.sata_fis = task->ata_task.fis;
    if (likely(!task->ata_task.device_control_reg_update))
        sata_cmd.sata_fis.flags |= 0x80;/* C=1: update ATA cmd reg */
    sata_cmd.sata_fis.flags &= 0xF0;/* PM_PORT field shall be 0 */

    /* Check if encryption is set */
    if (pm8001_ha->chip->encrypt && !(pm8001_ha->encrypt_info.status) && check_enc_sat_cmd(task))
    {
        PM8001_IO_DBG(pm8001_ha, pm8001_printk("Encryption enabled. Sending Encrypt SATA command 0x%x \n", sata_cmd.sata_fis.command));
        opc = OPC_INB_SATA_DIF_ENC_IO;

        /* set encryption bit */
        sata_cmd.ncqtag_atap_dir_m_dad = cpu_to_le32(((ncg_tag & 0xff)<<16)|((ATAP & 0x3f) << 10) | 0x20 | dir);        /* dad (bit 0-1) is 0 */
        /* fill in PRD (scatter/gather) table, if any */
        if (task->num_scatter > 1) {
            pm8001_chip_make_sg(task->scatter, ccb->n_elem, ccb->buf_prd);
            phys_addr = ccb->ccb_dma_handle +
                offsetof(struct pm8001_ccb_info, buf_prd[0]);
            sata_cmd.enc_addr_low = lower_32_bits(phys_addr);
            sata_cmd.enc_addr_high = upper_32_bits(phys_addr);
            sata_cmd.enc_esgl = cpu_to_le32(1 << 31);
        } else if (task->num_scatter == 1) {
            u64 dma_addr = sg_dma_address(task->scatter);
            sata_cmd.enc_addr_low = lower_32_bits(dma_addr);
            sata_cmd.enc_addr_high = upper_32_bits(dma_addr);
            sata_cmd.enc_len = cpu_to_le32(task->total_xfer_len);
            sata_cmd.enc_esgl = 0;

            /* Check 4G Boundary */
            start_addr = cpu_to_le64(dma_addr);
            end_addr = (start_addr + sata_cmd.enc_len) - 1;
            end_addr_low = cpu_to_le32(lower_32_bits(end_addr));
            end_addr_high = cpu_to_le32(upper_32_bits(end_addr));
            if ( end_addr_high != sata_cmd.enc_addr_high ) {
                PM8001_FAIL_DBG(pm8001_ha, pm8001_printk("The sg list address start_addr=0x%016llx data_len=0x%x end_addr_high=0x%08x end_addr_low=0x%08x has crossed 4G boundary\n", start_addr, sata_cmd.enc_len, end_addr_high, end_addr_low));
                pm8001_chip_make_sg(task->scatter, 1, ccb->buf_prd);
                phys_addr = ccb->ccb_dma_handle +
                                offsetof(struct pm8001_ccb_info, buf_prd[0]);
                sata_cmd.enc_addr_low = lower_32_bits(phys_addr);
                sata_cmd.enc_addr_high = upper_32_bits(phys_addr);
                sata_cmd.enc_esgl = cpu_to_le32(1 << 31);

            }

        } else if (task->num_scatter == 0) {
            sata_cmd.enc_addr_low = 0;
            sata_cmd.enc_addr_high = 0;
            sata_cmd.enc_len = cpu_to_le32(task->total_xfer_len);
            sata_cmd.enc_esgl = 0;
        }
        sata_cmd.key_index_mode = 0x6 << 4; /* XTS mode. All other fields are 0 */
        /* set tweak values. Should be the start lba */
        sata_cmd.twk_val0 = cpu_to_le32((sata_cmd.sata_fis.lbal_exp << 24) | (sata_cmd.sata_fis.lbah << 16) | (sata_cmd.sata_fis.lbam << 8) | (sata_cmd.sata_fis.lbal));
        sata_cmd.twk_val1 = cpu_to_le32((sata_cmd.sata_fis.lbah_exp << 8) | (sata_cmd.sata_fis.lbam_exp));
    }
    else
    {
        PM8001_IO_DBG(pm8001_ha, pm8001_printk("Sending Normal SATA command 0x%x inb %p \n", sata_cmd.sata_fis.command, inb));
        sata_cmd.ncqtag_atap_dir_m_dad =
            cpu_to_le32(((ncg_tag & 0xff)<<16)|((ATAP & 0x3f) << 10) | dir);    /* dad (bit 0-1) is 0 */
    
        /* fill in PRD (scatter/gather) table, if any */
        if (task->num_scatter > 1) {
            pm8001_chip_make_sg(task->scatter, ccb->n_elem, ccb->buf_prd);
            phys_addr = ccb->ccb_dma_handle +
                offsetof(struct pm8001_ccb_info, buf_prd[0]);
            sata_cmd.addr_low = lower_32_bits(phys_addr);
            sata_cmd.addr_high = upper_32_bits(phys_addr);
            sata_cmd.esgl = cpu_to_le32(1 << 31);
        } else if (task->num_scatter == 1) {
            u64 dma_addr = sg_dma_address(task->scatter);
            sata_cmd.addr_low = lower_32_bits(dma_addr);
            sata_cmd.addr_high = upper_32_bits(dma_addr);
            sata_cmd.len = cpu_to_le32(task->total_xfer_len);
            sata_cmd.esgl = 0;

            /* Check 4G Boundary */
            start_addr = cpu_to_le64(dma_addr);
            end_addr = (start_addr + sata_cmd.len) - 1;
            end_addr_low = cpu_to_le32(lower_32_bits(end_addr));
            end_addr_high = cpu_to_le32(upper_32_bits(end_addr));
            if ( end_addr_high != sata_cmd.addr_high ) {
                PM8001_FAIL_DBG(pm8001_ha, pm8001_printk("The sg list address start_addr=0x%016llx data_len=0x%x end_addr_high=0x%08x end_addr_low=0x%08x has crossed 4G boundary\n", start_addr, sata_cmd.len, end_addr_high, end_addr_low));                
                pm8001_chip_make_sg(task->scatter, 1, ccb->buf_prd);
                phys_addr = ccb->ccb_dma_handle +
                    offsetof(struct pm8001_ccb_info, buf_prd[0]);
                sata_cmd.addr_low = lower_32_bits(phys_addr);
                sata_cmd.addr_high = upper_32_bits(phys_addr);
                sata_cmd.esgl = cpu_to_le32(1 << 31);
            }

        } else if (task->num_scatter == 0) {
            sata_cmd.addr_low = 0;
            sata_cmd.addr_high = 0;
            sata_cmd.len = cpu_to_le32(task->total_xfer_len);
            sata_cmd.esgl = 0;
        }
            /* scsi cdb */
            sata_cmd.atapi_scsi_cdb[0] = cpu_to_le32(((task->ata_task.atapi_packet[0]) | (task->ata_task.atapi_packet[1] << 8) | 
                                                     (task->ata_task.atapi_packet[2] << 16) | (task->ata_task.atapi_packet[3] << 24)));
            sata_cmd.atapi_scsi_cdb[1] = cpu_to_le32(((task->ata_task.atapi_packet[4]) | (task->ata_task.atapi_packet[5] << 8) | 
                                                     (task->ata_task.atapi_packet[6] << 16) | (task->ata_task.atapi_packet[7] << 24)));
            sata_cmd.atapi_scsi_cdb[2] = cpu_to_le32(((task->ata_task.atapi_packet[8]) | (task->ata_task.atapi_packet[9] << 8) | 
                                                     (task->ata_task.atapi_packet[10] << 16) | (task->ata_task.atapi_packet[11] << 24)));
            sata_cmd.atapi_scsi_cdb[3] = cpu_to_le32(((task->ata_task.atapi_packet[12]) | (task->ata_task.atapi_packet[13] << 8) | 
                                                     (task->ata_task.atapi_packet[14] << 16) | (task->ata_task.atapi_packet[15] << 24)));
    }
    
    //print fields if it is read log
    if (sata_cmd.sata_fis.command == 0x2f)
    {
        
        PM8001_FAIL_DBG(pm8001_ha, pm8001_printk(" ************* READ LOG COMMAND DETAILS ***************** \n")); 
        PM8001_FAIL_DBG(pm8001_ha, pm8001_printk("READ LOG SATA CMD tag %x device id %x length %d dir_dad %x \n", sata_cmd.tag, sata_cmd.device_id, sata_cmd.data_len, sata_cmd.ncqtag_atap_dir_m_dad));
        PM8001_FAIL_DBG(pm8001_ha, pm8001_printk("READ LOG SATA FIS flags %x cmd %x lbal %x \n", sata_cmd.sata_fis.flags, sata_cmd.sata_fis.command, sata_cmd.sata_fis.lbal));

#if defined PM8001_RHEL63 || (LINUX_VERSION_CODE >= KERNEL_VERSION(3, 0, 76))  || \
    (LINUX_VERSION_CODE >= KERNEL_VERSION(3, 6, 0))
        //return from here itself if any of the flag is set
        if (pm8001_ha_dev && ((pm8001_ha_dev->id & NCQ_READ_LOG_FLAG) || 
            (pm8001_ha_dev->id & NCQ_ABORT_ALL_FLAG) || (pm8001_ha_dev->id & NCQ_2ND_RLE_FLAG)))
        {
            pm8001_ha_dev->id &= 0xDFFFFFFF;    //clear 2nd RLE flag. We will block only 2nd RLE for 
                                //NCQ medium error drive.
            ts = &task->task_status;
    
        spin_lock_irqsave(&task->task_state_lock, flags);
        ts->resp = SAS_TASK_COMPLETE;
        ts->stat = SAM_STAT_GOOD;
        task->task_state_flags &= ~SAS_TASK_STATE_PENDING;
        task->task_state_flags &= ~SAS_TASK_AT_INITIATOR;
        task->task_state_flags |= SAS_TASK_STATE_DONE;
        if (unlikely((task->task_state_flags & SAS_TASK_STATE_ABORTED))) {
            spin_unlock_irqrestore(&task->task_state_lock, flags);
            PM8001_FAIL_DBG(pm8001_ha,
                pm8001_printk("task 0x%p "
                " resp 0x%x stat 0x%x but aborted by upper layer!\n",
                task, ts->resp, ts->stat));
            pm8001_ccb_task_free(pm8001_ha, task, ccb, tag);
        }
        else if (task->uldd_task) {
            PM8001_FAIL_DBG(pm8001_ha, pm8001_printk("task with uldd task \n"));
            
            spin_unlock_irqrestore(&task->task_state_lock, flags);
            pm8001_ccb_task_free(pm8001_ha, task, ccb, tag);
            mb();/* ditto */
            spin_unlock_irqrestore(&pm8001_ha->lock, pm8001_ha->lock_flags);
            task->task_done(task);
            spin_lock_irqsave(&pm8001_ha->lock, pm8001_ha->lock_flags);
            return 0;
        } else if (!task->uldd_task) {
            PM8001_FAIL_DBG(pm8001_ha, pm8001_printk("task with no uldd task \n"));
            spin_unlock_irqrestore(&task->task_state_lock, flags);
            pm8001_ccb_task_free(pm8001_ha, task, ccb, tag);
            mb();/*ditto*/
            spin_unlock_irqrestore(&pm8001_ha->lock, pm8001_ha->lock_flags);
            task->task_done(task);
            spin_lock_irqsave(&pm8001_ha->lock, pm8001_ha->lock_flags);
            return 0;
        }
      }
#endif
    }

    //q_index = (u32) (pm8001_ha_dev->id & 0x00ffffff) % PM8001_MAX_OUTB_NUM;
    ret = pm8001_mpi_build_cmd(pm8001_ha, circularQ, opc, &sata_cmd, q_index);


    PM8001_MSG_DBG(pm8001_ha, pm8001_printk("SATA task:%p device %p tag %x device id %x OPC:0x%x FIS_CMD:0x%x\n", 
            task, pm8001_ha_dev, sata_cmd.tag, sata_cmd.device_id, opc, sata_cmd.sata_fis.command));

    
        
    return ret;
}

/**
 * pm80xx_chip_phy_start_req - start phy via PHY_START COMMAND
 * @pm8001_ha: our hba card information.
 * @num: the inbound queue number
 * @phy_id: the phy id which we wanted to start up.
 */
static int
pm80xx_chip_phy_start_req(struct pm8001_hba_info *pm8001_ha, u8 phy_id)
{
    struct phy_start_req payload;
    struct inbound_queue_table *circularQ;
    int ret;
    u32 tag = 0x01;
    u32 opcode = OPC_INB_PHYSTART;
    circularQ = &pm8001_ha->inbnd_q_tbl[0];
    memset(&payload, 0, sizeof(payload));
    payload.tag = cpu_to_le32(tag);


    PM8001_INIT_DBG(pm8001_ha,
            pm8001_printk("PHY START REQ for phy_id %d \n", phy_id));
    /*
     ** [0:7]   PHY Identifier
     ** [8:11]  link rate 1.5G, 3G, 6G, 12G.
     ** [12:13] link mode 01b SAS mode; 10b SATA mode; 11b Auto mode 
     ** [14]    0b disable spin up hold; 1b enable spin up hold
     ** [15] ob no change in current PHY analig setup 1b enable using SPAST
     */
        // Delray 12G support.
    payload.ase_sh_lm_slr_phyid = cpu_to_le32(SPINHOLD_DISABLE |
            LINKMODE_AUTO | LINKRATE_15 |
            LINKRATE_30 | LINKRATE_60 | LINKRATE_120 | phy_id);
    /* TBU - SSC Disable and SAS Analog ST configuration */ 
    /**
    payload.ase_sh_lm_slr_phyid = cpu_to_le32(SSC_DISABLE_30 | SAS_ASE | SPINHOLD_DISABLE | LINKMODE_AUTO | LINKRATE_15 | LINKRATE_30 | LINKRATE_60 | phy_id);
    Have to add "SAS PHY Analog Setup SPASTI 1 Byte" Based on need 
    **/
    #if (LINUX_VERSION_CODE < KERNEL_VERSION(3,10,0) )
    payload.sas_identify.dev_type = SAS_END_DEV;
    #else
    payload.sas_identify.dev_type = SAS_END_DEVICE ;
    #endif
    payload.sas_identify.initiator_bits = SAS_PROTOCOL_ALL;
    memcpy(payload.sas_identify.sas_addr, pm8001_ha->sas_addr, SAS_ADDR_SIZE);
    payload.sas_identify.phy_id = phy_id;
//#if 0
#ifdef DIAG_MODE
	switch(phy_id/4) {
	case 0:
		memcpy(payload.sas_identify.sas_addr, pm8001_ha->sas_addr_0, SAS_ADDR_SIZE);
		PM8001_MSG_DBG(pm8001_ha, printk(KERN_INFO "PHY %d sas_addr = %llx \n", phy_id, pm8001_ha->phy[phy_id].dev_sas_addr));
		break;
	case 1:
		memcpy(payload.sas_identify.sas_addr, pm8001_ha->sas_addr_1, SAS_ADDR_SIZE);
		PM8001_MSG_DBG(pm8001_ha, printk(KERN_INFO "PHY %d sas_addr = %llx \n", phy_id, pm8001_ha->phy[phy_id].dev_sas_addr));
		break;
	case 2:
		memcpy(payload.sas_identify.sas_addr, pm8001_ha->sas_addr_2, SAS_ADDR_SIZE);
		PM8001_MSG_DBG(pm8001_ha, printk(KERN_INFO "PHY %d sas_addr = %llx \n", phy_id, pm8001_ha->phy[phy_id].dev_sas_addr));
		break;
	case 3:
		memcpy(payload.sas_identify.sas_addr, pm8001_ha->sas_addr_3, SAS_ADDR_SIZE);
		PM8001_MSG_DBG(pm8001_ha, printk(KERN_INFO "PHY %d sas_addr = %llx \n", phy_id, pm8001_ha->phy[phy_id].dev_sas_addr));
		break;
	}
#endif
    ret = pm8001_mpi_build_cmd(pm8001_ha, circularQ, opcode, &payload, 0);
    return ret;
}

/**
 * pm8001_chip_phy_stop_req - start phy via PHY_STOP COMMAND
 * @pm8001_ha: our hba card information.
 * @num: the inbound queue number
 * @phy_id: the phy id which we wanted to start up.
 */
static int pm80xx_chip_phy_stop_req(struct pm8001_hba_info *pm8001_ha,
    u8 phy_id)
{
    struct phy_stop_req payload;
    struct inbound_queue_table *circularQ;
    int ret;
    u32 tag = 0x01;
    u32 opcode = OPC_INB_PHYSTOP;
    circularQ = &pm8001_ha->inbnd_q_tbl[0];
    memset(&payload, 0, sizeof(payload));
    payload.tag = cpu_to_le32(tag);
    //payload.phy_id = cpu_to_le32((1 << 8) | (phy_id & 0xFF));
    payload.phy_id = cpu_to_le32(phy_id);
    ret = pm8001_mpi_build_cmd(pm8001_ha, circularQ, opcode, &payload, 0);
    return ret;
}

/**
 * see comments on pm8001_mpi_reg_resp.
 */
static int pm80xx_chip_reg_dev_req(struct pm8001_hba_info *pm8001_ha,
    struct pm8001_device *pm8001_dev, u32 flag)
{
    struct reg_dev_req payload;
    u32 opc;
    u32 stp_sspsmp_sata = 0x4;
    struct inbound_queue_table *circularQ;
    u32 linkrate, phy_id;
    int rc, tag = 0xdeadbeef;
    struct pm8001_ccb_info *ccb;
    u8 retryFlag = 0x1;
    u16 firstBurstSize = 0;
    u16 ITNT = 2000;
    struct domain_device *dev = pm8001_dev->sas_device;
    struct domain_device *parent_dev = dev->parent;
    circularQ = &pm8001_ha->inbnd_q_tbl[0];

    memset(&payload, 0, sizeof(payload));
    rc = pm8001_tag_alloc(pm8001_ha, &tag);
    if (rc)
        return rc;
    ccb = &pm8001_ha->ccb_info[tag];
    ccb->device = pm8001_dev;
    ccb->ccb_tag = tag;
    payload.tag = cpu_to_le32(tag);
    
        if (flag == 1)
    {
        stp_sspsmp_sata = 0x02; /*direct attached sata */
    }
    else {
        #if (LINUX_VERSION_CODE < KERNEL_VERSION(3,10,0) )
        if (pm8001_dev->dev_type == SATA_DEV)
            stp_sspsmp_sata = 0x00; /* stp*/
        else if (pm8001_dev->dev_type == SAS_END_DEV ||
            pm8001_dev->dev_type == EDGE_DEV ||
            pm8001_dev->dev_type == FANOUT_DEV)
            stp_sspsmp_sata = 0x01; /*ssp or smp*/
        #else
        if (pm8001_dev->dev_type == SAS_PHY_UNUSED)
                stp_sspsmp_sata = 0x00; /* stp*/
        else if (pm8001_dev->dev_type == SAS_END_DEVICE  ||
            pm8001_dev->dev_type == SAS_EDGE_EXPANDER_DEVICE  ||
            pm8001_dev->dev_type == SAS_FANOUT_EXPANDER_DEVICE )
            stp_sspsmp_sata = 0x01; /*ssp or smp*/
        #endif
    }
    if (parent_dev && DEV_IS_EXPANDER(parent_dev->dev_type))
        phy_id = parent_dev->ex_dev.ex_phy->phy_id;
    else
        phy_id = pm8001_dev->attached_phy;
    
        opc = OPC_INB_REG_DEV;

    linkrate = (pm8001_dev->sas_device->linkrate < dev->port->linkrate) ?
            pm8001_dev->sas_device->linkrate : dev->port->linkrate;

    payload.phyid_portid =
        cpu_to_le32(((pm8001_dev->sas_device->port->id) & 0xFF) |
        ((phy_id & 0xFF) << 8));

    payload.dtype_dlr_mcn_ir_retry = cpu_to_le32((retryFlag & 0x01) |
        ((linkrate & 0x0F) << 24) |
        ((stp_sspsmp_sata & 0x03) << 28));      
    payload.firstburstsize_ITNexustimeout =
        cpu_to_le32(ITNT | (firstBurstSize * 0x10000));

    memcpy(payload.sas_addr, pm8001_dev->sas_device->sas_addr,
        SAS_ADDR_SIZE);
        
    rc = pm8001_mpi_build_cmd(pm8001_ha, circularQ, opc, &payload, 0);

    return rc;
}

/**
 * pm80xx_chip_phy_ctl_req - support the local phy operation
 * @pm8001_ha: our hba card information.
 * @num: the inbound queue number
 * @phy_id: the phy id which we wanted to operate
 * @phy_op:
 */
static int pm80xx_chip_phy_ctl_req(struct pm8001_hba_info *pm8001_ha,
    u32 phyId, u32 phy_op)
{
    struct local_phy_ctl_req payload;
    struct inbound_queue_table *circularQ;
    int ret;
    u32 opc = OPC_INB_LOCAL_PHY_CONTROL;
    memset(&payload, 0, sizeof(payload));
    circularQ = &pm8001_ha->inbnd_q_tbl[0];
    payload.tag = cpu_to_le32(1);
    payload.phyop_phyid =
        cpu_to_le32(((phy_op & 0xFF) << 8) | (phyId & 0xFF));
    ret = pm8001_mpi_build_cmd(pm8001_ha, circularQ, opc, &payload, 0);
    return ret;
}

static u32 pm80xx_chip_is_our_interupt(struct pm8001_hba_info *pm8001_ha)
{
    u32 value;
#ifdef PM8001_USE_MSIX
    return 1;
#endif
    value = pm8001_cr32(pm8001_ha, 0, MSGU_ODR);
    if (value)
        return 1;
    return 0;

}

/**
 * pm8001_chip_isr - PM8001 isr handler.
 * @pm8001_ha: our hba card information.
 * @irq: irq number.
 * @stat: stat.
 */
static irqreturn_t
pm80xx_chip_isr(struct pm8001_hba_info *pm8001_ha, u8 vec)
{
    pm80xx_chip_interrupt_disable(pm8001_ha, vec);
    PM8001_DEVIO_DBG(pm8001_ha, 
        pm8001_printk("irq vec %d, ODMR:0x%x\n", vec, pm8001_cr32(pm8001_ha, 0, 0x30)));
    process_oq(pm8001_ha, vec);
    pm80xx_chip_interrupt_enable(pm8001_ha, vec);
    return IRQ_HANDLED;
}
  
 
/**
 * get_phy_profile
*/

int pm8001_chip_get_phy_profile(struct pm8001_hba_info *pm8001_ha, 
                        int phy_id, int page_code)
{
        
    u32 tag;                            
    struct get_phy_profile_req payload;
    struct inbound_queue_table *circularQ;      
    int rc,ppc_phyid;
    u32 opc = OPC_INB_GET_PHY_PROFILE ;
    
    memset(&payload, 0, sizeof(payload));

    rc = pm8001_tag_alloc(pm8001_ha, &tag);
    if (rc)
        PM8001_FAIL_DBG(pm8001_ha,pm8001_printk("Invalid tag\n"));
    
    circularQ = &pm8001_ha->inbnd_q_tbl[0];

    payload.tag = cpu_to_le32(tag);
    ppc_phyid = (page_code & 0xFF)  << 8 | ( phy_id & 0xFF );
    payload.ppc_phyid = cpu_to_le32(ppc_phyid);

    pm8001_mpi_build_cmd(pm8001_ha, circularQ, opc, &payload, 0);                              

    return rc;
}

    
/**
 * set_phy_profile
 
 **/

    
void mpi_set_phy_profile_req(struct pm8001_hba_info *pm8001_ha, u32 operation, u32 phyid, u32 length, u32 * buf)
{
    u32 tag ,i,j = 0;
    int rc;
    struct set_phy_profile_req payload;
    struct inbound_queue_table *circularQ;      
    u32 opc = OPC_INB_SET_PHY_PROFILE ;
    memset(&payload, 0, sizeof(payload));
    rc = pm8001_tag_alloc(pm8001_ha, &tag);
    if (rc)
        PM8001_INIT_DBG(pm8001_ha,pm8001_printk("Invalid tag\n"));
    circularQ = &pm8001_ha->inbnd_q_tbl[0];
    payload.tag = cpu_to_le32(tag);
    payload.ppc_phyid = (((operation & 0xF) << 8 ) | (phyid  & 0xFF));
    PM8001_INIT_DBG(pm8001_ha, 
            pm8001_printk(" phy profile command for phy %x ,length is %d\n",payload.ppc_phyid,length));
             
    for(i = length ; i <(length + PHY_DWORD_LENGTH - 1 ) ; i++ ) 
    {        
            payload.reserved[j] =  cpu_to_le32(*((u32*)buf + i));              
            PM8001_INIT_DBG(pm8001_ha,
            pm8001_printk("length is %d , phy is %d , buffer content %x and payload content is %x\n",length,phyid,*((u32*)buf + i),(u32)payload.reserved[j]));
            j++;
        }        
    pm8001_mpi_build_cmd(pm8001_ha, circularQ, opc, &payload, 0);                              
}

void pm8001_set_phy_profile(struct pm8001_hba_info *pm8001_ha,u32 length,u8 *buf)
{
    u32 page_code,i;                            
    page_code = SAS_PHY_ANALOG_SETTINGS_PAGE;
    for (i = 0; i < pm8001_ha->chip->n_phy; i++){        
            mpi_set_phy_profile_req(pm8001_ha,SAS_PHY_ANALOG_SETTINGS_PAGE,i,length,(u32*)buf);
            length = length + PHY_DWORD_LENGTH;           
            PM8001_INIT_DBG(pm8001_ha,pm8001_printk("phy is %d \n",i)); 
        }
    PM8001_INIT_DBG(pm8001_ha,pm8001_printk("phy settings completed\n"));       
}

/**
 * pm80xx_chip_gpio_req - support for GPIO operation
 * @pm8001_ha: our hba card information.
 * @ioctl_payload: the payload for the GPIO operation
 */
int pm80xx_chip_gpio_req(struct pm8001_hba_info *pm8001_ha, struct pm8001_gpio *gpio_payload)
{
    struct gpio_req payload;
    struct inbound_queue_table *circularQ;
    int ret;
    u32 tag;
    u32 opc = OPC_INB_GPIO;

    if(pm8001_ha->pdev->subsystem_vendor == PCI_VENDOR_ID_ADAPTEC2)
        return -1;

    ret = pm8001_tag_alloc(pm8001_ha, &tag);
    if (ret)
        return -1;

    memset(&payload, 0, sizeof(payload));
    circularQ = &pm8001_ha->inbnd_q_tbl[0];
    payload.tag = cpu_to_le32(tag);

    switch(gpio_payload->operation)
    {
    case GPIO_READ:
        payload.eobid_ge_gs_gr_gw = cpu_to_le32(GPIO_GR_BIT);
        break;

    case GPIO_WRITE:
        payload.eobid_ge_gs_gr_gw = cpu_to_le32(GPIO_GW_BIT);
        payload.gpio_wr_msk = cpu_to_le32(gpio_payload->mask);
        payload.gpio_wr_val = cpu_to_le32(gpio_payload->rd_wr_val);
        break;

    case GPIO_PINSETUP:
        payload.eobid_ge_gs_gr_gw = cpu_to_le32(GPIO_GS_BIT);
        payload.gpio_in_enabled = cpu_to_le32(gpio_payload->input_enable);
        payload.gpio_pinsetup1 = cpu_to_le32(gpio_payload->pinsetup1);
        payload.gpio_pinsetup2 = cpu_to_le32(gpio_payload->pinsetup2);
        break;

    case GPIO_EVENTSETUP:
        payload.eobid_ge_gs_gr_gw = cpu_to_le32(GPIO_GE_BIT);
        payload.gpio_evt_change = cpu_to_le32(gpio_payload->event_level);
        payload.gpio_evt_rise = cpu_to_le32(gpio_payload->event_rising_edge);
        payload.gpio_evt_fall = cpu_to_le32(gpio_payload->event_falling_edge);
        break;
    }
    ret = pm8001_mpi_build_cmd(pm8001_ha, circularQ, opc, &payload, 0);
    if(ret != 0)
        pm8001_tag_free(pm8001_ha, tag);
    return ret;
}


const struct pm8001_dispatch pm8001_80xx_dispatch = {
    .name                   = "pmc80xx",
    .chip_init              = pm80xx_chip_init,
    .chip_soft_rst          = pm80xx_chip_soft_rst,
    .chip_rst               = pm80xx_hw_chip_rst,
    .chip_iounmap           = pm8001_chip_iounmap,
    .isr                    = pm80xx_chip_isr,
    .is_our_interupt        = pm80xx_chip_is_our_interupt,
    .isr_process_oq         = process_oq,
    .interrupt_enable       = pm80xx_chip_interrupt_enable,
    .interrupt_disable      = pm80xx_chip_interrupt_disable,
    .make_prd               = pm8001_chip_make_sg,
    .smp_req                = pm80xx_chip_smp_req,
    .ssp_io_req             = pm80xx_chip_ssp_io_req,
    .sata_req               = pm80xx_chip_sata_req,
    .phy_start_req          = pm80xx_chip_phy_start_req,
    .phy_stop_req           = pm80xx_chip_phy_stop_req,
    .reg_dev_req            = pm80xx_chip_reg_dev_req,
    .dereg_dev_req          = pm8001_chip_dereg_dev_req,
    .phy_ctl_req            = pm80xx_chip_phy_ctl_req,
    .task_abort             = pm8001_chip_abort_task,
    .ssp_tm_req             = pm8001_chip_ssp_tm_req,
    .get_nvmd_req           = pm8001_chip_get_nvmd_req,
    .set_nvmd_req           = pm8001_chip_set_nvmd_req,
    .fw_flash_update_req    = pm8001_chip_fw_flash_update_req,
    .get_dev_state_req      = pm8001_chip_get_dev_state_req,
    .set_dev_state_req      = pm8001_chip_set_dev_state_req,
    .sas_diag_start_end_req = pm8001_chip_sas_diag_start_end_req,
    .sas_diag_execute_req   = pm8001_chip_sas_diag_execute_req,
    .set_phy_state_req      = pm8001_chip_set_phy_state_req,
    .sas_re_init_req        = pm8001_chip_sas_re_initialization,
    .gpio_req               = pm80xx_chip_gpio_req,
    .smp_util_req           = pm80xx_chip_smputil_req,
    .sgpio_req              = pm8001_chip_sgpio_req,
    .get_phy_profile_req    = pm8001_chip_get_phy_profile,
    .ssp_util_req           = pm80xx_chip_ssp_util_req,
};
